

;	THE ARITHMETIC ROUTINES

;	THE 'E-FORMAT TO FLOATING-POINT' SUBROUTINE
;(Offset 3C - see CALCULATE below: 'e_to_fp')
;This subroutine gives a 'last value' on the top of the calc stack that is the
;result of converting a number given in the form xEm, where m is a positive
;or negative integer. The subroutine is entered with x at the top of the calc
;stack and m in the A reg.
;  The method used is to find the absolute value of m, say p, and to multiply
;or divide x by 10^p according to whether m is positive or negative.
;  To achieve this, p is shifted right until it is zero, and x is multiplied
;or divided by 10^(2^n) for each set bit b(n) of p. Since p is never much more
;than decimal 39, bits 6 & 7 of p will not normally be set.
L2D4F:;E_TO_FP
	RLCA			;Test the sign of m by rotating bit 7 of A
	RRCA			;into the carry without changing A.
	JR	NC,L2D55;E_SAVE	;Jump if m is positive.
	CPL			;Negate m in A without
	INC	A		;disturbing the carry flag.
L2D55:;E_SAVE
	PUSH	AF		;Save m in A briefly.
	LD	HL,L5C92	;This is MEMBOT: a sign flag is now stored
	CALL	L350B;FP_0/1	;in the 1st byte of mem_0, i.e. 0 for '+'
				;and 1 for '-'.
	RST	28H;FP_CALC	;The stack holds x.
	DB	0A4H;stk_ten	;x, 10
	DB	38H;end_calc
	POP	AF		;Restore m in A.
L2D60:;E_LOOP
	SRL	A		;In the loop, shift out the next bit of m,
				;modifying the carry and zero flags
	JR	NC,L2D71;E_TST_END ;appropriately; jump if carry reset.
	PUSH	AF		;Save the rest of m and the flags.
	RST	28H;FP_CALC	;The stack holds x' and 10^(2^n), where x'
				;is an interim stage in the multiplication
				;of x by 10^m, and n=0,1,2,3,4 or 5.
	DB	0C1h;st_mem_1	;(10^(2^n) is copied to mem_1)
	DB	0E0H;get_mem_0	;x', 10^(2^n), (1/0)
	DB	00H;jump_true	;x', 10^(2^n)
	DB	04H;to E_DIVSN	;x', 10^(2^n)
	DB	04H;multiply	;x' * 10^(2^n) = x"
	DB	33H;jump	;x"
	DB	02H;to E_FETCH	;x"
L2D6D:;E_DIVSN
	DB	05H;division	;x' / 10^(2^n) = x" (x" is N'*10^(2^n) or 
				;x' / 10^(2^n) according as m is '+' or '-')
L2D6E:;E_FETCH
	DB	0E1H;get_mem_1	;x", 10^(2^n)
	DB	38H;end_calc	;x", 10^(2^n)
	POP	AF		;Restore the rest of m in A, and the flags.
L2D71:;E_TST_END
	JR	Z,L2D7B;E_END	;Jump if m has been reduced to zero.
	PUSH	AF		;Save the rest of m in A.
	RST	28H;FP_CALC	;x'', 10^(2^n)
	DB	31H;duplicate	;x'', 10^(2^n), 10^(2^n)
	DB	04H;multiply	;x'', 10^(2^(n+1))
	DB	38H;end_calc
	POP	AF		;Restore the rest of m in A.
	JR	L2D60;E_LOOP	;Jump back for all bits of m.
L2D7B:;E_END
	RST	28H;FP_CALC	;Use the calc to delete the
	DB	02H;delete	;final power of 10 reached,
	DB	38H;end_calc	;leaving the 'last value' x*10^m
	RET			;on the stack.

;	THE 'INT_FETCH' SUBROUTINE
;This subroutine collects in DE a small integer n (-65535<=n<=65535) from
;the loc addressed by HL: i.e. n is normally the 1st (or 2nd) number at the
;top of the calc stack; but HL can also access (by exchange with DE) a
;number which has been deleted from the stack.
;  The subroutine does not itself delete the number from the stack or from
;memory; it returns HL pointing to the fourth byte of the number in its
;original position.
L2D7F:;INT_FETCH
	INC	HL		;Point to the sign byte of the number.
	LD	C,(HL)		;Copy the sign byte to C.

;The following mechanism will two's complement the number if it is negative
;(C is FF) but leave it unaltered if it is positive (C is 00).
	INC	HL		;Point to the less significant byte.
	LD	A,(HL)		;Collect the byte in A.
	XOR	C		;Ones complement it if negative.
	SUB	C		;This adds 1 for negative numbers;
				;it sets the carry unless the byte was 0.
	LD	E,A		;Less significant byte to E now.
	INC	HL		;Point to the more significant byte.
	LD	A,(HL)		;Collect it in A.
	ADC	A,C		;Finish two complementing in the case of
	XOR	C		;a negative number; note that the carry is
				;always left reset.
	LD	D,A		;More significant byte to D now.
	RET			;Finished.

;	THE 'INT_STORE' SUBROUTINE
;This subroutine stores a small integer n (-65535<=n<=65535) in the loc
;addressed by HL and the four following locs: i.e. n  replaces the 1st
;(or 2nd) number at the top of the calc stack.
;  The subroutine returns HL pointing to the 1st byte of n on the stack.
L2D8C:;P_INT_STO
	LD	C,0		;This entry point would store a number
				;known to be positive.
L2D8E:;INT_STORE
	PUSH	HL		;The pntr to the 1st loc is saved.
	LD	(HL),0		;The 1st byte is set to zero.
	INC	HL		;Point to the 2nd loc.
	LD	(HL),C		;Enter the sign byte.

;The same mechanism is now used as in 'INT_FETCH' to twos complement negative
;numbers. This is needed e.g. before and after the multiplication of small
;integers. Addition is however performed without any further twos
;complementing before or afterwards.
	INC	HL		;Point to the 3rd loc.
	LD	A,E		;Collect the less significant byte.
	XOR	C		;Twos complement it if the
	SUB	C		;number is negative.
	LD	(HL),A		;Store the byte.
	INC	HL		;Point to the 4th loc.
	LD	A,D		;Collect the more significant byte.
	ADC	A,C		;Twos complement it if the
	XOR	C		;number is negative.
	LD	(HL),A		;Store the byte.
	INC	HL		;Point to the 5th loc.
	LD	(HL),0		;The 5th byte is set to zero.
	POP	HL		;Return with HL pointing to the
	RET			;1st byte of n on the stack.

;	THE 'FLOATING-POINT TO BC' SUBROUTINE
;This subroutine is called from four different places for various purposes
;and is used to compress the flpnt 'last value' into the BC reg pair.
;  If the result is too large, i.e. greater than 65535 dec, then the
;subroutine returns with the carry flag set. If the 'last value' is negative
;then the zero flag is reset.
;  The low byte of the result is also copied to the A reg.
L2DA2:;FP_TO_BC
	RST	28H;FP_CALC	;Use the calc to make HL
	DB	38H;end_calc	;pointing to STKEND-5.
	LD	A,(HL)		;Collect the exponent byte of
	AND	A		;the 'last value'; jump if it is
	JR	Z,L2DAD;FP_DELETE ;zero, indicating a 'small integer'.
	RST	28H;FP_CALC	;Now use the calc to round
	DB	0A2H;stk_half	;the 'last value' to the nearest
	DB	0FH;addition	;integer, which also changes it to 'small
	DB	27H;int		;integer' form on the calc stack if that is
	DB	38H;end_calc	;possible i.e. if -65535.5<=x<65535.5.
L2DAD:;FP_DELETE
	RST	28H;FP_CALC	;Use the calc to delete the integer
	DB	02H;delete	;from the stack; DE still points to it in
	DB	38H;end_calc	;memory (at STKEND).
	PUSH	HL		;Save both stack pntrs.
	PUSH	DE
	EX	DE,HL		;HL now points to the number.
	LD	B,(HL)		;Copy the 1st byte to B.
	CALL	L2D7F;INT_FETCH	;Copy bytes 2,3 and 4 to C, E and D.
	XOR	A		;Clear the A reg.
	SUB	B		;This sets the carry unless B is zero.
	BIT	7,C		;This sets the zero flag if the number is
				;positive (NZ denotes negative).
	LD	B,D		;Copy the high byte to B.
	LD	C,E		;And the low byte to C.
	LD	A,E		;Copy the low byte to A too.
	POP	DE		;Restore the stack pntrs.
	POP	HL
	RET			;Finished.

;	THE 'LOG(2^A)' SUBROUTINE
;This subroutine is called by the 'PRINT_FP' subroutine to calculate the
;approximate number of digits before the decimal in x, the number to be
;printed, or, if there are no digits before the decimal, then the
;approximate number of leading zeros after the decimal. It is entered with
;the A reg containing e', the true exponent of x, or e'-2, and calculates
;z=log to the base 10 of (2^A). It then sets A equal to ABS INT (z+.5), as
;required, using FP_TO_A for this purpose.
L2DC1:;LOG(2^A)
	LD	D,A		;The integer A is stacked, either as 00 00
	RLA			;A 00 00 (for positive A) or as 00 FF A FF 00
	SBC	A,A		;(for negative A).
	LD	E,A		;These bytes are 1st loaded into A, E, D, C,
	LD	C,A		;B and then STK_STORE is called to put the
	XOR	A		;number on the calc stack.
	LD	B,A
	CALL	L2AB6;STK_STORE
	RST	28H;FP_CALC	;The calc is used.
	DB	34H;stk_data	;Log 2 to the base 1$ is now stacked.
	DB	0EFH;exponent+7F
	DB	1AH,20H,9AH,85H
	DB	04H;multiply	;A*log 2 i.e. log(2^A)
	DB	27H;int		;INT log(2^A)
	DB	38H;end_calc
;The subroutine continues on into FP_TO_A to complete the calculation.

;	THE 'FLOATING-POINT TO A' SUBROUTINE
;This short subroutine is called at least 8 times for various purposes. It
;uses the last but one subroutine, FP_TO_BC, to get the 'last value' into
;the A reg where this is possible. It therefore tests whether the modulus
;of the number rounds to more than 255 and if it does the subroutine returns
;with the carry flag set. Otherwise it returns with the modulus of the
;number, rounded to the nearest integer, in the A reg, and the zero flag set
;to imply that the number was positive, or reset to imply that it was
;negative.
L2DD5:;FP_TO_A
	CALL	L2DA2;FP_TO_BC	;Compress the 'last value' into BC.
	RET	C		;Return if out of range already.
	PUSH	AF		;Save the result and the flags.
	DEC	B		;Again it will be out of range
	INC	B		;if the B reg does not hold zero.
	JR	Z,L2DE1;FP_A_END ;Jump if in range.
	POP	AF		;Fetch the result and the flags.
	SCF			;Signal the result is out of range.
	RET			;Finished - unsuccessful.
L2DE1:;FP_A_END
	POP	AF		;Fetch the result and the flags.
	RET			;Finished - successful.

;	THE 'PRINT A FLOATING-POINT NUMBER' SUBROUTINE
;This subroutine is called by the PRINT command routine at 2039 and by STR$
;at 3630, which converts to a string the number as it would be printed. The
;subroutine prints x, the 'last value' on the calc stack. The print format
;never occupies more than 14 spaces.
;  The 8 most significant digits of x, correctly rounded, are stored in an
;ad hoc print buffer in mem_3 and mem_4. Small numbers, numerically less
;than 1, and large numbers, numerically greater than 2^27, are dealt with
;separately. The former are multiplied by 10^n, where n is the approximate
;number of leading zeros after the decimal, while the latter are divided
;by 10^(n-7), where n is the approximate number of digits before the decimal.
;This brings all numbers into the middle range, and the numbers of digits
;required before the decimal is built up in the 2nd byte of mem_5. Finally
;the printing is done, using E-format if there are more than 8 digits before
;the decimal or, for small numbers, more than 4 leading zeros after the
;decimal.
;  The following program shows the range of print formats:
;	10 FOR a=-11 TO 12: PRINT SGN a*9^a,: NEXT a
;   i. 1st the sign of x is taken care of:
;	If x is negative, the subroutine jumps to PF_NEGATIVE, takes ABS x
;and prints the minus sign.
;	If x is zero, x is deleted from the calc stack, a '0' ts printed
;and a return is made from the subroutine.
;	If x is positive, the subroutine just continues.
L2DE3:;PRINT_FP
	RST	28H;FP_CALC	;Use the calc.
	DB	31H;duplicate	;x, x
	DB	36H;less_0	;x, (1/0) Logical value of x.
	DB	00H;jump_true	;x
	DB	0BH;to PF_NEGATIVE
	DB	31H;duplicate	;x, x
	DB	37H;greater_0	;x, (1/0) Logical value of x.
	DB	00H;jump_true	;x
	DB	0DH;to PF_POSITIVE
	DB	02H;delete	;-
	DB	38H;end_calc
	LD	A,'0'		;Enter the char code for '0'.
	RST	10H;PRINT_A_1	;Print the '0'.
	RET			;Finished as the 'last value' is zero.
L2DF2:;PF_NEGATIVE
	DB	2Ah;abs		;x'=ABS x
	DB	38h;end_calc	;x'
	LD	A,'-'		;Enter the char code for '-'.
	RST	10H;PRINT_A_1	;Print the '-'.
	RST	28H;FP_CALC	;Use the calc again.
L2DF8:;PF_POSITIVE
	DB	0A0H;stk_zero	;The 15 bytes at mem_3, mem_4
	DB	0C3H;st_mem_3	;and mem_5 are now initialised to
	DB	0C4H;st_mem_4	;zero to be used for a print
	DB	0C5H;st_mem_5	;buffer and two counters.
	DB	02H;delete	;The stack is cleared, except for x'.
	DB	38H;end_calc	;x'
	EXX			;H'L', which is used to hold
	PUSH	HL		;calc offset (e.g. for
	EXX			;'STR$') is saved of the machine stack.

;  ii. This is the start of a loop which deals with large numbers. However
;every number is 1st split into its integer part i and the fractional part
;f. If i is a small integer, e.g. if -65535<=i<=65535, it is stored in D'E'
;for insertion into the print buffer.
L2E01:;PF_LOOP
	RST	28H;FP_CALC	;Use the calc again.
	DB	31H;duplicate	;x', x'
	DB	27H;int		;x', INT(x')=i
	DB	0C2H;st_mem_2	;(i is stored in mem_2)
	DB	03H;subtract	;x'-i=f
	DB	0E2H;get_mem_2	;f, i
	DB	01H;exchange	;i, f
	DB	0C2H;st_mem_2	;(f is stored in mem_2)
	DB	02H;delete	;i
	DB	38H;end_calc	;i
	LD	A,(HL)		;Is i a small integer (1st byte
	AND	A		;zero) i.e. if ABS i<=65535?
	JR	NZ,L2E56;PF_LARGE ;Jump if it is not.
	CALL	L2D7F;INT_FETCH	;i is copied to DE (i, like x',>=0).
	LD	B,10H		;B is set to count 16 bits.
	LD	A,D		;D is copied to A for testing:
	AND	A		;is it zero?
	JR	NZ,L2E1E;PF_SAVE ;Jump if it is not zero.
	OR	E		;Now test E.
	JR	Z,L2E24;PF_SMALL ;Jump if DE zero: it's a pure fraction.
	LD	D,E		;Move E to D and set B for 8 bits:
	LD	B,8		;D was zero and E was not.
L2E1E:;PF_SAVE
	PUSH	DE		;Transfer DE to D'E' via the
	EXX			;machine stack, to be moved
	POP	DE		;into the print buffer at
	EXX			;PF_BITS.
	JR	L2E7B;PF_BITS	;Jump forward.

; iii. Pure fractions are multiplied by 10^n, where n is the approximate
;number of leading zeros after the decimal; and -n is added to the 2nd byte
;of mem_5, which holds the number of digits needed before the decimal; a
;negative number here indicates leading zeros after the decimal;
L2E24:;PF_SMALL
	RST	28H;FP_CALC	;i (i=zero here).
	DB	0E2H;get_mem_2	;i, f
	DB	38H;end_calc
;Note that the stack is now unbalanced. An extra byte 'DB 2;delete' is needed
;at 2E25, immediately after the RST 28h. Now an expression like "2"+STR$ .5
;is evaluated incorrectly as 0.5; the zero left on the stack displaces the
;"2" and is treated as a null string. Similarly all the string comparisons
;can yield incorrect values if the 2nd string takes the form STR$ x where
;x is numerically less than 1; e.g. the expression "50"<STR$ 0.1 yields
;the logical value "true"; once again "" is used instead of "50".
	LD	A,(HL)		;The exponent byte e of f is copied to A.
	SUB	7EH		;A becomes e-126 dec i.e. e'+2, where e'
				;is the true exponent of f.
	CALL	L2DC1;LOG(2^A)	;The construction A=ABS INT(LOG(2^A)) is
				;performed (LOG is to base 10); i.e. A=n,
	LD	D,A		;say: n is copied from A to D.
	LD	A,(L5CAC);mem_5_2nd ;The current count is collected
	SUB	D		;from the 2nd byte of mem_5
	LD	(L5CAC),A;mem_5_2nd ;and n is subtracted from it.
	LD	A,D		;n is copied from D to A.
	CALL	L2D4F;E_TO_FP	;y=f*10^n is formed and stacked.
	RST	28H;FP_CALC	;i, y
	DB	31H;duplicate	;i, y, y
	DB	27H;int		;i, y, INT(y)=i2
	DB	0C1H;st_mem_1	;(i2 is copied to mem_1)
	DB	03H;subtract	;i, y-i2
	DB	0E1H;get_mem_1	;i, y-i2, i2
	DB	38H;end_calc	;i, f2, i2 (f2=y-i2)
	CALL	L2DD5;FP_TO_A	;i2 is transferred from the stack to A.
	PUSH	HL		;The pntr to f2 is saved.
	LD	(L5CA1),A;mem_3_1st ;i2 is stored in the 1st byte of mem_3:
				;a digit for printing.
	DEC	A		;i2 will not count as a digit for
	RLA			;printing if it is zero; A is
	SBC	A,A		;manipulated so that zero will produce zero
	INC	A		;but a non-zero digit will produce 1.
	LD	HL,L5CAB	;The zero or one is inserted into
	LD	(HL),A		;the 1st byte of mem_5 (the no.
	INC	HL		;of digits for printing) and added
	ADD	A,(HL)		;to the 2nd byte of mem_5 (the number of
	LD	(HL),A		;digits before the decimal).
	POP	HL		;The pntr to f2 is restored.
	JP	L2ECF;PF_FRACTN	;Jump to store f2 in buffer (HL now points
				;to f2, DE to i2).

;  iv. Numbers greater than 2^27 are similarly multiplied by 2^(-n+7),
;reducing the number of digits before the decimal to 8, and the loop is
;re-entered at PF_LOOP.
L2E56:;PF_LARGE
	SUB	80H		;e-80 hex=e', the true exponent of i.
	CP	1CH		;Is e' less than 28 dec?
	JR	C,L2E6F;PF_MEDIUM ;Jump if it is less.
	CALL	L2DC1;LOG(2^A)	;n is formed in A.
	SUB	7		;And reduced to n-7.
	LD	B,A		;Then copied to B.
	LD	HL,L5CAC	;n-7 is added in to the 2nd
	ADD	A,(HL)		;byte of mem_5, the number of
	LD	(HL),A		;digits required before the decimal in x.
	LD	A,B		;Then i is multiplied by 10^(-n+7).
	NEG			;This will bring it into medium
	CALL	L2D4F;E_TO_FP	;range for printing.
	JR	L2E01;PF_LOOP	;Round the loop again to deal with the
				;now medium-sited number.

;   v. The integer part of x is now stored in the print buffer in mem_3
;and mem_4.
L2E6F:;PF_MEDIUM
	EX	DE,HL		;DE now points to i, HL to f.
	CALL	L2FBA;FETCH_TWO	;The mantissa of i is now in D', E', D, E.
	EXX			;Get the exchange regs.
	SET	7,D		;True numerical bit 7 to D'.
	LD	A,L		;Exponent byte e of i to A.
	EXX			;Back to the main regs.
	SUB	80H		;True exponent e'=e-80 hex to A.
	LD	B,A		;This gives the required bit count.

;Note that the case where i is a small integer (<65536) re-enters here.
L2E7B:;PF_BITS
	SLA	E		;The mantissa of i is now rotated
	RL	D		;left and all the bits of i are thus
	EXX			;shifted into mem_4 and each byte of
	RL	E		;mem_4 is decimal adjusted at each shift.
	RL	D		;All four bytes of i.
	EXX			;Back to the main regs.
	LD	HL,L5CAA	;Addr of the 5th byte of mem_4 to HL;
	LD	C,5		;count of 5 bytes to C.
L2E8A:;PF_BYTES
	LD	A,(HL)		;Get the byte of mem_4.
	ADC	A,A		;Shift it left, taking in the new bit.
	DAA			;Decimal adjust the byte.
	LD	(HL),A		;Restore it to mem_4.
	DEC	HL		;Point to next byte of mem_4.
	DEC	C		;Decrease the byte count by one.
	JR	NZ,L2E8A;PF_BYTES ;Jump for each byte of mem_4.
	DJNZ	L2E7B;PF_BITS	;Jump for each bit of INT(x).

;Decimal adjusting each byte of mem_4 gave 2 dec digits per byte, there
;being at most 9 digits. The digits will now be re-packedd, one to a byte,
;in mem_3 and mem_4, using the instruction RLD.
	XOR	A		;A is cleared to receive the digits.
	LD	HL,L5CA6	;Source addr: 1st byte of mem_4.
	LD	DE,L5CA1	;Destination addr: 1st byte of mem_3.
	LD	B,9		;There are at most 9 digits.
	RLD			;The left nibble of mem_4 is discarded.
	LD	C,0FFH		;FF in C will signal a leading zero,
				;00 will signal a non-leading zero.
L2EA1:;PF_DIGITS
	RLD			;Left nibble of (HL) to A, right nibble
				;of (HL) to left.
	JR	NZ,L2EA9;PF_INSERT ;Jump if digit in A is not zero.
	DEC	C		;Test for a leading zero:
	INC	C		;it will now give zero reset.
	JR	NZ,L2EB3;PF_TEST_2 ;Jump if it was a leading zero.
L2EA9:;PF_INSERT
	LD	(DE),A		;Insert the digit now.
	INC	DE		;Point to next destination.
	INC	(IY+71H);mem_5_1st ;One more digit for printing, and
	INC	(IY+72H);mem_5_2nd ;one more before the decimal.
	LD	C,0		;Change the flag from leading zero
				;to other zero.
L2EB3:;PF_TEST_2
	BIT	0,B		;The source pntr needs to be
	JR	Z,L2EB8;PF_ALL_9 ;incremented on every 2nd passage
	INC	HL		;through the loop, when B is odd.
L2EB8:;PF_ALL_9
	DJNZ	L2EA1;PF_DIGITS	;Jump back for all 9 digits.
	LD	A,(L5CAB);mem_5_1st ;Get counter: were there 9 digits
	SUB	9		;excluding leading zeros?
	JR	C,L2ECB;PF_MORE	;If not, jump to get more digits.
	DEC	(IY+71H);mem_5_1st ;Prepare to round: reduce count to 8.
	LD	A,4		;Compare 9th digit, byte 4 of mem_4,
	CP	(IY+6FH);mem_4_4th ;with 4 to set carry for rounding up.
	JR	L2F0C;PF_ROUND	;Jump forward to round up.
L2ECB:;PF_MORE
	RST	28H;FP_CALC	;Use the calc again.
	DB	02H;delete	;- (i is now deleted)
	DB	0E2H;get_mem_2	;f
	DB	38H;end_calc

;  vi. The fractional part of x is now stored in the print buffer.
L2ECF:;PF_FRACTN
	EX	DE,HL		;DE now points to f.
	CALL	L2FBA;FETCH_TWO	;The mantissa of f is now in D', E', D, E.
	EXX			;Get the exchange regs.
	LD	A,80H		;The exponent of f is reduced to zero,
	SUB	L		;by shifting the bits of f 80 hex-e
	LD	L,0		;places right, where L' contained e.
	SET	7,D		;True numerical bit to bit 7 of D'.
	EXX			;Restore the main regs.
	CALL	L2FDD;SHIFT_FP	;Now make the shift.
L2EDF:;PF_FRN_LP
	LD	A,(IY+71H);mem_5_1st ;Get the digit count.
	CP	8		;Are there already 8 digits?
	JR	C,L2EEC;PR_FR_DGT ;If not, jump forward.
	EXX			;If 8 digits, just use f to round i up,
	RL	D		;rotating D' left to set the carry.
	EXX			;Restore main regs and jump
	JR	L2F0C;PF_ROUND	;forward to round up.
L2EEC:;PF_FR_DGT
	LD	BC,200h		;Initial zero to C, count of 2 to B.
L2EEF:;PF_FR_EXX
	LD	A,E		;D'E'DE is multiplied by 10 in 2 stages,
	CALL	L2F8B;CA=10*A+C	;1st DE then D'E', each
	LD	E,A		;byte by byte in 2 steps, and the
	LD	A,D		;integer part of the result is
	CALL	L2F8B;CA=10*A+C	;obtained in C to be passed into
	LD	D,A		;the print buffer.
	PUSH	BC		;The count and the result
	EXX			;alternate between BC and B'C'.
	POP	BC
	DJNZ	L2EEF;PF_FR_EXX	;Look back once through the
				;exchange regs.
	LD	HL,L5CA1	;The start - 1st byte of mem_3.
	LD	A,C		;Result to A for storing.
	LD	C,(IY+71H);mem_5_1st ;Count of digits so far in number to C.
	ADD	HL,BC		;Addr the 1st empty byte.
	LD	(HL),A		;Store the next digit.
	INC	(IY+71H);mem_5_1st ;Step up the count of digits.
	JR	L2EDF;PF_FRN_LP	;Loop back until there are 8 digits.
; vii. The digits stored in the print buffer are rounded to a maximum of 8
;digits for printing.
L2F0C:;PF_ROUND
	PUSH	AF		;Save the carry flag for the rounding.
	LD	HL,L5CA1	;Base addr of number: mem_3 byte 1.
	LD	C,(IY+71H);mem_5_1st ;Offset (number of digits in
	LD	B,0		;number) to BC.
	ADD	HL,BC		;Address the last byte of the number.
	LD	B,C		;Copy C to B as the counter.
	POP	AF		;Restore the carry flag.
L2F18:;PF_RND_LP
	DEC	HL		;This is the last byte of the number.
	LD	A,(HL)		;Get the byte into A.
	ADC	A,0		;Add in the carry i.e. round up.
	LD	(HL),A		;Store the rounded byte in the buffer.
	AND	A		;If the byte is 0 or 10, B will be
	JR	Z,L2F25;PF_R_BACK ;decremented and the final zero (or
	CP	0AH		;the 10) will not be counted for printing.
	CCF			;Reset the carry for a valid digit.
	JR	NC,L2F2D;PF_COUNT ;Jump if carry reset.
L2F25:;PF_R_BACK
	DJNZ	L2F18;PF_RND_LP	;Jump back for more rounding or more
				;final zeros.
	LD	(HL),1		;There is overflow to the left; an extra
	INC	B		;1 is needed here. It is also an extra
	INC	(IY+72H);mem_5_2nd ;digit before the decimal.
L2F2D:;PF_COUNT
	LD	(IY+71H),B;mem_5_1st ;B now sets the count of the digits to
			     ;be printed (final zeros will not be printed).
	RST	28H;FP_CALC	;1 is to be deleted.
	DB	02H;delete
	DB	38H;end_calc
	EXX			;The calculator offset saved on
	POP	HL		;the stack is restored to HL'.
	EXX
;viii. The number can now be printed. First C will be set to hold the number
;of digits to be printed, not counting final zeros, while B will hold the
;number of digits required before the decimal.
	LD	BC,(L5CAB);mem_5_1st ;The counters are set.
	LD	HL,L5CA1;mem_3_1st ;The start of the digits.
	LD	A,B		;If more than 9, or fewer than minus 4,
	CP	9		;digits are required before the decimal,
	JR	C,L2F46;PF_NOT_E ;then E-format will be needed.
	CP	0FCH		;Fewer than 4 means more than 4 leading
	JR	C,L2F6C;PF_E_FRMT ;zeros after the decimal.
L2F46:;PF_NOT_E
	AND	A		;Are there no digits before the decimal?
	CALL	Z,L15EF;OUT_CODE ;If so, print an initial zero.
;
; The next entry point is also used to print the digits needed for E-format
;printing.
L2F4A:;PF_E_SBRN
	XOR	A		;Start by setting A to zero.
	SUB	B		;Subtract B: minus will mean there are
	JP	M,L2F52;PF_OUT_LP ;digits before the decimal, jump
				;forward to print them.
	LD	B,A		;A is now required as a counter. Jump
	JR	L2F5E;PF_DC_OUT	;forward to print the decimal part.
L2F52:;PF_OUT_LP
	LD	A,C		;Copy the number of digits to be printed
	AND	A		;to A. If A is 0, there are still final
	JR	Z,L2F59;PF_OUT_DT ;zeros to print (B is non-zero), so jump.
	LD	A,(HL)		;Get a digit from the print buffer.
	INC	HL		;Point to the next digit.
	DEC	C		;Decrease the count by one.
L2F59:;PF_OUT_DT
	CALL	L15EF;OUT_CODE	;Print the appropriate digit.
	DJNZ	L2F52;PF_OUT_LP	;Loop back until B is zero.
L2F5E:;PF_DC_OUT
	LD	A,C		;It is time to print the decimal, unless
	AND	A		;C is now zero; in that case,
	RET	Z		;return - finished.
	INC	B		;Add 1 to B - include the decimal.
	LD	A,'.'		;Put the code for '.' to A.
L2F64:;PF_DEC_0S
	RST	10H;PRINT_A_1	;Print it.
	LD	A,'0'		;Enter the char code for zero.
	DJNZ	L2F64;PF_DEC_0S	;Loop back to print all needed zeros.
	LD	B,C		;Set the count for all remaining digits.
	JR	L2F52;PF_OUT_LP	;Jump back to print them.
L2F6C:;PF_E_FRMT
	LD	D,B		;The count of digits is copied to D.
	DEC	D		;It is decremented to give the exponent.
	LD	B,1		;One digit is required before the decimal
				;in E-format. All the part of the number
	CALL	L2F4A;PF_E_SBRN	;before the 'E' is now printed.
	LD	A,'E'
	RST	10H;PRINT_A_1	;Print the 'E'.
	LD	C,D		;Exponent to C now for printing.
	LD	A,C		;And to A for testing.
	AND	A		;Its sign is tested.
	JP	P,L2F83;PF_E_POS ;Jump if it is positive.
	NEG			;Otherwise, negate it in A.
	LD	C,A		;Then copy it back to C for printing.
	LD	A,'-'
	JR	L2F85;PF_E_SIGN	;Jump to print the sign.
L2F83:;PF_E_POS
	LD	A,'+'
L2F85:;PF_E_SIGN
	RST	10H;PRINT_A_1	;Now print the sign: '+' or '-'.
	LD	B,0		;BC holds the exponent for printing.
	JP	L1A1B;OUT_NUM_1	;Jump back to print it and finish.

;	THE 'CA=10*A+C' SUBROUTINE
;This subroutine is called by the PRINT_FP subroutine to multiply each byte
;of D'E'DE by 10 and return the integer part of the result in the C reg. On
;entry, the A reg contains the byte to be multiplied by 10 and the C reg
;contains the carry over from the previous byte. On return, the A reg
;contains the resulting byte and the C reg the carry forward to the next
;byte.
L2F8B:;CA=10*A+C
	PUSH	DE		;Save whichever DE pair is in use.
	LD	L,A		;Copy the multiplicand from A
	LD	H,0		;to HL.
	LD	E,L		;Copy it to DE too.
	LD	D,H
	ADD	HL,HL		;Double HL.
	ADD	HL,HL		;Double it again.
	ADD	HL,DE		;Add in DE to give HL=5*A.
	ADD	HL,HL		;Double again: now HL=10*A.
	LD	E,C		;Copy C to DE (D is zero) for addition.
	ADD	HL,DE		;Now HL=10*A+C.
	LD	C,H		;H is copied to C.
	LD	A,L		;L is copied to A, completing the task.
	POP	DE		;The DE reg pair is restored.
	RET			;Finished.

