
;	THE 'PREPARE TO ADD' SUBROUTINE
;This subroutine is the 1st of four subroutines that are used by the main
;arithmetic operation routines - SUBTRACTION, ADDITION, MULTIPLICATION and
;DIVISION.
;  This particular subroutine prepares a flpnt number for addition, mainly
;by replacing the sign bit with a true numerical bit 1, and negating the
;number (two's complement) if it is negative. The exponent is returned in
;the A reg and the 1st byte is set to 0 for a positive number and FF hex
;for a negative number.
L2F9B:;PREP_ADD
	LD	A,(HL)		;Transfer the exponent to A.
	LD	(HL),0		;Presume a positive number.
	AND	A		;If the number is zero then the
	RET	Z		;preparation is already finished.
	INC	HL		;Now point to the sign byte.
	BIT	7,(HL)		;Set the zero flag for positive number.
	SET	7,(HL)		;Restore the true numeric bit.
	DEC	HL		;Point to the 1st byte again.
	RET	Z		;Positive numbers have been prepared,
				;but negative numbers need to be twos
				;complemented.
	PUSH	BC		;Save any earlier exponent.
	LD	BC,5		;There are 5 bytes to be handled.
	ADD	HL,BC		;Point one-past the last byte.
	LD	B,C		;Transfer the '5' to B.
	LD	C,A		;Save the exponent in C.
	SCF			;Set carry flag for negation.
L2FAF:;NEG_BYTE
	DEC	HL		;Point to each byte in turn.
	LD	A,(HL)		;Get each byte.
	CPL			;One's complement the byte.
	ADC	A,0		;Add in carry for negation.
	LD	(HL),A		;Restore the byte.
	DJNZ	L2FAF;NEG_BYTE	;Loop the '5' times.
	LD	A,C		;Restore the exponent to A.
	POP	BC		;Restore any earlier exponent.
	RET			;Finished.

;	THE 'FETCH TWO NUMBERS' SUBROUTINE
;This subroutine is called by ADDITION, MULTIPLICATION and DIVISION to get
;two numbers from the calc stack and put them into the reg, including the
;exchange regs.
;  On entry to the subroutine the HL reg pair points to the 1st byte of the
;1st number and the DE reg pair points to the 1st byte of the 2nd number.
;  When the subroutine is called from MULTIPLICATION or DIVISION the sign
;of the result is saved in the 2nd byte of the 1st number.
L2FBA:;FETCH_TWO
	PUSH	HL		;HL is preserved.
	PUSH	AF		;AF is preserved.

;Call the five bytes of the 1st number - M1, M2, M3, M4 & M5.
;and the 2nd number		       - N1, N2, N3, N4 & N5.
	LD	C,(HL)		;M1 to C.
	INC	HL		;Next.
	LD	B,(HL)		;M2 to B.
	LD	(HL),A		;Copy the sign of the result to (HL).
	INC	HL		;Next.
	LD	A,C		;M1 to A.
	LD	C,(HL)		;M3 to C.
	PUSH	BC		;Save M2 & M3 on the machine stack.
	INC	HL		;Next.
	LD	C,(HL)		;M4 to C.
	INC	HL		;Next.
	LD	B,(HL)		;M5 to B.
	EX	DE,HL		;HL now points to N1.
	LD	D,A		;M1 to D.
	LD	E,(HL)		;N1 to E.
	PUSH	DE		;Save M1 & N1 on the machine stack.
	INC	HL		;Next.
	LD	D,(HL)		;N2 to D.
	INC	HL		;Next.
	LD	E,(HL)		;N3 to E.
	PUSH	DE		;Save N2 & N3 on the machine stack.
	EXX			;Get the exchange regs.
	POP	DE		;N2 to D' & N3 to E'.
	POP	HL		;M1 to H' & N1 to L'.
	POP	BC		;M2 to B' & M3 to C'.
	EXX			;Get the original set of regs.
	INC	HL		;Next.
	LD	D,(HL)		;N4 to D.
	INC	HL		;Next.
	LD	E,(HL)		;N5 to E.
	POP	AF		;Restore the original AF.
	POP	HL		;Restore the original HL.
	RET			;Finished.
;Summary:	M1-M5 are in H', B', C', C, B.
;		N1-N5 are in L', D', E', D, E.
;		HL points to the 1st byte of the 1st number.

;	THE 'SHIFT ADDEND' SUBROUTINE
;This subroutine shifts a flpnt number up to 32 dec, 20 hex, places right
;to line it up properly for addition. The number with the smaller exponent
;has been put in the addend position before this subroutine is called.
;Any overflow to the right, into the carry, is added back into the number.
;If the exponent difference is greater than 32 dec, or the carry ripples
;right back to the beginning of the number then the number is set to zero
;so that the addition will not alter the other number (the augend).
L2FDD:;SHIFT_FP
	AND	A		;If the exponent difference is
	RET	Z		;zero, the subroutine returns at
	CP	21h		;once. If the difference is greater
	JR	NC,L2FF9;ADDEND_0 ;than 20 hex, jump forward.
	PUSH	BC		;Save BC briefly.
	LD	B,A		;Transfer the exponent difference to B
				;to count the shifts right.
L2FE5:;ONE_SHIFT
	EXX			;Arithmetic shift right for L',
	SRA	L		;preserving the sign marker bits.
	RR	D		;Rotate right with carry D', E',
	RR	E		;D & E.
	EXX			;Thereby shifting the whole five
	RR	D		;bytes of the number to the right
	RR	E		;as many times as B counts.
	DJNZ	L2FE5;ONE_SHIFT	;Loop back until B reaches zero.
	POP	BC		;Restore the original BC.
	RET	NC		;Done if no carry to retrieve.
	CALL	L3004;ADD_BACK	;Retrieve carry.
	RET	NZ		;Return unless the carry rippled
				;right back. (In this case there is
				;nothing to add).
L2FF9:;ADDEND_0
	EXX			;Fetch L', D' & E'.
	XOR	A		;Clear the A reg.
L2FFB:;ZEROS_4/5
	LD	L,0		;Set the addend to zero in D', E',
	LD	D,A		;D & E, together with its marker
	LD	E,L		;byte (sign indicator) L', which
	EXX			;was 0 for a positive number and FF hex
	LD	DE,0		;for a negative number. ZEROS_4/5 produces
				;only 4 zero bytes when called for near
				;underflow at 3160.
	RET			;Finished.

;	THE 'ADD_BACK' SUBROUTINE
;This subroutine adds back into the number any carry which has overflowed
;to the right. In the extreme case, the carry ripples right back to the left
;of the number.
;  When this subroutine is called during addition, this ripple means that a
;mantissa of 0.5 was shifted a full 32 places right, and the addend will now
;be set to zero; when called from MULTIPLICATION, it means that the exponent
;must be incremented, and this may result in overflow.
L3004:;ADD_BACK
	INC	E		;Add carry to rightmost byte.
	RET	NZ		;Return if no overflow to left.
	INC	D		;Continue to the next byte.
	RET	NZ		;Return if no overflow to left.
	EXX			;Get the next byte.
	INC	E		;Increment it too.
	JR	NZ,L300D;ALL_ADDED ;Jump if no overflow.
	INC	D		;Increment the last byte.
L300D:;ALL_ADDED
	EXX			;Restore the original regs.
	RET			;Finished.

;	THE 'SUBTRACTION' OPERATION
;(Offset 03 - see CALCULATE below: 'subtract')
;This subroutine simply changes the sign of the subtrahend and carried on
;into ADDITION.
;  Note that HL points to the minuend and DE points to the subtrahend.
;(See ADDITION for more details.)
L300F:;SUBTRACT
	EX	DE,HL		;Exchange the pntrs.
	CALL	L346E;NEGATE	;Change the sign of the subtrahend.
	EX	DE,HL		;Exchange the pntrs back and continue
				;into ADDITION.

;	THE 'ADDITION' OPERATION
;(Offset 0F - see CALCULATE below: 'addition')
;The 1st of three major arithmetical subroutines, this subroutine carries out
;the flpnt addition of two numbers, each with a 4-byte mantissa and a 1-byte
;exponent. In these three subroutines, the two numbers at the top of the calc
;stack are added/multiplied/divided to give one number at the top of the calc
;stack, a 'last value'. HL points to the 2nd number from the top, the augend/
;multiplier/dividend. DE points to the number at the top of the calc stack,
;the addend/multiplicand/divisor. Afterwards HL points to the resultant 'last
;value' whose addr can also be considered to be STKEND-5.
;  But the addition subroutine 1st tests whether the 2 numbers to be added
;are 'small integers'. If they are, it adds them quite simply in HL and BC,
;and puts the result directly on the stack. No twos complementing is needed
;before or after the addition, since such numbers are held on the stack in
;twos complement form, ready for addition.
L3014:;addition
	LD	A,(DE)		;Test whether the 1st bytes of
	OR	(HL)		;both numbers are zero.
	JR	NZ,L303E;FULL_ADDN ;If not, jump for full addition.
	PUSH	DE		;Save the pntr to the 2nd number.
	INC	HL		;Point to the 2nd byte of the 1st
	PUSH	HL		;number and save that pntr too.
	INC	HL		;Point to the less significant byte.
	LD	E,(HL)		;Fetch it in E.
	INC	HL		;Point to the more significant byte.
	LD	D,(HL)		;Fetch it in D.
	INC	HL		;Move on to the 2nd byte of
	INC	HL		;the 2nd number.
	INC	HL
	LD	A,(HL)		;Fetch it in A (this is the sign byte).
	INC	HL		;Point to the less significant byte.
	LD	C,(HL)		;Fetch it in C.
	INC	HL		;Point to the more significant byte.
	LD	B,(HL)		;Fetch it in B.
	POP	HL		;Fetch the pntr to the sign byte of the 1st
	EX	DE,HL		;number; put it in DE, and the number in HL.
	ADD	HL,BC		;Perform the addition: result in HL.
	EX	DE,HL		;Result to DE, sign byte to HL.
	ADC	A,(HL)		;Add the sign bytes and the carry into A;
	RRCA			;this will detect any overflow.
	ADC	A,0		;A non-zero A now indicates overflow.
	JR	NZ,L303C;ADDN_OVFL ;Jump to reset the pntrs and to do
				;full addition.
	SBC	A,A		;Define the correct sign byte for the result.
	LD	(HL),A		;Store it on the stack.
	INC	HL		;Point to the next loc.
	LD	(HL),E		;Store the low byte of the result.
	INC	HL		;Point to the next loc.
	LD	(HL),D		;Store the high byte of the result.
	DEC	HL		;Move the pntr back to
	DEC	HL		;addr the 1st byte of the
	DEC	HL		;result.
	POP	DE		;Restore STKEND to DE.
	RET			;Finished.
;Note that the number 65536 decimal can arise here in the form 00 FF 00 00 00
;as the result of the addition of two smaller negative integers, e.g. -65000
;and -536. It is simply stacked in this form. This is a mistake. The Spectrum
;system cannot handle this number. Most functions threat it as zero, and it
;is printed as -1E-38, obtained by treating it as 'minus zero' in an
;illegitimatex format.
;  One possible remedy would be to test for this number at about byte 3032
;and, if it is present, to make the 2nd byte 80 hex and the 1st byte 91 hex,
;so producing the full five byte flpnt form of the number, i.e. 91 80 00 00
;00, which causes no problems. See also the remarks in 'truncate' below,
;before byte 3225, and the Appendix.
L303C:;ADDN_OVFL
	DEC	HL		;Restore the pntr to the 1st byte.
	POP	DE		;Restore the pntr to the 2nd byte.
L303E:;FULL_ADDN
	CALL	L3293;RE_ST_TWO	;Re-stack both numbers in full five-byte
        			;flpnt form.

;The full ADDITION subroutine 1st calls PREP_ADD for each number, then gets
;the two numbers from the calc stack and puts the one with the smaller
;exponent into the addend position. It then calls SHIFT_FP to shift the
;addend up to 32 decimal places right to line it up for addition. The actual
;addition is done in a few bytes, a single shift is made for carry (overflow
;to the left) if needed, the result is twos complemented if negative, and
;any arithmetic overflow is reported; otherwise the subroutine jumps to
;TEST_NORM to normalise the result and return it to the stack with the
;correct sign bit inserted into the 2nd byte.
	EXX			;Exchange the regs.
	PUSH	HL		;Save the next literal addr.
	EXX			;Exchange the regs.
	PUSH	DE		;Save pntr to the addend.
	PUSH	HL		;Save pntr to the augend.
	CALL	L2F9B;PREP_ADD	;Prepare the augend.
	LD	B,A		;Save its exponent in B.
	EX	DE,HL		;Exchange the pntrs.
	CALL	L2F9B;PREP_ADD	;Prepare the addend.
	LD	C,A		;Save its exponent in C.
	CP	B		;If the 1st exponent is smaller,
	JR	NC,L3055;SHIFT_LEN ;keep the 1st number in the
	LD	A,B		;addend position; otherwise
	LD	B,C		;change the exponents and the
	EX	DE,HL		;pntrs back again.
L3055:;SHIFT_LEN
	PUSH	AF		;Save the larger exponent in A.
	SUB	B		;The difference between the exponents is the
				;length of the shift right.
	CALL	L2FBA;FETCH_TWO	;Get the two numbers from the stack.
	CALL	L2FDD;SHIFT_FP	;Shift the addend right.
	POP	AF		;Restore the larger exponent.
	POP	HL		;HL is to point to the result.
	LD	(HL),A		;Store the exponent of the result.
	PUSH	HL		;Save the pntr again.
	LD	L,B		;M4 to H & M5 to L.
	LD	H,C		;(see FETCH_TWO).
	ADD	HL,DE		;Add the two right bytes.
	EXX			;N2 to H' & N3 to L'.
	EX	DE,HL		;(see FETCH_TWO).
	ADC	HL,BC		;Add left bytes withc carry.
	EX	DE,HL		;Result back in D'E'.
	LD	A,H		;Add H', L' and the carry; the
	ADC	A,L		;resulting mechanisms will ensure
	LD	L,A		;that a single shift right is called
	RRA			;if the sum of 2 positive numbers
	XOR	L		;has overflowed left, or the sum of 2
	EXX			;negative numbers has not overflowed left.
	EX	DE,HL		;The result is now in DED'E'.
	POP	HL		;Get the pntr to the exponent.
	RRA			;The test for shift (H', L' were 0 for
	JR	NC,L307C;TEST_NEG ;positive numbers and FF hex for negative
				;numbers).
	LD	A,1		;A counts a single shift right.
	CALL	L2FDD;SHIFT_FP	;The shift is called.
	INC	(HL)		;Add 1 to the exponent; this
	JR	Z,L309F;ADD_REP_6 ;may lead to arithmetic overflow.
L307C:;TEST_NEG
	EXX			;Test for negative result: get
	LD	A,L		;sign bit of L' into A (this now
	AND	80H		;correctly indicates the sign of
	EXX			;the result).
	INC	HL		;Store it in the 2nd byte
	LD	(HL),A		;position of the result on
	DEC	HL		;the calc stack. If it is zero, then do not
	JR	Z,L30A5;GO_NC_MLT ;twos complement the result.
	LD	A,E		;Get the 1st byte.
	NEG			;Negate it.
	CCF			;Complement the carry for continued
	LD	E,A		;negation, and store byte.
	LD	A,D		;Get the next byte.
	CPL			;Ones complement it.
	ADC	A,0		;Add in the carry for negation.
	LD	D,A		;Store the byte.
	EXX			;Proceed to get next byte into
	LD	A,E		;the A reg.
	CPL			;Ones complement it.
	ADC	A,0		;Add in the carry for negation.
	LD	E,A		;Store the byte.
	LD	A,D		;Get the last byte.
	CPL			;Ones complement it.
	ADC	A,0		;Add in the carry for negation.
	JR	NC,L30A3;END_COMPL ;Done if no carry.
	RRA			;Else, get .5 into mantissa and add 1 to
	EXX			;the exponent; this will be needed when two
	INC	(HL)		;negative numbers add to give an exact power
				;of 2, and it may lead to arithmetic ovfl.
L309F:;ADD_REP_6
	JP	Z,L31AD;REPORT_6 ;Give the error if required.
	EXX
L30A3:;END_COMPL
	LD	D,A		;Store the last byte.
	EXX
L30A5:;GO_NC_MLT
	XOR	A		;Clear the carry flag.
	JP	L3155;TEST_NORM	;Exit via TEST_NORM.

;	THE 'HL=HL*DE' SUBROUTINE
;This subroutine is called by 'GET_HL*DE' and by 'MULTIPLICATION' to perform
;the 16-bit multiplication as stated.
;  Any overflow of the 16 bits available is dealt with on return from the
;subroutine.
L30A9:;HL=HL*DE
	PUSH	BC		;BC is saved.
	LD	B,10H		;It is to be a 16 bit multiplication.
	LD	A,H		;A holds the high byte.
	LD	C,L		;C holds the low byte.
	LD	HL,0		;Initialise the result to zero.
L30B1:;HL_LOOP
	ADD	HL,HL		;Double the result.
	JR	C,L30BE;HL_END	;Jump if overflow.
	RL	C		;Rotate bit 7 of C into the carry.
	RLA			;Rotate the carry bit into bit 0 and bit 7
				;into the carry flag.
	JR	NC,L30BC;HL_AGAIN ;Jump if the carry flag is reset.
	ADD	HL,DE		;Otherwise add DE in once.
	JR	C,L30BE;HL_END	;Jump if overflow.
L30BC:;HL_AGAIN
	DJNZ	L30B1;HL_LOOP	;Until 16 passes have been made.
L30BE:;HL_END
	POP	BC		;Restore BC.
	RET			;Finished.

;	THE 'PREPARE TO MULTIPLY OR DIVIDE' SUBROUTINE
;Thims subroutine prepares a flpnt number for multiplication or division,
;returning with carry set if the number is zero, getting the sign of the
;result into the A reg, and replacing the sign bit in the number by the true
;numeric bit 1.
L30C0:;PREP_M/D
	CALL	L34E9;TEST_ZERO	;If the number is zero, return
	RET	C		;with the carry flag set.
	INC	HL		;Point to the sign byte.
	XOR	(HL)		;Get sign for result into A (like signs give
				;plus, unlike give minus); also reset the
				;carry flag.
	SET	7,(HL)		;Set the true numeric bit.
	DEC	HL		;Point to the exponent again.
	RET			;Return with carry flag reset.

;	THE 'MULTIPLICATION' OPERATION
;(Offset 04 - see CALCULATE below: 'multiply')
;This subroutine 1st tests whether the two numbers to be multiplied are
;'small integers'. If they are, it uses INT_FETCH to get them from the
;stack, HL=HL*DE to multiply them and INT_STORE to return the result to the
;stack. Any overflow of this 'short multiplication' (i.e. if the result is
;not itself a 'small integer') causes a jump to multiplication in full five
;byte flpnt form (see below).
L30CA:;multiply
	LD	A,(DE)		;Test whether the 1st bytes of
	OR	(HL)		;both numbers are zero.
	JR	NZ,L30F0;MULT_LONG ;If not, jump for 'long' multiplication.
	PUSH	DE		;Save the pntrs: to the 2nd number.
	PUSH	HL		;And to the 1st number.
	PUSH	DE		;And to the 2nd number yet again.
	CALL	L2D7F;INT_FETCH	;Fetch sign in C, number in DE.
	EX	DE,HL		;Number to HL now.
	EX	(SP),HL		;Number to stack, 2nd pntr to HL.
	LD	B,C		;Save 1st sign in B.
	CALL	L2D7F;INT_FETCH	;Fetch 2nd sign in C, number in DE.
	LD	A,B		;Form sign of result in A: like signs
	XOR	C		;give plus (00), unlike give minus (FF).
	LD	C,A		;Store sign of result in C.
	POP	HL		;Restore the 1st number to HL.
	CALL	L30A9;HL=HL*DE	;Perform the actual multiplication.
	EX	DE,HL		;Store the result in DE.
	POP	HL		;Restore the pntr to the 1st number.
	JR	C,L30EF;MULT_OFLW ;Jump on overflow to 'full' multiplication.
	LD	A,D		;These 5 bytes ensure that
	OR	E		;00 FF 00 00 00 is replaced by zero;
	JR	NZ,L30EA;MULT_RSLT ;that they should not be needed if this
	LD	C,A		;number were excluded from the system
				;(see after 303B above).
L30EA:;MULT_RSLT
	CALL	L2D8E;INT_STORE	;Now store the result on the stack.
	POP	DE		;Restore STKEND to DE.
	RET			;Finished.
L30EF:;MULT_OFLW
	POP	DE		;Restore the pntr to the 2nd number.
L30F0:;MULT_LONG
	CALL	L3293;RE_ST_TWO	;Re-stack both numbers in full five byte
				;flpnt form.

;The full MULTIPLICATION subroutine prepares the 1st number for
;multiplication by calling PREP_M/D, returning if it is zero; otherwise the
;2nd number is prepared by again calling PREP_M/D, and if it is zero the
;subroutine goes to set the result to zero. Next it fetches the two numbers
;from the calc stack and multiplies their mantissas in the usual way,
;rotating the 1st number (treated as the multiplier) right and adding in
;the 2nd number (the multiplicand) to the result whenever the multiplier
;bit is set. The exponents are then added together and checks are made for
;overflow and for underflow (giving the result zero). Finally, the result
;is normalised and returned to the calc stack with the correct sign bit in
;the 2nd byte.
	XOR	A		;A is set to 0 so that the sign of the
				;1st number will go into A.
	CALL	L30C0;PREP_M/D	;Prepare the 1st number, and return if
	RET	C		;zero. (Result already zero.)
	EXX     		;Exchange the regs.
	PUSH	HL		;Save the next literal addr.
	EXX			;Exchange the regs.
	PUSH	DE		;Save the pntr to the multiplicand.
	EX	DE,HL		;Exchange the pntrs.
	CALL	L30C0;PREP_M/D	;Prepare the 2nd number.
	EX	DE,HL		;Exchange the pntrs again.
	JR	C,L315D;ZERO_RSLT ;Jump forward if 2nd number is zero.
	PUSH	HL		;Save the pntr to the result.
	CALL	L2FBA;FETCH_TWO	;Get the two numbers from the stack.
	LD	A,B		;M5 to A (see FETCH_TWO).
	AND	A		;Prepare for a subtraction.
	SBC	HL,HL		;Initialise HL to zero for the result.
	EXX			;Exchange the regs.
	PUSH	HL		;Save M1 & N1 (see FETCH_TWO).
	SBC	HL,HL		;Also initialise H'L' for the result.
	EXX			;Exchange the regs.
	LD	B,21h		;B counts 33 decimal, 21 hex shifts.
	JR	L3125;STRT_MLT	;Jump forward into the loop.

;Now enter the multiplier loop.
L3114:;MLT_LOOP
	JR	NC,L311B;NO_ADD	;Jump forward to NO_ADD if no carry, i.e.
				;the multiplier bit was reset.
	ADD	HL,DE		;Else, add the multiplicand in D'E'DE
	EXX			;(see FETCH_TWO) into the result
	ADC	HL,DE		;being built up on H'L'HL.
	EXX
L311B:;NO_ADD
	EXX			;Whether multiplicand was added or not,
	RR	H		;shift result right in H'L'HL, i.e. the
	RR	L		;shift is done by rotating each byte with
	EXX			;carry, so that any bit that drops into the
	RR	H		;carry is picked up by the next byte, and
	RR	L		;the shift continued into B'C'CA.
L3125:;STRT_MLT
	EXX			;Shift right the multiplier in B'C'CA
	RR	B		;(see FETCH_TWO & above).
	RR	C		;A final bit dropping into the
	EXX			;carry will trigger another add of
	RR	C		;the multiplicand to the result.
	RRA
	DJNZ	L3114;MLT_LOOP	;Loop 33 times to get all the bits.
	EX	DE,HL		;Move the result from:
	EXX
	EX	DE,HL		;H'L'HL to D'E'DE.
	EXX

;Now add the exponents together.
	POP	BC		;Restore the exponents - M1 & N1.
	POP	HL		;Restore the pntr to the exponent byte.
	LD	A,B		;Get the sum of the two exponent bytes
	ADD	A,C		;in A, and the correct carry.
	JR	NZ,L313B;MAKE_EXPT ;If the sum equals zero then clear the
	AND	A		;carry; else leave it unchanged.
L313B:;MAKE_EXPT
	DEC	A		;Prepare to increase the
	CCF			;exponent by 80 hex.

;The rest of the subroutine is common to both MULTIPLICATION and DIVISION.
L313D:;DIVN_EXPT
	RLA			;These few bytes very cleverly
	CCF			;make the correct exponent byte.
	RRA			;Rotating left then right gets the exponent
				;byte (true exponent plus 80 hex) into A.
	JP	P,L3146;OFLW1_CLR ;If the sign flag is reset, no report
				;of arithmetic overflow needed.
	JR	NC,L31AD;REPORT_6 ;Report the overflow if carry reset.
	AND	A		;Clear the carry now.
L3146:;OFLW1_CLR
	INC	A		;The exponent byte is now complete;
	JR	NZ,L3151;OFLW2_CLR ;but if A is zero a further check
	JR	C,L3151;OFLW2_CLR ;for overflow is needed.
	EXX			;If there is no carry set and the result is
	BIT	7,D		;already in normal form (bit 7 of D' set)
	EXX			;then there is overflow to report; but if
	JR	NZ,L31AD;REPORT_6 ;bit 7 of D' is reset, the result is just
				;in range, i.e. just under 2**127.
L3151:;OFLW2_CLR
	LD	(HL),A		;Store the exponent byte, at last.
	EXX			;Pass the 5th result byte to A
	LD	A,B		;for the normalisation sequence,
	EXX			;i.e. the overflow from L into B'.

;The remainder of the subroutine deals with normalisation and is common to
;all the arithmetic routines.
L3155:;TEST_NORM
	JR	NC,L316C;NORMALISE ;If no carry then normalise now.
	LD	A,(HL)		;Else, deal with underflow (zero
	AND	A		;result) or near underflow
L3159:;NEAR_ZERO
	LD	A,80H		;(result 2**-128):
	JR	Z,L315E;SKIP_ZERO ;return exponent to A, test if A
L315D:;ZERO_RSLT
	XOR	A		;is zero (case 2**-128) and if so
L315E:;SKIP_ZERO
	EXX			;produce 2**-128 if number is
	AND	D		;normal; otherwise produce zero.
	CALL	L2FFB;ZEROS_4/5	;The exponent must then be set to zero
	RLCA			;(for zero) or 1 (for 2**-128).
	LD	(HL),A		;Restore the exponent byte.
	JR	C,L3195;OFLOW_CLR ;Jump if case 2**-128.
	INC	HL		;Otherwise, put zero into 2nd
	LD	(HL),A		;byte of result on the calc stack.
	DEC	HL
	JR	L3195;OFLOW_CLR	;Jump forward to transfer the result.

;The actual normalisation operation.
L316C:;NORMALISE
	LD	B,20h		;Normalise the result by up to 32
L316E:;SHIFT_ONE
	EXX			;dec, 20 hex, shifts left of
	BIT	7,D		;D'E'DE (with A adjoined) until
	EXX			;bit 7 of D' is set. A holds zero
	JR	NZ,L3186;NORML_NOW ;after addition so no precision is
	RLCA			;gained or lost; A holds the 5th
	RL	E		;byte from B' after multiplication
	RL	D		;or division; but as only
	EXX			;about 32 bits can be correct, no
	RL	E		;precision is lost. Note that A is
	RL	D		;rotated circularly, with branch
	EXX			;at carry... eventually a random process.
	DEC	(HL)		;The exponent is decremented on each shift.
	JR	Z,L3159;NEAR_ZERO ;If the exponent becomes zero, then number
				;from 2**-129 are rounded up to 2**-128.
	DJNZ	L316E;SHIFT_ONE	;Loop back, up to 32 times.
	JR	L315D;ZERO_RSLT	;If bit 7 never became 1 then the whole
				;result is to be zero.

;Finish the normalisation by considering the 'carry'.
L3186:;NORML_NOW
	RLA			;After normalisation add back
	JR	NC,L3195;OFLOW_CLR ;any final carry that went into A.
	CALL	L3004;ADD_BACK	;Jump forward if the carry does
	JR	NZ,L3195;OFLOW_CLR ;not ripple right back.
	EXX			;If it should ripple right back
	LD	D,80H		;then set mantissa to 0.5 and
	EXX			;increment the exponent.
	INC	(HL)		;This action may lead to arithmetic
	JR	Z,L31AD;REPORT_6 ;overflow (final case).

;The final part of the subroutine involves passing the result to the bytes
;reserved for it on the calc stack and resetting the pntrs.
L3195:;OFLOW_CLR
	PUSH	HL		;Save the result pntr.
	INC	HL		;Point to the sign byte in the result.
	EXX			;The result is moved from its
	PUSH	DE		;present regs, D'E'DE, to
	EXX			;BCDE; and then to ACDE.
	POP	BC
	LD	A,B		;The sign bit is retieved from
	RLA			;its temporary store and transferred
	RL	(HL)		;to its correct position of bit 7
	RRA			;of the 1st byte of the mantissa.
	LD	(HL),A		;The 1st byte is stored.
	INC	HL		;Next.
	LD	(HL),C		;The 2nd byte is stored.
	INC	HL		;Next.
	LD	(HL),D		;The 3rd byte is stored.
	INC	HL		;Next.
	LD	(HL),E		;The 4th byte is stored.
	POP	HL		;Restore the pntr to the result.
	POP	DE		;Restore the pntr to the 2nd number.
	EXX			;Exchange the regs.
	POP	HL		;Restore the next literal addr.
	EXX			;Exchange the regs.
	RET			;Finished.

;Report 6 - Arithmetic overflow
L31AD:;ERROR_6
	RST	8;ERROR_1	;Call the error handling
	DB	05H		;routine.

