
;	THE 'DIVISION' OPERATION
;(Offset 05 - see CALCULATE below: 'divide')
;This subroutine 1st prepared the divisor by calling PREP_M/D, reporting
;arithmetic overflow if it is zero; then it prepares the dividend by again
;calling PREP_M/D, returning if it is zero. Next fetches the two numbers
;from the calc stack and divides their mantissa by means of the usual
;restoring division, trial subtracting the divisor from the dividend and
;restoring if there is carry, otherwise adding 1 to the quotient. The
;maximum precision is obtained for a 4-byte division, and after subtracting
;the exponents the subroutine exits by joining the later part of
;MULTIPLICATION.
L31AF:;division
	CALL	L3293;RE_ST_TWO	;Use full flpnt forms.
	EX	DE,HL		;Exchange the pntrs.
	XOR	A		;A is set to 0, so that the sign of the
				;1st number will go into A.
	CALL	L30C0;PREP_M/D	;Prepare the divisor and give the report
	JR	C,L31AD;REPORT_6 ;for arithmetic overflow if it is zero.
	EX	DE,HL		;Exchange the pntrs.
	CALL	L30C0;PREP_M/D	;Prepare the dividend and return
	RET	C		;if it is zero (result already zero).
	EXX			;Exchange the regs.
	PUSH	HL		;Save the next literal addr.
	EXX			;Exchange the regs.
	PUSH	DE		;Save pntr to divisor.
	PUSH	HL		;Save pntr to dividend.
	CALL	L2FBA;FETCH_TWO	;Get the two numbers from the stack.
	EXX			;Exchange the regs.
	PUSH	HL		;Save M1 & N1 on the machine stack.
	LD	H,B		;Copy the four bytes of the
	LD	L,C		;dividend from regs B'C'BC
	EXX			;(i.e. M2, M3, M4 & M5; see FETCH_TWO)
	LD	H,C		;to the regs H'L'HL.
	LD	L,B
	XOR	A		;Clear A and reset the carry flag.
	LD	B,0DFH		;B will count upwards from -33 to -1,
				;twos complement, hex DF to FF, looping
				;on minus and will jump again on zero
				;for extra precision.
	JR	L31E2;DIV_START	;Jump forward into the division loop
				;for the 1st trial subtraction.

;Now enter the division loop.
L31D2:;DIV_LOOP
	RLA			;Shift the result left into B'C'CA,
	RL	C		;shifting out the bits already there,
	EXX			;picking up 1 from the carry whenever
	RL	C		;it is set, and rotating left each byte
	RL	B		;with carry to achieve the 32 bit shift.
	EXX
L31DB:;DIV_34TH
	ADD	HL,HL		;Move what remains of the dividend left
	EXX			;in H'L'HL before the next trial
	ADC	HL,HL		;subtraction; if a bit drops into the carry,
	EXX			;force no restore and a bit for the
				;quotient, thus retrieving the lost bit
	JR	C,L31F2;SUBN_ONLY ;and allowing a full 32-bit divisor.
L31E2:;DIV_START
	SBC	HL,DE		;Trial subtract divisor in D'E'DE from
	EXX			;rest of dividend in H'L'HL; there is no
	SBC	HL,DE		;initial carry (see previous step).
	EXX
	JR	NC,L31F9;NO_RSTORE ;Jump forward if there is no carry.
	ADD	HL,DE		;Otherwise restore, i.e. add back the
	EXX			;divisor. Then clear the carry so that
	ADC	HL,DE		;there will be no bit for the quotient
	EXX			;(the divisor 'did not go').
	AND	A
	JR	L31FA;COUNT_ONE	;Jump forward to the counter.
L31F2:;SUBN_ONLY
	AND	A		;Just subtract with no restore
	SBC	HL,DE		;and go on to set the carry flag
	EXX			;because the lost bit of the dividend
	SBC	HL,DE		;is to be retrieved and used
	EXX			;for the quotient.
L31F9:;NO_RSTORE
	SCF			;One for the quotient in B'C'CA.
L31FA:;COUNT_ONE
	INC	B		;Step the loop count up by one.
	JP	M,L31D2;DIV_LOOP ;Loop 32 times for all bits.
	PUSH	AF		;Save any 33rd bit for extra
				;precision (the present carry).
	JR	Z,L31E2;DIV_START ;Trial subtract yet again for any 34th
				;bit; the PUSH AF above saves this bit too.
;Note: This jump is made to the wrong place. No 34th bit will ever be
;obtained without 1st shifting the dividend. Hence important results like
;1/10 and 1/1000 are not rounded up as they should be. Rounding up never
;occurs when it depends on the 34th bit. The jump should have been to 31DB
;DIV_34TH above: i.e. byte 3200 hex in the ROM should read DA hex (218 dec)
;instead of E1 hex (225 dec).
	LD	E,A		;Now move the four bytes that
	LD	D,C		;form the mantissa bytes of the
	EXX			;result from B'C'CA to D'E'DE.
	LD	E,C
	LD	D,B
	POP	AF		;The put the 34th and 33rd bits
	RR	B		;into B' to be picked up on
	POP	AF		;normalisation.
	RR	B
	EXX
	POP	BC		;Restore the exponent bytes, M1 & N1.
	POP	HL		;Restore the pntr to the result.
	LD	A,B		;Get the difference between the	two exponent
	SUB	C		;bytes into A and set the carry flag if
	JP	L313D;DIVN_EXPT	;required. Exit via DIVN_EXPT.

;	THE 'INTEGER TRUNCATION TOWARDS ZERO' SUBROUTINE
;(Offset 3A - see CALCULATE below: 'truncate')
;This subroutine (say I(x)) returns the result of integer truncation of x,
;the 'last value', towards zero. Thus, I(2.4) is 2 and I(-2.4) is -2.
;The subroutine returns at once if x is in the form of a 'short integer'.
;It returns zero if the exponent byte of x is less than 81 hex (ABS x is
;less than 1). If I(x) is a 'short integer' the subroutine returns it in
;that form. It returns x if the exponent byte is A0 hex or greater (x has
;no significant non-integral part). Otherwise the correct number of bytes
;of x are set to zero and, if needed, one more byte is split with a mask.
L3214:;truncate
	LD	A,(HL)		;Get the exponent byte of x into A.
	AND	A		;If A is zero, return since x is
	RET	Z		;already a small integer.
	CP	81H		;Compare e, the exponent, to 81 hex.
	JR	NC,L3221;T_GR_ZERO ;Jump if e is greater than 80 hex.
	LD	(HL),0		;Else, set the exponent to zero;
	LD	A,20h		;enter 32 dec, 20 hex, into A and jump
	JR	L3272;NIL_BYTES	;forward to NIL_BYTES to make all the bits
				;of x be zero.
L3221:;T_GR_ZERO
	CP	91H		;Compare e to 91 hex, 145 dec.
	JR	NZ,L323F;T_SMALL ;Jump if e not 91 hex.
;The next 26 bytes seem designed to test whether x is in fact -65536 dec,
;i.e. 91 80 00 00 00, and if it is, to set it to 00 FF 00 00 00. This is
;a mistake. As already stated at byte 303B above, the Spectrum system cannot
;handle this number. The result here is simply to make INT(-65536) return
;the value -1. This is a pity, since the number would have been perfectly
;all right if left alone. The remedy would seem to be simply to omit the
;28 bytes from 3223 to 323E inclusive from the program.
	INC	HL		;HL is pointed at the 4th byte
	INC	HL		;of x, where the 17 bits of the
	INC	HL		;integer part of x end after the 1st bit.
	LD	A,80H		;The 1st bit is obtained in A,
	AND	(HL)		;using 80 hex as a mask.
	DEC	HL		;That bit and the previous 8 bits
	OR	(HL)		;are tested together for zero.
	DEC	HL		;HL is pointed at the 2nd byte of x.
	JR	NZ,L3233;T_FIRST ;If already non-zero, the test can end.
	LD	A,80H		;Otherwise, the test for -65536 is now
	XOR	(HL)		;completed: 91 80 00 00 00 will leave
				;the zero flag set now.
L3233:;T_FIRST
	DEC	HL		;HL is pointed at the 1st byte of x.
	JR	NZ,L326C;T_EXPNENT ;If zero reset, the jump is made.
	LD	(HL),A		;The 1st byte is set to zero.
	INC	HL		;HL points to the 2nd byte.
	LD	(HL),0FFH	;The 2nd byte is set to FF.
	DEC	HL		;HL again points to the 1st byte.
	LD	A,18H		;The last 24 bits are to be zero.
	JR	L3272;NIL_BYTES	;The jump to NIL_BYTES completes the
				;number 00 FF 00 00 00.

;If the exponent byte of x is between 81 and 90 hex (129 and 144 dec)
;inclusive, I(x) is a 'small integer', and will be compressed into one
;or two bytes. But 1st a test is made to see whether x is, after all, large.
L323F:;T_SMALL
	JR	NC,L326D;X_LARGE ;Jump with exponent byte 92 or more
				;(it would be better jump with 91 too).
	PUSH	DE		;Save STKEND in DE.
	CPL			;Range 129<=A<=144 becomes 126>=A>=111.
	ADD	A,91H		;Range is now 15>=A>=0.
	INC	HL		;Point HL at 2nd byte.
	LD	D,(HL)		;2nd byte to D.
	INC	HL		;Point HL at 3rd byte.
	LD	E,(HL)		;3rd byte to E.
	DEC	HL		;Point HL at 1st byte again.
	DEC	HL
	LD	C,0		;Assume a positive number.
	BIT	7,D		;Now test for negative (bit 7 set).
	JR	Z,L3252;T_NUMERIC ;Jump if positive after all.
	DEC	C		;Change the sign.
L3252:;T_NUMERIC
	SET	7,D		;Insert true numeric bit, 1, in D.
	LD	B,8		;Now test whether A>=8 (one byte only)
	SUB	B		;or two bytes needed.
	ADD	A,B		;Leave A unchanged.
	JR	C,L325E;T_TEST	;Jump if two bytes needed.
	LD	E,D		;Put the one byte into E.
	LD	D,0		;And set D to zero.
	SUB	B		;Now 1<=A<=7 to count the shifts needed.
L325E:;T_TEST
	JR	Z,L3267;T_STORE	;Jump if no shift needed.
	LD	B,A		;B will count the shifts.
L3261:;T_SHIFT
	SRL	D		;Shift D and E right B times to
	RR	E		;produce the correct number.
	DJNZ	L3261;T_SHIFT	;Loop until B is zero.
L3267:;T_STORE
	CALL	L2D8E;INT_STORE	;Store the result on the stack.
	POP	DE		;Restore STKEND to DE.
	RET			;Finished.

;Large values of x remains to be considered.
L326C:;T_EXPNENT
	LD	A,(HL)		;Get the exponent byte of x into A.
L326D:;X_LARGE
	SUB	0A0H		;Subtract 160 dec, A0 hex, from e.
	RET	P		;Return on plus - x has no significant
				;non-integral part. (If the true exponent
				;were reduced to zero, the 'binary point'
				;would come at or after the end of the four
				;bytes of the mantissa.)
	NEG			;Else, negate the remainder; this gives the
				;number of bits to become zero (the number
				;of bits after the 'binary point').

;Now the bits of the mantissa can be cleared.
L3272:;NIL_BYTES
	PUSH	DE		;Save the current value of DE (STKEND).
	EX	DE,HL		;Make HL point one past the 5th byte.
	DEC	HL		;HL now points to the 5th byte of x.
	LD	B,A		;Get the number of bits to be set
	SRL	B		;to zero into B and divide it by 8
	SRL	B		;to give the number of whole
	SRL	B		;bytes implied.
	JR	Z,L3283;BITS_ZERO ;Jump forward if the result is zero.
L327E:;BYTE_ZERO
	LD	(HL),0		;Else, set the bytes to zero;
	DEC	HL		;B counts them.
	DJNZ	L327E;BYTE_ZERO
L3283:;BITS_ZERO
	AND	7		;Get A (mod 8); this is the number of bits
				;still to be set to zero.
	JR	Z,L3290;IX_END	;Jump to the end if nothing more to do.
	LD	B,A		;B will count the bits now.
	LD	A,0FFH		;Prepare the mask.
L328A:;LESS_MASK
	SLA	A		;With each loop a zero enters the mask from
	DJNZ	L328A;LESS_MASK	;the right and thereby a mask of the correct
				;length is produced.
	AND	(HL)		;The unwanted bits of (HL) are lost
	LD	(HL),A		;as the masking is performed.
L3290:;IX_END
	EX	DE,HL		;Return the pntr to HL.
	POP	DE		;Return STKEND to DE.
	RET			;Finished.

;	THE 'RE-STACK TWO' SUBROUTINE
;This subroutine is called to re-stack two 'small integer' in full five byte
;flpnt form for the binary operations of addition, multiplication and
;division. It does so by calling the following subroutine twice.
L3293:;RE_ST_TWO
	CALL	L3296;RESTK_SUB	;Call the subroutine, and then continue into
				;it for the 2nd call.
L3296:;RESTK_SUB
	EX	DE,HL		;Exchange the pntrs at each call.

;	THE 'RE-STACK' SUBROUTINE
;(Offset 3D - see CALCULATE below: 're_stack')
;This subroutine is called to re-stack one number (which could be a 'small
;integer') in full five byte flpnt form. It is used for a single number by
;ARCTAN and also, through the calc offset, by EXP, LN and 'get_argt'.
L3297:;RE_STACK
	LD	A,(HL)		;If the 1st byte is not zero,
	AND	A		;return - the number cannot be
	RET	NZ		;a 'small integer'.
	PUSH	DE		;Save the 'other' pntr in DE.
	CALL	L2D7F;INT_FETCH	;Fetch the sign in C and the number in DE.
	XOR	A		;Clear the A reg.
	INC	HL		;Point to the 5th loc.
	LD	(HL),A		;Set the 5th byte to zero.
	DEC	HL		;Point to the 4th loc.
	LD	(HL),A		;Set the 4th byte to zero:
				;bytes 2 and 3 will hold the mantissa.
	LD	B,91H		;Set B to 145 dec for the exponent i.e. for
				;up to 16 bits in the integer.
	LD	A,D		;Test whether D is zero so that
	AND	A		;at most 8 bits would be needed.
	JR	NZ,L32B1;RS_NRMLSE ;Jump if more than 8 bits needed.
	OR	E		;Now test E too.
	LD	B,D		;Save the zero in B (it will give
				;zero exponent if E too is zero).
	JR	Z,L32BD;RS_STORE ;Jump if E is indeed zero.
	LD	D,E		;Move E to D (D was zero, E not).
	LD	E,B		;Set E to zero now.
	LD	B,89H		;Set B to 137 dec for the exponent -
				;no more than 8 bits now.
L32B1:;RS_NRMLSE
	EX	DE,HL		;Pntr to DE, number to HL.
L32B2:;RSTK_LOOP
	DEC	B		;Decrement the exponent on each shift.
	ADD	HL,HL		;Shift the number right one position.
	JR	NC,L32B2;RSTK_LOOP ;Until the carry is set.
	RRC	C		;Sign bit to carry flag now.
	RR	H		;Insert it in place as the number is
	RR	L		;shifted back one place - normal now.
	EX	DE,HL		;Pntr to byte 4 back to HL.
L32BD:;RS_STORE
	DEC	HL		;Point to the 3rd loc.
	LD	(HL),E		;Store the 3rd byte.
	DEC	HL		;Point to the 2nd loc.
	LD	(HL),D		;Store the 2nd byte.
	DEC	HL		;Point to the 1st loc.
	LD	(HL),B		;Store the exponent byte.
	POP	DE		;Restore the 'other' pntr to DE.
	RET			;Finished.

