

;	THE LOUDSPEAKER ROUTINES

;  The two subrs in this section are the BEEPER subroutine, that actually
;controls the loudspeaker, and the BEEP command routine.
;  The loudspeaker is activated by having D4 low during an OUT instruction
;that is using port '254'. When D4 is high in a similar situation the
;loudspeaker is deactivated. A 'beep' can therefore be produced by
;regularly changing the level of D4.
;  Consider now the note 'middle C' which has the frequency 261.63 hz.
;In order to get this note the loudspeaker will have to be alternately
;activated and deactivated every 1/523.26 th. of a second. In the SPECTRUM
;the system clock is set to run at 3.5 mhz. and the note 'middle C' will
;require that the requisite OUT instruction be executed as close as possible
;to every 6.689 T states. This last value, when reduced slightly for
;unavoidable overheads, represents the 'length of the timing loop' in the
;BEEPER subroutine.
L03B5:;BEEPER
	DI			;Disable the interrupt for the
				;duration of a 'beep'.
	LD	A,L		;Save L temporarily.
	SRL	L		;Each '1' in the L reg is to count '4' T
	SRL	L		;states, but take INT(L/4) and count '16'
				;T states instead.
	CPL			;Go back to the original value
	AND	3		;in L and find how many were
	LD	C,A		;lost by taking INT(L/4).
	LD	B,0
	LD	IX,L03D1;BE_IX+3;The base addr of the timing loop.
	ADD	IX,BC		;Alter the length of the timing loop.
				;Use an earlier starting point for each
				;'1' lost by taking INT(L/4).
	LD	A,(L5C48);BORDCR;Fetch the present border
	AND	38H		;colour and move it to bits
	RRCA			;2, 1 & 0 of the A reg.
	RRCA
	RRCA
	OR	8		;Ensure the MIC output is 'off'.

;Now enter the sound generation loop. 'DE' complete passes are made, i.e.
;a pass for each cycle of the note.
;  The HL reg holds the 'length of the timing loop' with '16' T states
;being used for each '1' in the L reg and '1,024' T states for each '1'
;in the H reg.
L03D1:;BE_IX+3
	NOP			;Add '4' T states for each
L03D2:;BE_IX+2
	NOP			;earlier entry point
L03D3:;BE_IX+1
	NOP			;that is used.
L03D4:;BE_IX+0
	INC	B		;The values in the B & C regs will come
	INC	C		;from H & L regs - see below.
L03D6:;BE_H&L_LP
	DEC	C		;The 'timing loop'.
	JR	NZ,L03D6;BE_H&L_LP ;i.e. 'BC'*'4' T states.
	LD	C,3FH		;(But note that at the half-cycle
	DEC	B		;print - C will be equal to
	JP	NZ,L03D6;BE_H&L_LP ;'L+1'.)

;The loudspeaker is now alternately activated and deactivated.
	XOR	10H		;Flip bit 4.
	OUT	(0FEH),A	;Perform the OUt operation;
				;leaving the border unchanged.
	LD	B,H		;Reset the B reg.
	LD	C,A		;Save the A reg.
	BIT	4,A		;Jump if at the half-cycle
	JR	NZ,L03F2;BE_AGAIN ;point.

;After a full cycle the DE reg pair is tested.
	LD	A,D		;Jump forward if the last
	OR	E		;complete pass has been
	JR	Z,L03F6;BE_END	;made already.
	LD	A,C		;Fetch the saved value.
	LD	C,L		;Reset the C reg.
	DEC	DE		;Decrease the pass counter.
	JP	(IX)		;Jump back to the required
				;starting location of the loop.

;The parameters for the second half-cycle are set up.
L03F2:;BE_AGAIN
	LD	C,L		;Reset the C reg.
	INC	C		;Add '16' T states as this path is shorter.
	JP	(IX)		;Jump back.

;Upon completion of the 'beep' the maskable interrupt has to be enabled.
L03F6:;BE_END
	EI			;Enable interrupts.
	RET			;Finally return.

;	THE 'BEEP' COMMAND ROUTINE
;This subroutine is entered with two numbers on the calculator stack.
;The topmost number represents the 'pitch' of the note and the number
;underneath it represents the 'duration'.
L03F8:;BEEP
	RST	28H;FP_CALC	;The FP calculator is used to manipulate
				;the two values - t & P.
	DB	31H;duplicate	;t, P, P
	DB	27H;int		;t, P, i (where i = INT P)
	DB	0C0H;st_mem_0	;t, P, i (mem_0 holds i)
	DB	03H;subtract	;t, p (where p is the fractional part of P)
	DB	34H;stk_data	;Stack the decimal value 'K'.
	DB	0ECH;exponent+7C;0.0577622606 (which is a
	DB	6CH,98H,1FH,0F5H;little below 12*sqrt(2)-1)
	DB	04H;multiply	;t, pK
	DB	0A1H;stk_one	;t, pK, 1
	DB	0FH;addition	;t, pK+1
	DB	38H;end_calc

;Now perform several tests on i, the integer part of the 'pitch'.
	LD	HL,L5C92	;This is 'mem_0_1st' (MEMBOT).
	LD	A,(HL)		;Fetch the exponent of i.
	AND	A		;Give an error if i is not in the
	JR	NZ,L046C;REPORT_B ;integral (short) form.
	INC	HL		;Copy the sign byte to the
	LD	C,(HL)		;C reg.
	INC	HL		;Copy the low-byte to the
	LD	B,(HL)		;B reg; and to the A
	LD	A,B		;reg.
	RLA			;Again give report B if i does not
	SBC	A,A		;satisfy the test:
	CP	C		;-128<=i<=+127
	JR	NZ,L046C;REPORT_B
	INC	HL
	CP	(HL)
	JR	NZ,L046C;REPORT_B
	LD	A,B		;Fetch the low-byte and test it further.
	ADD	A,3CH
	JP	P,L0425;BE_i_OK	;Accept -60<=i<=67.
	JP	PO,L046C;REPORT_B ;Reject -128 to -61.

;Note: The range 70 to 127 will be rejected later on.

;The correct frequency for the 'pitch' I can now be found.
L0425:;BE_i_OK
	LD	B,0FAH		;Start '6' octaves below middle C.
L0427:;BE_OCTAVE
	INC	B		;Repeatedly reduce i in order to
	SUB	0CH		;find the correct octave.
	JR	NC,L0427;BE_OCTAVE
	ADD	A,0CH		;Add back the last subtraction.
	PUSH	BC		;Save the octave number.
	LD	HL,L046E	;The base addr of the 'semi-tone table'.
	CALL	L3406;LOC_MEM	;Consider the table and pass the
	CALL	L33B4;STACK_NUM	;'A th.' value to the calculator
				;stack. (Call in C.)

;Now the fractional part of the 'pitch' can be taken into consideration.
	RST	28H;FP_CALC	;t, pK+1, C
	DB	04H;multiply	;t, C(pK+1)
	DB	38H;end_calc

;The final frequency f is found by modifying the 'last value' according
;to the octave number.
	POP	AF		;Fetch the octave number.
	ADD	A,(HL)		;Multiply the 'last value' by '2 to the
	LD	(HL),A		;power of the octave number'.
	RST	28H;FP_CALC	;t, f
	DB	0C0H;st_mem_0	;The frequency is put aside for
	DB	02H;delete	;the moment in mem_0.

;Attention is now turned to the 'duration'.
	DB	31H;duplicate	;t, t
	DB	38H;end_calc
	CALL	L1E94;FIND_INT1	;The value 'INT t' must be in
	CP	0BH		;the range 00-0A.
	JR	NC,L046C;REPORT_B

;The number of complete cycles in the 'beep' is given by 'f*t' so this
;valueis now found.
	RST	28H;FP_CALC	;t
	DB	0E0H;get_mem_0	;t, f
	DB	04H;multiply	;f*t

;The result is left on the calculator stack whilst the length of the
;'timing loop' required for the 'beep' is computed;
	DB	0E0H;get_mem_0	;f*t, f
	DB	34H;stk_data	;The value '3.5*10^6/8'
	DB	80H;four bytes	;is formed on the top of
	DB	43H;exponent +93;the calculator stack.
	DB	55H,9FH,80H;00	;f*t, f, 437,500 (dec.)
	DB	01H;exchange	;f*t, 437,500, f
	DB	05H;division	;f*t, 437,500/f
	DB	34H;stk_data
	DB	35H;exponent +85
	DB	71H;00,00,00	;f*t, 437,500/f, 30.125 (dec.)
	DB	03H;subtract	;f*t, 437,500/f-30.125
	DB	38H;end_calc

;Note: The value '437,500/f' gives the 'half-cycle' length of the note and
;reducing it by '30.125' allows for '120.5' T states in which to actually
;produce the not and adjust the counters etc.
;  The values can now be transferred to the required regs.
	CALL	L1E99;FIND_INT2	;The 'timing loop' value is
				;compressed into the BC
	PUSH	BC		;reg pair; and saved.

;Note: If the timing loop value is too large then an error will occur
;(returning via ERROR_1); thereby excluding 'pitch' values of '70 to 127'.
	CALL	L1E99;FIND_INT2	;The 'f*t' value is compressed into
				;the BC reg pair.
	POP	HL		;Move the 'timing loop' value to
				;the HL reg pair.
	LD	D,B		;Move the 'f*t' value to the
	LD	E,C		;DE reg pair.

;However before making the 'beep' test the value 'f*t'.
	LD	A,D		;Return if 'f*t' has given the
	OR	E		;result of 'no cycles'
	RET	Z		;required.
	DEC	DE		;Decrease the cycle number and jump
	JP	L03B5;BEEPER	;to the BEEPER subroutine
				;(making, at least, one pass).

;Report B - Integer out of range
L046C:;REPORT_B
	RST	8;ERROR_1	;Call the error handling
	DB	0AH		;routine.

;	THE 'SEMI_TONE' TABLE
;This table holds the frequencies of the twelve semi-tones in an octave.
L046E:
;					frequency hz.	note
	DB	89H,02H,0D0H,12H,86H	;261.63		C
	DB	89H,0AH,97H,60H,75H	;277.18		C#
	DB	89H,12H,0D5H,17H,1FH	;293.66		D
	DB	89H,1BH,90H,41H,02H	;311.13		D#
	DB	89H,24H,0D0H,53H,0CAH	;329.63		E
	DB	89H,2EH,9DH,36H,0B1H	;349.23		F
	DB	89H,38H,0FFH,49H,3EH	;369.99		F#
	DB	89H,43H,0FFH,6AH,73H	;392		G
	DB	89H,4FH,0A7H,00H,54H	;415.30		G#
	DB	89H,5CH,00H,00H,00H	;440		A
	DB	89H,69H,14H,0F6H,24H	;466.16		A#
	DB	89H,76H,0F1H,10H,05H	;493.88		B


;	THE 'PROGRAM NAME' SUBROUTINE (ZX81)
;The following subroutine applies to the ZX81 and was not removed when
;the program was rewritten for the SPECTRUM.
	CALL	24FBH
	LD	A,(5C3BH)
	ADD	A,A
	JP	M,1C8AH
	POP	HL
	RET	NC
	PUSH	HL
	CALL	2BF1H
	LD	H,D
	LD	L,E
	DEC	C
	RET	M
	ADD	HL,BC
	SET	7,(HL)
	RET

