

;	THE CASSETTE HANDLING ROUTINES
;
;The 16K monitor program has an extensive set of routines for handling the
;cassette interface. In effect these routines form the SAVE, LOAD, VERIFY
;and MERGE command routines.
;  The entry point to the routines is at SAVE_ETC (0605). However before
;this point are the subroutines concerned with the actual SAVEing and LOADing
;(or VERIFYing) of bytes.
;  In all cases the bytes to be handled by these subroutines are described by
;the DE reg pair holding the 'length' of the block, the IX reg pair holding
;the 'base addr' and the A reg holding 00 for a header block, or FF for a
;program/data block.

;	THE 'SA_BYTES' SUBROUTINE
;This subroutine is called to SAVE the header information (from 098A) and
;later the actual program/data block (from 099E).
L04C2:;SA_BYTES
	LD	HL,L053F	;Pre-load the machine stack with
	PUSH	HL		;the addr - SA/LD_RET.
	LD	HL,1F80H	;This constant will give a leader
				;of about 5 secs. for a 'leader'.
	BIT	7,A		;Jump forward if SAVEing a
	JR	Z,L04D0;SA_FLAG	;header.
	LD	HL,0C98H	;This constant will give a leader of about
				;2 secs. for a program/data block.
L04D0:;SA_FLAG
	EX	AF,AF'		;The flag is saved.
	INC	DE		;The 'length' is incremented and the
	DEC	IX		;'base addr' reduced to allow for the flag.
	DI			;The maskable interrupt is disabled
				;during the SAVE.
	LD	A,2		;Signal 'MIC on' and border to be RED.
	LD	B,A		;Give a value to B.

;A loop is now entered to create the pulses of the leader. Both the 'MIC on'
;and the 'MIC off' pulses are 2,168 T states in length. The colour of the
;border changes from RED to CYAN with each 'edge'.
;
;Note: An 'edge' will be a transition either from 'on' to 'off',
;      or from 'off' to 'on'.
L04D8:;SA_LEADER
	DJNZ	L04D8;SA_LEADER	;The main timing period.
	OUT	(0FEH),A	;MIC on/off, border RED/CYAN,
	XOR	0FH		;on each pass.
	LD	B,0A4H		;The main timing constant.
	DEC	L		;Decrease the low counter.
	JR	NZ,L04D8;SA_LEADER ;Jump back for another pulse.
	DEC	B		;Allow for the longer path
				;(- reduce by 13 T states).
	DEC	H		;Decrease the high counter.
	JP	P,L04D8;SA_LEADER ;Jump back for another pulse until
				;completion of the leader.

;A sync pulse is now sent.
	LD	B,2FH
L04EA:;SA_SYNC_1
	DJNZ	L04EA;SA_SYNC_1	;MIC off for 687 T states from 'OUT to OUT'.
	OUT	(0FEH),A	;MIC on and RED.
	LD	A,0DH		;Signal 'MIC off & CYAN'.
	LD	B,37H		;MIC on for 735 T states from
L04F2:;SA_SYNC_2		;'OUT to OUT'.
	DJNZ	L04F2;SA_SYNC_2
	OUT	(0FEH),A	;Now MIC off & border CYAN.

;The header v. program/data flag will be the 1st byte to be SAVEd.
	LD	BC,3B0EH	;3B is a timing constant; 0E signals
				;'MIC off & YELLOW'.
	EX	AF,AF'		;Fetch the flag and pass it to the
	LD	L,A		;L reg for 'sending'.
	JP	L0507;SA_START	;Jump forward int the SAVEing loop.

;The byte SAVEing loop is now entered. The 1st byte to be SAVEd is the flag;
;this is followed by the actual data byte and the final byte sent is the parity
;byte that is built up by considering the values of all the earlier bytes.
L04FE:;SA_LOOP
	LD	A,D		;The 'length' counter is tested
	OR	E		;and the jump taken when it
	JR	Z,L050E;SA_PARITY ;has reached zero.
	LD	L,(IX+0)	;Fetch the next byte that is to be SAVEd.
L0505:;SA_LOOP_P
	LD	A,H		;Fetch the current 'parity'.
	XOR	L		;Include the present byte.
L0507:;SA_START
	LD	H,A		;Restore the 'parity'. Note that on entry
				;here the 'flag' value initialises 'parity'.
	LD	A,1		;Signal 'MIC on & BLUE'.
	SCF			;Set the carry flag. This will act as a
				;'marker' for the 8 bits of a byte.
	JP	L0525;SA_8_BITS	;Jump forward.

;When it is time to send the 'parity' byte then it is transferred to the L
;reg for SAVEing.
L050E:;SA_PARITY
	LD	L,H		;Get final 'parity' value.
	JR	L0505;SA_LOOP_P	;Jump back.

;The following inner loop produces the actual pulses. The loop is entered at
;SA_BIT_1 with the type of the bit to be SAVEd indicated by the carry flag.
;Two passes of the loop are made for each bit thereby making an 'off pulse'
;and an 'on pulse'. The pulses for a reset bit are shorter by 855 T states.
L0511:;SA_BIT_2
	LD	A,C		;Come here on the 2nd pass and fetch
				;'MIC off & YELLOW'.
	BIT	7,B		;Set the zero flag to show '2nd pass'.
L0514:;SA_BIT_1
	DJNZ	L0514;SA_BIT_1	;The main timing loop; always 801 T states
				;on a 2nd. pass.
	JR	NC,L051C;SA_OUT	;Jump, taking the shorter path, if
				;SAVEing a '0'.
	LD	B,42H		;However if SAVEing a '1' then
L051A:;SA_SET			;add 855 T states.
	DJNZ	L051A;SA_SET
L051C:;SA_OUT
	OUT	(0FEH),A	;On the 1st. pass 'MIC on & BLUE' and on
				;the 2nd. pass 'MIC off & YELLOW'.
	LD	B,3EH		;Set the timing constant for the 2nd. pass.
	JR	NZ,L0511;SA_BIT_2 ;Jump back at the end of the 1st pass;
	DEC	B		;otherwise reclaim 13 T states.
	XOR	A		;Clear the carry flag and set A to hold 01
	INC	A		;(MIC on&BLUE) before continuing into
				;the '8 bit loop'.

;The '8 bit loop' is entered initially with the whole byte in the L reg and
;the carry flag set. However it is re-entered after each bit has been SAVEd
;until the point is reached when the 'marker' passes to the carry flag leaving
;the L reg empty.
L0525:;SA_8_BITS
	RL	L		;Move bit 7 to the carry and the
				;'marker' leftwards.
	JP	NZ,L0514;SA_BIT_1 ;SAVE the bit unless finished with the byte.
	DEC	DE		;Decrease the 'counter'.
	INC	IX		;Advance the 'base addr'.
	LD	B,31H		;Set the timing constant for the 1st bit
				;of the next byte.
	LD	A,7FH		;Return (to SA/LD_RET) if the
	IN	A,(0FEH)	;BREAK key is being pressed.
	RRA
	RET	NC
	LD	A,D		;Otherwise test the 'counter'
	INC	A		;and jump back even if it has reached
	JP	NZ,L04FE;SA_LOOP;zero (so as to send the 'parity' byte).
	LD	B,3BH		;Exit when the 'counter'
L053C:;SA_DELAY
	DJNZ	L053C;SA_DELAY	;reaches FFFF. But 1st
	RET			;give a short delay.

;Note: A reset bit will give a 'MIC off' pulse of 855 T states followed by a
;'MIC on' pulse of 855 T states. Whereas a set bit will give pulses of exactly
;twice as long. Note also that there are no gaps either between the sync pulse
;and the 1st bit of the flag, or between bytes.

;	THE 'SA/LD_RET' SUBROUTINE
;This subroutine is common to both SAVEing and LOADing.
;  The border is set to its original colour and the BREAK key tested
;for a last time.
L053F:;SA/LD_RET
	PUSH	AF		;Save the carry flag. (It is reset
				;after a LOADing error.)
	LD	A,(L5C48);BORDCR;Fetch the original border colour
	AND	38H		;from its system var.
	RRCA			;Move the border colour
	RRCA			;to bits 2, 1 & 0.
	RRCA
	OUT	(0FEH),A	;Set the border to its original colour.
	LD	A,7FH		;Read the BREAK key for a
	IN	A,(0FEH)	;last time.
	RRA
	EI			;Enable the maskable interrupt.
	JR	C,L0554;SA/LD_END ;Jump unless a break is to be made.

;Report D - BREAK-CONT repeats.
L0552:;REPORT_D
	RST	8;ERROR_1	;Call the error handling routine.
	DB	0CH

;Continue here.
L0554:;SA/LD_END
	POP	AF		;Retrieve the carry flag.
	RET			;Return to the calling routine.

;	THE 'LD_BYTES' SUBROUTINE
;This subroutine is called to LOAD the header information (from 076E) and
;later LOAD, or VERIFY, an actual block of data (from 0802).
L0556:;LD_BYTES
	INC	D		;This resets the zero flag. (D can't hold FF.)
	EX	AF,AF'		;The A reg holds 00 for a header and FF
				;for a block of data.
	DEC	D		;Restore D its original value.
	DI			;The maskable interrupt is now disabled.
	LD	A,0FH		;The border is made WHITE.
	OUT	(0FEH),A
	LD	HL,L053F	;Pre-load the machine stack
	PUSH	HL		;with the addr - SA/LD_RET.
	IN	A,(0FEH)	;Make an initial read of port '254'.
	RRA			;Rotate the byte obtained but
	AND	20H		;keep only the EAR bit.
	OR	2		;Signal 'RED' border. Store the value in the
	LD	C,A		;C reg. (22 for 'off' and 02 for 'on' -
				;the present EAR state.)
	CP	A		;Set the zero flag.

;The 1st stage of reading a tape involves showing that a pulsing signal
;actually exists. (i.e. 'On/off' or 'off/on' edges.)
L056B:;LD_BREAK
	RET	NZ		;Return if the BREAK key is being pressed.
L056C:;LD_START
	CALL	L05E7;LD_EDGE_1	;Return with the carry flag reset if there is
	JR	NC,L056B;LD_BREAK ;no 'edge' within approx. 14,000 T states.
				;But if an 'edge' is found the border
				;will go CYAN.

;The next stages involves waiting a while and then showing that the signal
;is still pulsing.
	LD	HL,415H		;The length of this waiting period
L0574:;LD_WAIT
	DJNZ	L0574;LD_WAIT	;will be almost one second in duration.
	DEC	HL
	LD	A,H
	OR	L
	JR	NZ,L0574;LD_WAIT
	CALL	L05E3;LD_EDGE_2	;Continue only if two edges are found within
	JR	NC,L056B;LD_BREAK ;the allowed time period.

;Now accept only a 'leader signal'.
L0580:;LD_LEADER
	LD	B,9CH		;The timing constant.
	CALL	L05E3;LD_EDGE_2	;Continue only if two edges are found
	JR	NC,L056B;LD_BREAK ;within the allowed time period.
	LD	A,0C6H		;However the edges must have
	CP	B		;been found within about
	JR	NC,L056C;LD_START ;3,000 T states of each other.
	INC	H		;Count the pair of edges in the H reg until
	JR	NZ,L0580;LD_LEADER ;'256' pairs have been found.

;After the leader come the 'off' and 'on' parts of the sync pulse.
L058F:;LD_SYNC
	LD	B,0C9H		;The timing constant.
	CALL	L05E7;LD_EDGE_1	;Every edge is considered until
	JR	NC,L056B;LD_BREAK ;two edges are found close
	LD	A,B		;together - these will be the
	CP	0D4H		;start and finishing edges of
	JR	NC,L058F;LD_SYNC;the 'off' sync pulse.
	CALL	L05E7;LD_EDGE_1	;The finishing edge of the 'on' pulse must
	RET	NC		;exists. (Return carry flag reset.)

;The bytes of the header or the program/data block can now be LOADed or
;VERIFYed. But the 1st byte is the type flag.
	LD	A,C		;The border colours from now
	XOR	3		;on will be BLUE & YELLOW.
	LD	C,A
	LD	H,0		;Initialise the 'parity matching' byte to zero.
	LD	B,0B0H		;Set the timing constant for the flag byte.
	JR	L05C8;LD_MARKER	;Jump forward into the byte LOADING loop.

;The byte LOADing loop is used to fetch the bytes one at a time. The flag
;byte is 1st. This is followed by the data bytes and the last byte is the
;'parity' byte.
L05A9:;LD_LOOP
	EX	AF,AF'		;Fetch the flags.
	JR	NZ,L05B3;LD_FLAG;Jump forward only when handling
				;the 1st byte.
	JR	NC,L05BD;LD_VERIFY ;Jump forward if VERIFYing a tape.
	LD	(IX+0),L	;Make the actual LOAD when required.
	JR	L05C2;LD_NEXT	;Jump forward to LOAD the next byte.
L05B3:;LD_FLAG
	RL	C		;Keep the carry flag in a safe
				;place temporarily.
	XOR	L		;Return now if the type flag does not match
	RET	NZ		;the 1st byte on the tape.(Carry flag reset.)
	LD	A,C		;Restore the carry flag now.
	RRA
	LD	C,A
	INC	DE		;Increase the counter to compensate
	JR	L05C4;LD_DEC	;for its 'decrease' after the jump.

;If a data block is being verified then then the freshly loaded byte is
;tested against the original byte.
L05BD:;LD_VERIFY
	LD	A,(IX+0)	;Fetch the original byte.
	XOR	L		;Match it against the new byte.
	RET	NZ		;Return if 'no match'. (Carry flag reset.)

;A new byte can now be collected from the tape.
L05C2:;LD_NEXT
	INC	IX		;Increase the 'destination'.
L05C4:;LD_DEC
	DEC	DE		;Decrease the 'counter'.
	EX	AF,AF'		;Save the flags.
	LD	B,0B2H		;Set the timing constant.
L05C8:;LD_MARKER
	LD	L,1		;Clear the 'object' reg apart from
				;a 'marker' bit.

;The 'LD_8_BITS' loop is used to build up a byte in the L reg.
L05CA:;LD_8_BITS
	CALL	L05E3;LD_EDGE_2	;Find the length of the 'off' and 'on'
				;pulses of the next bit.
	RET	NC		;Return if the time period is
				;exceeded. (Carry flag reset.)
	LD	A,0CBH		;Compare the length against approx.
				;2,400 T states; resetting the carry flag
	CP	B		;for a '0' and setting for a '1'.
	RL	L		;Include the new bit in the L reg.
	LD	B,0B0H		;Set the timing constant for the next bit.
	JP	NC,L05CA;LD_8_BITS ;Jump back whilst there are still bits
				;to be fetched.

;The 'parity matching' byte has to be updated with each new byte.
	LD	A,H		;Fetch the 'parity matching'
	XOR	L		;byte and include the new byte.
	LD	H,A		;Save it once again.

;Passes round the loop are made until the 'counter' reaches zero. At that
;point the 'parity matching' byte should be holding zero.
	LD	A,D		;Make a further pass if the DE
	OR	E		;reg pair does not hold zero.
	JR	NZ,L05A9;LD_LOOP
	LD	A,H		;Fetch the 'parity matching' byte.
	CP	1		;Return with the carry flag set if the value
	RET			;is zero. (Carry flag reset if in error.)

;	THE 'LD_EDGE_2' AND 'LD_EDGE_1' SUBROUTINES
;These two subroutines form the most important part of the LOAD/VERIFY
;operation.
;  The subroutines are entered with a timing constant in the B reg, and the
;previous border colour and 'edge-type' in the C reg.
;  The subroutines return with the carry flag set if the required number of
;'edges' have been found in the time allowed; and the change to the value
;in the B reg shows just how long it took to find the 'edge(s)'.
;  The carry flag will be reset if there is an error. The zero flag then
;signals 'BREAK pressed' by being reset, or 'time-up' by being set.
;  The entry point LD_EDGE_2 is used when the length of a complete pulse is
;required and LD_EDGE_1 is used to find the time before the next 'edge'.
L05E3:;LD_EDGE_2
	CALL	L05E7;LD_EDGE_1	;In effect call LD_EDGE_1 twice; returning
	RET	NC		;in between if there is an error.
L05E7:;LD_EDGE_1
	LD	A,16H		;Wait 358 T states before
L05E9:;LD_DELAY
	DEC	A		;entering the sampling loop.
	JR	NZ,L05E9;LD_DELAY
	AND	A

;The sampling loop is now entered. The value in the B reg is incremented
;for each pass; 'time-up' is given when B reaches zero.
L05ED:;LD_SAMPLE
	INC	B		;Count each pass.
	RET	Z		;Return carry reset & zero set if 'time-up'.
	LD	A,7FH		;Read from port 7FFE.
	IN	A,(0FEH)	;i.e. BREAK & EAR.
	RRA			;Shift the byte.
	RET	NC		;Return carry reset & zero reset if BREAK
				;was pressed.
	XOR	C		;Now test the byte against the 
	AND	20H		;'last edge-type'; jump back
	JR	Z,L05ED;LD_SAMPLE ;unless it has changed.

;A new 'edge' has been found within the time period allowed for the search.
;So change the border colour and set the carry flag.
	LD	A,C		;Change the 'last edge-type'
	CPL			;and border colour.
	LD	C,A
	AND	7		;Keep only the border colour.
	OR	8		;Signal 'MIC off'.
	OUT	(0FEH),A	;Change the border colour (RED/CYAN
				;or BLUE/YELLOW).
	SCF			;Signal the successful search
	RET			;before returning.
;
;Note: The LD_EDGE_1 subroutine takes 465 T states, plus an additional 58
;T states for each unsuccessful pass around the sampling loop.
;  For example, therefore, when awaiting the sync pulse (see LD_SYNC at
;058F) allowance is made for ten additional passes through the sampling
;loop. The search is thereby for the next edge to be found within <???>
;1,100 T states (465 + 10*58 + overhead). This will prove successful for
;the sync 'off' pulse that comes after the long 'leader pulses'.

