
;	THE 'EDITOR' ROUTINES
;The editor is called in two occasions:
;i.   From the main execution routine so that the user can enter a BASIC
;     line into the system.
;ii.  From the INPUT command routine.
;
;First the 'error stack pointer' is saved and an alternative addr provided.
L0F2C:;EDITOR
	LD	HL,(L5C3D);ERR_SP ;The current value is saved on
	PUSH	HL		;the machine stack.
L0F30:;ED_AGAIN
	LD	HL,L107F	;This is ED_ERROR.
	PUSH	HL		;Any event that leads to the error
	LD	(L5C3D),SP;ERR_SP ;handling routine being used will
				;come back to ED_ERROR.

;A loop is now entered to handle each keystroke.
L0F38:;ED_LOOP
	CALL	L15D4;WAIT_KEY	;Return once a key has been pressed.
	PUSH	AF		;Save the code temporarily.
	LD	D,0		;Fetch the duration of the
	LD	E,(IY+0FFH);PIP	;keyboard click.
	LD	HL,0C8H		;And the pitch.
	CALL	L03B5;BEEPER	;Now make the 'pip'.
	POP	AF		;Restore the code.
	LD	HL,L0F38	;Pre-load the machine stack with
	PUSH	HL		;the addr of ED_LOOP.

;Now analyse the code obtained.
	CP	18H		;Accept all char codes,
	JR	NC,L0F81;ADD_CHAR ;graphic codes and tokens.
	CP	7		;Also accept ','.
	JR	C,L0F81;ADD_CHAR
	CP	10H		;Jump forward if the code
	JR	C,L0F92;ED_KEYS	;represents an editing key.

;The control keys - INK to TAB - are now considered.
	LD	BC,2		;INK & PAPER will require two locs.
	LD	D,A		;Copy the code to D.
	CP	16H		;Jump forward with INK &
	JR	C,L0F6C;ED_CONTR;PAPER.

;AT & TAB would be handled as follows:
	INC	BC		;Three locs required.
	BIT	7,(IY+37H);FLAGX;Jump forward unless dealing
	JP	Z,L101E;ED_IGNORE ;with INPUT LINE... .
	CALL	L15D4;WAIT_KEY	;Get the second code.
	LD	E,A		;and put it in E.

;The other bytes for the control chars are now fetched.
L0F6C:;ED_CONTR
	CALL	L15D4;WAIT_KEY	;Get another code.
	PUSH	DE		;Save the previous codes.
	LD	HL,(L5C5B)	;Fetch K_CUR.
	RES	0,(IY+07H);MODE	;Signal 'K mode'.
	CALL	L1655;MAKE_ROOM	;Make two or three spaces.
	POP	BC		;Restore the previous codes.
	INC	HL		;Point to the 1st loc.
	LD	(HL),B		;Enter 1st code.
	INC	HL		;Then enter the second code which will be
	LD	(HL),C		;overwritten if there are only two codes -
				;i.e. with INK & PAPER.
	JR	L0F8B;ADD_CH_1	;Jump forward.

;	THE 'ADD_CHAR' SUBROUTINE
L0F81:;ADD_CHAR
	RES	0,(IY+07H);MODE	;Signal 'K mode'.
	LD	HL,(L5C5B);K_CUR;Fetch the cursor position.
	CALL	L1652;ONE_SPACE	;Make a single space.
L0F8B:;ADD_CH_1
	LD	(DE),A		;Enter the code into the space
	INC	DE		;and signal that the cursor is to
	LD	(L5C5B),DE;K_CUR;occur at the loc after. Then
	RET			;return indirectly to ED_LOOP.

;The editing keys are dealt with as follows:
L0F92:;ED_KEYS
	LD	E,A		;The code is transferred to
	LD	D,0		;the DE reg pair.
	LD	HL,L0FA0-7	;0F99:The base addr of the editing
				;key table.
	ADD	HL,DE		;The entry is addressed and
	LD	E,(HL)		;then fetched into E.
	ADD	HL,DE		;The addr of the handling routine
	PUSH	HL		;is saved on the machine stack.
	LD	HL,(L5C5B);K_CUR;The HL reg pair is set and an indirect
	RET			;jump made to the required routine.

;	THE 'EDITING KEYS' TABLE
L0FA0:
	DB	L0FA9-$	;09H		;EDIT
	DB	L1007-$	;66H		;cursor left
	DB	L100C-$	;6AH		;cursor right
	DB	L0FF3-$	;50H		;cursor down
	DB	L1059-$	;0B5H		;cursor up
	DB	L1015-$	;70H		;DELETE
	DB	L1024-$	;7EH		;ENTER
	DB	L1076-$	;0CFH		;SYMBOL SHIFT
	DB	L107C-$	;0D4H		;GRAPHICS

;	THE 'EDIT KEY' SUBROUTINE
;When in 'editing mode' pressing the EDIT key will bring down the 'current
;BASIC line'. However in 'INPUT mode' the action of the EDIT key is to
;clear the current reply and allow a fresh one.
L0FA9:;ED_EDIT
	LD	HL,(L5C49);E_PPC;Fetch the current line number.
	BIT	5,(IY+37H);FLAGX;But jump forward if in
	JP	NZ,L1097;CLEAR_SP ;'INPUT mode'.
	CALL	L196E;LINE_ADDR	;Find the addr of the start of
	CALL	L1695;LINE_NO	;the current line and hence its number.
	LD	A,D		;If the line number returned is
	OR	E		;zero then simply clear the
	JP	Z,L1097;CLEAR_SP;editing area.
	PUSH	HL		;Save the addr of the line.
	INC	HL		;Move on to collect the
	LD	C,(HL)		;length of the line.
	INC	HL
	LD	B,(HL)
	LD	HL,0AH		;Add 0A to the length and test
	ADD	HL,BC		;that there is sufficient room
	LD	B,H		;for a copy of the line.
	LD	C,L
	CALL	L1F05;TEST_ROOM
	CALL	L1097;CLEAR_SP	;Now clear the editing area.
	LD	HL,(L5C51);CURCHL ;Fetch the current channel addr and
	EX	(SP),HL		;exchange it for the addr of the line.
	PUSH	HL		;Save it temporarily.
	LD	A,0FFH		;Open channel 'R' so that the
	CALL	L1601;CHAN_OPEN	;line will be copied to the editing area.
	POP	HL		;Fetch the addr of the line.
	DEC	HL		;Go to before the line.
	DEC	(IY+0FH);E_PPC_lo ;Decrement the current line number
				;so as to avoid printing the cursor.
	CALL	L1855;OUT_LINE	;Print the BASIC line
	INC	(IY+0FH);E_PPC_lo ;Increment the current line number.
			;Note: The decrementing of the line number
			;does not always stop the cursor from
			;being printed.
	LD	HL,(L5C59);E_LINE ;Fetch the start of the line in
	INC	HL		;the editing area and step past
	INC	HL		;the line number and the length
	INC	HL		;to find the addr for K_CUR.
	INC	HL
	LD	(L5C5B),HL;K_CUR
	POP	HL		;Fetch the former channel addr and
	CALL	L1615;CHAN_FLAG	;set the appropriate flags before
	RET			;returning to ED_LOOP.

;	THE 'CURSOR DOWN EDITING' SUBROUTINE
L0FF3:;ED_DOWN
	BIT	5,(IY+37H);FLAGX;Jump forward if in
	JR	NZ,L1001;ED_STOP;'INPUT' mode.
	LD	HL,L5C49	;This is E_PPC.
	CALL	L190F;LN_FETCH	;The next line number is found and a new
	JR	L106E;ED_LIST	;automatic listing produced.
L1001:;ED_STOP
	LD	(IY+0),10H	;'STOP in INPUT' report.
	JR	L1024;ED_ENTER	;Jump forward.

;	THE 'CURSOR LEFT EDITING' SUBROUTINE
L1007:;ED_LEFT
	CALL	L1031;ED_EDGE	;The cursor is moved.
	JR	L1011;ED_CUR	;Jump forward.

;	THE 'CURSOR RIGHT EDITING' SUBROUTINE
L100C:;ED_RIGHT
	LD	A,(HL)		;The current char is tested
	CP	0DH		;and if it is 'carriage return'
	RET	Z		;then return. Otherwise make the
	INC	HL		;cursor come after the char.
L1011:;ED_CUR
	LD	(L5C5B),HL;K_CUR;Set the system variable K_CUR.
	RET

;	THE 'DELETE EDITING' SUBROUTINE
L1015:;ED_DELETE
	CALL	L1031;ED_EDGE	;Move the cursor leftwards.
	LD	BC,1		;Reclaim the current
	JP	L19E8;RECLAIM_2	;char.

;	THE 'ED_IGNORE' SUBROUTINE
L101E:;ED_IGNORE
	CALL	L15D4;WAIT_KEY	;The next two codes from the
	CALL	L15D4;WAIT_KEY	;key-input routine are ignored.

;	THE 'ENTER EDITING' SUBROUTINE
L1024:;ED_ENTER
	POP	HL		;The addr of ED_LOOP and ED_ERROR
	POP	HL		;are discarded.
L1026:;ED_END
	POP	HL		;The old value of ERR_SP
	LD	(L5C3D),HL;ERR_SP ;is restored.
	BIT	7,(IY+0);ERR_NR	;Now return if there were
	RET	NZ		;no errors.
	LD	SP,HL		;Otherwise make an indirect
	RET		        ;jump to the error routine.

;	THE 'ED_EDGE' SUBROUTINE
;The addr of the cursor is in the HL reg pair and will be decremented
;unless the cursor is already at the start of the line. Care is taken
;not to put the cursor between control char and their parameters.
L1031:;ED_EDGE
	SCF			;DE will hold either E_LINE
	CALL	L1195;SET_DE	;(for editing) or WORKSP (for INPUTing).
	SBC	HL,DE		;The carry flag will become set
				;if the cursor is already to be at
	ADD	HL,DE		;the start of the line.
	INC	HL		;Correct for the subtraction.
	POP	BC		;Drop the return addr.
	RET	C		;Return via ED_LOOP if the carry is set.
	PUSH	BC		;Restore the return addr.
	LD	B,H		;Move the current addr of
	LD	C,L		;the cursor to BC.

;Now enter a loop to check that control chars are not split from their
;parameters.
L103E:;ED_EDGE_1
	LD	H,D		;HL will point to the
	LD	L,E		;char in the line after
	INC	HL		;that addressed by DE.
	LD	A,(DE)		;Fetch a char code.
	AND	0F0H		;Jump forward if the code
	CP	10H		;does not represent
	JR	NZ,L1051;ED_EDGE_2 ;INK to TAB.
	INC	HL		;Allow for one parameter.
	LD	A,(DE)		;Fetch the code anew.
	SUB	17H		;Carry is reset for TAB.
	ADC	A,0		;Note: This splits off AT & TAB but AT
				;& TAB in this form are not implemented
				;anyway so it makes no difference.
	JR	NZ,L1051;ED_EDGE_2 ;Jump forward unless dealing with AT &
	INC	HL		;TAB which would have 2 parameters, if used.
L1051:;ED_EDGE_2
	AND	A		;Prepare for true subtraction.
	SBC	HL,BC		;The carry flag will be reset when
	ADD	HL,BC		;the 'updated pointer' reaches K_CUR.
	EX	DE,HL		;For the next loop use the 'updated pntr',
	JR	C,L103E;ED_EDGE_1 ;but if exiting use the 'present pntr'
				;for K_CUR.
			;Note: It is the control char that is deleted
			;when using DELETE.
	RET

;	THE 'CURSOR UP EDITING' SUBROUTINE
L1059:;ED_UP
	BIT	5,(IY+37H);FLAGX;Return if in 'INPUT mode'.
	RET	NZ
	LD	HL,(L5C49);E_PPC;Fetch the current line
	CALL	L196E;LINE_ADDR	;number and its start addr.
	EX	DE,HL		;HL now points to the previous line.
	CALL	L1695;LINE_NO	;This line's number is fetched.
	LD	HL,L5C4A	;This is E_PPC_hi.
	CALL	L191C;LN_STORE	;The line number is stored.
L106E:;ED_LIST
	CALL	L1795;AUTO_LIST	;A new automatic listing is
	LD	A,0		;now produced and channel 'K'
	JP	L1601;CHAN_OPEN	;re-opened before returning to ED_LOOP.

;	THE 'ED_SYMBOL' SUBROUTINE
;If SYMBOL & GRAPHICS codes were used they would be handled as follows:
L1076:;ED_SYMBOL
	BIT	7,(IY+37H);FLAGX;Jump back unless dealing with
	JR	Z,L1024;ED_ENTER;INPUT...LINE.
L107C:;ED_GRAPH
	JP	L0F81;ADD_CHAR	;Jump back.

;	THE 'ED_ERROR' SUBROUTINE
;Come here when there has been some kind of error.
L107F:;ED_ERROR
	BIT	4,(IY+30H);FLAGS2 ;Jump back if using other than
	JR	Z,L1026;ED_END	;channel 'K'.
	LD	(IY+0),0FFH;ERR_NR ;Cancel the error number and
	LD	D,0		;give a 'rasp' before going
	LD	E,(IY+0FEH);RASP;around the editor again.
	LD	HL,1A90H
	CALL	L03B5;BEEPER
	JP	L0F30;ED_AGAIN

;	THE 'CLEAR_SP' SUBROUTINE
L1097:;CLEAR_SP
	PUSH	HL		;Save the pointer to the space.
	CALL	L1190;SET_HL	;DE will point to the 1st
				;char and HL the last.
	DEC	HL		;The correct amount is now
	CALL	L19E5;RECLAIM_1	;reclaimed.
	LD	(L5C5B),HL;K_CUR;The system variables K_CUR
	LD	(IY+07H),0;MODE	;and MODE ('K mode') are
	POP	HL		;initialised before fetching
	RET			;the pointer and returning.

;	THE 'KEYBOARD INPUT' SUBROUTINE
;This important subroutine returns the code of the last key to have been
;pressed but note that CAPS LOCK, the changing of the mode and the colour
;control parameters are handled within the subroutine.
L10A8:;KEY_INPUT
	BIT	3,(IY+02H);TV_FLAG ;Copy the edit-line or the
	CALL	NZ,L111D;ED_COPY;INPUT-line to the screen if
				;the mode has changed.
	AND	A		;Return with both carry
	BIT	5,(IY+1);FLAGS	;and zero flags reset if no
	RET	Z		;new key has been pressed.
	LD	A,(L5C08);LAST_K;Otherwise fetch the code and
	RES	5,(IY+1);FLAGS	;signal that it has been taken.
	PUSH	AF		;Save the code temporarily.
	BIT	5,(IY+2);TV_FLAG;Clear the lower part of the display if
	CALL	NZ,L0D6E;CLS_LOWER ;necessary; e.g. after 'scroll?';
	POP	AF		;Fetch the code.
	CP	' '		;Accept all chars and token codes.
	JR	NC,L111B;KEY_DONE
	CP	10H		;Jump forward with most of the
	JR	NC,L10FA;KEY_CONTR ;control char codes.
	CP	6		;Jump forward with the 'mode'
	JR	NC,L10DB;KEY_M&CL ;codes and the CAPS LOCK code.

;Now deal with the FLASH, BRIGHT & INVERSE codes.
	LD	B,A		;Save the code.
	AND	1		;Keep only bit 0.
	LD	C,A		;C holds 00 (=OFF) or 01 (=ON).
	LD	A,B		;Fetch the code.
	RRA			;Rotate it once (losing bit 0).
	ADD	A,12H		;Increase it by 12 giving for FLASH-12,
	JR	L1105;KEY_DATA	;BRIGHT-13 and INVERSE-14.

;The CAPS LOCK code and the mode codes are dealt with 'locally'.         
L10DB:;KEY_M&CL
	JR	NZ,L10E6;KEY_MODE ;Jump forward with 'mode' codes.
	LD	HL,L5C6A	;This is FLAGS2.
	LD	A,8		;Flip bit 3 of FLAGS2. This is
	XOR	(HL)		;the CAPS LOCK flag.
	LD	(HL),A
	JR	L10F4;KEY_FLAG	;Jump forward.
L10E6:;KEY_MODE
	CP	0EH		;Check the lower limit.
	RET	C
	SUB	0DH		;Reduce the range.
	LD	HL,L5C41	;This is MODE.
	CP	(HL)		;Has it been changed?
	LD	(HL),A		;Enter the new 'mode' code.
	JR	NZ,L10F4;KEY_FLAG ;Jump if it has changed;
	LD	(HL),0		;otherwise make it 'L mode'.
L10F4:;KEY_FLAG
	SET	3,(IY+2);TV_FLAG;Signal 'the mode might have changed'.
	CP	A		;Reset the carry flag and
	RET			;return.

;The ctrl key codes (apart from FLASH, BRIGHT & INVERSE) are manipulated.
L10FA:;KEY_CONTR
	LD	B,A		;Save the code.
	AND	7		;Make the C reg hold the
	LD	C,A		;parameter. (00 to 07)
	LD	A,10H		;A now holds the INK codes.
	BIT	3,B		;But if the code was an
	JR	NZ,L1105;KEY_DATA ;'unshifted' code then make A
	INC	A		;hold the PAPER code.

;The parameter is saved in K_DATA and the channel addr changed from
;KEY_INPUT to KEY_NEXT.
L1105:;KEY_DATA
	LD	(IY+0D3H),C;K_DATA ;Save the parameter.
	LD	DE,L110D	;This is KEY_NEXT.
	JR	L1113;KEY_CHAN	;Jump forward.

;Note: On the 1st pass entering at KEY_INPUT the A reg is returned
;holding a 'control code' and then on the next pass, entering at KEY_NEXT,
;it is the parameter that is returned.
L110D:;KEY_NEXT
	LD	A,(L5C0D);K_DATA;Fetch the parameter.
	LD	DE,L10A8	;This is KEY_INPUT.

;Now set the input addr in the 1st channel area.
L1113:;KEY_CHAN
	LD	HL,(L5C4F);CHANS;Fetch the channel addr.
	INC	HL
	INC	HL
	LD	(HL),E		;Now set the input addr.
	INC	HL
	LD	(HL),D

;Finally exit with the required code in the A reg.
L111B:;KEY_DONE
	SCF			;Show a code has been found
	RET			;and return.

;	THE 'LOWER SCREEN COPYING' SUBROUTINE
;This subroutine is called whenever the line in the editing area or the
;INPUT area is to be printed in the lower part of the screen.
L111D:;ED_COPY
	CALL	L0D4D;TEMPS	;Use the permanent colours.
	RES	3,(IY+02H);TV_FLAG ;Signal that the 'mode is to be
	RES	5,(IY+02H);TV_FLAG ;considered unchanged' and the
				;'lower screen does not need clearing'.
	LD	HL,(L5C8A)	;Save the current value of S_POSNL.
	PUSH	HL
	LD	HL,(L5C3D)	;Keep the current value of ERR_SP.
	PUSH	HL
	LD	HL,L1167	;This is ED_FULL.
	PUSH	HL		;Push this addr on to the machine stack
	LD	(L5C3D),SP;ERR_SP ;to make ED_FULL the entry point
				;following an error.
	LD	HL,(L5C82);ECHO_E ;Push the value of ECHO_E
	PUSH	HL		;on to the stack.
	SCF			;Make HL point to the start
	CALL	L1195;SET_HL	;of the space and DE the end.
	EX	DE,HL
	CALL	L187D;OUT_LINE2	;Now print the line.
	EX	DE,HL		;Exchange the pointers and
	CALL	L18E1;OUT_CURS	;print the cursor.
	LD	HL,(L5C8A);S_POSNL ;Next fetch the current value of
	EX	(SP),HL		;S_POSNL and exchange it with ECHO_E.
	EX	DE,HL		;Pass ECHO_E to DE.
	CALL	L0D4D;TEMPS	;Again fetch the permanent colours.

;The remainder of any line that has been started is now completed with
;spaces printed with the 'permanent' PAPER colour.
L1150:;ED_BLANK
	LD	A,(L5C8B);S_POSNL_hi ;Fetch the current line number
	SUB	D		;and subtract the old line number.
	JR	C,L117C;ED_C_DONE ;Jump forward if no 'blanking'
				;of lines required.
	JR	NZ,L115E;ED_SPACES ;Jump forward if not on the same line.
	LD	A,E		;Fetch the old column number
	SUB	(IY+50H);S_POSNL_lo ;and subtract the new column number.
	JR	NC,L117C;ED_C_DONE  ;Jump if no spaces required.
L115E:;ED_SPACES
	LD	A,' '		;A 'space'.
	PUSH	DE		;Save the old values.
	CALL	L09F4;PRINT_OUT	;Print it.
	POP	DE		;Fetch the old values.
	JR	L1150;ED_BLANK	;Back again.

;New deal with any errors.
L1167:;ED_FULL
	LD	D,0		;Give out a 'rasp'.
	LD	E,(IY+0FEH);RASP
	LD	HL,1A90H
	CALL	L03B5;BEEPER
	LD	(IY+00H),0FFH;ERR_NR ;Cancel the error number.
	LD	DE,(L5C8A);S_POSNL ;Fetch the current value of
	JR	L117E;ED_C_END	;S_POSNL and jump forward.

;The normal exit upon completion of the copying over of the edit or the
;INPUT line.
L117C:;ED_C_DONE
	POP	DE		;The new position value.
	POP	HL		;The 'error addr'.

;But come here after an error.
L117E:;ED_C_END
	POP	HL		;The old value of ERR_SP is
	LD	(L5C3D),HL;ERR_SP ;restored.
	POP	BC		;Fetch the old value of S_POSNL.
	PUSH	DE		;Save the new position values.
	CALL	L0DD9;CL_SET	;Set the system variables.
	POP	HL		;The old values of S_POSNL
	LD	(L5C82),HL;ECHO_E ;goes into ECHO_E.
	LD	(IY+26H),0;X_PTR_hi ;X_PTR is cleared in a
	RET			;suitable manner and the return made.

;	THE 'SET-HL' AND 'SET-DE' SUBROUTINES
;These subroutines return with HL pointing to the 1st loc and DE the
;'last' loc of either the editing area or the work space.
L1190:;SET_HL
	LD	HL,(L5C61);WORKSP ;Point to the last loc
	DEC	HL		;of the editing area.
	AND	A		;Clear the carry flag.
L1195:;SET_DE
	LD	DE,(L5C59);E_LINE ;Point to the start of the
	BIT	5,(IY+37H);FLAGX  ;editing area and return if
	RET	Z		;in 'editing mode'.
	LD	DE,(L5C61);WORKSP ;Otherwise change DE.
	RET	C		;Return if now intended.
	LD	HL,(L5C63);STKBOT ;Fetch STKBOT and then
	RET			;return.

;	THE 'REMOVE_FP' SUBROUTINE
L11A7:;REMOVE_FP
	LD	A,(HL)		;Each char in turn is examined.
	CP	0EH		;Is it a number marker?
	LD	BC,6		;It will occupy six locs.
	CALL	Z,L19E8;RECLAIM_2 ;Reclaim the F-P number.
	LD	A,(HL)		;Fetch the code again.
	INC	HL		;Update the pointer.
	CP	0DH		;'Carriage return'?
	JR	NZ,L11A7;REMOVE_FP ;Back if not. But make a
	RET			;simple return if it is.

