

;	THE EXECUTIVE ROUTINES

;	THE 'INITIALISATION' ROUTINE
;The main entry point to this routine is at START/NEW (11CB). When entered
;from START (0000), as when power is first applied to the system, the A reg
;holds zero and the DE reg the value 0FFFFh. However the main entry point
;can also be reached following the execution of the NEW command routine.

;	THE 'NEW COMMAND' ROUTINE

L11B7:;NEW
	DI			;Disable the maskable interrupt.
	LD	A,0FFH		;The NEW flag.
	LD	DE,(L5CB2);RAMTOP ;The existing value of RAMTOP is
				;preserved.
	EXX			;Load the alternate regs
	LD	BC,(L5CB4);P_RAMT ;with the following system
	LD	DE,(L5C38);RASP/PIP ;vars. All of which will
	LD	HL,(L5C7B);UDG	;also be preserved.
	EXX

;The main entry point.
L11CB:;START/NEW
	LD	B,A		;Save the flag for later.
	LD	A,7		;Make the border white in colour.
	OUT	(0FEH),A
	LD	A,3Fh		;Set the I reg to hold
	LD	I,A		;the value of 3Fh.
	DB	0,0,0,0,0,0	;Wait 24 T states.

;Now the memory is checked.
L11DA:;RAM_CHECK
	LD	H,D		;Transfer the value in DE
	LD	L,E		;(START=0FFFFh, NEW=RAMTOP)
L11DC:;RAM_FILL
	LD	(HL),2		;Enter the value of 2 into
	DEC	HL		;every location above 3FFFh.
	CP	H
	JR	NZ,L11DC;RAM_FILL
L11E2:;RAM_READ
	AND	A		;Prepare for true subtraction.
	SBC	HL,DE		;The carry flag will become
	ADD	HL,DE		;reset when the top is reached.
	INC	HL		;Update the pntr.
	JR	NC,L11EF;RAM_DONE ;Jump when at top.
	DEC	(HL)		;2 goes to 1.
	JR	Z,L11EF;RAM_DONE ;But if zero then RAM is faulty.
				;Use curr HL as top.
	DEC	(HL)		;1 goes to 0.
	JR	Z,L11E2;RAM_READ ;Step to the next test unless it fails.
L11EF:;RAM_DONE
	DEC	HL		;HL points to the last actual location
				;in working order.

;Next restore the 'preserved' system vars. (Meaningless when coming
;from START.)
	EXX			;Switch regs.
	LD	(L5CB4),BC;P_RAMT ;Restore P_RAMT, RASP/PIP & UDG.
	LD	(L5C38),DE;RASP/PIP
	LD	(L5C7B),HL;UDG
	EXX
	INC	B		;Test the START/NEW flag. Jump forward
	JR	Z,L1219;RAM_SET	;if coming from the NEW command routine.

;Overwrite the system vars when coming from START and initialise the
;user-defined graphics area.
	LD	(L5CB4),HL;P_RAMT ;Top of physical RAM.
	LD	DE,L3EAF	;Last byte of 'U' in char set.
	LD	BC,0A8H		;There are this number of bytes
				;in twenty one letters.
	EX	DE,HL		;Switch the pntrs.
	LDDR			;Now copy the char forms of the
				;letters 'A' to 'U'.
	EX	DE,HL		;Switch the pntrs back.
	INC	HL		;Point to the 1st byte.
	LD	(L5C7B),HL;UDG	;Now set UDG.
	DEC	HL		;Down one location.
	LD	BC,40h		;Set the system vars RASP & PIP.
	LD	(L5C38),BC;RASP/PIP

;The remainder of the routine is common to both the START and the NEW
;operations.
L1219:;RAM_SET
	LD	(L5CB2),HL;RAMTOP ;Set RAMTOP.
	LD	HL,L3D00-100h	;Initialise the system var CHARS.
	LD	(L5C36),HL;CHARS

;Next the machine stack is set up.
	LD	HL,(L5CB2);RAMTOP ;The top location is made to
	LD	(HL),3Eh	;hold 3Eh.
	DEC	HL		;The next location is left holding zero.
	LD	SP,HL		;These two locs represent the 'last entry'.
	DEC	HL		;Step down two locs to find the
	DEC	HL		;correct value for ERR_SP.
	LD	(L5C3D),HL;ERR_SP

;The initialisation routine continues with:
	IM	1		;Interrupt mode 1 is used.
	LD	IY,L5C3A;ERR_NR	;IY holds ERR_NR always.
	EI			;The maskable interrupt can now be
				;enabled. The real-time clock will be
				;updated and the kbrd scanned every
				;1/50th of a second.
	LD	HL,5CB6h	;The base addr of the channel
	LD	(L5C4F),HL;CHANS ;information area.
	LD	DE,L15AF	;The initial channel data
	LD	BC,15H		;is moved from the table
	EX	DE,HL		;(15AF) to the channel
	LDIR			;information area.
	EX	DE,HL		;The system var DATADD
	DEC	HL		;is made to point to the last
	LD	(L5C57),HL;DATADD ;loc of the channel data.
	INC	HL		;And PROG & VARS to the loc after that.
	LD	(L5C53),HL;PROG
	LD	(L5C4B),HL;VARS
	LD	(HL),80H	;The end-marker of the vars area.
	INC	HL		;Move on one loc to find
	LD	(L5C59),HL;E_LINE ;the value for E_LINE.
	LD	(HL),0DH	;Make the edit-line be a single
	INC	HL		;'carriage return' char.
	LD	(HL),80H	;Now enter an end-marker.
	INC	HL		;Move on one loc to find
	LD	(L5C61),HL;WORKSP ;the value for WORKSP, STKBOT & STKEND.
	LD	(L5C63),HL;STKBOT
	LD	(L5C65),HL;STKEND
	LD	A,38h		;Initialise the colour system vars to:
	LD	(L5C8D),A;ATTR_P ;FLASH 0:BRIGHT 0:PAPER 7:INK 0
	LD	(L5C8F),A;ATTR_T
	LD	(L5C48),A;BORDCR
	LD	HL,0523h	;Init the system vars REPDEL & REPPER.
	LD	(L5C09),HL;REPDEL
	DEC	(IY+0C6H);KSTATE_0 ;Make KSTATE_0 hold 0FFh.
	DEC	(IY+0CAH);KSTATE_4 ;Make KSTATE_4 hold 0FFh.
	LD	HL,L15C6	;Next move the initial stream
	LD	DE,L5C10	;data from its table to the
	LD	BC,0Eh		;streams area.
	LDIR
	SET	1,(IY+01H);FLAGS ;Signal 'printer in use'
	CALL	L0EDF;CLEAR_PRB	 ;and clear the printer buffer.
	LD	(IY+31H),2;DF_SZ ;Set the size of the lower part of the
	CALL	L0D6B;CLS	;display and clear the whole display.
	XOR	A		;Now print the message
	LD	DE,L1538	;'(c) 1982 Sinclair Research Ltd'
	CALL	L0C0A;PO_MSG	;on the bottom line.
	SET	5,(IY+02H);TV_FLAG ;Signal 'the lower part will
				;required to be cleared.
	JR	L12A9;MAIN_1	;Jump forward into the main execution loop.

;	THE 'MAIN EXECUTION' LOOP
;The main loop extends from loc 12A2 to loc 15AE and it controls the
;'editing mode', the execution of direct commands and the production
;of reports.
L12A2:;MAIN_EXEC
	LD	(IY+31H),2;DF_SZ ;The lower part of the screen is to be
				;two lines in size.
	CALL	L1795;AUTO_LIST	;Produce an automatic listing.
L12A9:;MAIN_1
	CALL	L16B0;SET_MIN	;All the areas from E_LINE onwards are
				;given their minimum configuration.
L12AC:;MAIN_2
	LD	A,0		;Channel 'K' is opened before
	CALL	L1601;CHAN_OPEN	;calling the EDITOR.
	CALL	L0F2C;EDITOR	;The EDITOR is called to allow the user
				;to build up a BASIC line.
	CALL	L1B17;LINE_SCAN	;The current line is scanned
				;for correct syntax.
	BIT	7,(IY+00H);ERR_NR ;Jump forward if the syntax is correct.
	JR	NZ,L12CF;MAIN_3
	BIT	4,(IY+30H);FLAGS2 ;Jump forward if other than channel 'K'
	JR	Z,L1303;MAIN_4	;is being used.
	LD	HL,(L5C59);E_LINE ;Point to the start of the line
				;with the error.
	CALL	L11A7;REMOVE_FP	;Remove the flpnt forms from this line.
	LD	(IY+00H),0FFH;ERR_NR ;Reset ERR_NR and jump back to
	JR	L12AC;MAIN_2	;MAIN_2 leaving the listing unchanged.

;The 'edit_line' has passed syntax and the three types of line that are
;possible have to be distinguished from each other.
L12CF:;MAIN_3
	LD	HL,(L5C59);E_LINE ;Point to the start of the line.
	LD	(L5C59),HL;CH_ADD ;Set CH_ADD to the start also.
	CALL	L19FB;E_LINE_NO	  ;Fetch any line number into BC.
	LD	A,B		;Is the line number a valid
	OR	C		;one? Jump if it is so, and add
	JP	NZ,L155D;MAIN_ADD ;the new line to the existing program.
	RST	18H		;Fetch the 1st char of the line and see
	CP	0DH		;if the line is 'carriage return only'.
	JR	Z,L12A2;MAIN_EXEC ;If it is then jump back.

;The 'edit_line' must start with a direct BASIC command so this line
;becomes the 1st line to be interpreted.
	BIT	0,(IY+30H);FLAGS2 ;Clear the whole display unless
	CALL	NZ,L0DAF;CL_ALL	  ;the flag says it is unnecessary.
	CALL	L0D6E;CLS_LOWER	  ;Clear the lower part anyway.
	LD	A,19H		;Set the appropriate value
	SUB	(IY+4FH);S_POSN_hi ;for the scroll cntr.
	LD	(L5C8C),A;SCR_CT
	SET	7,(IY+01H);FLAGS ;Signal 'line execution'.
	LD	(IY+00H),0FFH;ERR_NR ;Ensure ERR_NR is correct.
	LD	(IY+0AH),1;NSPPC ;Deal with the 1st stmt in the line.
	CALL	L1B8A;PROG_RUN	;Now the line is interpreted.
;Note: The addr 1303h goes on to the machine stack and is adressed
;by ERR_SP.

;After the line has been interpreted and all the actions consequential
;to it have been completed a return is made to MAIN_4, so that a report
;can be made.
L1303:;MAIN_4
	HALT			;The maskable interrupt must be enabled.
	RES	5,(IY+01H);FLAGS ;Signal 'ready for a new key'.
	BIT	1,(IY+30H);FLAGS2 ;Empty the printer buffer if
	CALL	NZ,L0ECD;COPY_BUFF ;it has been used.
	LD	A,(L5C3A);ERR_NR ;Fetch the error number and
	INC	A		;increment it.
L1313:;MAIN_G
	PUSH	AF		;Save the new value.
	LD	HL,0		;The system vars FLAGX, X_PTR_hi & DEFADD
	LD	(IY+37H),H;FLAGX ;are all set to zero.
	LD	(IY+26H),H;X_PTR_hi
	LD	(L5C0B),HL;DEFADD
	LD	HL,1		;Ensure that stream 0 points to
	LD	(L5C16),HL;STRMS_6 ;channel 'K'.
	CALL	L16B0;SET_MIN	;Clear all the work areas and the
				;calculator stack.
	RES	5,(IY+37H);FLAGX ;Signal 'editing mode'.
	CALL	L0D6E;CLS_LOWER	;Clear the lower screen.
	SET	5,(IY+02H);TV_FLAG ;Signal 'the lower screen will
				;require clearing'.
	POP	AF		;Fetch the report value.
	LD	B,A		;Make a copy in B.
	CP	0AH		;Jump forward with report
	JR	C,L133C;MAIN_5	;numbers '0 to 9'.
	ADD	A,7		;Add the ASCII letter offset value.
L133C:;MAIN_5
	CALL	L15EF;OUT_CODE	;Print the report code and
	LD	A,' '		;follow it with a 'space'.
	RST	10H;PRINT_A_1
	LD	A,B		;Fetch the report value and
	LD	DE,L1391	;use it to identify the
	CALL	L0C0A;PO_MSG	;required report message.
	XOR	A		;Print the message and follow
	LD	DE,L1536	;it by a 'comma' and a 'space'.
	CALL	L0C0A;PO_MSG
	LD	BC,(L5C45);PPC	;Now fetch the current line
	CALL	L1A1B;OUT_NUM1	;number and print it as well.
	LD	A,':'		;Follow it by a ':'.
	RST	10H;PRINT_A_1
	LD	C,(IY+0DH);SUBPPC ;Fetch the current stmt
	LD	B,0		;number into the BC reg pair
	CALL	L1A1B;OUT_NUM1	;and print it.
	CALL	L1097;CLEAR_SP	;Clear the editing area.
	LD	A,(L5C3A);ERR_NR ;Fetch the error number again.
	INC	A		;Increment it as usual.
	JR	Z,L1386;MAIN_9	;If the program was completed
				;successfully there cannot be
				;any 'CONTinuing' so jump.
	CP	9		;If the program halted with
	JR	Z,L1373;MAIN_6	;'STOP stmt' or 'BREAK into program'
	CP	15H		;CONTinuing will be from the next stmt;
	JR	NZ,L1376;MAIN_7
L1373:;MAIN_6
	INC	(IY+0DH);SUBPPC	;otherwise SUBPPC is unchanged.
L1376:;MAIN_7
	LD	BC,3		;The system vars OLDPPC & OSPPC have
	LD	DE,L5C70	;now to be made to hold the CONTinuing
				;line and stmt number.
	LD	HL,L5C44	;The values used will be those in
	BIT	7,(HL);NSPPC	;PPC & SUBPPC unless NSPPC
	JR	Z,L1384;MAIN_8	;indicates that the 'break'
	ADD	HL,BC		;occured before a 'jump'.
L1384:;MAIN_8
	LDDR			;(i.e. after a GO TO stmt etc.)
L1386:;MAIN_9
	LD	(IY+0AH),0FFH;NSPPC ;NSPPC is reset to indicate 'no jump'.
	RES	3,(IY+01H);FLAGS ;'K mode' is selected.
	JP	L12AC;MAIN_2	;And finally the jump back is made but
				;no program listing will appear until
				;requested.

;	THE REPORT MESSAGES
;Each message is given with the last char inverted (+80h).
L1391:	
	DB	x
	DB	'O','k'+x			;Report 0
	DB	'NEXT without FO','R'+x		;1
	DB	'Variable not foun','d'+x	;2
	DB	'Subscript wron','g'+x		;3
	DB	'Out of memor','y'+x		;4
	DB	'Out of scree','n'+x		;5
	DB	'Number too bi','g'+x		;6
	DB	'RETURN without GOSU','B'+x	;7
	DB	'End of fil','e'+x		;8
	DB	'STOP statemen','t'+x		;9
	DB	'Invalid argumen','t'+x		;A
	DB	'Integer out of rang','e'+x	;B
	DB	'Nonsense in BASI','C'+x	;C
	DB	'BREAK - CONT repeat','s'+x	;D
	DB	'Out of DAT','A'+x		;E
	DB	'Invalid file nam','e'+x	;F
	DB	'No room for lin','e'+x		;G
	DB	'STOP in INPU','T'+x		;H
	DB	'FOR without NEX','T'+x		;I
	DB	'Invalid I/O devic','e'+x	;J
	DB	'Invalid colou','r'+x		;K
	DB	'BREAK into progra','m'+x	;L
	DB	'RAMTOP no goo','d'+x		;M
	DB	'Statement los','t'+x		;N
	DB	'Invalid strea','m'+x		;O
	DB	'FN without DE','F'+x		;P
	DB	'Parameter erro','r'+x		;Q
	DB	'Tape loading erro'		;R
L1536:
	DB	'r'+x

;There are also the following two messages:
	DB	','		;A 'comma' and a 'space'.
L1538:
	DB	' '+x
	DB	'  KBAPK-4      HTTM-87/H-CK'
	DB	0AFH

;Report G - No room for line.
L1555:;REPORT_G
	LD	A,10H		;'G' has the code '10+07+30'.
	LD	BC,0		;Clear BC.
	JP	L1313;MAIN_G	;Jump back to give the report.

;	THE 'MAIN_ADD' SUBROUTINE
;This subroutine allows for a new BASIC line to be added to the existing
;BASIC program in the program area. If a line has both an old and a new
;version then the old one is 'reclaimed'. A new line that consists of
;only a line number does not go into the program area.
L155D:;MAIN_ADD
	LD	(L5C49),BC;E_PPC ;Make the new line number the
				;'current' line.
	LD	HL,(L5C5D);CH_ADD ;Fetch CH_ADD and save the
	EX	DE,HL		;addr in DE.
	LD	HL,L1555	;Push the addr of REPORT_G
	PUSH	HL		;on to the machine stack.
				;ERR_SP will now point to REPORT_G.
	LD	HL,(L5C61);WORKSP ;Fetch WORKSP.
	SCF			;Find the length of the line from after
	SBC	HL,DE		;the line number to the 'carriage return'
				;char inclusively.
	PUSH	HL		;Save the length.
	LD	H,B		;Move the line number to the HL
	LD	L,C		;reg pair.
	CALL	L196E;LINE_ADDR	;Is there an existing line with this
	JR	NZ,L157D;MAIN_ADD1 ;number? Jump if there was not.
	CALL	L19B8;NEXT_ONE	;Find the length of the 'old'
	CALL	L19E8;RECLAIM_2	;line and reclaim it.
L157D:;MAIN_ADD1
	POP	BC		;Fetch the length of the
	LD	A,C		;'new' line and jump forward
	DEC	A		;if it is only a 'line number
	OR	B		;and a carriage return'.
	JR	Z,L15AB;MAIN_ADD2
	PUSH	BC		;Save the length.
	INC	BC		;Four extra locs will be needed.
	INC	BC		;i.e. two for the number &
	INC	BC		;two for the length.
	INC	BC
	DEC	HL		;Make HL point to the loc
				;before the 'destination'.
	LD	DE,(L5C53);PROG	;Save the curr value of PROG to avoid
	PUSH	DE		;corruption when adding a 1st line.
	CALL	L1655;MAKE_ROOM	;Space for the new line is created.
	POP	HL		;The old value of PROG is
	LD	(L5C53),HL;PROG	;fetched and restored.
	POP	BC		;A copy of the line length
	PUSH	BC		;(without parameters) is taken.
	INC	DE		;Make DE point to the end loc of the new
	LD	HL,(L5C61);WORKSP ;area and HL to the 'carriage
	DEC	HL		;return' char of the new
	DEC	HL		;line in the editing area.
	LDDR			;Now copy over the line.
	LD	HL,(L5C49);E_PPC ;Fetch the line's number.
	EX	DE,HL		;Destination into HL & number into DE.
	POP	BC		;Fetch the new line's length.
	LD	(HL),B		;The high length byte.
	DEC	HL
	LD	(HL),C		;The low length byte.
	DEC	HL
	LD	(HL),E		;The low line number byte.
	DEC	HL
	LD	(HL),D		;The high line number byte.
L15AB:;MAIN_ADD2
	POP	AF		;Drop the addr of REPORT_G.
	JP	L12A2;MAIN_EXEC	;Jump back and this time do
				;produce an automatic listing.

;	THE 'INITIAL CHANNEL INFORMATION'
;Initially there are four channels - 'K', 'S', 'R' & 'P' - for communicating
;with the 'keyboard', 'screen', 'work space' & 'printer'. For each channel
;the output routine addr comes before the input routine addr and the
;channel's code.
L15AF:	
	DW	L09F4;PRINT_OUT
	DW	L10A8;KEY_INPUT
	DB	'K'
	DW	L09F4;PRINT_OUT
	DW	L15C4;REPORT_J
	DB	'S'
	DW	L0F81;ADD_CHAR
	DW	L15C4;REPORT_J
	DB	'R'
	DW	L09F4;PRINT_OUT
	DW	L15C4;REPORT_J
	DB	'P'
	DB	80H		;End marker.

;Report J - Invalid I/O device
L15C4:;REPORT_J
	RST	8;ERROR_1	;Call the error handling
	DB	12H		;routine.

;	THE 'INITIAL STREAM DATA'
;Initially there are seven streams - 0FDh to 03h.
L15C6:	
	DW	1		;stream 0FDh leads to channel 'K'
	DW	6		;stream 0FEh 		      'S'
	DW	0BH		;stream 0FFh		      'R'
	DW	1		;stream  00h		      'K'
	DW	1		;stream  01h		      'K'
	DW	6		;stream  02h		      'S'
	DW	10H		;stream  03h		      'P'

;	THE 'WAIT_KEY' SUBROUTINE
;This subroutine is the controlling subroutine for calling the current
;input subroutine.
L15D4:;WAIT_KEY
	BIT	5,(IY+02H);TV_FLAG ;Jump forward if the flag indicates the
	JR	NZ,L15DE;WAIT_KEY1 ;lower screen does not require clearing.
	SET	3,(IY+02H);TV_FLAG ;Otherwise signal 'consider the mode
				;as having changed'.
L15DE:;WAIT_KEY1
	CALL	L15E6;INPUT_AD	;Call the input subroutine indirectly
				;via INPUT_AD.
	RET	C		;Return with acceptable codes.
	JR	Z,L15DE;WAIT_KEY1 ;Both the carry flag and the zero flag
				;are reset if 'no key is being pressed';
				;otherwise signal an error.

;Report 8 - End of file
	RST	8;ERROR_1	;Call the error handling
	DB	7		;routine.

;	THE 'INPUT_AD' SUBROUTINE
;The regs are saved and HL made to point to the input addr.
L15E6:;INPUT_AD
	EXX			;Save the regs.
	PUSH	HL
	LD	HL,(L5C51);CURCHL ;Fetch the base addr for the current
	INC	HL		;channel info. Step past the output addr.
	INC	HL
	JR	L15F7;CALL_SUB	;Jump forward.

;	THE 'MAIN PRINTING' SUBROUTINE
;The subroutine is called with either an absolute value or a proper char
;code in the A reg.
L15EF:;OUT_CODE
	LD	E,30h		;Increase the value in the
	ADD	A,E		;A reg by 30h.
L15F2:;PRINT_A_2
	EXX			;Again save the regs.
	PUSH	HL
	LD	HL,(L5C51);CURCHL ;Fetch the base addr for the current
				;channel. This will point to an output addr.

;Now call the actual subroutine. HL points to the output or the input addr
;as directed.
L15F7:;CALL_SUB
	LD	E,(HL)		;Fetch the low byte.
	INC	HL
	LD	D,(HL)		;Fetch the high byte.
	EX	DE,HL		;Move the addr to the HL reg pair.
	CALL	L162C;CALL_JUMP	;Call the actual subroutine.
	POP	HL		;Restore the regs.
	EXX
	RET			;Return will be from here unless
				;an error occured.

;	THE 'CHAN_OPEN' SUBROUTINE
;This subroutine is called with the A reg holding a valid stream number -
;normally 0FDh to 03h. Then depending on the stream data a particular
;channel will be made the current channel.
L1601:;CHAN_OPEN
	ADD	A,A		;The value in the A reg is
	ADD	A,16H		;doubled and then increased by 16h.
	LD	L,A		;The result is moved to L.
	LD	H,5CH		;The addr 5C16h is the base addr for
	LD	E,(HL)		;stream 0. Fetch the 1st byte of the
	INC	HL		;required stream's data; then the
	LD	D,(HL)		;2nd byte.
	LD	A,D		;Give an error if both bytes are zero;
	OR	E		;otherwise jump forward.
	JR	NZ,L1610;CHAN_OP_1

;Report O - Invalid stream
L160E:;REPORT_O
	RST	8;ERROR_1	;Call the error handling
	DB	17H		;routine.

;Using the stream data now find the base addr of the channel information
;associated with that stream.
L1610:;CHAN_OP_1
	DEC	DE		;Reduce the stream data.
	LD	HL,(L5C4F);CHANS ;The base addr of the whole channel
				;information area.
	ADD	HL,DE		;Form the required addr in this area.

;	THE 'CHAN_FLAG' SUBROUTINE
;The appropriate flags for the different channels are set by this subroutine.
L1615:;CHAN_FLAG
	LD	(L5C51),HL;CURCHL ;The HL reg pair holds the base addr
				;for a particular channel.
	RES	4,(IY+30H);FLAGS2 ;Signal 'using other than channel 'K''.
	INC	HL		;Step past the output
	INC	HL		;and the input addrs and
	INC	HL		;make HL point to the
	INC	HL		;channel code.
	LD	C,(HL)		;Fetch the code.
	LD	HL,L162D	;The base addr of the 'channel code
				;look-up table'.
	CALL	L16DC;INDEXER	;Index into this table and locate the
				;required offset; but return if there is
	RET	NC		;not a matching channel code.
	LD	D,0		;Pass the offset to the
	LD	E,(HL)		;DE reg pair.
	ADD	HL,DE		;Jump forward to the appropriate
L162C:;CALL_JUMP
	JP	(HL)		;flag setting routine.

;	THE 'CHANNEL CODE LOOK-UP' TABLE
L162D:
	DB	'K',6		;L1634
	DB	'S',12H		;L1642
	DB	'P',1BH		;L164D
	DB	0		;end marker.

;	THE 'CHANNEL 'K' FLAG' SUBROUTINE
L1634:;CHAN_K
	SET	0,(IY+2);TV_FLAG ;Signal 'using lower screen'.
	RES	5,(IY+1);FLAGS	;Signal 'ready for a key'.
	SET	4,(IY+30H);FLAGS2  ;Signal 'using channel 'K''.
	JR	L1646;CHAN_S_1	   ;Jump forward.

;	THE 'CHANNEL 'S' FLAG' SUBROUTINE
L1642:;CHAN_S
	RES	0,(IY+2);TV_FLAG ;Signal 'using main screen'.
L1646:;CHAN_S_1
	RES	1,(IY+1);FLAGS	;Signal 'printer not being used'.
	JP	L0D4D;TEMPS	;Exit via TEMPS so as to set the
				;colour system vars.

;	THE 'CHANNEL 'P' FLAG' SUBROUTINE
L164D:;CHAN_P
	SET	1,(IY+1);FLAGS	;Signal 'printer in use'.
	RET

;	THE 'MAKE_ROOM' SUBROUTINE
;This is a very important subroutine. It is called on many occasions to
;'open up' an area. In all cases the HL reg pair points to the loc after
;the place where 'room' is required and the BC reg pair holds the length
;of the 'room' needed.
;  When a single space only is required then the subroutine is entered
;at ONE_SPACE.
L1652:;ONE_SPACE
	LD	BC,1		;Just the single extra loc is required.
L1655:;MAKE_ROOM
	PUSH	HL		;Save the pntr.
	CALL	L1F05;TEST_ROOM	;Make sure that there is sufficient memory
				;available for the task being undertaken.
	POP	HL		;Restore the pntr.
	CALL	L1664;POINTERS	;Alter all the pntrs before
				;making the 'room'.
	LD	HL,(L5C65);STKEND ;Make HL hold the new STKEND.
	EX	DE,HL		;Switch 'old' and 'new'.
	LDDR			;Now make the 'room'
	RET			;and return.
;Note: This subroutine returns with the HL reg pair pointing to the loc
;before the new 'room' and the DE reg pair pointing to the last of the
;new locs. The new 'room' therefore has the description: '(HL)+1' to
;'(DE)' inclusively.
;  However as the 'new locs' still retain their 'old values' it is also
;possible to consider the new 'room' as having been made after the original
;location '(HL)' and it thereby has the description: '(HL)+2' to '(DE)+1'.
;  In fact the programmer appears to have a preference for the '2nd
;description' and this can be confusing.

