
;	THE 'POINTERS' SUBROUTINE
;Whenever an area has to be 'made' or 'reclaimed' the system vars that
;addr locs beyond the 'position' of the change have to be amended as
;required. On entry the BC reg pair holds the number of bytes involved
;and the HL reg pair addresses the location before the 'position'.
L1664:;POINTERS
	PUSH	AF		;These regs are saved.
	PUSH	HL		;Copy the addr of the 'position'.
	LD	HL,L5C4B	;This is VARS, the first of the
	LD	A,14		;fourteen system pntrs.

;A loop is now entered to consider each pntr in turn. Only those pntrs
;that point beyond the 'position' are changed.
L166B:;PTR_NEXT
	LD	E,(HL)		;Fetch the two bytes of the current pntr.
	INC	HL
	LD	D,(HL)
	EX	(SP),HL		;Exchange the system var with the addr
				;of the 'position'.
	AND	A		;The carry flag will become
	SBC	HL,DE		;set if the system var's
	ADD	HL,DE		;addr is to be updated.
	EX	(SP),HL		;Restore the 'position'.
	JR	NC,L167F;PTR_DONE ;Jump forward if the pntr is to be left;
				;otherwise change it.
	PUSH	DE		;Save the old value.
	EX	DE,HL		;Now add the value in BC
	ADD	HL,BC		;to the old value.
	EX	DE,HL
	LD	(HL),D		;Enter the new value into the
	DEC	HL		;system var - high byte
	LD	(HL),E		;before low byte.
	INC	HL		;Point again to the high byte.
	POP	DE		;Fetch the old value.
L167F:;PTR_DONE
	INC	HL		;Point to the next system var
	DEC	A		;and jump back until all fourteen
	JR	NZ,L166B;PTR_NEXT ;have been considered.

;Now find the size of the block to be moved.
	EX	DE,HL		;Put the old value of STKEND in
	POP	DE		;HL and restore the other
	POP	AF		;regs.
	AND	A		;Now find the difference
	SBC	HL,DE		;between the old value of
	LD	B,H		;STKEND and the 'position'.
	LD	C,L		;Transfer the result to BC
	INC	BC		;and add '1' for the inclusive byte.
	ADD	HL,DE		;Reform the old value of STKEND
	EX	DE,HL		;and pass it to DE
	RET			;before returning.

;	THE 'COLLECT A LINE NUMBER' SUBROUTINE
;On entry the HL reg pair points to the loc under-consideration. If the
;loc holds a value that constitutes a suitable high byte for a line number
;then the line number is returned in DE. However if this is not so then
;the location addressed by DE is tried instead; and should this also be
;unsuccessful line number zero is returned.
L168F:;LINE_ZERO
	DW	0		;Line number zero.
L1691:;LINE_NO_A
	EX	DE,HL		;Consider the other pntr.
	LD	DE,L168F	;Use line number zero.
L1695:;LINE_NO
	LD	A,(HL)		;Fetch the high byte and
	AND	0C0H		;test it.
	JR	NZ,L1691;LINE_NO_A ;Jump back if not suitable.
	LD	D,(HL)		;Fetch the high byte.
	INC	HL
	LD	E,(HL)		;Fetch the low byte and
	RET			;return.

;	THE 'RESERVE' SUBROUTINE
;This subroutine is normally called by using RST 30h, BC_SPACES.
;  On entry here the last value on the machine stack is WORKSP and the
;value above it is the number of spaces that is to be 'reserved'.
;  This subroutine always makes 'room' between the existing work space
;and the calculator stack.
L169E:;RESERVE
	LD	HL,(L5C63);STKBOT ;Fetch the curr value of STKBOT and
	DEC	HL		;decrement it to get the last loc
				;of the work space.
	CALL	L1655;MAKE_ROOM	;Now make 'BC spaces'.
	INC	HL		;Point to the 1st new space
	INC	HL		;and then the 2nd.
	POP	BC		;Fetch the old value of
	LD	(L5C61),BC;WORKSP ;WORKSP and restore it.
	POP	BC      	;Restore BC - number of spaces.
	EX	DE,HL		;Switch the pntrs.
	INC	HL		;Make HL point to the 1st of the
	RET			;displaced bytes. Now return.
;Note: It can also be considered that the subroutine returns with the DE
;reg pair pointing to a '1st extra byte' and the HL reg pair pointing to
;a 'last extra byte', these extra bytes having been added after the
;original '(HL)+1' loc.

;	THE 'SET_MIN' SUBROUTINE
;This subroutine resets the editing area and the areas after it to their
;minimum sizes. In effect it 'clears' the areas.
L16B0:;SET_MIN
	LD	HL,(L5C59)	;Fetch E_LINE.
	LD	(HL),0DH	;Make the editing area hold
	LD	(L5C5B),HL;K_CUR ;only the 'carriage return'
	INC	HL		;char and the end marker.
	LD	(HL),80H
	INC	HL		;Move on to clear the work space.
	LD	(L5C61),HL;WORKSP

;Entering here will 'clear' the work space and the calculator stack.
L16BF:;SET_WORK
	LD	HL,(L5C61)	;Fetch WORKSP.
	LD	(L5C63),HL;STKBOT ;This clears the work space.

;Entering here will 'clear' only the calculator stack.
L16C5:;SET_STK
	LD	HL,(L5C63)	;Fetch STKBOT.
	LD	(L5C65),HL;STKEND ;This clears the stack.

;In all cases make MEM addr the calculator's memory area.
	PUSH	HL		;Save STKEND.
	LD	HL,L5C92	;The base of the memory area.
	LD	(L5C68),HL;MEM	;Set MEM to this addr.
	POP	HL		;Restore STKEND to the HL
	RET			;reg pair before returning.

;	THE 'RECLAIM THE EDIT_LINE' SUBROUTINE
L16D4:;REC_EDIT
	LD	DE,(L5C59)	;Fetch E_LINE.
	JP	L19E5;RECLAIM_1	;Reclaim the memory.

;	THE 'INDEXER' SUBROUTINE
;This subroutine is used on several occasions to look through tables.
;The entry point is at INDEXER.
L16DB:;INDEXER_1
	INC	HL		;Move on to consider the next pair
				;of entries.
L16DC:;INDEXER
	LD	A,(HL)		;Fetch the 1st of a pair of entries
	AND	A		;but return if it is zero - the
	RET	Z		;end marker.
	CP	C		;Compare it to the supplied code.
	INC	HL		;Point to the second entry.
	JR	NZ,L16DB;INDEXER_1 ;Jump back if the correct entry has
				;not been found.
	SCF			;The carry flag is set upon a
	RET			;successful search.

;	THE 'CLOSE #' COMMAND ROUTINE
;This command allows the user to CLOSE streams. However for streams 00 to
;03 the 'initial' stream data is restored and these streams cannot therefore
;be CLOSEd.
L16E5:;CLOSE
	CALL	L171E;STR_DATA	;The existing data for stream is fetched.
	CALL	L1701;CLOSE_2	;Check the code in that stream's channel.
	LD	BC,0		;Prepare to make the stream's data zero.
	LD	DE,0A3E2h	;Prepare to identify the use of
	EX	DE,HL		;streams 00 to 03.
	ADD	HL,DE		;The carry flag will be set
				;with streams 04 to 0F.
	JR	C,L16FC;CLOSE_1 ;Jump forward with these streams;
	LD	BC,L15D4	;otherwise find the correct entry in the
	ADD	HL,BC		;'initial stream data' table.
	LD	C,(HL)		;Fetch the initial data
	INC	HL		;for streams 00 to 03.
	LD	B,(HL)
L16FC:;CLOSE_1
	EX	DE,HL		;Now enter the data; either
	LD	(HL),C		;zero & zero, or the initial
	INC	HL		;values.
	LD	(HL),B
	RET

;	THE 'CLOSE_2' SUBROUTINE
;The code of the channel associated with the stream being closed
;has to be 'K', 'S' or 'P'.
L1701:;CLOSE_2
	PUSH	HL		;Save the addr of the stream's data.
	LD	HL,(L5C4F);CHANS ;Fetch the base addr of the channel
	ADD	HL,BC		;information area and find the channel
				;data for the stream being CLOSEd.
	INC	HL		;Step past the subroutine addrs
	INC	HL		;and get the channel code.
	INC	HL
	LD	C,(HL)
	EX	DE,HL		;Save the pntr.
	LD	HL,L1716	;The base addr of the 'CLOSE stream
				;look-up' table.
	CALL	L16DC;INDEXER	;Index into this table and locate
				;the required offset.
	LD	C,(HL)		;Pass the offset to the BC reg pair.
	LD	B,0
	ADD	HL,BC		;Jump forward to the
	JP	(HL)		;appropriate routine.

;	THE 'CLOSE STREAM LOOK_UP' TABLE
L1716:
	DB	'K',5		;171C
	DB	'S',3		;171C
	DB	'P',1		;171C
;Note: There is no end marker at the end of this table.

;	THE 'CLOSE STREAM' SUBROUTINE
L171C:;CLOSE_STR
	POP	HL		;Fetch the channel information
	RET			;pntr and return.

;	THE 'STREAM DATA' SUBROUTINE
L171E:;STR_DATA
	CALL	L1E94;STK_TO_A	;The given stream number is taken off
				;the calculator stack.
	CP	10H		;Give an error if the stream
	JR	C,L1727;STR_DATA1 ;number is greater than 0Fh.

;Report O - Invalid stream
L1725:;REPORT_O
	RST	8;ERROR_1	;Call the error handling
	DB	17H		;routine.

;Continue with valid stream numbers.
L1727:;STR_DATA1
	ADD	A,3		;Range now 03 to 12h;
	RLCA			;and now 06 to 24h.
	LD	HL,L5C10	;The base addr of the stream data area.
	LD	C,A		;Move the stream code to the
	LD	B,0		;BC reg pair.
	ADD	HL,BC		;Index into the data area
	LD	C,(HL)		;and fetch the two data bytes
	INC	HL		;into the BC reg pair.
	LD	B,(HL)
	DEC	HL		;Make the pntr addr the 1st of the data
	RET			;bytes before returning.

;	THE 'OPEN #' COMMAND ROUTINE
;This command allows the user to OPEN streams. A channel code must be
;supplied and it must be 'K', 'k', 'S', 's', 'P' or 'p'.
;  Note that no attempt is made to give streams 00 to 03 their initial
;data.
L1736:;OPEN
	RST	28H;FP_CALC	;Use the CALCULATOR.
	DB	01H;exchange	;Exchange the stream number
	DB	38H;end_calc	;and the channel code.
	CALL	L171E;STR_DATA	;Fetch the data for the stream.
	LD	A,B		;Jump forward if both bytes of
	OR	C		;the data are zero, i.e. the
	JR	Z,L1756;OPEN_1	;stream was in a closed state.
	EX	DE,HL		;Save DE.
	LD	HL,(L5C4F);CHANS ;Fetch CHANS - the base
	ADD	HL,BC		;addr of the channel
	INC	HL		;information and find the
	INC	HL		;code of the channel
	INC	HL		;associated with the stream
	LD	A,(HL)		;being OPENed.
	EX	DE,HL		;Return DE.
	CP	'K'		;The code fetched from the
	JR	Z,L1756;OPEN_1	;channel information area
	CP	'S'		;must be 'K', 'S' or 'P';
	JR	Z,L1756;OPEN_1	;give an error if it is not.
	CP	'P'
	JR	NZ,L1725;REPORT_O
L1756:;OPEN_1
	CALL	L175D;OPEN_2	;Collect the appropriate data in DE.
	LD	(HL),E		;Enter the data into the
	INC	HL		;two bytes in the stream
	LD	(HL),D		;information area.
	RET			;Finally return.

;	THE 'OPEN_2' SUBROUTINE
;The appropriate stream data bytes for the channel that is associated
;with the stream being OPENed are found.
L175D:;OPEN_2
	PUSH	HL		;Save HL.
	CALL	L2BF1;STK_FETCH	;Fetch the parameters of the channel code.
	LD	A,B		;Give an error if the
	OR	C		;expression supplied is a null
	JR	NZ,L1767;OPEN_3	;expression; i.e. OPEN# 5,"".

;Report F - Invalid file name
L1765:;REPORT_F
	RST	8;ERROR_1	;Call the error handling
	DB	0EH		;routine.

;Continue if no error occured.
L1767:;OPEN_3
	PUSH	BC		;The length of the expression is saved.
	LD	A,(DE)		;Fetch the 1st char.
	AND	0DFH		;Convert lower case codes to
				;upper case ones.
	LD	C,A		;Move code to the C reg.
	LD	HL,L177A	;The base addr of the 'OPEN stream
				;look-up' table.
	CALL	L16DC;INDEXER	;Index into this table and locate
				;the required offset.
	JR	NC,L1765;REPORT_F ;Jump back if not found.
	LD	C,(HL)		;Pass the offset to the BC
	LD	B,0		;reg pair.
	ADD	HL,BC		;Make HL point to the start of
				;the appropriate subroutine.
	POP	BC		;Fetch the length of the expression
	JP	(HL)		;before jumping to the subroutine.

;	THE 'OPEN STREAM LOOK-UP' TABLE
L177A:
	DB	'K',6		;L1781
	DB	'S',8		;L1785
	DB	'P',0AH		;L1789
	DB	0		;end marker.

;	THE 'OPEN_K' SUBROUTINE
L1781:;OPEN_K
	LD	E,1		;The data bytes will be 01 & 00.
	JR	L178B;OPEN_END

;	THE 'OPEN_S' SUBROUTINE
L1785:;OPEN_S
	LD	E,6		;The data bytes will be 06 & 00.
	JR	L178B;OPEN_END

;	THE 'OPEN_P' SUBROUTINE
L1789:;OPEN_P
	LD	E,10H		;The data bytes will be 10h & 00.

L178B:;OPEN_END
	DEC	BC		;Decrease the length of the
	LD	A,B		;expression and give an error
	OR	C		;if it was not a single
	JR	NZ,L1765;REPORT_F ;char; otherwise clear the
	LD	D,A		;D reg, fetch HL and
	POP	HL		;return.
	RET

;	THE 'CAT, ERASE, FORMAT & MOVE' COMMAND ROUTINES
;In the standard SPECTRUM system the use of these commands leads to the
;production of report O - Invalid stream.
L1793:;CAT_ETC.
	JR	L1725;REPORT_O	;Give this report.

;	THE 'LIST & LLIST' COMMAND ROUTINES
;The routines in this part of the 16K program are used to produce listings
;of the current BASIC program. Each line has to have its line number
;evaluated, its tokens expanded and the appropriate cursors positioned.
;  The entry point AUTO_LIST is used by both the MAIN EXECUTION routine
;and the EDITOR to produce a single page of the listing.
L1795:;AUTO_LIST
	LD	(L5C3F),SP;LIST_SP ;The stack pntr is saved allowing
				;the machine stack to be reset when the
				;listing is finished. (see PO_SCR, 0C55).
	LD	(IY+02H),10H;TV_FLAG ;Signal 'automatic listing in the
				;main screen'.
	CALL	L0DAF;CL_ALL	;Clear this part of the screen.
	SET	0,(IY+02H);FLAGS ;Switch to the editing area.
	LD	B,(IY+31H);DF_SZ ;Now clear the lower part
	CALL	L0E44;CL_LINE	;of the screen as well.
	RES	0,(IY+02H);FLAGS ;Then switch back.
	SET	0,(IY+30H);FLAGS2 ;Signal 'screen is clear'.
	LD	HL,(L5C49);E_PPC ;Now fetch the 'current' line number
	LD	DE,(L5C6C);S_TOP ;and the 'automatic' line number.
	AND	A		;If the 'current' number is less
	SBC	HL,DE		;than the 'automatic' number then
	ADD	HL,DE		;jump forward to
	JR	C,L17E1;AUTO_L_2 ;update the 'automatic' number.

;The 'automatic' number has now to be altered to give a listing with the
;'current' line appearing near the bottom of the screen.
	PUSH	DE		;Save the 'automatic' number.
	CALL	L196E;LINE_ADDR	;Find the addr of the
	LD	DE,L02C0	;start of the 'current' line
	EX	DE,HL		;and produce an addr roughly
	SBC	HL,DE		;a 'screen before it' (negated).
	EX	(SP),HL		;Save the 'result' on the machine
	CALL	L196E;LINE_ADDR	;stack whilst the 'automatic' line addr
				;is also found (in HL).
	POP	BC		;The 'result' goes to the BC reg pair.

;A loop is now entered. The 'automatic' line number is incremented
;until it is likelyx that the 'current' line will show on a listing.
L17CE:;AUTO_L_1
	PUSH	BC		;Save the 'result'.
	CALL	L19B8;NEXT_ONE	;Find the addr of the start of the line
				;after the present 'automatic' line (in DE).
	POP	BC		;Restore the 'result'.
	ADD	HL,BC		;Perform the computation and
	JR	C,L17E4;AUTO_L_3 ;jump forward if finished.
	EX	DE,HL		;Move the next line's addr
	LD	D,(HL)		;to the HL reg pair and
	INC	HL		;collect its line number.
	LD	E,(HL)
	DEC	HL
	LD	(L5C6C),DE;S_TOP ;Now S_TOP can be updated and the test
	JR	L17CE;AUTO_L_1	;repeated with the new line.

;Now the 'automatic' listing can be made.
L17E1:;AUTO_L_2
	LD	(L5C6C),HL;S_TOP ;When E_PPC is less than S_TOP.
L17E4:;AUTO_L_3
	LD	HL,(L5C6C);S_TOP ;Fetch the top line's number
	CALL	L196E;LINE_ADDR	;and hence its addr.
	JR	Z,L17ED;AUTO_L_4 ;If the line cannot be found
	EX	DE,HL		;use DE instead.
L17ED:;AUTO_L_4
	CALL	L1833;LIST_ALL	;The listing is produced.
	RES	4,(IY+02H);TV_FLAG ;The return will be to here unless
	RET			;scrolling was needed to show the
				;current line.

;	THE 'LLIST' ENTRY POINT
;The printer channel will need to be opened.
L17F5:;LLIST
	LD	A,3		;Use stream 03.
	JR	L17FB;LIST_1	;Jump forward.

;	THE 'LIST' ENTRY POINT
;The 'main screen' channel will need to be opened.
L17F9:;LIST
	LD	A,2		;Use stream 02.
L17FB:;LIST_1
	LD	(IY+02H),0;TV_FLAG ;Signal 'an ordinary listing in
				;the main part of the screen'.
	CALL	L2530;SYNTAX_Z	;Open the channel unless
	CALL	NZ,L1601;CHAN_OPEN ;checking syntax.
	RST	18H;GET_CHAR	;With the present char in the A reg
	CALL	L2070;STR_ALTER	;see if the stream is to be changed.
	JR	C,L181F;LIST_4	;Jump forward if unchanged.
	RST	18H;GET_CHAR	;Is the present char a ';'?
	CP	';'
	JR	Z,L1814;LIST_2	;Jump if it is.
	CP	','		;Is it a ','?
	JR	NZ,L181A;LIST_3	;Jump if it is not.
L1814:;LIST_2
	RST	20H;NEXT_CHAR	;A numeric expression must
	CALL	L1C82;EXPT_1NUM	;follow, e.g. LIST#5, 20
	JR	L1822;LIST_5	;Jump forward with it.
L181A:;LIST_3
	CALL	L1CE6;USE_ZERO	;Otherwise use zero and also jump
	JR	L1822;LIST_5	;forward.

;Come here if the stream was unaltered.
L181F:;LIST_4
	CALL	L1CDE;FETCH_NUM	;Fetch any line or use zero
				;if none supplied.
L1822:;LIST_5
	CALL	L1BEE;CHECK_END	;If checking the syntax of the edit-line
				;move on to the next stmt.
	CALL	L1E99;FIND_INT	;Line number to BC.
	LD	A,B		;High byte to A.
	AND	3Fh		;Limit the high byte to the
	LD	H,A		;correct range and pass the
	LD	L,C		;whole line number to HL.
	LD	(L5C49),HL;E_PPC ;Set E_PPC and find the addr of the start
	CALL	L196E;LINE_ADDR	;of this line or the 1st line after it if
				;the actual line does not exist.
	LD	E,1		;Flag 'before current line'.

;Now the controlling loop for printing a series of lines is entered.
L1835:;LIST_ALL
	CALL	L1855;OUT_LINE	;Print the whole of a BASIC line.
	RST	10H;PRINT_A_1	;This will be a 'carriage return'.
	BIT	4,(IY+02H);TV_FLAG ;Jump back unless dealing
	JR	Z,L1835;LIST_ALL ;with an automatic listing.
	LD	A,(L5C6B);DF_SZ	;Also jump back if there is still
	SUB	(IY+4FH);S_POSN_hi ;part of the main screen
	JR	NZ,L1835;LIST_ALL ;that can be used.
	XOR	E		;A return can be made at this point if the
	RET	Z		;screen is full and the current line
				;has been printed (E=00).
	PUSH	HL		;However if the current line is
	PUSH	DE		;missing from the listing
	LD	HL,L5C6C	;then S_TOP has to be updated
	CALL	L190F;LN_FETCH	;and a further line printed
	POP	DE		;(using scrolling).
	POP	HL
	JR	L1835;LIST_ALL

;	THE 'PRINT A WHOLE BASIC LINE' SUBROUTINE
;The HL reg pair points to the start of the line - the loc holding the high
;byte of the line number.
;  Before the line number is printed it is tested to determine whether it
;comes before the 'current' line, is the 'current' line or comes after.
L1855:;OUT_LINE
	LD	BC,(L5C49);E_PPC ;Fetch the 'current' line
	CALL	L1980;CP_LINES	;number and compare it.
	LD	D,'>'		;Pre-load then D reg with the
				;current line cursor. Jump forward
	JR	Z,L1865;OUT_LINE1 ;if printing the 'current' line.
	LD	DE,0		;Load the D reg with zero (it is not the
	RL	E		;cursor) and set E to hold 01 if the line
				;is before the 'current' line and 00 if
				;after. (The carry flag comes from CP_LINES).
L1865:;OUT_LINE1
	LD	(IY+2DH),E;BREG	;Save the line marker.
	LD	A,(HL)		;Fetch the high byte of the
	CP	40h		;line number and make a full
	POP	BC		;return if the listing has been
	RET	NC		;finished.
	PUSH	BC
	CALL	L1A28;OUT_NUM_2	;The line number can now be printed -
				;with leading spaces.
	INC	HL		;Move the pntr on to addr
	INC	HL		;the 1st-command code in
	INC	HL		;the line.
	RES	0,(IY+01H);FLAGS ;Signal 'leading space allowed'.
	LD	A,D		;Fetch the cursor code and jump
	AND	A		;forward unless the
	JR	Z,L1881;OUT_LINE3 ;cursor is to be printed.
	RST	10H;PRINT_A_1	;So print the cursor now.
L187D:;OUT_LINE2
	SET	0,(IY+01H);FLAGS ;Signal 'no leading space now'.
L1881:;OUT_LINE3
	PUSH	DE		;Save the regs.
	EX	DE,HL		;Move the pntr to DE.
	RES	2,(IY+30H);FLAGS2 ;Signal 'not in quotes'.
	LD	HL,L5C3B	;This is FLAGS.
	RES	2,(HL)		;Signal 'print in K-mode'.
	BIT	5,(IY+37H);FLAGX ;Jump forward unless in
	JR	Z,L1894;OUT_LINE4 ;INPUT mode.
	SET	2,(HL)		;Signal 'print in L-mode'.

;Now enter a loop to print all the codes in the rest of the BASIC line -
;jumping over flpnt forms as necessary.
L1894:;OUT_LINE4
	LD	HL,(L5C5F);X_PTR ;Fetch the syntax error
	AND	A		;pntr and jump forward
	SBC	HL,DE		;unless it is time to print
	JR	NZ,L18A1;OUT_LINE5 ;the error marker.
	LD	A,'?'		;Print the error marker now.
	CALL	L18C1;OUT_FLASH	;It is a flashing '?'.
L18A1:;OUT_LINE5
	CALL	L18E1;OUT_CURS	;Consider whether to print the cursor.
	EX	DE,HL		;Move the pntr to HL now.
	LD	A,(HL)		;Fetch each char in turn.
	CALL	L18B6;NUMBER	;If the char is a 'number marker' then
				;the hidden flpnt form is not to be printed.
	INC	HL		;Update the pntr for the next pass.
	CP	0DH		;Is the char a 'carriage return'.
	JR	Z,L18B4;OUT_LINE6 ;Jump if it is.
	EX	DE,HL		;Switch the pntr to DE.
	CALL	L1937;OUT_CHAR	;Print the char.
	JR	L1894;OUT_LINE4	;Go around the loop for at least one
				;further pass.

;The line has now been printed.
L18B4:;OUT_LINE6
	POP	DE		;Restore the DE reg pair
	RET			;and return.

;	THE 'NUMBER' SUBROUTINE
;If the A reg holds the 'number marker' then the HL reg pair is advanced
;past the flpnt form.
L18B6:;NUMBER
	CP	0EH		;Is the char a 'number marker'?
	RET	NZ		;Return if not.
	INC	HL		;Advance the pntr six
	INC	HL		;times so as to step past the
	INC	HL		;'number marker' and the five
	INC	HL		;locs holding the
	INC	HL		;flpnt form.
	INC	HL
	LD	A,(HL)		;Fetch the current code before
	RET			;returning.

;	THE 'PRINT A FLASHING CHARACTER' SUBROUTINE
;The 'error cursor' and the 'mode cursors' are printed using this subroutine.
L18C1:;OUT_FLASH
	EXX			;Save the current regs.
	LD	HL,(L5C8F);ATTR_T ;Save the ATTR_T & MASK_T on
	PUSH	HL		;the machine stack.
	RES	7,H		;Ensure that FLASH is
	SET	7,L		;active.
	LD	(L5C8F),HL;ATTR_T ;Use these modified values
				;for ATTR_T & MASK_T.
	LD	HL,L5C91	;This is P_FLAG.
	LD	D,(HL)		;Save P_FLAG also on the
	PUSH	DE		;machine stack.
	LD	(HL),0		;Ensure INVERSE 0, OVER 0
				;and not PAPER 9 nor INK 9.
	CALL	L09F4;PRINT_OUT	;The char is printed.
	POP	HL		;The former value of P_FLAG
	LD	(IY+57H),H;P_FLAG ;is restored.
	POP	HL		;The former values of ATTR_T
	LD	(L5C8F),HL;ATTR_T ;& MASK_T are also restored
	EXX			;before returning.
	RET

;	THE 'PRINT THE CURSOR' SUBROUTINE
;A return is made if it is not the correct place to print the cursor
;but if it is then either 'C', 'E', 'G', 'K' or 'L' will be printed.
L18E1:;OUT_CURS
	LD	HL,(L5C5B);K_CUR ;Fetch the addr of the
	AND	A		;cursor but return if the
	SBC	HL,DE		;correct place is not being
	RET	NZ		;considered.
	LD	A,(L5C41);MODE	;The current value of MODE is
	RLC	A		;fetched and doubled. Jump forward unless
	JR	Z,L18F3;OUT_C_1	;dealing with Extended mode or Graphics.
	ADD	A,'C'		;Add the appropriate offset to give 'E'
	JR	L1909;OUT_C_2	;or 'G'. Jump forward to print it.
L18F3:;OUT_C_1
	LD	HL,L5C3B	;This is FLAGS.
	RES	3,(HL)		;Signal 'K-mode'.
	LD	A,'K'		;The 'K' char.
	BIT	2,(HL)		;Jump forward to print 'K' if 'the
	JR	Z,L1909;OUT_C_2	;printing is to be in K-mode'.
	SET	3,(HL)		;The 'printing is to be in L-mode'
				;so signal 'L-mode'.
	INC	A		;Form the char 'L'.
	BIT	3,(IY+30H);FLAGS2 ;Jump forward if not in
	JR	Z,L1909;OUT_C_2	;'C-mode'.
	LD	A,'C'		;The char 'C'.
L1909:;OUT_C_2
	PUSH	DE		;Save the DE reg pair
	CALL	L18C1;OUT_FLASH	;whilst the cursor is printed
	POP	DE		;- FLASHing.
	RET			;Return once it has been done.
;Note: It is the action of considering which cursor-letter is to be printed
;that determines the mode - 'K' vs. 'L/C'.

;	THE 'LN_FETCH' SUBROUTINE
;This subroutine is entered with the HL reg pair addressing a system var
;S_TOP or E_PPC.
;  The subroutine returns with the system var holding the line number
;of the following line.
L190F:;LN_FETCH
	LD	E,(HL)		;The line number held by the
	INC	HL		;system var is collected.
	LD	D,(HL)
	PUSH	HL		;The pntr is saved.
	EX	DE,HL		;The line number is moved to the HL
	INC	HL		;reg pair and incremented.
	CALL	L196E;LINE_ADDR	;The addr of the start of this line is
				;found, or the next line if the actual
				;line number is not being used.
	CALL	L1695;LINE_NO	;The number of that line is fetched.
	POP	HL		;The pntr to the system var is restored.

;The entry point LN_STORE is used by the EDITOR.
L191C:;LN_STORE
	BIT	5,(IY+37H);FLAGX ;Return if in 'INPUT mode';
	RET	NZ		;otherwise proceed to
	LD	(HL),D		;enter the line number into
	DEC	HL		;the two locs of the
	LD	(HL),E		;system var.
	RET			;Return when it has been done.

;	THE 'PRINTING CHARACTERS IN A BASIC LINE' SUBROUTINE
;All of the char/token codes in a BASIC line are printed by repeatedly
;calling this subroutine.
;  The entry point OUT_SP_NO is used when printing line numbers which
;may require leading spaces.
L1925:;OUT_SP_2
	LD	A,E		;The A reg will hold 20h for a space
				;or 0FFh for no-space.
	AND	A		;Test the value and return if
	RET	M		;there is not to be a space.
	JR	L1937;OUT_CHAR	;Jump forward to print a space.
L192A:;OUT_SP_NO
	XOR	A		;Clear the A reg.

;The HL reg pair holds the line number and the BC reg the value for
;'repeated subtraction'. (BC holds '-1000, -100 or -10'.)
L192B:;OUT_SP_1
	ADD	HL,BC		;The 'trial subtraction'.
	INC	A		;Count each 'trial'.
	JR	C,L192B;OUT_SP_1 ;Jump back until exhausted.
	SBC	HL,BC		;Restore last 'subtraction'
	DEC	A		;and discount it.
	JR	Z,L1925;OUT_SP_2 ;If no 'subtractions' were possible
				;jump back to see if a space is to
				;be printed.
	JP	L15EF;OUT_CODE	;Otherwise print the digit.

;The entry point OUT_CHAR is used for all chars, tokens and control chars.
L1937:;OUT_CHAR
	CALL	L2D1B;NUMERIC	;Return carry reset if handling a digit code.
	JR	NC,L196C;OUT_CH_3 ;Jump forward to print the digit.
	CP	21h		;Also print the control chars
	JR	C,L196C;OUT_CH_3 ;and 'space'.
	RES	2,(IY+01H);FLAGS ;Signal 'print in K-mode'.
	CP	0CBH		;Jump forward if dealing with
	JR	Z,L196C;OUT_CH_3 ;the token 'THEN'.
	CP	':'		;Jump forward unless dealing
	JR	NZ,L195A;OUT_CH_1 ;with ':'.
	BIT	5,(IY+37H);FLAGX ;Jump forward to print the
	JR	NZ,L1968;OUT_CH_2 ;':' if in 'INPUT mode'.
	BIT	2,(IY+30H);FLAGS2 ;Jump forward if the ':' is 'not in
	JR	Z,L196C;OUT_CH_3 ;quotes', i.e. an inter-stmt marker.
	JR	L1968;OUT_CH_2	;The ':' is inside quotes and can
				;now be printed.
L195A:;OUT_CH_1
	CP	'"'		;Accept for printing all
	JR	NZ,L1968;OUT_CH_2 ;chars except '"'.
	PUSH	AF		;Save the char code whilst
				;changing the 'quote mode'.
	LD	A,(L5C6A);FLAGS2 ;Fetch FLAGS2 and flip
	XOR	4		;bit 2.
	LD	(L5C6A),A;FLAGS2 ;Enter the amended value and
	POP	AF		;restore the char code.
L1968:;OUT_CH_2
	SET	2,(IY+01H);FLAGS ;Signal 'the next char is to be
				;printed in L-mode'.
L196C:;OUT_CH_3
	RST	10H;PRINT_A_1	;The present char is
	RET			;printed before returning.
;Note: It is the consequence of the tests on the present char that
;determines whether the next char is to be 'printed in 'K' or 'L' mode'.
;  Also note how the program does not cater for ':' in REM stmts.

;	THE 'LINE_ADDR' SUBROUTINE
;For a given line number, in the HL reg pair, this subroutine returns the
;starting addr of that line or the 'first line after', in the HL reg pair,
;and the start of the previous line in the DE reg pair.
L196E:;LINE_ADDR
	PUSH	HL		;Save the given line number.
	LD	HL,(L5C53);PROG	;Fetch the system var
	LD	D,H		;PROG and transfer the addr
	LD	E,L		;to the DE reg pair.

;Now enter a loop to test the line number of each line of the program
;against the given line number until the line number is matched or exceeded.
L1974:;LINE_AD_1
	POP	BC		;The given line number.
	CALL	L1980;CP_LINES	;Compare the given line number against the
	RET	NC		;addressed line number. Return if carry
	PUSH	BC		;reset; otherwise address the next
	CALL	L19B8;NEXT_ONE	;line's number.
	EX	DE,HL		;Switch the pntrs and jump back to consider
	JR	L1974;LINE_AD_1	;the next line of the program.

;	THE 'COMPARE LINE NUMBERS' SUBROUTINE
;The given line number in the BC reg pair is matched against the addressed
;line number.
L1980:;CP_LINES
	LD	A,(HL)		;Fetch the high byte of the
	CP	B		;addressed line number and compare it.
	RET	NZ		;Return if they do not match.
	INC	HL		;Next compare the low bytes.
	LD	A,(HL)		;Return with the carry flag
	DEC	HL		;set if the addressed line
	CP	C		;number has yet to reach the
	RET			;given line number.

