
;	THE 'FIND EACH STATEMENT' SUBROUTINE
;This subroutine has two distinct functions.
;   i. It can be used to find the 'D'th. statement in a BASIC line -
;      returning with the HL reg pair addressing the loc before the start
;      of the stmt and the zero flag set.
;  ii. Also the subroutine can be used to find a stmt, if any, that starts
;      with a given token code (in the E reg).
	INC	HL		;Not used.
	INC	HL
	INC	HL
L198B:;EACH_STMT
	LD	(L5C5D),HL;CH_ADD ;Set CH_ADD to the current byte.
	LD	C,0		;Set a 'quotes off' flag.

;Enter a loop to handle each stmt in the BASIC line.
L1990:;EACH_S_1
	DEC	D		;Decrease 'D' and return if the required
	RET	Z		;stmt has been found.
	RST	20H;NEXT_CHAR	;Fetch the next char code
	CP	E		;and jump if it does not match
	JR	NZ,L199A;EACH_S_3 ;the given token code.
	AND	A		;But should it match then return with the
	RET			;carry and the zero flags both reset.

;Now enter another loop to consider the individual chars in the line to
;find where the stmt ends.
L1998:;EACH_S_2
	INC	HL		;Update the pntr and fetch
	LD	A,(HL)		;the new code.
L199A:;EACH_S_3
	CALL	L18B6;NUMBER	;Step over any number.
	LD	(L5C5D),HL;CH_ADD ;Update CH_ADD.
	CP	'"'		;Jump forward if the char
	JR	NZ,L19A5;EACH_S_4 ;is not a '"'.
	DEC	C		;Otherwise set the 'quotes flag'.
L19A5:;EACH_S_4
	CP	':'		;Jump forward if the char
	JR	Z,L19AD;EACH_S_5 ;is a ':'.
	CP	0CBH		;Jump forward unless the code
	JR	NZ,L19B1;EACH_S_6 ;is the token 'THEN'.
L19AD:;EACH_S_5
	BIT	0,C		;Read the 'quotes flag' and jump back at
	JR	Z,L1990;EACH_S_1 ;the end of each stmt (incl. after 'THEN').
L19B1:;EACH_S_6
	CP	0DH		;Jump back unless at the end
	JR	NZ,L1998;EACH_S_2 ;of a BASIC line.
	DEC	D		;Decrease the stmt
	SCF			;counter and set the carry
	RET			;flag before returning.

;	THE 'NEXT_ONE' SUBROUTINE
;This subroutine can be used to find the 'next line' in the program area or
;the 'next var' in the vars area. The subroutine caters for the six different
;types of var that are used in the SPECTRUM system.
L19B8:;NEXT_ONE
	PUSH	HL		;Save the addr of the current line or var.
	LD	A,(HL)		;Fetch the 1st byte.
	CP	40h		;Jump forward if searching
	JR	C,L19D5;NEXT_O_3 ;for a 'next line'.
	BIT	5,A		;Jump forward if searching for the next
	JR	Z,L19D6;NEXT_O_4 ;string or array var.
	ADD	A,A		;Jump forward with simple numeric and
	JP	M,L19C7;NEXT_O_1 ;FOR-NEXT vars.
	CCF			;Long name numeric vars only.
L19C7:;NEXT_O_1
	LD	BC,5		;A numeric var will occupy five locs but a
	JR	NC,L19CE;NEXT_O_2 ;FOR-NEXT control var will need
	LD	C,18		;eighteen locs.
L19CE:;NEXT_O_2
	RLA			;The carry flag becomes reset for long
				;named vars only; until the final char
				;of the long name is reached.
	INC	HL		;Increment the pntr and
	LD	A,(HL)		;fetch the new code. Jump back unless the
	JR	NC,L19CE;NEXT_O_2 ;previous code was the last code of the
				;var's name.
	JR	L19DB;NEXT_O_5	;Now jump forward (BC=0005 or 0012h).
L19D5:;NEXT_O_3
	INC	HL		;Step past the low byte of the line number.
L19D6:;NEXT_O_4
	INC	HL		;Now point to the low byte of the length.
	LD	C,(HL)		;Fetch the length into the BC reg pair.
	INC	HL
	LD	B,(HL)
	INC	HL		;Allow for the inclusive byte.

;In all cases the addr of the 'next' line or var is found.
L19DB:;NEXT_O_5
	ADD	HL,BC		;Point to the 1st byte of the 'next'
				;line or var.
	POP	DE		;Fetch the addr of the previous one and
				;continue into the 'difference' subroutine.

;	THE 'DIFFERENCE' SUBROUTINE
;The 'length' between two 'starts' is formed in the BC reg pair. The pntrs
;are reformed but returned exchanged.
L19DD:;DIFFER
	AND	A		;Prepare for a true subtraction.
	SBC	HL,DE		;Find the length from one
	LD	B,H		;'start' to the next and pass
	LD	C,L		;it to the BC reg pair.
	ADD	HL,DE		;Reform the addr and
	EX	DE,HL		;exchange them before
	RET			;returning.

;	THE 'RECLAIMING' SUBROUTINE
;The entry point RECLAIM_1 is used when the addr of the 1st loc to be
;reclaimed is in the DE reg pair and the addr of the 1st loc to be left
;alone is in the HL reg pair. The entry point RECLAIM_2 is used when the
;HL reg pair points to the 1st loc to be reclaimed and the BC reg pair
;holds the number of the bytes that are to be reclaimed.
L19E5:;RECLAIM_1
	CALL	L19DD;DIFFER	;Use the 'difference' subroutine to
				;develop the appropriate values.
L19E8:;RECLAIM_2
	PUSH	BC		;Save the number of bytes to be reclaimed.
	LD	A,B		;All the system var
	CPL			;pntrs above the area
	LD	B,A		;have to be reduced by 'BC'
	LD	A,C		;so this number is 2's
	CPL			;complemented before the
	LD	C,A		;pntrs are altered.
	INC	BC
	CALL	L1664;POINTERS
	EX	DE,HL		;Return the '1st loc'
	POP	HL		;addr to the DE reg pair and reform
	ADD	HL,DE		;the addr of the 1st loc to be left.
	PUSH	DE		;Save the '1st loc'
	LDIR			;whilst the actual reclamation
	POP	HL		;occurs.
	RET			;Now return.

;	THE 'E_LINE_NO' SUBROUTINE
;This subroutine is used to read the line number of the line in the editing
;area. If there is no line number, i.e. a direct BASIC line, then the line
;number is considered to be zero.
;  In all cases the line number is returned in the BC reg pair.
L19FB:;E_LINE_NO
	LD	HL,(L5C59);E_LINE ;Pick up the pntr to the edit-line.
	DEC	HL		;Set CH_ADD to point to the
	LD	(L5C5D),HL;CH_ADD ;loc before any number.
	RST	20H;NEXT_CHAR	;Pass the 1st code to the A reg.
	LD	HL,L5C92	;However before considering the code make
	LD	(L5C65),HL;STKEND ;the calculator's memory area a temporary
				;calculator stack area.
	CALL	L2D3B;INT_TO_FP	;Now read the digits of the line number.
				;Return zero if no number exists.
	CALL	L2DA2;FP_TO_BC	;Compress the line number into BC.
	JR	C,L1A15;E_L_1	;Jump forward if the number exceeds '65536'.
	LD	HL,0D8F0h	;Otherwise test it again
	ADD	HL,BC		;'-10000'.
L1A15:;E_L_1
	JP	C,L1C8A;REPORT_C ;Give report C if over '9999'.
	JP	L16C5;SET_STK	;Return via SET_STK that restores the
				;calculator stack to its rightful place.

;	THE 'REPORT AND LINE NUMBER PRINTING' SUBROUTINE
;The entry point OUT_NUM_1 will lead to the number in the BC reg pair being
;printed. Any value over '9999' will not however be printed correctly.
;  The entry point OUT_NUM_2 will lead to the number indirectly addressed
;by the HL reg pair being printed. This time any necessary leading spaces
;will appear. Again the limit of correctly printed numbers is '9999'.
L1A1B:;OUT_NUM_1
	PUSH	DE		;Save the other regs
	PUSH	HL		;throughout the subroutine.
	XOR	A		;Clear the A reg. Jump forward to print a
	BIT	7,B		;zero rather than '-2' when
	JR	NZ,L1A42;OUT_NUM_4 ;reporting on the edit-line.
	LD	H,B		;Move the number to the
	LD	L,C		;HL reg pair.
	LD	E,0FFH		;Flag 'no leading spaces'.
	JR	L1A30;OUT_NUM_3	;Jump forward to print the number.
L1A28:;OUT_NUM_2
	PUSH	DE		;Save the DE reg pair.
	LD	D,(HL)		;Fetch the number into the
	INC	HL		;DE reg pair and save
	LD	E,(HL)		;the pntr (updated).
	PUSH	HL
	EX	DE,HL		;Move the number to the HL reg pair and
	LD	E,' '		;flag 'leading space are to be printed'.

;Now the integer form of the number in the HL reg pair is printed.
L1A30:;OUT_NUM_3
	LD	BC,0FC18h	;This is '-1000'.
	CALL	L192A;OUT_SP_NO	;Print a 1st digit.
	LD	BC,0FF9Ch	;This is '-100'.
	CALL	L192A;OUT_SP_NO	;Print the 2nd digit.
	LD	C,0F6H		;This is '-10'.
	CALL	L192A;OUT_SP_NO	;Print the 3rd digit.
	LD	A,L		;Move any remaining part of the number
				;to the A reg.
L1A42:;OUT_NUM_4
	CALL	L15EF;OUT_CODE	;Print the digit.
	POP	HL		;Restore the regs
	POP	DE		;before returning.
	RET

