

;		EXPRESSION EVALUATION

;	THE 'SCANNING' SUBROUTINE
;This subroutine is used to produce an evaluation result of the 'next
;expression'.
;  The result is returned as the 'last value' on the calc stack. For a
;numerical result, the last value will be the actual flpnt number. However,
;for a string result the last value will consist of a set of params. The
;1st of the five bytes is unspecified, the 2nd and 3rd bytes hold the addr
;of the start of the string and the 4th and 5th bytes hold the length
;of the string.
;  Bit 6 of FLASG is set for a numeric result and reset for a string result.
;  When a next expression consists of only a single operand, e.g. .... A
;...., ... RND ..., ...A$(4, 3 TO 7)..., then the last value is simply the
;value that is obtained from evaluating the operand.
;  However when the next expression contains a function and an operand, e.g.
;... CHR$ A ..., ... NOT A ..., ... SIN 1 ..., the operation code of the
;function is stored on the machine stack until the last value of the operand
;has been calculated. This last value is then subjected to the appropriate
;operation to give a new last value.
;  In the case of there being an arithmetic or logical operation to be
;performed, e.g. ...A+B..., ...A*B..., ...A=B..., then both the last value
;of the 1st argument and the operation code have to be kept until the last
;value of the 2nd argument has been found. Indeed the calculation of the
;last value of the 2nd argument may also involve the storing of last values
;and operation codes whilst the calculation is being performed.
;  It can therefore be shown that as a complex expression is evaluated, e.g.
;... CHR$ (T+A-26*INT ((T+A)/26)+65) ..., a hierarchy of operations yet to be
;performed is built up until the point is reached from which it must be
;dismantled to produce the final last value.
;  Each operation code has associated with it an appropriate priority code
;and operations of higher priority are always performed before those of
;lower priority.
;  The subroutine begins with the A reg being set to hold the 1st char of
;the expression and a starting priority marker - zero - being put on the
;machine stack.
L24FB:;SCANNING
	RST	18H;GET_CHAR	;The 1st char is fetched.
	LD	B,0		;The starting priority marker.
	PUSH	BC		;It is stacked.
L24FF:;S_LOOP_1
	LD	C,A		;The main re-entry point.
	LD	HL,L2596	;Index into scanning function
	CALL	L16DC;INDEXER	;table with the code in C.
	LD	A,C		;Restore the code to A.
	JP	NC,L2684;S_ALPHNUM ;Jump if code not found in table.
	LD	B,0		;Use the entry found in the table
	LD	C,(HL)		;to build up the required addr
	ADD	HL,BC		;in HL, and jump to it.
	JP	(HL)

;Four subroutines follow; they are called by routines from the scanning
;function table. The 1st one, the 'scanning quotes subroutine', is used
;by S_QUOTE to check that every string quote is matched by another one.
L250F:;S_QUOTE_S
	CALL	L0074;CH_ADD+1	;Point to the next char.
	INC	BC		;Increase the length count by one.
	CP	0DH		;Is it a carriage return?
	JP	Z,L1C8A;REPORT_C ;Report the error if so.
	CP	'"'		;Is it another '"'?
	JR	NZ,L250F;S_QUOTE_S ;Loop back if it is not.
	CALL	L0074;CH_ADD+1	;Point to next char; set zero
	CP	'"'		;flag if it is another '"'.
	RET			;Finished.

;The next subroutine, the 'scanning: two co-ordinates' subroutine, is
;called by S_SCREEN$, S_ATTR and S_POINT to make sure the required two
;co-ordinates are given in their proper form.
L2522:;S_2_COORD
	RST	20H;NEXT_CHAR	;Fetch the next char.
	CP	'('		;Is it a '('?
	JR	NZ,L252D;S_RPORT_C ;Report the error if it is not.
	CALL	L1C79;NEXT_2NUM	;Co-ordinates to calc stack.
	RST	18H;GET_CHAR	;Fetch the current char.
	CP	')'		;Is it a ')'?
L252D:;S_RPORT_C
	JP	NZ,L1C8A;REPORT_C ;Report the error if it is not.

;	THE 'SYNTAX_Z' SUBROUTINE
;At this point the 'SYNTAX_Z' subroutine is interpolated. It is called 32
;times, with a saving of just one byte each call. A simple test of bit 7
;of FLAGS will give the zero flag reset during execution and set during
;syntax checking.
L2530:;SYNTAX_Z
	BIT	7,(IY+1);FLAGS	;Test bit 7 of FLAGS.
	RET			;Finished.

;The next subroutine is the 'scanning SCREEN$ subroutine', which is used by
;S_SCREEN$ to find the char that appears at line x, column y of the screen.
;It only searches the char set 'pointed to' to CHARS.
;Note: This is normally the chars 20h (space to 7Fh (c)) although the user
;can alter CHARS to match for other chars, including the user-defined
;graphics.
L2535:;S_SCRN$_S
	CALL	L2307;STK_TO_BC	;x to C, y to B; 0<=x<=23; 0<=y<=31.
	LD	HL,(L5C36);CHARS
	LD	DE,0100H	;CHARS plus 256 dec gives
	ADD	HL,DE		;HL pointing to the char set.
	LD	A,C		;x is copied to A.
	RRCA			;The number 32*(x mod 8)+y is formed
	RRCA			;in A and copied to E.
	RRCA			;This is the low byte of the
	AND	0E0H		;required screen addr.
	XOR	B
	LD	E,A
	LD	A,C		;x is copied to A again.
	AND	18H		;Now the number 64+8*INT(x/8) is
	XOR	40h		;inserted into D
	LD	D,A		;DE now holds the screen addr.
	LD	B,60H		;B counts the 96 chars.
L254F:;S_SCRN_LP
	PUSH	BC		;Save the count.
	PUSH	DE		;And the screen pntr.
	PUSH	HL		;And the char set pntr.
	LD	A,(DE)		;Get 1st row of screen char.
	XOR	(HL)		;Match with row from char set.
	JR	Z,L255A;S_SC_MATCH ;Jump if direct match found.
	INC	A		;Now test for match with inverse char
				;(get 0 in A from 0FFh).
	JR	NZ,L2573;S_SCR_NXT ;Jump if neither match found.
	DEC	A		;Restore 0FFh to A.
L255A:;S_SC_MATCH
	LD	C,A		;Inverse status (0 or 0FFh) to C.
	LD	B,7		;B counts through the other 7 rows.
L255D:;S_SC_ROWS
	INC	D		;Move DE to next row (add 256 dec).
	INC	HL		;Move HL to next row (i.e. next byte).
	LD	A,(DE)		;Get the screen row.
	XOR	(HL)		;Match with row from the ROM.
	XOR	C		;Include the inverse status.
	JR	NZ,L2573;S_SCR_NXT ;Jump if row fails to match.
	DJNZ	L255D;S_SC_ROWS	;Jump back till all rows done.
	POP	BC		;Discard char set pntr.
	POP	BC		;And screen pntr.
	POP	BC		;Final count to BC.
	LD	A,80H		;Last char code in set plus one.
	SUB	B		;A now holds required code.
	LD	BC,1		;One space is now needed in the work space.
	RST	30H;BC_SPACES	;Make the space.
	LD	(DE),A		;Put the char into it.
	JR	L257D;S_SCR_STO	;Jump to stack the char.
L2573:;S_SCR_NXT
	POP	HL		;Restore char set pntr.
	LD	DE,8		;Move it on 8 bytes, to the next
	ADD	HL,DE		;char in the set.
	POP	DE		;Restore the screen pntr.
	POP	BC		;And the counter.
	DJNZ	L254F;S_SCRN_LP	;Loop back for the 96 chars.
	LD	C,B		;Stack the empty string (length zero).
L257D:;S_SCR_STO
	JP	L2AB2;STK_STO_$	;Jump to stack the matching char, or the
				;null string if no match is found.
;Note: The exit, via STK_STO_$, is a mistake as it leads to 'double storing'
;of the string result (see S_STRING, 25DB). The instruction line should be
;'RET'.

;  The last of these four subroutines is the 'scanning attributes
;subroutine'. It is called by S_ATTR to return the value of ATTR(x,y) which
;codes the attributes of line x, column y on the television screen.
L2580:;S_ATTR_S
	CALL	L2307;STK_TO_BC	;x to C, y to B. 0<=x<=23, 0<=y<=31.
	LD	A,C		;x is copied to A and the number
	RRCA			;32*x(mod 8)+y is formed in A and
	RRCA			;copied to L.
	RRCA
	LD	C,A		;32*x(mod 8)+INT(x/8) is also
	AND	0E0H		;copied to C.
	XOR	B
	LD	L,A		;L holds low byte of attribute addr.
	LD	A,C		;32*x(mod 8)+INT(x/8) is copied to A.
	AND	3		;88+INT(x/8) is
	XOR	58h		;formed in A and copied to H.
	LD	H,A		;H holds high byte of attribute addr.
	LD	A,(HL)		;The attribute byte is copied to A.
	JP	L2D28;STACK_A	;Exit, stacking the required byte.

;	THE SCANNING FUNCTION TABLE
;This table contains 8 functions and 4 operators. It thus incorporates 5
;new Spectrum functions and provides a neat way of accessing some functions
;and operators which already existed on the ZX81.
L2596:	
;		code	offset	name		addr
	DB	22H,	1CH	;S_QUOTE	25B3
	DB	28H,	4FH	;S_BRACKET	25E8
	DB	2EH,	0F2H	;S_DECIMAL	268D
	DB	2BH,	12H	;S_U_PLUS	25AF
	DB	0A8H,	56H	;S_FN		25F5
	DB	0A5H,	57H	;S_RND		25F8
	DB	0A7H,	84H	;S_PI		2627
	DB	0A6H,	8FH	;S_INKEY$	2634
	DB	0C4H,	0E6H	;S_BIN		268D (EQU S_DECIMAL)
	DB	0AAH,	0BFH	;S_SCREEN$	2668
	DB	0ABH,	0C7H	;S_ATTR		2672
	DB	0A9H,	0CEH	;S_POINT	267B
	DB	00H		;End marker.

;	THE SCANNING FUNCTION ROUTINES

L25AF:;S_U_PLUS
	RST	20H;NEXT_CHAR	;For unary plus, simply move on to the
	JP	L24FF;S_LOOP_1	;next char and jump back to the main
				;re-entry point of SCANNING.

;The 'scanning QUOTE routine': This routine deals with string quotes,
;whether simple like "name" or more complex like "a ""white"" lie" or the
;seemingly redundant VAL$ """a""".
L25B3:;S_QUOTE
	RST	18H;GET_CHAR	;Fetch the current char.
	INC	HL		;Point to the start of the string.
	PUSH	HL		;Save the start addr.
	LD	BC,0		;Set the length to zero.
	CALL	L250F;S_QUOTE_S	;Call the "matching" subroutine.
	JR	NZ,L25D9;S_Q_PRMS ;Jump if zero resets - no more quotes.
L25BE:;S_Q_AGAIN
	CALL	L250F;S_QUOTE_S	;Call it again for a third quote.
	JR	Z,L25BE;S_Q_AGAIN ;And again for the fifth, seventh etc.
	CALL	L2530;SYNTAX_Z	;If testing syntax, jump to reset bit 6
	JR	Z,L25D9;S_Q_PRMS ;of FLAGS and to continue scanning.
	RST	30H;BC_SPACES	;Make space in the work space for the
				;string and the terminating quote.
	POP	HL		;Get the pntr to the start.
	PUSH	DE		;Save the pntr to the 1st space.
L25CB:;S_Q_COPY
	LD	A,(HL)		;Get a char from the string.
	INC	HL		;Point to the next one.
	LD	(DE),A		;Copy last one to work space.
	INC	DE		;Point to the next space.
	CP	'"'		;Is last char a '"'?
	JR	NZ,L25CB;S_Q_COPY ;If not, jump to copy next one.
	LD	A,(HL)		;But if it was, don't copy next one;
	INC	HL		;if next one is a '"', jump to copy
	CP	'"'		;the one after it;
	JR	Z,L25CB;S_Q_COPY ;otherwise, finished with copying.
L25D9:;S_Q_PRMS
	DEC	BC		;Get true length to BC.

;Note that the 1st quote was not counted into the length; the final quote
;was, and is discarded now. Inside the string, the 1st, 3rd, 5th, etc.,
;quotes were counted in but the 2nd, 4th, etc., were not.
	POP	DE		;Restore the start of copied string.
L25DB:;S_STRING
	LD	HL,L5C3B	;This is FLAGS; this entry point is used
	RES	6,(HL)		;whenever bit 6 is to be reset and a
	BIT	7,(HL)		;string stacked if executing line.
	CALL	NZ,L2AB2;STK_STO_$ ;This is done now.
	JP	L2712;S_CONT_2	;Jump to continue scanning the line
;
;Note that in copying the string to the work space, every two pairs of
;string quotes inside the string ("") have been reduced to one pair of
;string quotes (").
L25E8:;S_BRACKET
	RST	20H;NEXT_CHAR	;The 'scanning BRACKET routine' simply
	CALL	L24FB;SCANNING	;gets the next char and calls SCANNING
	CP	')'		;recursively.
	JP	NZ,L1C8A;REPORT_C ;Report the error if no matching
	RST	20H;NEXT_CHAR	;bracket; then continue scanning.
	JP	L2712;S_CONT_2

L25F5:;S_FN			;The 'scanning FN routine'.
	JP	L27BD;S_FN_SBRN
;This routine, for user-defined functions, just jumps to the 'scanning
;FN subroutine'

L25F8:;S_RND
	CALL	L2530;SYNTAX_Z	;Unless syntax is being checked, jump
	JR	Z,L2625;S_RND_END ;to calculate a random number.
	LD	BC,(L5C76);SEED	;Fetch the current value of SEED.
	CALL	L2D2B;STACK_BC	;Put it on the calc stack.
	RST	28H;FP_CALC	;Now use the calculator.
	DB	0A1H;stk_one	;The 'last value' is now
	DB	0FH;addition	;SEED + 1.
	DB	34H;stk_data	;Put the decimal number 75
	DB	37H;exponent+87	;on the calc stack.
	DB	16H;{00H,00H,00H}
	DB	04H;multiply	;'Last value' (SEED+1)*75.
	DB	34H;stk_data	;See STACK LITERALS to see how
	DB	80H;four bytes	;bytes are expanded so as to put
	DB	41H;exponent+91	;the decimal 65537 on the calc stack.
	DB	00H,00H,80H;{00H}
	DB	32H;n_mod_m	;Divide (SEED+1)*75 by 65537 to give
				;a 'remainder' and an 'answer'.
	DB	02H;delete	;Discard the 'answer'.
	DB	0A1H;stk_one	;The 'last value' is now
	DB	03H;subtract	;'remainder' - 1.
	DB	31H;duplicate	;Make a copy of the 'last value'.
	DB	38H;end_calc	;The calculation is finished.
	CALL	L2DA2;FP_TO_BC	;Use the 'last value' to give the
	LD	(L5C76),BC;SEED	;new value for SEED.
	LD	A,(HL)		;Fetch the exponent of 'last value'.
	AND	A		;Jump forward if the exponent
	JR	Z,L2625;S_RND_END ;is zero.
	SUB	10H		;Reduce the exponent, i.e. divide 'last
	LD	(HL),A		;value' by 65536 to give the required
				;'last value'.
L2625:;S_RND_END
	JR	L2630;S_PI_END	;Jump past the 'PI' routine.

;The 'scanning PI routine': unless syntax is being checked the value of
;'PI' is calculated and forms the 'last value' on the calc stack.
L2627:;S_PI
	CALL	L2530;SYNTAX_Z	;Test for syntax checking.
	JR	Z,L2630;S_PI_END ;Jump if required.
	RST	28H;FP_CALC	;Now use the calc.
	DB	0A3H;stk_pi/2	;The value of PI/2 is put on the calc
	DB	38H;end_calc	;stack as the 'last value'.
	INC	(HL)		;The exponent is incremented thereby
				;doubling the 'last value' giving PI.
L2630:;S_PI_END
	RST	20H;NEXT_CHAR	;Move on to the next char.
	JP	L26C3;S_NUMERIC	;Jump forward.

L2634:;S_INKEY$
	LD	BC,105Ah	;Priority 10h, operation code 5Ah
	RST	20H;NEXT_CHAR	;for the 'read-in' subroutine.
	CP	'#'		;If next char is '#', jump. There
	JP	Z,L270D;S_PUSH_PO ;will be a numerical arg.
	LD	HL,L5C3B	;This is FLAGS.
	RES	6,(HL)		;Reset bit 6 for a string result.
	BIT	7,(HL)		;Test for syntax checking.
	JR	Z,L2665;S_INK$_EN ;Jump if required.
	CALL	L028E;KEY_SCAN	;Fetch a key-value in DE.
	LD	C,0		;Prepare empty string; stack it if
	JR	NZ,L2660;S_IK$_STK ;too many keys pressed.
	CALL	L031E;K_TEST	;Test the key value; stack empty
	JR	NC,L2660;S_IK$_STK ;string if unsatisfactory.
	DEC	D		;0FFh to D for L mode (bit 3 set).
	LD	E,A		;Key-value to E for decoding.
	CALL	L0333;K_DECODE	;Decode the key-value.
	PUSH	AF		;Save the ASCII value briefly.
	LD	BC,1		;One space is needed in the work space.
	RST	30H;BC_SPACES	;Make it now.
	POP	AF		;Restore the ASCII value.
	LD	(DE),A		;Prepare to stack it as a string.
	LD	C,1		;Its length is one.
L2660:;S_IK$_STK
	LD	B,0		;Complete the length parameter.
	CALL	L2AB2;STK_STO_$	;Stack the required string.
L2665:;S_INK$_EN
	JP	L2712;S_CONT_2	;Jump forward.

L2668:;S_SCREEN$
	CALL	L2522;S_2_COORD	;Check that 2 co-ordinates are given.
	CALL	NZ,L2535;S_SCRN$_S ;Call the subroutine unless
	RST	20H;NEXT_CHAR	;checking syntax; then get next
	JP	L25DB;S_STRING	;char and jump back.

L2672:;S_ATTR
	CALL	L2522;S_2_COORD	;Check that 2 co-ordinates are given.
	CALL	NZ,L2580;S_ATTR_S ;Call the subroutine unless
	RST	20H;NEXT_CHAR	;checking syntax; then get the next
	JR	L26C3;S_NUMERIC	;char and jump forward.

L267B:;S_POINT
	CALL	L2522;S_2_COORD	;Check that 2 co-ordinates are given.
	CALL	NZ,L22CB;POINT_SUB ;Call the subroutine unless
	RST	20H;NEXT_CHAR	;checking syntax; then get the next
	JR	L26C3;S_NUMERIC	;char and jump forward.

L2684:;S_ALPHNUM
	CALL	L2C88;ALPHANUM	;Is the char alphanumeric?
	JR	NC,L26DF;S_NEGATE ;Jump if not a letter or a digit.
	CP	'A'		;Now jump if it a letter;
	JR	NC,L26C9;S_LETTER ;otherwise continue on into S_DECIMAL.

;The 'scanning DECIMAL routine' which follows deals with a decimal point or
;a number that starts with a digit. It also takes care of the expression
;'BIN', which is dealt with in the 'decimal to flpnt' subroutine.
L268D:;S_DECIMAL (EQU S_BIN)
	CALL	L2530;SYNTAX_Z	;Jump forward if a line is
	JR	NZ,L26B5;S_STK_DEC ;being executed.

;The action taken is now very different for syntax checking and line
;execution. If syntax is being checked then the flpnt form has to be
;calculated and copied into the actual BASIC line. However when a line is
;being executed the flpnt form will always be available so it is copied
;to the calc stack to form a 'last value'.
;	During syntax checking:
	CALL	L2C9B;DEC_TO_FP	;The flpnt form is found.
	RST	18H;GET_CHAR	;Set HL to point one past the last digit.
	LD	BC,6		;Six locs are required.
	CALL	L1655;MAKE_ROOM	;Make the room in the BASIC line.
	INC	HL		;Point to the 1st free space.
	LD	(HL),0EH	;Enter the number marker code.
	INC	HL		;Point to the 2nd loc.
	EX	DE,HL		;This pntr is wanted in DE.
	LD	HL,(L5C65);STKEND ;Fetch the 'old' STKEND.
	LD	C,5		;There are 5 bytes to move.
	AND	A		;Clear the carry flag.
	SBC	HL,BC		;The 'new' STKEND = 'old' STKEND -5.
	LD	(L5C65),HL;STKEND ;Move the flpnt number from the
	LDIR			;calc stack to the line.
	EX	DE,HL		;Put the line pntr in HL.
	DEC	HL		;Point to the last byte added.
	CALL	L0077;TEMP_PTR1	;This sets CH_ADD.
	JR	L26C3;S_NUMERIC	;Jump forward.

;	During line execution:
L26B5:;S_STK_DEC
	RST	18H;GET_CHAR	;Get the current char.
L26B6:;S_SD_SKIP
	INC	HL		;Now move on to the next
	LD	A,(HL)		;char in turn until
	CP	0EH		;the number marker code
	JR	NZ,L26B6;S_SD_SKIP ;is found.
	INC	HL		;Point to the 1st byte of the number.
	CALL	L33B4;STACK_NUM	;Move the flpnt number.
	LD	(L5C5D),HL;CH_ADD ;Set CH_ADD.

;A numeric result has now been identified, coming from RND, PI, ATTR,
;POINT or a decimal number, therefore bit 6 of FLAGS must be set.
L26C3:;S_NUMERIC
	SET	6,(IY+1);FLAGS	;Set the numeric marker bit.
	JR	L26DD;S_CONT_1	;Jump forward.

;	THE SCANNING VARIABLE ROUTINE
;When a var name has been identified a call is made to LOOK_VARS which looks
;through those vars that already exist in the vars area (or in the program
;area at DEF FN stmts for a user-defined function FN). If an appropriate
;numeric value is found then it is copied to the calc stack using STACK_NUM.
;However a string or string array entry has to have the appropriate params
;passed to the calc stack by the STK_VAR subroutine (or in the case of a
;user-defined function, by the STK_F_ARG subr as called from LOOK_VARS).
L26C9:;S_LETTER
	CALL	L28B2;LOOK_VARS	;Look in the existing vars for
				;the matching entry.
	JP	C,L1C2E;REPORT_2 ;An error is reported if there is no
				;existing entry.
	CALL	Z,L2996;STK_VARS ;Stack the params of the string entry/
				;return numeric element base addr.
	LD	A,(L5C3B);FLAGS	;Fetch FLAGS.
	CP	0C0H		;Test bits 6 and 7 together.
	JR	C,L26DD;S_CONT_1 ;One or both bits are reset.
	INC	HL		;A numeric value is to be stacked.
	CALL	L33B4;STACK_NUM	;Move the number.
L26DD:;S_CONT_1
	JR	L2712;S_CONT_2	;Jump forward.

;The char is tested against the code for '-', thus identifying the 'unary
;minus' operation.
;  Before the actual test the B reg is set to hold the priority 9 and the
;C reg the operation code 0DBh that are required for this operation.
L26DF:;S_NEGATE
	LD	BC,9DBh		;Priority 9, operation code 0DBh.
	CP	'-'		;Is it a '-'?
	JR	Z,L270D;S_PUSH_PO ;Jump forward if it is 'unary minus'.

;Next the char is tested against the code for 'VAL$', with priority 16 dec
;and operation code 18 hex.
	LD	BC,1018h	;Priority 16 dec, operation code 18h.
	CP	0AEH		;Is it 'VAL$'?
	JR	Z,L270D;S_PUSH_PO ;Jump forward if it is 'VAL$'.

;The present char must now represent one of the functions CODE to NOT, with
;codes 0AFh to 0C3h.
	SUB	0AFH		;The range of the functions is changed from
				;0AFh to 0C3h to range 0 to 14h.
	JP	C,L1C8A;REPORT_C ;Report an error if out of range.

;The function 'NOT' is identified and dealt with separately from the others.
	LD	BC,4F0h		;Priority 4, operation code 0F0h
	CP	14H		;Is it the function 'NOT'?
	JR	Z,L270D;S_PUSH_PO ;Jump if it is so.
	JP	NC,L1C8A;REPORT_C ;Check the range again.

;The remaining functions have priority 16 dec. The operation codes for these
;functions are now calculated. Functions that operate on strings need bit 6
;reset and functions that give string results need bit 7 reset in their
;operation codes.
	LD	B,10H		;Priority 16 dec.
	ADD	A,0DCH		;The function range is now 0DCh to 0EFh.
	LD	C,A		;Transfer the operation code.
	CP	0DFH		;Separate CODE, VAL and LEN
	JR	NC,L2707;S_NO_TO_$ ;which operate on strings to give
	RES	6,C		;numerical results.
L2707:;S_NO_TO_$
	CP	0EEH		;Separate STR$ and CHR$ which operate
	JR	C,L270D;S_PUSH_PO ;on numbers to give string results.
	RES	7,C

;The priority code and the operation code for the function being considered
;are now pushed on to the machine stack. A hierarchy of operations is
;thereby built up.
L270D:;S_PUSH_PO
	PUSH	BC		;Stack the priority and operation
	RST	20H;NEXT_CHAR	;codes before moving on to
	JP	L24FF;S_LOOP_1	;consider the next part of the expression.

;The scanning of the line now continues. The present argument may be
;followed by a '(', a binary operator or, if the end of the expression has
;been reached, then e.g. a carriage return char or a colon, a separator or
;a 'THEN'.
L2712:;S_CONT_2
	RST	18H;GET_CHAR	;Fetch the present char.
L2713:;S_CONT_3
	CP	'('		;Jump forward if it is not a '(', which
	JR	NZ,L2723;S_OPERTR ;indicates a parenthesised expression.

;If the 'last value' is numeric then the parenthesised expression is a true
;sub-expression and must be evaluated by itself. However if the 'last value'
;is a string then the parenthesised expression represents an element of an
;array or a slice of a string. A call to SLICING modifies the params of the
;string as required.
	BIT	6,(IY+1);FLAGS	;Jump forward if dealing with a numeric
	JR	NZ,L2734;S_LOOP	;parenthesised expression.
	CALL	L2A52;SLICING	;Modify the params of the 'last value'.
	RST	20H;NEXT_CHAR	;Move on to consider the next char.
	JR	L2713;S_CONT_3

;If the present char is indeed a binary operation it will be given an
;operation code in the range 0C3h - 0CFh, and the appropriate priority code.
L2723:;S_OPERTR
	LD	B,0		;Original code to BC to index
	LD	C,A		;into table of operators.
	LD	HL,L2795	;The pntr to the table.
	CALL	L16DC;INDEXER	;Index into the table.
	JR	NC,L2734;S_LOOP	;Jump forward if no operation found.
	LD	C,(HL)		;Get required code from the table.
	LD	HL,L27B0-0C3h	;The pntr to the priority table: i.e.
				;26ED+C3 gives 27B0 as the 1st addr.
	ADD	HL,BC		;Index into the table.
	LD	B,(HL)		;Fetch the appropriate priority.

;The main loop of this subroutine is now entered. At this stage there are:
;   i. A 'last value' on the calc stack.
;  ii. The starting priority marker on the machine stack below a hierarchy,
;      of unknown size, of function and binary operation codes. This
;      hierarchy may be null.
; iii. The BC reg pair holding the 'present' operation and priority, which
;      if the end of an expression has been reached will be priority zero.
;Initially the 'last' operation and priority are taken off the machine stack
;and compared against the 'present' operation and priority.
;  If the 'present' priority is higher than the 'last' priority then an exit
;is made from the loop as the 'present' priority is considered to bind
;tighter than the 'last' priority.
;  However, if the present priority is less binding, then the operation
;specified as the 'last' operation is performed. The 'present' operation and
;priority go back on the machine stack to be carried round the loop again.
;In this manner the hierarchy of functions and binary operations that have
;been queued are dealt with in the correct order.
L2734:;S_LOOP
	POP	DE		;Get the 'last' operation and priority.
	LD	A,D		;The priority goes to the A reg.
	CP	B		;Compare 'last' against 'present'.
	JR	C,L2773;S_TIGHTER ;Exit to wait for the argument.
	AND	A		;Are both priorities zero?
	JP	Z,L0018;GET_CHAR ;Exit via GET_CHAR thereby making
				;'last value' the required result.

;Before the 'last' operation is performed, the 'USR' function is separated
;into 'USR number' and 'USR string' according as bit 6 of FLAGS was set or
;reset when the argument of the function was stacked as the 'last value'.
	PUSH	BC		;Stack the 'present' values.
	LD	HL,L5C3B	;This is FLAGS.
	LD	A,E		;The 'last' operation is compared with the
	CP	0EDH		;code for USR, which will give 'USR number'
	JR	NZ,L274C;S_STK_LST ;unless modified; jump if not 'USR'.
	BIT	6,(HL)		;Test bit 6 of FLAGS.
	JR	NZ,L274C;S_STK_LST ;Jump if it is set ('USR number').
	LD	E,99H		;Modify the 'last' operation code: 'offset'
				;19, +80 for string input and numerical
				;result ('USR string').
L274C:;S_STK_LST
	PUSH	DE		;Stack the 'last' values briefly.
	CALL	L2530;SYNTAX_Z	;Do not perform the actual operation
	JR	Z,L275B;S_SYNTEST ;if syntax is being checked.
	LD	A,E		;The 'last' operation code.
	AND	3Fh		;Strip off bits 6 and 7 to convert the
	LD	B,A		;operation code to a calc offset.
	RST	28H;FP_CALC	;Now use the calc.
	DB	3BH;fp_calc_2	;Perform the actual operation.
	DB	38H;end_calc	;It has been done.
	JR	L2764;S_RUNTEST	;Jump forward.

;An important part of syntax checking involves the testing of the operation
;to ensure that the nature of the 'last value' is of the correct type for
;the operation under consideration.
L275B:;S_SYNTEST
	LD	A,E		;Get the 'last' operation code.
	XOR	(IY+1);FLAGS	;This tests the nature of the 'last value'
	AND	40h		;against the requirement of the operation.
				;They are to be the same for correct syntax.
L2761:;S_RPORT_C
	JP	NZ,L1C8A;REPORT_C ;Jump if syntax fails.

;Before jumping back to go round the loop again the nature of the 'last
;value' must be recorded in FLAGS.
L2764:;S_RUNTEST
	POP	DE		;Get the 'last' operation code.
	LD	HL,L5C3B	;This is FLAGS.
	SET	6,(HL)		;Assume result to be numeric.
	BIT	7,E		;Jump forward if the nature of
	JR	NZ,L2770;S_LOOPEND ;'last value' is numeric.
	RES	6,(HL)		;It is string.
L2770:;S_LOOPEND
	POP	BC		;Get the 'present' values into BC.
	JR	L2734;S_LOOP	;Jump back.

;Whenever the 'present' operation binds tighter, the 'last' and the 'present'
;values go back on the machine stack. However if the 'present' operation
;requires a string as its operand then the operation code is modified to
;indicate this requirement.
L2773:;S_TIGHTER
	PUSH	DE		;The 'last' values go on the stack.
	LD	A,C		;Get the 'present' operation code.
	BIT	6,(IY+1);FLAGS	;Do not modify the operation code if
	JR	NZ,L2790;S_NEXT	;dealing with a numeric operand.
	AND	3Fh		;Clear bits 6 and 7.
	ADD	A,8		;Increase the code by 8.
	LD	C,A		;Return the code to the C reg.
	CP	10H		;Is the operation 'AND'?
	JR	NZ,L2788;S_NOT_AND ;Jump if it is not so.
	SET	6,C		;'AND' requires a numeric operand.
	JR	L2790;S_NEXT	;Jump forwards.
L2788:;S_NOT_AND
	JR	C,L2761;S_RPORT_C ;The operations -, *, /, ^ and OR are
				;not possible between strings.
	CP	17H		;Is the operation a '+'?
	JR	Z,L2790;S_NEXT	;Jump if it is so.
	SET	7,C		;The other operations yield a numeric
				;result.
L2790:;S_NEXT
	PUSH	BC		;The 'present' values go on the machine
				;stack.
	RST	20H;NEXT_CHAR	;Consider the next char.
	JP	L24FF;S_LOOP_1	;Go around the loop again.

;	THE TABLE OF OPERATORS
L2795:;		code	op code	operator
	DB	2BH,	0CFH;	+
	DB	2DH,	0C3H;	-
	DB	2AH,	0C4H;	*
	DB	2FH,	0C5H;	/
	DB	5EH,	0C6H;	^
	DB	3DH,	0CEH;	=
	DB	3EH,	0CCH;	>
	DB	3CH,	0CDH;	<
	DB	0C7H,	0C9H;	<=
	DB	0C8H,	0CAH;	>=
	DB	0C9H,	0CBH;	<>
	DB	0C5H,	0C7H;	OR
	DB	0C6H,	0C8H;	AND
	DB	00H;		End marker.

;	THE TABLE OF PRIORITIES (precedence table)
L27B0:;		- * /  ^  OR AND <= >= <> > < = +
	DB	6,8,8,0Ah,2 ,3  ,5 ,5 ,5 ,5,5,5,6

