
;	THE 'SCANNING FUNCTION' SUBROUTINE
;This subroutine is called by the 'scanning FN routine' to evaluate a user
;defined function which occurs in a BASIC line. The subroutine can be
;considered in four stages:
;    i. The syntax of the FN stmt is checked during syntax checking.
;   ii. During line execution, a search is made of the program area for a
;	DEF FN stmt, and the names of the functions are compared, until
;	a match is found - or an error is reported.
;  iii. The arguments of the FN are evaluated by calls to SCANNING.
;   iv. The function itself is evaluated by calling SCANNING, which in turn
;	calls LOOK_VARS and so the 'STACK FUNCTION ARGUMENT' subroutine.
L27BD:;S_FN_SBRN
	CALL	L2530;SYNTAX_Z	;Unless syntax is being checked,
	JR	NZ,L27F7;SF_RUN	;a jump is made to SF_RUN.
	RST	20H;NEXT_CHAR	;Get the 1st char of the name.
	CALL	L2C8D;ALPHA	;If it is not alphabetic, then
	JP	NC,L1C8A;REPORT_C ;report the error.
	RST	20H;NEXT_CHAR	;Get the next char.
	CP	'$'		;Is it a '$'?
	PUSH	AF		;Save the zero flag on the stack.
	JR	NZ,L27D0;SF_BRKT_1 ;Jump if it was not a '$'.
	RST	20H;NEXT_CHAR	;But get the next char if it was.
L27D0:;SF_BRKT_1
	CP	'('		;If the char is not a '(', then
	JR	NZ,L27E6;SF_RPRT_C ;report the error.
	RST	20H;NEXT_CHAR	;Get the next char.
	CP	')'		;Is it a ')'?
	JR	Z,L27E9;SF_FLAG_6 ;Jump if it is; there are no arguments.
L27D9:;SF_ARGMTS
	CALL	L24FB;SCANNING	;Within the loop, call SCANNING to check the
				;syntax of each argument and to insert
				;flpnt numbers.
	RST	18H;GET_CHAR	;Get the char which follows the argument;
	CP	','		;if it is not a ','
	JR	NZ,L27E4;SF_BRKT_2 ;then jump - no more arguments.
	RST	20H;NEXT_CHAR	;Get the 1st char in the next argument.
	JR	L27D9;SF_ARGMTS	;Loop back to consider this argument.
L27E4:;SF_BRKT_2
	CP	')'		;Is the current char a ')'?
L27E6:;SF_RPRT_C
	JP	NZ,L1C8A;REPORT_C ;Report the error if it is not.
L27E9:;SF_FLAG_6
	RST	20H;NEXT_CHAR	;Point to the next char in the BASIC line.
	LD	HL,L5C3B	;This is FLAGS; assume a string-valued
	RES	6,(HL)		;function and reset bit 6 of FLAGS.
	POP	AF		;Restore the zero flag; jump if
	JR	Z,L27F4;SF_SYN_EN ;the FN is indeed string-valued.
	SET	6,(HL)		;Otherwise, set bit 6 of FLAGS.
L27F4:;SF_SYN_EN
	JP	L2712;S_CONT_2	;Jump back to continue scanning the line.

;   ii. During line execution, a search must 1st be made for a DEF FN stmt.
L27F7:;SF_RUN
	RST	20H;NEXT_CHAR	;Get the 1st char of the name.
	AND	0DFH		;Reset bit 5 for upper case.
	LD	B,A		;Copy the name to B.
	RST	20H;NEXT_CHAR	;Get the next char.
	SUB	'$'		;Subtract 24 hex, the code for '$'.
	LD	C,A		;Copy the result to C (zero for a string,
				;non-zero for a numerical function).
	JR	NZ,L2802;SF_ARGMT1 ;Jump if non-zero: numerical function.
	RST	20H;NEXT_CHAR	;Get the next char, the '('.
L2802:;SF_ARGMT1
	RST	20H;NEXT_CHAR	;Get 1st char of 1st argument.
	PUSH	HL		;Save the pntr to it on the stack.
	LD	HL,(L5C53);PROG	;Point to the start of the program.
	DEC	HL		;Go back one loc.
L2808:;SF_FND_DF
	LD	DE,0CEH		;The search will be for 'DEF FN'.
	PUSH	BC		;Save the name and 'string status'.
	CALL	L1D86;LOOK_PROG	;Search the program now.
	POP	BC		;Restore the name and status.
	JR	NC,L2814;SF_CP_DEF ;Jump if a DEF FN stmt found.

;Report P - FN without DEF
L2812:;REPORT_P
	RST	8;ERROR_1	;Call the error handling
	DB	18H		;routine.

;When a DEF FN stmt is found, the name and status of the two functions are
;compared: if they do not match, the search is resumed.
L2814:;SF_CP_DEF
	PUSH	HL		;Save the pntr to the DEF FN char in case
				;the search has to be resumed.
	CALL	L28AB;FN_SKPOVR	;Get the name of the DEF FN function.
	AND	0DFH		;Reset bit 5 for upper case.
	CP	B		;Does it match the FN name?
	JR	NZ,L2825;SF_NOT_FD ;Jump if it does not match.
	CALL	L28AB;FN_SKPOVR	;Get the next char in the DEF FN.
	SUB	'$'		;Subtract 24 hex, the code for '$'.
	CP	C		;Compare the status with that of FN.
	JR	Z,L2831;SF_VALUES ;Jump if complete match now found.
L2825:;SF_NOT_FD
	POP	HL		;Restore the pntr to the 'DEF FN'.
	DEC	HL		;Step back one loc.
	LD	DE,200h		;Use the search routine to find the end of
	PUSH	BC		;the DEF FN stmt, preparing for the next
	CALL	L198B;EACH_STMT	;search.
	POP	BC		;Save the name and status meanwhile.
	JR	L2808;SF_FND_DF	;Jump back for a further search.

;  iii. The correct DEF FN stmt has now been found. The aruments of the FN
;stmt will be evaluated by repeated calls of SCANNING, and their 5 byte
;values (or params, for strings) will be inserted into the DEF FN stmt
;(calling FN_SKPOVR as needed) while CH_ADD points along the FN stmt
;(calling RST 20, NEXT_CHAR, as needed).
L2831:;SF_VALUES
	AND	A		;If HL is now pointing to a '$',
	CALL	Z,L28AB;FN_SKPOVR ;move on to the '('.
	POP	DE		;Discard the pntr to 'DEF FN'.
	POP	DE		;Get the pntr to the 1st argument
	LD	(L5C5D),DE;CH_ADD ;of FN, and copy it to CH_ADD.
	CALL	L28AB;FN_SKPOVR	;Move past the '(' now.
	PUSH	HL		;Save this pntr on the stack.
	CP	')'		;Is it pointing to a ')'?
	JR	Z,L2885;SF_R_BR_2 ;If so, jump: FN has no arguments.
L2843:;SF_ARG_LP
	INC	HL		;Point to the next code.
	LD	A,(HL)		;Put the code into A.
	CP	0EH		;Is it the 'number marker' code, 0E hex?
	LD	D,40h		;Set bit 6 of D for a numerical argument.
	JR	Z,L2852;SF_ARG_VL ;Jump on zero: numerical argument.
	DEC	HL		;Now ensure that HL is pointing to the
				;'$' char (not e.g. to a control code).
	CALL	L28AB;FN_SKPOVR
	INC	HL		;HL now points to the 'number marker'.
	LD	D,0		;Bit 6 of D is reset: string argument.
L2852:;SF_ARG_VL
	INC	HL		;Point to the 1st of the 5 bytes in DEF FN.
	PUSH	HL		;Save this pntr on the stack.
	PUSH	DE		;Save the 'string status' of the argument.
	CALL	L24FB;SCANNING	;Now evaluate the argument.
	POP	AF		;Get the no./string flag into A.
	XOR	(IY+1);FLAGS	;Test bit 6 of it against the result
	AND	40h		;of SCANNING.
	JR	NZ,L288B;REPORT_Q ;Give report Q if they did not match.
	POP	HL		;Get the pntr to the 1st of the
	EX	DE,HL		;5 spaces in DEF FN into DE.
	LD	HL,(L5C65)	;Point HL at STKEND.
	LD	BC,5		;BC will count 5 bytes to be moved.
	SBC	HL,BC		;1st, decrease STKEND by 5, so deleting
	LD	(L5C65),HL;STKEND ;the 'last value' from the stack.
	LDIR			;Copy the 5 bytes into the spaces in DEF FN.
	EX	DE,HL		;Point HL at the next code.
	DEC	HL		;Ensure that HL points to the
	CALL	L28AB;FN_SKPOVR	;char after the 5 bytes.
	CP	')'		;Is it a ')'?
	JR	Z,L2885;SF_R_BR_2 ;Jump if it is: no more arguments in the
				;DEF FN stmt.
	PUSH	HL		;It is a ',': save the pntr to it.
	RST	18H;GET_CHAR	;Get the char after the last argument that
				;was evaluated from FN.
	CP	','		;If it is not a ',' jump: mismatched
	JR	NZ,L288B;REPORT_Q ;arguments of FN and DEF FN.
	RST	20H;NEXT_CHAR	;Point CH_ADD to the next argument of FN.
	POP	HL		;Point HL to the ',' in DEF FN again.
	CALL	L28AB;FN_SKPOVR	;Move HL on to the next argument in DEF FN.
	JR	L2843;SF_ARG_LP	;Jump back to consider this argument.
L2885:;SF_R_BR_2
	PUSH	HL		;Save the pntr to the ')' in DEF FN.
	RST	18H;GET_CHAR	;Get the char after the last argument in FN.
	CP	')'		;Is it a ')'?
	JR	Z,L288D;SF_VALUE ;If so, jump to evaluate the function;
				;but if not, give report Q.

;Report Q - Parameter error
L288B:;REPORT_Q
	RST	8;ERROR_1	;Call the error handling
	DB	19H		;routine.

;   iv. Finally, the function itself is evaluated by calling SCANNING,
;after 1st setting DEFADD to hold the addr of the arguments as they occur
;in the DEF FN stmt. This ensures that LOOK_VARS, when called by SCANNING,
;will 1st search these arguments for the required values, before making
;a search of the vars area.
L288D:;SF_VALUE
	POP	DE		;Restore pntr to ')' in DEF FN.
	EX	DE,HL		;Get this pntr into HL.
	LD	(L5C5D),HL;CH_ADD ;Insert it into CH_ADD.
	LD	HL,(L5C0B);DEFADD ;Get the old value of DEFADD.
	EX	(SP),HL		;Stack it, and get the start addr of the
	LD	(L5C0B),HL;DEFADD ;arguments area of DEF FN into DEFADD.
	PUSH	DE		;Save addr of ')' in FN.
	RST	20H;NEXT_CHAR	;Move CH_ADD on past ')' and '=' to the
	RST	20H;NEXT_CHAR	;start of the expression for the function
	CALL	L24FB;SCANNING	;in DEF FN. Now evaluate the function.
	POP	HL		;Restore the addr of ')' in FN.
	LD	(L5C5D),HL;CH_ADD ;Store it in CH_ADD.
	POP	HL		;Restore original value of DEFADD.
	LD	(L5C0B),HL;DEFADD ;Put it back into DEFADD.
	RST	20H;NEXT_CHAR	;Get the next char in the BASIC line.
	JP	L2712;S_CONT_2	;Jump back to continue scanning.

;	THE 'FUNCTION SKIPOVER' SUBROUTINE
;This subroutine is used by FN and by STK_F_ARG to move HL along the DEF FN
;stmt while leaving CH_ADD undisturbed, as it points along the FN stmt.
L28AB:;FN_SKPOVR
	INC	HL		;Point to the next code in the stmt.
	LD	A,(HL)		;Copy the code to A.
	CP	21h		;Jump back to skip over it if it is a
	JR	C,L28AB;FN_SKPOVR ;control code or a space.
	RET			;Finished.

;	THE 'LOOK_VARS' SUBROUTINE
;This subroutine is called whenever a search of the vars area or of the
;arguments of a DEF FN stmt is required. The subroutine is entered with
;the system var CH_ADD pointing to the 1st letter of the name of the var
;whose loc is being sought. The name will be in the program area or the
;work space. The subroutine initially builds up a discriminator byte, in
;the C reg, that is based on the 1st letter of the var's name. Bits 5 & 6
;of this byte indicate the type of the var that is being handled.
;  The B reg is used as a bit register to hold flags.
L28B2:;LOOK_VARS
	SET	6,(IY+1);FLAGS	;Presume a numeric var.
	RST	18H;GET_CHAR	;Get the 1st char into A.
	CALL	L2C8D;ALPHA	;Is it alphabetic?
	JP	NC,L1C8A;REPORT_C ;Give an error report if it is not so.
	PUSH	HL		;Save the pntr to the 1st letter.
	AND	1FH		;Transfer bits 0 to 4 of the letter to
	LD	C,A		;the C reg; bits 5 & 7 are always reset.
	RST	20H;NEXT_CHAR	;Get the 2nd char into A.
	PUSH	HL		;Save this pntr also.
	CP	'('		;Is the 2nd char a '('?
	JR	Z,L28EF;V_RUN/SYN ;Separate arrays of numbers.
	SET	6,C		;Now set bit 6.
	CP	'$'		;Is the 2nd char a '$'?
	JR	Z,L28DE;V_STR_VAR ;Separate all the strings.
	SET	5,C		;Now set bit 5.
	CALL	L2C88;ALPHANUM	;If the var's name has only one char
	JR	NC,L28E3;V_TEST_FN ;then jump forward.

;Now find the end char of a name that has more than one char.
L28D4:;V_CHAR
	CALL	L2C88;ALPHANUM	;Is the char alphanumeric?
	JR	NC,L28EF;V_RUN/SYN ;Jump out of the loop when the end of
				;the name is found.
	RES	6,C		;Mark the discriminator byte.
	RST	20H;NEXT_CHAR	;Get the next char.
	JR	L28D4;V_CHAR	;Go back to test it.

;Simple strings and arrays of strings require that bit 6 of FLAGS is reset.
L28DE:;V_STR_VAR
	RST	20H;NEXT_CHAR	;Step CH_ADD past the '$'.
	RES	6,(IY+1);FLAGS	;Reset the bit 6 to indicate a string.

;If DEFADD-hi is non-zero, indicating that a 'function' (a 'FN') is being
;evaluated, and if in 'run-time', a search will be made of the arguments
;in the DEF FN stmt.
L28E3:;V_TEST_FN
	LD	A,(L5C0C);DEFADD_hi ;Is DEFADD_hi zero?
	AND	A
	JR	Z,L28EF;V_RUN/SYN ;If so, jump forward.
	CALL	L2530;SYNTAX_Z	;In 'run-time'?
	JP	NZ,L2951;STK_F_ARG ;If so, jump forward to search the
				;DEF FN stmt.

;Otherwise (or if the var was not found in the DEF FN stmt) a search of
;the vars area will be made, unless syntax is being checked.
L28EF:;V_RUN/SYN
	LD	B,C		;Copy the discriminator byte to the B reg.
	CALL	L2530;SYNTAX_Z	;Jump forward if in
	JR	NZ,L28FD;V_RUN	;'run-time'.
	LD	A,C		;Move the discriminator to A.
	AND	0E0H		;Drop the char code part.
	SET	7,A		;Indicate syntax by setting bit 7.
	LD	C,A		;Restore the discriminator.
	JR	L2934;V_SYNTAX	;Jump forward to continue.

;A BASIC line is being executed so make a search of the vars area.
L28FD:;V_RUN
	LD	HL,(L5C4B);VARS	;Pick up the VARS pntr.

;Now enter a loop to consider the names of the existing vars.
L2900:;V_EACH
	LD	A,(HL)		;The 1st letter of each existing var.
	AND	7FH		;Match on bits 0 to 6.
	JR	Z,L2932;V_80_BYTE ;Jump when the '80-byte' is reached.
	CP	C		;The actual comparison.
	JR	NZ,L292A;V_NEXT	;Jump forward if the 1st chars do not match.
	RLA			;Rotate A leftwards and then
	ADD	A,A		;double it to test bits 5 & 6.
	JP	P,L293F;V_FOUND_2 ;Strings and array vars.
	JR	C,L293F;V_FOUND_2 ;Simple numeric and FOR-NEXT vars.

;Long names are required to be matched fully.
	POP	DE		;Take a copy of the pntr
	PUSH	DE		;to the 2nd char.
	PUSH	HL		;Save the 1st letter pntr.
L2912:;V_MATCHES
	INC	HL		;Consider the next char.
L2913:;V_SPACES
	LD	A,(DE)		;Fetch each char in turn.
	INC	DE		;Point to the next char.
	CP	' '		;Is the char a 'space'?
	JR	Z,L2913;V_SPACES ;Ignore the spaces.
	OR	20h		;Set bit 5 so as to match lower and upper
	CP	(HL)		;case letters; make the comparison.
	JR	Z,L2912;V_MATCHES ;Back for another char if it does match.
	OR	80H		;Will it match with bit 7 set?
	CP	(HL)		;Try it.
	JR	NZ,L2929;V_GET_PTR ;Jump forward if the 'last chars'
				;do not match.
	LD	A,(DE)		;Check that the end of the
	CALL	L2C88;ALPHANUM	;name has been reached before
	JR	NC,L293E;V_FOUND_1 ;jumping forward.

;In all cases where the names fail to match the HL reg pair has to be made
;to point to the next var in the vars area.
L2929:;V_GET_PTR
	POP	HL		;Fetch the pntr.
L292A:;V_NEXT
	PUSH	BC		;Save B & C briefly.
	CALL	L19B8;NEXT_ONE	;DE is made to point to the next var.
	EX	DE,HL		;Switch the two pntrs.
	POP	BC		;Get B & C back.
	JR	L2900;V_EACH	;Go around the loop again.

;Come here if no entry was found with the correct name.
L2932:;V_80_BYTE
	SET	7,B		;Signal 'var not found'.

;Come here if checking syntax.
L2934:;V_SYNTAX
	POP	DE		;Drop the pntr to the 2nd char.
	RST	18H;GET_CHAR	;Fetch the present char.
	CP	'('		;Is it a '('?
	JR	Z,L2943;V_PASS	;Jump forward.
	SET	5,B		;Indicate not dealing with an
	JR	L294B;V_END	;array and jump forward.

;Come here when an entry with the correct name was found.
L293E:;V_FOUND_1
	POP	DE		;Drop the saved var pntr.
L293F:;V_FOUND_2
	POP	DE		;Drop the 2nd char pntr.
	POP	DE		;Drop the 1st letter pntr.
	PUSH	HL		;Save the 'last' letter pntr.
	RST	18H;GET_CHAR	;Fetch the current char.

;If the matching var name has more than a single letter then the other
;chars must be passed-over.
;Note: This appears to have been done already at V_CHAR.
L2943:;V_PASS
	CALL	L2C88;ALPHANUM	;Is it alphanumeric?
	JR	NC,L294B;V_END	;Jump when the end of the name
				;has been found.
	RST	20H;NEXT_CHAR	;Fetch the next char.
	JR	L2943;V_PASS	;Go back and test it.

;The exit-parameters are now set.
L294B:;V_END
	POP	HL		;HL holds the pntr to the letter of a short
				;name or the 'last' char of a long name.
	RL	B		;Rotate the whole reg.
	BIT	6,B		;Specify the state of bit 6.
	RET			;Finished.
;The exit-params for the subroutine can be summarised as follows: The
;system var CH_ADD points to the 1st loc after the name of the var as it
;occurs in the BASIC line.
;When 'var not found':
;    i. The carry flag is set.
;   ii. The zero flag is set only when the search was for an array var.
;  iii. The HL reg pair points to the 1st letter of the name of the var
;	as it occurs in the BASIC line.
;When 'var found':
;    i. The carry flag is reset.
;   ii. The zero flag is set for both simple string vars and all array vars.
;  iii. The HL reg pair points to the letter of a 'short' name, or the last
;	char of a 'long' name, of the existing entry that was found in the
;	var area.
;In all cases bits 5&6 of the C reg indicate the type of var being handled.
;Bit 7 is the complement of the SYNTAX/RUN flag. But only when the
;subroutine is used in 'run-time' will bits 0 to 4 hold the code of the
;var's letter.
;  In syntax time the return is always made with the carry flag reset. The
;zero flag is set for arrays and reset for all other vars, except that a
;simple string name incorrectly followed by a '(' sets the zero flag and,
;in the case of SAVE "name" DATA a$(), passes syntax as well.

;	THE 'STACK FUNCTION ARGUMENT' SUBROUTINE
;This subroutine is called by LOOK_VARS when DEFADD_hi in non-zero, to
;make a search of the args area of a DEF FN stmt, before searching in the
;vars area. If the var is found in the DEF FN stmt, then the params of a
;string var a stacked and a signal is given that there is no need to call
;STK_VAR. But it is left to SCANNING to stack the value of a numerical
;var at 26DA in the usual way.
L2951:;STK_F_ARG
	LD	HL,(L5C0B);DEFADD ;Point to the 1st char in the
	LD	A,(HL)		;args area and put it into A.
	CP	')'		;Is it a ')'?
	JP	Z,L28EF;V_RUN/SYN ;Jump to search the vars area.
L295A:;SFA_LOOP
	LD	A,(HL)		;Get the next arg in the loop.
	OR	60H		;Set bits 5&6, assuming a simple
	LD	B,A		;numeric var; copy it to B.
	INC	HL		;Point to the next code.
	LD	A,(HL)		;Put it into the A reg.
	CP	0EH		;Is it the 'number marker' code 0Eh?
	JR	Z,L296B;SFA_CP_VR ;Jump if so: numeric var.
	DEC	HL		;Ensure that HL points to the '$' char,
	CALL	L28AB;FN_SKPOVR	;not to a space or control code.
	INC	HL		;HL now points to the 'number marker'.
	RES	5,B		;Reset bit 5 of B: string var.
L296B:;SFA_CP_VR
	LD	A,B		;Get the var name into A.
	CP	C		;Is it the one we are looking for?
	JR	Z,L2981;SFA_MATCH ;Jump if it matches.
	INC	HL		;Now pass over the 5 bytes of
	INC	HL		;the flpnt number or
	INC	HL		;string params to get to the
	INC	HL		;next arg.
	INC	HL
	CALL	L28AB;FN_SKPOVR	;Pass on to the next char.
	CP	')'		;Is it a ')'?
	JP	Z,L28EF;V_RUN/SYN ;If so, jump to search the vars area.
	CALL	L28AB;FN_SKPOVR	;Point to the next arg.
	JR	L295A;SFA_LOOP	;Jump back to consider it.

;A match has been found. The params of a string var are stacked, avoiding
;the need to call the STK_VAR subroutine.
L2981:;SFA_MATCH
	BIT	5,C		;Test for a numeric var.
	JR	NZ,L2991;SFA_END ;Jump if the var is numeric; SCANNING
				;will stack it.
	INC	HL		;Point to the 1st of the 5 bytes to be
	LD	DE,(L5C65);STKEND ;stacked. Point DE to STKEND.
	CALL	L33C0;MOVE_FP	;Stack the 5 bytes.
	EX	DE,HL		;Point HL to the new position of STKEND,
	LD	(L5C65),HL;STKEND ;and reset the system var.
L2991:;SFA_END
	POP	DE		;Discard the LOOK_VARS pntrs (2nd & 1st
	POP	DE		;char pntrs).
	XOR	A		;Return from the search with both the
	INC	A		;carry and zero flags reset - signalling
	RET			;that a call to STK_VAR is no required.

;	THE 'STK_VAR' SUBROUTINE
;This subroutine is usually used either to find the params that define an
;existing string entry in the vars area or to return in the HL reg pair
;the base addr of a particular element or an array of numbers. When
;called from DIM the subroutine only checks the syntax of the BASIC stmt.
;  Note that the params that define a string may be altered by calling
;SLICING if this should be specified.
;  Initially the A and the B regs are cleared and bit 7 of the C reg is
;tested to determine whether syntax is being checked.
L2996:;STK_VAR
	XOR	A		;Clear the array flag.
	LD	B,A		;Clear the B reg for later.
	BIT	7,C		;Jump forward if syntax is
	JR	NZ,L29E7;SV_COUNT ;being checked.

;Next, simple strings are separated from array vars.
	BIT	7,(HL)		;Jump forward if dealing with
	JR	NZ,L29AE;SV_ARRAYS ;an array var.

;The params for a simple string are readily found.
	INC	A		;Signal 'a simple string'.
L29A1:;SV_SIMPLE$
	INC	HL		;Move along the entry.
	LD	C,(HL)		;Pick up the low length counter.
	INC	HL		;Advance the pntr.
	LD	B,(HL)		;Pick up the high length counter.
	INC	HL		;Advance the pntr.
	EX	DE,HL		;Transfer the pntr to the actual string.
	CALL	L2AB2;STK_STORE	;Pass these params to the calc stack.
	RST	18H;GET_CHAR	;Fetch the present char and jump forward
	JP	L2A49;SV_SLICE?	;to see if a 'slice' is required.

;The base addr of an element in an array is now found. Initially the
;'number of dimensions' is collected.
L29AE:;SV_ARRAYS
	INC	HL		;Step past the length bytes.
	INC	HL
	INC	HL
	LD	B,(HL)		;Collect the 'number of dimensions'.
	BIT	6,C		;Jump forward if handling an
	JR	Z,L29C0;SV_PTR	;array of numbers.

;If an array of strings has its 'number of dimensions' equal to '1' then
;such an array can be handled as a simple string.
	DEC	B		;Decrease the 'number of dimensions'
	JR	Z,L29A1;SV_SIMPLE$ ;and jump if the number is now zero.

;Next a check is made to ensure that in the BASIC line the var is followed
;by a subscript.
	EX	DE,HL		;Save the pntr in DE.
	RST	18H;GET_CHAR	;Get the present char.
	CP	'('		;Is it a '('?
	JR	NZ,L2A20;REPORT_3 ;Report the error if it is not so.
	EX	DE,HL		;Restore the pntr.

;For both numeric arrays and arrays of strings the var pntr is transferred
;to the DE reg pair before the subscript is evaluated.
L29C0:;SV_PTR
	EX	DE,HL		;Pass the pntr to DE.
	JR	L29E7;SV_COUNT	;Jump forward.

;The following loop is used to find the params of a specified element
;within an array. The loop is entered at the mid-point - SV_COUNT - where
;the element count is set to zero.
;  The loop is accessed 'B' times, this being, for a numeric array, equal
;to the number of dimensions that are being used, but for an array of
;strings 'B' is one less than the number of dimensions in use as the last
;subscript is used to specify a 'slice' of the string.
L29C3:;SV_COMMA
	PUSH	HL		;Save the counter.
	RST	18H;GET_CHAR	;Get the present char.
	POP	HL		;Restore the counter.
	CP	','		;Is the present char a ','?
	JR	Z,L29EA;SV_LOOP	;Jump forward to consider another subscript.
	BIT	7,C		;If a line is being executed
	JR	Z,L2A20;REPORT_3 ;then there is an error.
	BIT	6,C		;Jump forward if dealing with
	JR	NZ,L29D8;SV_CLOSE ;an array of strings.
	CP	')'		;Is the present char a ')'?
	JR	NZ,L2A12;SV_RPT_C ;Report an error if not so.
	RST	20H;NEXT_CHAR	;Advance CH_ADD.
	RET			;Return as the syntax is correct.

;For an array of strings the present subscript may represent a 'slice',
;or the subscript for a 'slice' may yet be present in the BASIC line.
L29D8:;SV_CLOSE
	CP	')'		;Is the present char a ')'?
	JR	Z,L2A48;SV_DIM	;Jump forward and check whether there is
				;another subscript.
	CP	0CCH		;Is the present char a 'TO'?
	JR	NZ,L2A12;SV_RPT_C ;It must not be otherwise.
L29E0:;SV_CH_ADD
	RST	18H;GET_CHAR	;Get the present char.
	DEC	HL		;Point to the preceding
	LD	(L5C5D),HL;CH_ADD ;char and set CH_ADD.
	JR	L2A45;SV_SLICE	;Evaluate the 'slice'.

;Enter the loop here.
L29E7:;SV_COUNT
	LD	HL,0		;Set the counter to zero.
L29EA:;SV_LOOP
	PUSH	HL		;Save the counter briefly.
	RST	20H;NEXT_CHAR	;Advance CH_ADD.
	POP	HL		;Restore the counter.
	LD	A,C		;Fetch the discriminator byte.
	CP	0C0H		;Jump unless checking the
	JR	NZ,L29FB;SV_MULT ;syntax for an array of strings.
	RST	18H;GET_CHAR	;Get the present char.
	CP	')'		;Is it a ')'?
	JR	Z,L2A48;SV_DIM	;Jump forward as finished counting elements.
	CP	0CCH		;Is to 'TO'?
	JR	Z,L29E0;SV_CH_ADD ;Jump back if dealing with a 'slice'.
L29FB:;SV_MULT
	PUSH	BC		;Save the dimension-number counter and the
	PUSH	HL		;discriminator byte. Save element-counter.
	CALL	L2AEE;DE,(DE+1)	;Get a dimension-size into DE.
	EX	(SP),HL		;The counter moves to HL and the var pntr is
	EX	DE,HL		;stacked. The counter moves to DE and the
				;dimension-size to HL.
	CALL	L2ACC;INT_EXP1	;Evaluate the next subscript.
	JR	C,L2A20;REPORT_3 ;Give an error if out of range.
	DEC	BC		;The result of the evaluation is decremented
				;as the counter is to count the elements
				;occuring before the specified element.
	CALL	L2AF4;GET_HL*DE	;Multiply the counter by the dimension-size.
	ADD	HL,BC		;Add the result of 'INT_EXP1' to the present
	POP	DE		;counter. Fetch the var pntr.
	POP	BC		;Fetch the dimension-number and the
				;discriminator byte.
	DJNZ	L29C3;SV_COMMA	;Keep going round the loop until
				;'B' equals zero.

;The SYNTAX/RUN flag is checked before arrays of strings are separated
;from arrays of numbers.
	BIT	7,C		;Report an error if checking
L2A12:;SV_RPT_C
	JR	NZ,L2A7A;SL_RPT_C ;syntax at this point.
	PUSH	HL		;Save the counter.
	BIT	6,C		;Jump forward if handling
	JR	NZ,L2A2C;SV_ELEM$ ;an array of strings.

;When dealing with an array of numbers the present char must be a ')'.
	LD	B,D		;Transfer the var pntr
	LD	C,E		;to the BC reg pair.
	RST	18H;GET_CHAR	;Fetch the present char.
	CP	')'		;Is it a ')'? Jump past the error
	JR	Z,L2A22;SV_NUMBER ;report unless it is needed.

;Report 3 - Subscript out of range
L2A20:;REPORT_3
	RST	8;ERROR_1	;Call the error handling
	DB	02H		;routine.

;The addr of the loc before the actual flpnt form can now be calculated.
L2A22:;SV_NUMBER
	RST	20H;NEXT_CHAR	;Advance CH_ADD.
	POP	HL		;Fetch the counter.
	LD	DE,5		;There are 5 bytes to each element
				;in an array of numbers.
	CALL	L2AF4;GET_HL*DE	;Compute the total number of bytes before
				;the required element.
	ADD	HL,BC		;Make HL point to the loc before the
	RET			;required element. Return with this addr.

;When dealing with an array of strings the length of an element is given by
;the last 'dimension-size'. The appropriate params are calculated and then
;passed to the calc stack.
L2A2C:;SV_ELEM$
	CALL	L2AEE;DE,(DE+1)	;Fetch the last dimension-size.
	EX	(SP),HL		;The var pntr goes on the stack and
				;the counter to HL.
	CALL	L2AF4;GET_HL*DE	;Multiply 'counter' by 'dimension-size'.
	POP	BC		;Fetch the var pntr.
	ADD	HL,BC		;This gives HL pointing to the loc
				;before the string.
	INC	HL		;So point to the actual 'start'.
	LD	B,D		;Transfer the last dimension-size
	LD	C,E		;to BC to form the 'length'.
	EX	DE,HL		;Move the 'start' to DE.
	CALL	L2AB1;STK_ST_0	;Pass these params to the calc stack. Note:
				;The 1st param is zero indicating a string
				;from an 'array of strings' and hence the
				;existing entry is not to be reclaimed.

;There are three possible forms of the last subscript. The 1st is illustrated
;by - A$(2,4 TO 8), the 2nd by - A$(2)(4 TO 8), and the 3rd by - A$(2) -
;which is the default form and indicates that the whole string is required.
	RST	18H;GET_CHAR	;Get the present char.
	CP	')'		;Is it a ')'?
	JR	Z,L2A48;SV_DIM	;Jump if it is so.
	CP	','		;Is it a ','?
	JR	NZ,L2A20;REPORT_3 ;Report the error if not so.
L2A45:;SV_SLICE
	CALL	L2A52;SLICING	;Use SLICING to modify the set of params.
L2A48:;SV_DIM
	RST	20H;NEXT_CHAR	;Fetch the next char.
L2A49:;SV_SLICE?
	CP	'('		;Is it a '('? Jump back if there is
	JR	Z,L2A45;SV_SLICE ;a 'slice' to be considered.

;When finished considering the last subscript a return can be made.
	RES	6,(IY+1);FLAGS	;Signal - string result.
	RET			;Return with the params of the required
				;string forming a 'last value' on the
				;calc stack.

