
;	THE 'ALPHANUM' SUBROUTINE
;This subroutine returns with the carry flag set if the present value of
;A reg denotes a valid digit or letter.
L2C88:;ALPHANUM
	CALL	L2D1B;NUMERIC	;Test for a digit; carry will be reset for
	CCF			;a digit. Complement the carry flag.
	RET	C		;Return if a digit; otherwise continue
				;on into 'ALPHA'.

;	THE 'ALPHA' SUBROUTINE
;This subroutine returns with the carry flag set if the present value of
;the A reg denotes a valid letter of the alphabet.
L2C8D:;ALPHA
	CP	'A'		;Test against 41h, the code for 'A'.
	CCF			;Complement the carry flag.
	RET	NC		;Return if not a valid char code.
	CP	5BH		;Test against 5Bh, 1 more than the code
	RET	C		;for 'Z'. Return if an upper case letter.
	CP	61H		;Test against 61h, the code for 'a'.
	CCF			;Complement the carry flag.
	RET	NC		;Return if not a valid char code.
	CP	7BH		;Test against 7Bh, 1 more than the code
	RET			;for 'z'. Finished.

;	THE 'DECIMAL TO FLOATING-POINT' SUBROUTINE
;As part of syntax checking decimal numbers that occur in a BASIC line are
;converted to their flpnt forms. This subroutine reads the decimal number
;digit by digit and gives its result as a 'last value' on the calc stack.
;But 1st it deals with the alternative notation BIN, which introduces a
;sequence of 0's and 1's giving the binary representation of the
;required number.
L2C9B:;DEC_TO_FP
	CP	0C4H		;Is the char a 'BIN'?
	JR	NZ,L2CB8;NOT_BIN ;Jump if it is not 'BIN'.
	LD	DE,0		;Initialise result to zero in DE.
L2CA2:;BIN_DIGIT
	RST	20H;NEXT_CHAR	;Get the next char.
	SUB	'1'		;Subtract the char code for '1'.
	ADC	A,0		;0 now gives 0 with carry set; 1 gives 0
				;with carry reset.
	JR	NZ,L2CB3;BIN_END ;Any other char causes a jump to BIN_END
				;and will be checked for syntax during or
				;after scanning.
	EX	DE,HL		;Result so far to HL now.
	CCF			;Complement the carry flag.
	ADC	HL,HL		;Shift the result left, with the carry
				;going to bit 0.
	JP	C,L31AD;REPORT_6 ;Report overflow if more than 65535.
	EX	DE,HL		;Return the result so far to DE.
	JR	L2CA2;BIN_DIGIT	;Jump back for next 0 or 1.
L2CB3:;BIN_END
	LD	B,D		;Copy result to BC for stacking.
	LD	C,E
	JP	L2D2B;STACK_BC	;Jump forward to stack the result.

;For other numbers, 1st any integer part is converted; if the nexti char is
;a decimal, then the decimal fraction is considered.
L2CB8:;NOT_BIN
	CP	'.'		;Is the 1st char a '.'?
	JR	Z,L2CCB;DECIMAL	;If so, jump forward.
	CALL	L2D3B;INT_TO_FP	;Otherwise, form a 'last value'
				;of the integer.
	CP	'.'		;Is the next char a '.'?
	JR	NZ,L2CEB;E_FORMAT ;Jump forward to see if it is an 'E'.
	RST	20H;NEXT_CHAR	;Get the next char.
	CALL	L2D1B;NUMERIC	;Is it a digit?
	JR	C,L2CEB;E_FORMAT ;Jump if not (e.g. 1.E4 is allowed).
	JR	L2CD5;DEC_STO_1	;Jump forward to deal with the
				;digits after the decimal point.
L2CCB:;DECIMAL
	RST	20H;NEXT_CHAR	;If the number started with a decimal,
	CALL	L2D1B;NUMERIC	;see if the next char is a digit.
L2CCF:;DEC_RPT_C
	JP	C,L1C8A;REPORT_C ;Report the error if it is not.
	RST	28H;FP_CALC	;Use the calc to stack zero
	DB	0A0H;stk_zero	;as the integer part of such
	DB	38H;end_calc	;numbers.
L2CD5:;DEC_STO_1
	RST	28H;FP_CALC	;Use the calc again.
	DB	0A1H;stk_one	;Find the flpnt form of
	DB	0C0H;st_mem_0	;the decimal number '1', and
	DB	02H;delete	;save it in the memory area.
	DB	38H;end_calc
L2CDA:;NXT_DGT_1
	RST	18H;GET_CHAR	;Get the present char.
	CALL	L2D22;STK_DIGIT	;If it is a digit then stack it.
	JR	C,L2CEB;E_FORMAT ;If not jump forward.
	RST	28H;FP_CALC	;Now use the calc.
	DB	0E0H;get_mem_0	;For each passage of the loop,
	DB	0A4H;stk_ten	;the number saved in the memory
	DB	05H;division	;are is fetched, divided by 10
	DB	0C0H;st_mem_0	;and restored; i.e. going from .1
				;to .01 to .001 etc.
	DB	04H;multiply	;The present digit is multiplied
	DB	0FH;addition	;by the 'saved number' and
	DB	38H;end_calc	;added to the 'last value'.
	RST	20H;NEXT_CHAR	;Get the next char.
	JR	L2CDA;NXT_DGT_1	;Jump back (one more byte than needed)
				;to consider it.

;Next consider any 'E notation', i.e. the form xEm or xem where m is a
;positive or negative integer.
L2CEB:;E_FORMAT
	CP	'E'		;Is the present char an 'E'?
	JR	Z,L2CF2;SIGN_FLAG ;Jump forward if it is.
	CP	65H		;Is it an 'e'?
	RET	NZ		;Finished unless it is so.
L2CF2:;SIGN_FLAG
	LD	B,0FFH		;Use B as a sign flag, FF for '+'.
	RST	20H;NEXT_CHAR	;Get the next char.
	CP	'+'		;Is it a '+'?
	JR	Z,L2CFE;SIGN_DONE ;Jump forward.
	CP	'-'		;Is it a '-'?
	JR	NZ,L2CFF;ST_E_PART ;Jump if neither '+' not '-'.
	INC	B		;Change the sign of the flag.
L2CFE:;SIGN_DONE
	RST	20H;NEXT_CHAR	;Point to the 1st digit.
L2CFF:;ST_E_PART
	CALL	L2D1B;NUMERIC	;Is it indeed a digit?
	JR	C,L2CCF;DEC_RPT_C ;Report the error if not.
	PUSH	BC		;Save the flag in B briefly.
	CALL	L2D3B;INT_TO_FP	;Stack ABS m, where m is the exponent.
	CALL	L2DD5;FP_TO_A	;Transfer ABS m to A.
	POP	BC		;Restore the sign flag to B.
	JP	C,L31AD;REPORT_6 ;Report the overflow now if ABS m is
	AND	A		;greater than 255 or indeed greater than
	JP	M,L31AD;REPORT_6 ;127 (other values greater than about 39
				;will be detected later).
	INC	B		;Test the sign flag in B; '+' (i.e. 0FFh)
				;will now set the zero flag.
	JR	Z,L2D18;E_FP_JUMP ;Jump if sign of m is '+'.
	NEG			;Negate m if sign is '-'.
L2D18:;E_FP_JUMP
	JP	L2D4F;E_TO_FP	;Jump to assign to the 'last value'
				;the result of x*10^m.

;	THE 'NUMERIC' SUBROUTINE
;This subroutine returns with the carry flag reset if the present value
;of the A reg denotes a valid digit.
L2D1B:;NUMERIC
	CP	'0'		;Test against 30h, the code for '0'.
	RET	C		;Return if not a valid char code.
	CP	':'		;Test against the upper limit.
	CCF			;Complement the carry flag.
	RET			;Finished.

;	THE 'STK_DIGIT' SUBROUTINE
;This subroutine simply returns if the current value held in the A reg does
;not represent a digit but if it does then the flpnt form for the digit
;becomes the 'last value' on the calc stack.
L2D22:;STK_DIGIT
	CALL	L2D1B;NUMERIC	;Is the char a digit?
	RET	C		;Return if not in range.
	SUB	'0'		;Replace the code by the actual digit.

;	THE 'STACK_A' SUBROUTINE
;This subroutine gives the flpnt form for the absolute binary value
;currently held in the A reg.
L2D28:;STACK_A
	LD	C,A		;Transfer the value to the C reg.
	LD	B,0		;Clear the B reg.

;	THE 'STACK_BC' SUBROUTINE
;This subroutine gives the flpnt form for the absolute binary value
;currently held in the BC reg pair.
;  The form used in this and hence in the two previous subroutines as well
;is the one reserved in the Spectrum for small integers n, where
;-65535<=n<=65535. The 1st and 5th bytes are zero; the 3rd and 4th bytes
;are the less significant and more significant bytes of the 16 bit integer
;n in two's complement form (if n is negative, these two bytes hold
;65536+n); and the 2nd byte is a sign byte, 0 for '+' and FF for '-'.
L2D2B:;STACK_BC
	LD	IY,L5C3A;ERR_NR	;Re-initialise IY.
	XOR	A		;Clear the A reg.
	LD	E,A		;And the E reg, to indicate '+'.
	LD	D,C		;Copy the less significant byte to D.
	LD	C,B		;And the more significant byte to C.
	LD	B,A		;Clear the B reg.
	CALL	L2AB6;STK_STORE	;Now stack the number.
	RST	28H;FP_CALC	;Make HL point to
	DB	38h;end_calc	;STKEND-5.
	AND	A		;Clear the carry flag.
	RET			;Finished.

;	THE 'INTEGER TO FLOATING-POINT' SUBROUTINE
;This subroutine returns a 'last value' on the calc stack that is the result
;of converting an integer in a BASIC line, i.e. the integer part of the
;decimal number or the line number, to its flpnt form.
;  Repeated calls to CH_ADD+1 fetch each digit of the integer in turn. An
;exit is made when a code that does not represent a digit has been fetched.
L2D3B:;INT_TO_FP
	PUSH	AF		;Save the 1st digit - in A.
	RST	28H;FP_CALC	;Use the calc.
	DB	0A0H;stk_zero	;The 'last value' is now zero.
	DB	38H;end_calc
	POP	AF		;Restore the 1st digit.

;Now a loop is set up. As long as the code represents a digit then the flpnt
;form is found and stacked under the 'last value'. The 'last value' is then
;multiplied by decimal 10 and added to the 'digit' to form a new 'last
;value' which is carried back to the start of the loop.
L2D40:;NXT_DGT_2
	CALL	L2D22;STK_DIGIT	;If the code represents a digit then stack
	RET	C		;the flpnt form.
	RST	28H;FP_CALC	;Use the calc.
	DB	01H;exchange	;'Digit' goes under 'last value'.
	DB	0A4H;stk_ten	;Define decimal 10.
	DB	04H;multiply	;'Last value'='last value'*10
	DB	0FH;addition	;'Last value'='last value'+'digit'
	DB	38H;end_calc
	CALL	L0074;CH_ADD+1	;The next code goes into A.
	JR	L2D40;NXT_DGT_2	;Loop back with this code.

