

;	THE FLOATING-POINT CALCULATOR

;	THE TABLE OF CONSTANTS
;This first table holds the five useful and frequently needed numbers zero,
;one, a half, a half of pi and ten. The numbers are held in a condensed form
;which is expanded by the STACK LITERALS subroutine, see below, to give
;the required floating-point form.
L32C5:;stk_zero			when expanded gives:
	DB	00H;		00 00 00 00 00
	DB	0B0H
	DB	00H
L32C8:;stk_one
	DB	40H;		00 00 01 00 00
	DB	0B0H
	DB	00H
	DB	01H
L32CC:;stk_half
	DB	30H;		80 00 00 00 00
	DB	00H
L32CE:;stk_pi/2
	DB	0F1H;		81 49 0F DA A2
	DB	49H
	DB	0FH
	DB	0DAH
	DB	0A2H
L32D3:;stk_ten
	DB	40H;		00 00 0A 00 00
	DB	0B0H
	DB	00H
	DB	0AH

;	THE TABLE OF ADDRESSES:
;This second table is a look-up table of the addrs of the 66 operational
;subroutines of the calculator. The offsets used to index into the table are
;derived either from the operation codes used in SCANNING, see 2734, etc.,
;or from the literals that follow a RST 28 instruction.
L32D7:
	DW	L368F			;00 jump_true
	DW	L343C			;01 exchange
	DW	L33A1			;02 delete
	DW	L300F			;03 subtract
	DW	L30CA			;04 multiply
	DW	L31AF			;05 division
	DW	L3851			;06 to_power
	DW	L351B			;07 or
	DW	L3524			;08 no_&_no
	DW	L353B			;09 no_l_eql
	DW	L353B			;0A no_gr_eq
	DW	L353B			;0B nos_neql
	DW	L353B			;0C no_grtr
	DW	L353B			;0D no_less
	DW	L353B			;0E nos_eql
	DW	L3014			;0F addition
	DW	L352D			;10 str_&_no
	DW	L353B			;11 str_l_eql
	DW	L353B			;12 str_gr_eq
	DW	L353B			;13 strs_neql
	DW	L353B			;14 str_grtr
	DW	L353B			;15 str_less
	DW	L353B			;16 strs_eql
	DW	L359C			;17 strs_add
	DW	L35DE			;18 val$
	DW	L34BC			;19 usr$
	DW	L3645			;1A read_in
	DW	L346E			;1B negate
	DW	L3669			;1C code
	DW	L35DE			;1D val
	DW	L3674			;1E len
	DW	L37B5			;1F sin
	DW	L37AA			;20 cos
	DW	L37DA			;21 tan
	DW	L3833			;22 asn
	DW	L3843			;23 acs
	DW	L37E2			;24 atn
	DW	L3713			;25 ln
	DW	L36C4			;26 exp
	DW	L36AF			;27 int
	DW	L384A			;28 sqr
	DW	L3492			;29 sgn
	DW	L346A			;2A abs
	DW	L34AC			;2B peek
	DW	L34A5			;2C in
	DW	L34B3			;2D usr_no
	DW	L361F			;2E str$
	DW	L35C9			;2F chr$
	DW	L3501			;30 not
	DW	L33C0			;31 duplicate
	DW	L36A0			;32 n_mod_m
	DW	L3686			;33 jump
	DW	L33C6			;34 stk_data
	DW	L367A			;35 dec_jr_nz
	DW	L3506			;36 less_0
	DW	L34F9			;37 greater_0
	DW	L369B			;38 end_calc
	DW	L3783			;39 get_argt
	DW	L3214			;3A truncate
	DW	L33A2			;3B fp_calc_2
	DW	L2D4F			;3C e_to_fp
	DW	L3297			;3D re_stack
	DW	L3449			;3E series-06 etc.
	DW	L341B			;3F stk_zero etc
	DW	L342D			;40 st_mem_0 etc.
	DW	L340F			;41 get_mem_0 etc.

;Note: The last four subroutines are multi-purpose subroutine and are
;entered with a parameter that is a copy of the right hand five bits of the
;original literal. The full set follows:
;Offset 3E:	series-06, series-08 & series-0C; literals 86, 88 & 8C.
;Offset 3F:	stk_zero, stk_one, stk_half, stk_pi/2 & stk_ten;
		literals A0-A4.
;Offset 40:	st_mem_0, ..., st_mem_5; literals C0-C5.
;Offset 41:	get_mem_0, ..., get_mem_5; literals E0-E5.



;	THE 'CALCULATE' SUBROUTINE
;This subroutine  is used to perform floating-point calculations. These can be
;considered to be of three types:
;i.	Binary operations, e.g. addition, where two numbers in floating-point
;	form are added together to give one 'last value'.
;ii.	Unary operations, e.g. sin, where the 'last value' is changed to give
;	the appropriate function result as a new 'last value'.
;iii.	Manipulatory operation, e.g. st_mem_0, where the 'last value' is
;	copied to the first five bytes of the calculator's memory area.
;
;The operations to be performed are specified as a series of data-bytes, the
;literals, that follow an RST 28 instruction that calls this subroutine. The
;last literal in the list is always '38' which leads to an end to the whole
;operation.
;  In the case of a single operation needing to be performed, the operation
;offset can be passed to the CALCULATOR in the B reg, and operation '3B', the
;SINGLE CALCULATION operation, performed.
;  It is also possible to call this subroutine recursively, i.e. from within
;itself, and in such a case it is possible to use the system variable BREG
;as a counter that controls how many operations are performed before returning.
;  The first part of this subroutine is complicated but essentially it performs
;the two tasks of setting the regs to hold their required values, and to
;produce an offset, and possibly a parameter, from the literal that is
;currently being considered.
;  The offset is used to index into the calculator's table of addrs, see above,
;to find the required subroutine addr.
;  The parameter is used when the multi-purpose subroutines are called.
;
;Note: A floating-point number may in reality be a set of string parameters.
;
L335B:;CALCULATE
	CALL	L35BF;STK_PNTRS	;Presume a unary operation and therefore
				;set HL to point to the start of the 'last
				;value' on the calc stack and DE one-past
				;this floating-point number (STKEND).
L335E:;GEN_ENT_1
	LD	A,B		;Either, transfer a single operation offset
	LD	(L5C67),A;BREG	;to BREG temporarily, or, when using the
				;subroutine recursively pass the parameter
				;to BREG to be used as a counter.
L3362:;GEN_ENT_2
	EXX			;The return addr of the subroutine is store
	EX	(SP),HL		;in H'L'. This saves the pointer to the first
	EXX			;literal. Entering the CALCULATOR at GEN_ENT_2
				;is used whenever BREG is in use as a counter
				;and is not to be disturbed.
L3365:;RE_ENTRY
	LD	(L5C65),DE;STKEND ;A loop is now entered to handle each
				;literal in the list that follows the calling
				;instruction; so first, always set STKEND.
	EXX			;Go to the alternate reg set,
	LD	A,(HL)		;and fetch the literal for this loop.
	INC	HL		;Make H'L' point to the next literal.
L336C:;SCAN_ENT
	PUSH	HL		;This pointer is saved briefly on the machine
				;stack. SCAN_ENT is used by the SINGLE
				;CALCULATION subroutine to find the subroutine
				;that is required.
	AND	A		;Test the A reg.
	JP	P,L3380;FIRST_3D;Separate the simple literals from the multi-
				;purpose literals. Jump with literals 00-3D.
	LD	D,A		;Save the literal in D.
	AND	60H		;Continue only with bits 5 & 6.
	RRCA			;Four right shifts make them
	RRCA			;now bits 1 & 2.
	RRCA
	RRCA
	ADD	A,7CH		;The offsets required are 3E-41 and L
	LD	L,A		;will now hold double the required offset.
	LD	A,D		;Now produce the parameter by taking bits
	AND	1FH		;0, 1, 2, 3 & 4 of the literal; keep the
				;parameter in A.
	JR	L338E;ENT_TABLE	;Jump forward to find the addr of the
				;required subroutine.
L3380:;FIRST_3D
	CP	18H		;Jump forward if performing a
	JR	NC,L338C;DOUBLE_A ;unary operation.
	EXX			;All of the subroutines that perform
	LD	BC,0FFFBH	;binary operations require
	LD	D,H		;that HL points to the first operand
	LD	E,L		;and DE points to the second
	ADD	HL,BC		;operand (the 'last value') as they
	EXX			;appear on the calc stack.
L338C:;DOUBLE_A
	RLCA			;As each entry in the table of addrs takes
	LD	L,A		;up two bytesthe offset produced is doubled.
L338E:;ENT_TABLE
	LD	DE,L32D7	;The base addr of the table.
	LD	H,0		;The addr of the required
	ADD	HL,DE		;table entry if formed in HL; and
	LD	E,(HL)		;the required subrouine addr
	INC	HL		;is loaded into the DE reg pair.
	LD	D,(HL)
	LD	HL,L3365	;The RE_ENTRY addr of 3365 is put on the
	EX	(SP),HL		;machine stack underneath
	PUSH	DE		;the subroutine addr.
	EXX			;Return to the main set of regs.
	LD	BC,(L5C66);STKEND_hl ;The current value of BREG is transferred
				;to the B reg thereby returning the single
				;operation offset.
				;(See COMPARISON at 353B).
L33A1:;delete
	RET			;An indirect jump to the required subroutine.

;	THE 'DELETE' SUBROUTINE
;(Offset 02: 'delete')
;This subroutine contains only a single RET instruction at 33A1, above.
;The literal '02' results in this subroutine being considered as a binary
;operation that is to be entered with a first number addressed by the HL
;reg pair and a second number addressed by the DE reg pair, and the result
;produced again addressed by the HL reg pair.
;  The single RET instruction thereby leads to the first number being
;considered as the resulting 'last value' and the second number considered
;as being deleted. Of course the number has not been deleted from the memory
;but remains inactive and will probably soon be overwritten.

;	THE 'SINGLE OPERATION' SUBROUTINE
;(Offset 3B: 'fp_calc_2')
;This subroutine is only called from SCANNING at 2757H and is used to perform
;a single arithmetic operation. The offset that specifies which operation is
;to be performed is supplied to the calculator in the B reg and subsequently
;transferred to the system variable BREG.
;  The effect of calling this subroutine is essentially to make a jump to the
;appropriate subroutine for the single operation.
L33A2:;FP_CALC_2
	POP	AF		;Discard the RE_ENTRY addr.
	LD	A,(L5C67);BREG	;Transfer the offset to A.
	EXX			;Enter the alternate reg set.
	JR	L336C;SCAN_ENT	;Jump back to find the required addr; stack
				;the RE_ENTRY addr and jump to the subroutine
				;for the operation.

;	THE 'TEST 5_SPACES' SUBROUTINE
;This subroutine tests whether there is sufficient room in memory for another
;5-byte floating-point number to be added to the calculator stack.
L33A9:;TEST_5_SP
	PUSH	DE		;Save DE briefly.
	PUSH	HL		;Save HL briefly.
	LD	BC,5		;Specifyx the test if for 5 bytes.
	CALL	L1F05;TEST_ROOM	;Make the test.
	POP	HL		;Restore HL.
	POP	DE		;Restore DE.
	RET			;Finished.

;	THE 'STACK NUMBER' SUBROUTINE
;This subroutine is called by BEEP and by SCANNING twice to copy STKEND to DE,
;move a floating-point number to the calculator stack, and reset STKEND from
;DE. It calls 'MOVE_FP' to do the actual move.
L33B4:;STACK_NUM
	LD	DE,(L5C65);STKEND ;Copy STKEND to DE as destination addr.
	CALL	L33C0;MOVE_FP	;Move the number.
	LD	(L5C65),DE;STKEND ;Reset STKEND from DE.
	RET			;Finished.

;	THE 'MOVE A FLOATING-POINT NUMBER' SUBROUTINE
;(Offset 31:'duplicate')
;This subroutine moves a floating-point number to the top of the calculator
;stack (3 cases) or from the top of the stack to the calculator's memory area
;(1 case). It is also called through the calculator when it simply duplicates
;the number at the top of the calculator stack, the 'last value', thereby
;extending the stack by 5 bytes.
L33C0:;MOVE_FP
	CALL	L33A9;TEST_5_SP	;A test is made for room.
	LDIR			;Move the five bytes involved.
	RET			;Finished.

;	THE 'STACK LITERALS' SUBROUTINE
;(Offset 34:'stk_data')
;This subroutine places on the calc stack, as a 'last value', the floating
;point number supplied to it as 2, 3, 4 or 5 literals.
;  When called by using offset '34' the literals follow the '34' in the list
;of literals; when called by the SERIES GENERATOR, see below, the literals
;are supplied by the subroutine that called for a series to be generated;
;and when calling by SKIP CONSTANTS & STACK A CONSTANT the literals are
;obtained from the calc's table of consts (32C5-32D6).
;  In each case, the first literal supplied is divided by Hex 40, and the
;integer quotient plus 1 determines whether 1, 2, 3 or 4 further literals will
;be taken from the source to form the mantissa of the number. Any unfilled
;bytes of the 5 bytes that go to form a 5-byte floating-point number are set
;to zero. The first literal is also used to determine the exponent, after
;reducing mod Hex 40, unless the remainder is zero, in which case the second
;literal is used, as it stands, without reducing mod Hex 40. In either case,
;Hex 50 is added to the literal, giving the augmented exponent byte, e (the
;true exponent e' plus Hex 80). The rest of the 5 bytes are stacked,
;including any zeros needed, and the subroutine returns.
L33C6:;STK_DATA
	LD	H,D		;This subroutine performs the manipulatory
	LD	L,E		;operation of adding a 'last value' to the
				;calc stack; hence HL is set to point one-past
				;the present 'last value' and hence point
				;to the result.
L33C8:;STK_CONST
	CALL	L33A9;TEST_5_SP	;Now test that there is indeed room.
	EXX			;Go to the alternate reg set
	PUSH	HL		;and stack the pointer to the
	EXX			;next literal.
	EX	(SP),HL		;Switch over the result pointer and the next
	PUSH	BC		;literal pointer. Save BC briefly.
	LD	A,(HL)		;The first literal is put into A
	AND	0C0H		;and divided by Hex 40 to give
	RLCA			;the integer values 0, 1, 2 or 3.
	RLCA
	LD	C,A		;The integer value is transferred to C and
	INC	C		;incremented, thereby giving the range 1, 2,
				;3 or 4 for the number of literals that will
				;be needed.
	LD	A,(HL)		;The literal is fetch anew,
	AND	3FH		;reduced mod Hex 40 and discarded
	JR	NZ,L33DE;FORM_EXP ;as inappropriate if the
	INC	HL		;remainder is zero; in which case the next
	LD	A,(HL)		;literal is fetched and used unreduced.
L33DE:;FORM_EXP
	ADD	A,50H		;The exponent, e, is formed by the addition of
	LD	(DE),A		;Hex 50 and passed to the calc stack as the
				;first of the 5 bytes of the result.
	LD	A,5		;The number of literals specified
	SUB	C		;in C are taken from the source
	INC	HL		;and entered into the bytes of
	INC	DE		;the result.
	LD	B,0
	LDIR
	POP	BC		;Restore BC.
	EX	(SP),HL		;Return the result pointer to HL
	EXX			;and the next literal pointer to
	POP	HL		;its usual position in H' & L'.
	EXX
	LD	B,A		;The number of zero bytes
	XOR	A		;required at this stage is given by
L33F1:;STK_ZEROS
	DEC	B		;5_C_1; and this number of zeros
	RET	Z		;is added to the result to make
	LD	(DE),A		;up the required five bytes.
	INC	DE
	JR	L33F1;STK_ZEROS

;	THE 'SKIP CONSTANTS' SUBROUTINE
;This subroutine is entered with the HL reg pair holding the base addr of the
;calc's table of consts and the A reg holding a parameter that shows which of
;the 5 consts is being requested.
;  The subroutine performs the null operations of loading the 5 bytes of each
;unwanted constant into the locations 0000 thru 0004 at the beginning of the
;ROM until the requested const is reached.
;  The subroutine returns with the HL reg pair holding the base addr of the
;requested const within the table of consts.
L33F7:;SKIP_CONS
	AND	A		;The subroutine returns if the parameter is
L33F8:;SKIP_NEXT
	RET	Z		;zero, or when the requested const has
				;been reached.
	PUSH	AF		;Save the parameter.
	PUSH	DE		;Save the result pointer.
	LD	DE,0		;The dummy addr.
	CALL	L33C8;STK_CONST	;Perform imaginary stacking of an
				;expanded const.
	POP	DE		;Restore the result pointer.
	POP	AF		;Restore the parameter.
	DEC	A		;Count the loops.
	JR	L33F8;SKIP_NEXT

;	THE 'MEMORY LOCATION' SUBROUTINE
;This subroutine finds the base address for each five byte portion of
;the calculator's memory area to or from which a fl-pnt number is to be
;moved from or to the calc's stack. It does this operation by adding
;five times the parameter supplied to the base addr for the area which
;is held in the HL reg pair.
;  Note that when a FOR-NEXT variable is being handled then the pointers
;are changed so that the variable is treated as if it were the calc's
;memory area (see addr 1D20).
L3406:;LOC_MEM
	LD	C,A		;Copy the parameter to C.
	RLCA			;Double the parameter.
	RLCA			;Double the result.
	ADD	A,C		;Add the value of the parameter to give
				;five times the original value.
	LD	C,A		;This result is wanted in the
	LD	B,0		;BC reg pair.
	ADD	HL,BC		;Produce the new base addr.
	RET			;Finished.

;	THE 'GET FROM MEMORY AREA' SUBROUTINE
;(Offsets E0 to E5: 'get_mem_0' to 'get_mem_5')
;This subroutine is called using the literals E0 to E5 and the parameter
;derived from these literals is held in the A reg. The subroutine calls
;MEMORY LOCATION to put the required source addr into the HL reg pair
;and MOVE A FLOATING-POINT NUMBER to copy the five bytes involved from
;the calc's memory area to the top of calc stack to form a new 'last value'.
L340F:;get_mem_0 ...
	PUSH	DE		;Save the result pointer.
	LD	HL,(L5C68);MEM	;Fetch the pointer to the current
				;memory area (see above).
	CALL	L3406;LOC_MEM	;The base addr is found.
	CALL	L33C0;MOVE_FP	;The five bytes are moved.
	POP	HL		;Set the result pointer.
	RET			;Return.

;	THE 'STACK A CONSTANT' SUBROUTINE
;(Offsets A0 to A4: 'stk_zero', 'stk_one', 'stk_half', 'stk_pi/2' & 'stk_ten')
;This subroutine uses SKIP CONSTANTS to find the base addr of the
;requested constants from the calc's table of constants and then calls
;STACK LITERALS, entering at STK_CONST, to make the expanded form of the
;constant the 'last value' on the calc stack.
L341B:;stk_zero  ...
	LD	H,D		;Set HL to hold the result pointer.
	LD	L,E
	EXX			;Go to the alternative reg set
	PUSH	HL		;and save the next literal pointer.
	LD	HL,L32C5	;The base addr of the calc's table
				;of constants.
	EXX			;Back to the main set of regs.
	CALL	L33F7;SKIP_CONS	;Find the requested base addr.
	CALL	L33C8;STK_CONST	;Expand the constant.
	EXX
	POP	HL		;Restore the next literal pointer.
	EXX
	RET			;Finished.

;	THE 'STORE IN MEMORY AREA' SUBROUTINE
;(Offsets C0 to C5: 'st_mem_0' to 'st_mem_5')
;This subroutine is called using the literals C0 to C5 and the parameter
;derived from these literals is held in the A reg. This subroutine is very
;similar to the GET FROM MEMORY subroutine but the source and destination
;pointers are exchanged.
L342D:;st_mem_0 ...
	PUSH	HL		;Save the result pointer.
	EX	DE,HL		;Source to DE briefly.
	LD	HL,(L5C68);MEM	;Fetch the pnt to the current memory area.
	CALL	L3406;LOC_MEM	;The base addr is found.
	EX	DE,HL		;Exchange source & destination pnts.
	CALL	L33C0;MOVE_FP	;The five bytes are moved.
	EX	DE,HL		;'Last value'+5, i.e. STKEND, to DE.
	POP	HL		;Result pnt to HL.
	RET			;Finished.
;Note that the pnts HL & DE remain as they were, pointing to STKEND-5
;and STKEND respectively, so that the 'last value' remains on the
;calc stack. If required it can be removed by using 'delete'.

;	THE 'EXCHANGE' SUBROUTINE
;(Offset 01: 'exchange')
;This binary operation 'exchanges' the first number with the second
;number, i.e. the topmost two numbers on the calc stack are exchanged.
L343C:;EXCHANGE
	LD	B,5		;There are five bytes involved.
L343E:;SWAP_BYTE
	LD	A,(DE)		;Each byte of the second number.
	LD	C,(HL)		;Each byte of the first number.
	EX	DE,HL		;Switch source & destination.
	LD	(DE),A		;Now to the first number.
	LD	(HL),C		;Now to the second number.
	INC	HL		;Move to consider the next pair
	INC	DE		;  of bytes.
	DJNZ	L343E;SWAP_BYTE	;Exchange the five bytes.
	EX	DE,HL		;Get the pointers correct as the number
				;5 is an odd number.
	RET			;Finished.

