
;	THE 'STR$' FUNCTION
;(Offset 2E: 'str$')
;This subroutine handles the function STR$ X and returns a 'last value'
;which is a set of params that define a string containing what would
;appear on the screen if X were displayed by a PRINT command.
L361F:;str$
	LD	BC,1		;One space is made in the work
	RST	30H;BC_SPACES	;space and its addr is copied to
	LD	(L5C5B),HL;K_CUR ;K_CUR, the addr of the cursor.
	PUSH	HL		;This addr is saved on the stack too.
	LD	HL,(L5C51);CURCHL ;The curr channel addr is
	PUSH	HL		;saved on the machine stack.
	LD	A,0FFH		;Channel 'R' is opened, allowing the
	CALL	L1601;CHAN_OPEN	;string to be 'printed' out into the
				;work space.
	CALL	L2DE3;PRINT_FP	;The 'last value', X, is now printed
				;out in the work space and the work
				;space is expanded with each char.
	POP	HL		;Restore CURCHL to HL and restore the
	CALL	L1615;CHAN_FLAG	;flags that are appropriate to it.
	POP	DE		;Restore the starting addr of the string.
	LD	HL,(L5C5B);K_CUR ;Now the cursor addr is one past the
	AND	A		;end of the string and hence the
	SBC	HL,DE		;difference is the length.
	LD	B,H		;Transfer the length to BC.
	LD	C,L
	CALL	L2AB2;STK_STO_$	;Pass the params of the new string to
				;the calc stack.
	EX	DE,HL		;Reset the pntrs.
	RET			;Finished.
;Note: See PRINT_FP for an explantation of the 'PRINT "A"+STR$ 0.1' error.

;	THE 'READ_IN' SUBROUTINE
;(Offset 1A: 'read_in')
;This subroutine is called via the calculator offset through the 1st line
;of the S_INKEY$ routine in SCANNING. It appears to provide for the reading
;in of data through different streams from those available on the standard
;Spectrum. Like INKEY$ the subroutine returns a string.
L3645:;read_in
	CALL	L1E94;FIND_INT1	;The numerical param is compressed
				;into the A reg.
	CP	10H		;Is it smaller than 16 dec?
	JP	NC,L1E9F;REPORT_B ;If not, report the error.
	LD	HL,(L5C51);CURCHL ;The curr channel addr is saved
	PUSH	HL		;on the machine stack.
	CALL	L1601;CHAN_OPEN	;The channel specified by the
				;param is opened.
	CALL	L15E6;INPUT_AD	;The signal is now accepted,
				;like a 'key_value'.
	LD	BC,0		;The default length of the resulting
				;string is zero.
	JR	NC,L365F;R_I_STORE ;Jump if there was no signal.
	INC	C		;Set the length to 1 now.
	RST	30H;BC_SPACES	;Make a space in the work space.
	LD	(DE),A		;Put the string into it.
L365F:;R_I_STORE
	CALL	L2AB2;STK_STO_$	;Pass the params of the string
				;to the calc stack.
	POP	HL		;Restore CURCHL and the
	CALL	L1615;CHAN_FLAG	;appropriate flags.
	JP	L35BF;STK_PNTRS	;Exit, setting the pntrs.

;	THE 'CODE' FUNCTION
;(Offset 1C: 'code')
;This subroutine handles the function CODE A$ and returns the Spectrum
;code of the 1st char in A$, or zero if A$ should be null.
L3669:;code
	CALL	L2BF1;STK_FETCH	;The params of the string are fetched.
	LD	A,B		;The length is tested and the A reg
	OR	C		;holding zero is carried forward
	JR	Z,L3671;STK_CODE ;if A$ is a null string.
	LD	A,(DE)		;The code of the 1st char is put
				;into A otherwise.
L3671:;STK_CODE
	JP	L2D28;STACK_A	;The subroutine exits via STACK_A
				;which gives the correct 'last value'.

;	THE 'LEN' FUNCTION
;(Offset 1E: 'len')
;This subroutine handles the function LEN A$ and returns a 'last value'
;that is equal to the length of the string.
L3674:;len
	CALL	L2BF1;STK_FETCH	;The params of the string are fetched.
	JP	L2D2B;STACK_BC	;The subroutine exits via STACK_BC which
				;gives the correct 'last value'.

;	THE 'DECREASE THE COUNTER' SUBROUTINE
;(Offset 35: 'dec_jr_nz')
L367A:;dec_jr_nz
	EXX			;Go to the alternative reg set and
	PUSH	HL		;save the next literal pntr on the
				;machine stack.
	LD	HL,L5C67	;Make HL point to BREG.
	DEC	(HL)		;Decrease BREG.
	POP	HL		;Restore the next literal pntr.
	JR	NZ,L3687;JUMP_2	;The jump is made on non-zero.
	INC	HL		;The next literal is passed over.
	EXX			;Return to the main reg set.
	RET			;Finished.

;	THE 'JUMP' SUBROUTINE
;(Offset 33: 'jump')
;This subroutine executes an unconditional jump when called by the
;literal '33'. It is also used by the subroutine DECREASE THE COUNTER
;and JUMP ON TRUE.
L3686:;JUMP
	EXX			;Go to the alternative reg set.
L3687:;JUMP_2
	LD	E,(HL)		;The next literal (jump length) is put
				;in the E' reg.
	LD	A,E		;The number 0 or FF hex is formed in A
	RLA			;according as E' is positive or negative,
	SBC	A,A		;and is then copied to D'.
	LD	D,A
	ADD	HL,DE		;The regs H' & L' now hold the next
	EXX			;literal pntr.
	RET			;Finished.

;	THE 'JUMP ON TRUE' SUBROUTINE
;(Offset 00: 'jump_true')
;This subroutine executes a conditional jump if the 'last value' on
;the calc stack, or more precisely the number addressed currently by
;the DE reg pair, is true.
L368F:;jump_true
	INC	DE		;Point to the 3rd byte, which is zero
	INC	DE		;or one.
	LD	A,(DE)		;Collect this byte in the A reg.
	DEC	DE		;Point to the 1st byte once again.
	DEC	DE
	AND	A		;Test the 3rd byte: is it zero?
	JR	NZ,L3686;JUMP	;Make the jump if the byte is non-zero,
				;i.e. if the number is not-false.
	EXX			;Go to the alternate reg set.
	INC	HL		;Pass over the jump length.
	EXX			;Back to the main set of regs.
	RET			;Finished.

;	THE 'END_CALC' SUBROUTINE
;(Offset 38: 'end_calc')
;This subroutine ends a RST 28h operation.
L369B:;end_calc
	POP	AF		;The return addr to the calculator
				;('RE_ENTRY') is discarded.
	EXX			;Instead, the addr in H'L' is put on
	EX	(SP),HL		;the machine stack and an indirect jump
	EXX			;is made to it. H'L' will now hold any
				;earlier addr in the calculator chain
				;of addrs.
	RET			;Finished.

;	THE 'MODULUS' SUBROUTINE
;(Offset 32: 'n_mod_m')
;The subroutine calculates N MOD M, where M is a positive integer held at
;the top of the calc stack, the 'last value', and N is an integer held on
;the stack beneath M.
;  The subroutine returns the integer quotient INT(N/M) at the top of the
;calc stack, the 'last value', and the remainder N-M*INT(N/M) in the 2nd
;place on the stack.
;  This subroutine is called during the calculation of a random number to
;reduce N mod 65537 dec.
L36A0:;n_mod_m
	RST	28H;FP_CALC	;N, M
	DB	0C0H;st_mem_0	;N, M		mem_0 holds M
	DB	02H;delete	;N
	DB	31H;duplicate	;N, N
	DB	0E0H;get_mem_0	;N, N, M
	DB	05H;division	;N, N/M
	DB	27H;int		;N, INT(N/M)
	DB	0E0H;get_mem_0	;N, INT(N/M), M
	DB	01H;exchange	;N, M, INT(N/M)
	DB	0C0H;st_mem_0	;N, M, INT(N/M)	mem_0 holds INT(N/M)
	DB	04H;multiply	;N, M*INT(N/M)
	DB	03H;subtract	;N-M*INT(N/M)
	DB	0E0H;get_mem_0	;N-M*INT(N/M), INT(N/M)
	DB	38H;end_calc
	RET			;Finished.

;	THE 'INT' FUNCTION
;(Offset 27: 'int')
;This subroutine handles the function INT X and returns a 'last value'
;that is the 'integer part' of the value supplied. Thus INT 2.4 gives 2
;but as the subroutine always rounds the result down INT -2.4 gives -3.
;  The subroutine uses the INTEGER TRUNCATION TOWARDS ZERO subroutine at
;3214 to produce I(X) such that I(2.4) gives 2 and I(-2.4) gives -2.
;Thus, INT X is gives by I(X) for values of X that are greater than or
;equal to zero, and I(X)-1 for negative values of X that are not already
;integers, when the result is, of course, I(X).
L36AF:;int
	RST	28H;FP_CALC	;X
	DB	31H;duplicate	;X, X
	DB	36H;less_0	;X, (1/0)
	DB	00H;jump_true	;X
	DB	04H;to 36B7,X_NEG

;For values of X that have been shown to be greater than or equal to
;zero there is no jump and I(X) is readily found.
	DB	3AH;truncate	;I(X)
	DB	38H;end_calc
	RET			;Finished.

;When X is a neg integer I(X) is returned, otherwise I(X)-1 is returned.
L36B7:;X_NEG
	DB	31H;duplicate	;X, X
	DB	3AH;truncate	;X, I(X)
	DB	0C0H;st_mem_0	;X, I(X)	mem_0 holds I(X)
	DB	03H;subtract	;X-I(X)
	DB	0E0H;get_mem_0	;X-I(X), I(X)
	DB	01H;exchange	;I(X), X-I(X)
	DB	30H;not		;I(X), (1/0)
	DB	00H;jump_true	;I(X)
	DB	03H;to 36C2,EXIT

;The jump is made for values of X that are negative integers, otherwise
;there is no jump and I(X)-1 is calculated.
	DB	0A1H;stk_one	;I(X), 1
	DB	03H;subtract	;I(X)-1

;In either case the subroutine finishes with:
	DB	38H;end_calc	;I(X) or I(X)-1
	RET

;	THE 'EXPONENTIAL' FUNCTION
;(Offset 26: 'exp')
;This subroutine handles the function EXP X and is the first of four
;routines that use SERIES GENERATOR to produce Chebyshev polynomials.
;  The approximation to EXP X is found as follows:
;  i. X is divided by LN 2 to give Y, so that 2 to the power Y is now
;     the required result.
; ii. The value N is found, such that N=INT Y.
;iii. The value W is found, such that W=Y-N, where 0<=W<=1, as required
;     for the series to converge.
; iv. The argument Z is formed, such that Z=2*W-1.
;  v. The SERIES GENERATOR is used to return 2**W.
; vi. Finally N is added to the exponent, giving 2**(N+W), which is 2**Y
;     and therefore the required answer for EXP X.
;The method is illustrated using a BASIC program in the Appendix.
L36C4:;EXP
	RST	28H;FP_CALC	;X
	DB	3DH;re_stack	;X (in full flpnt form)
	DB	34H;stk_data	;X, 1/LN 2
	DB	0F1H;exponent+81
	DB	38H,0AAH,3BH,29H
	DB	04H;multiply	;X/LN 2 = Y
	DB	31H;duplicate	;Y, Y
	DB	27H;int,1C46	;Y, INT Y = N
	DB	0C3H;st_mem_3	;Y, N		mem_3 holds N
	DB	03H;subtract	;Y-N = W
	DB	31H;duplicate	;W, W
	DB	0FH;addition	;2*W
	DB	0A1H;stk_one	;2*W, 1
	DB	03H;subtract	;2*W-1 = Z

;Perform step v, passing to the SERIES GENERATOR the parameter '8' and
;the eight constants required.
	DB	88H;series_8	;Z
	DB	13H;exponent+63
	DB	36H;00,00,00,00
	DB	58H;exponent+68
	DB	65H,66H;00,00
	DB	9DH;exponent+6D
	DB	78H,65H,40H;00
	DB	0A2H;exponent+72
	DB	60H,32H,0C9H;00
	DB	0E7H;exponent+77
	DB	21H,0F7H,0AFH,24H
	DB	0EBH;exponent+7B
	DB	2FH,0B0H,0B0H,14H
	DB	0EEH;exponent+7E
	DB	7EH,0BBH,94H,58H
	DB	0F1H;exponent+81
	DB	3AH,7EH,0F8H,0CFH

;At the end of the last loop the 'last value' is 2**W.
;Perform step vi.
	DB	0E3H;get_mem_3	;2**W, N
	DB	38H;end_calc
	CALL	L2DD5;FP_TO_A	;The absolute value of N mod 256 dec,
				;is put into the A reg.
	JR	NZ,L3705;N_NEGTV ;Jump forward if N was negative.
	JR	C,L3703;REPORT6	;Error if ABS N greater than 255 dec.
	ADD	A,(HL)		;Now add ABS N to the exponent.
	JR	NC,L370C;RESULT_OK ;Jump unless e greater than 255 dec.

;	Report 6 - Number too big
L3703:;REPORT_6
	RST	8;ERROR_1	;Call the error handling
	DB	05h		;routine.
L3705:;N_NEGTV
	JR	C,L370E;RSLT_ZERO ;The result is to be zero if N is
				;less than 255 dec.
	SUB	(HL)		;Subtract ABS N from the exponent
				;as N was negative.
	JR	NC,L370E;RSLT_ZERO ;Zero result if e less than zero.
	NEG			;Minus e is changed to e.
L370C:;RESULT_OK
	LD	(HL),A		;The exponent, e, is entered.
	RET			;Finished: 'last value' is EXP X.
L370E:;RSLT_ZERO
	RST	28H;FP_CALC	;Use the calculator to make the
	DB	02H;delete	;'last value' zero.
	DB	0A0H;stk_zero
	DB	38H;end_calc
	RET			;Finished, with EXP X = 0.

;	THE 'NATURAL LOGARITHM' FUNCTION
;(Offset 25: 'ln')
;This subroutine handles the function LN X and is the 2nd of the four
;routines that use SERIES GENERATOR to produce Chebyshev polynomials.
;  The approximation to LN X is found as follows:
;   i. X is tested and report A is given if X is not positive.
;  ii. X is then split into its true exponent, e', and its mantissa
;      X'=X/(2**e'), where X' is greater than, or equal to, 0.5 but
;      still less than 1.
; iii. The required value Y1 or Y2 is formed. If X' is greater than
;      0.8 then Y1=e'*LN 2 and if otherwise Y2=(e'-1)*LN 2.
;  iv. If X' is greater than 0.8 then the quantity X'-1 is stacked;
;      otherwise 2*X'-1 is stacked.
;   v. Now the argument Z is formed, being if X' is greater than 0.8,
;      Z=2.5*X'-3; otherwise Z=5*X'-3. In each case, -1<=Z<=1, as
;      required for the series to converge.
;  vi. The SERIES GENERATOR is used to produce the required function.
; vii. Finally a simply multiplication and addition leads to LN X being
;      returned as the 'last value'.
L3713:;ln
	RST	28H;FP_CALC
	DB	3DH;re_stack	;X (in full flpnt form)
	DB	31H;duplicate	;X, X
	DB	37H;greater_0	;X, (1/0)
	DB	00H;jump_true	;X
	DB	04H;to 371C,VALID
	DB	38H;end_calc	;X

;	Report A - Invalid argument
	RST	8;ERROR_1	;Call the error handling
	DB	09H		;routine.
L371C:;VALID
	DB	0A0H;stk_zero	;X, 0		The deleted 1 is
	DB	02H;delete	;X	    overwritten with zero.
	DB	38H;end_calc	;X
	LD	A,(HL)		;The exponent, e, goes into A.
	LD	(HL),80H	;X is reduced to X'.
	CALL	L2D28;STACK_A	;The stack holds: X', e.
	RST	28H;FP_CALC	;X', e
	DB	34H;stk_data	;X', e, 128 (dec)
	DB	38H;exponent+88
	DB	00H;00,00,00
	DB	03H;subtract	;X', e'

;Perform step iii.
	DB	01H;exchange	;e', X'
	DB	31H;duplicate	;e', X', X'
	DB	34H;stk_data	;e', X', X', 0.8 (dec)
	DB	0F0H;exponent+80
	DB	4CH,0CCH,0CCH,0CDH
	DB	03H;subtract	;e', X', X'-0.8
	DB	37H;greater_0	;e', X', (1/0)
	DB	00H;jump_true	;e', X'
	DB	08H;to 373D,GRE_8
	DB	01H;exchange	;X', e'
	DB	0A1H;stk_one	;X', e', 1
	DB	03H;subtract	;X', e'-1
	DB	01H;exchange	;e'-1, X'
	DB	38H;end_calc	;e'-1, X'
	INC	(HL)		;Double X' to give 2*X'.
	RST	28H;FP_CALC
L373D:;GRE_8
	DB	01H;exchange	;X', e'		- X' large.
				;2*X', e'-1	- X' small.
	DB	34H;stk_data	;X', e', LN 2
	DB	0F0H;exponent+80
	DB	31H,72H,17H,0F8H
	DB	04H;multiply	;X', e'*LN 2=Y1
				;2*X', (e'-1)*LN 2=Y2

;Perform step iv.
	DB	01H;exchange	;Y1, X'		- X' large.
				;Y2, 2*X'	- X' small.
	DB	0A2H;stk_half	;Y1, X', .5
				;Y2, 2*X', .5
	DB	03H;subtract	;Y1, X'-.5
				;Y2, 2*X'-.5
	DB	0A2H;stk_half	;Y1, X'-.5
				;Y2, 2*X'-.5
	DB	03H;subtract	;Y1, X'-1
				;Y2, 2*X'-1

;Perform step v.
	DB	31H;duplicate	;Y1, X'-1, X'-1
				;Y2, 2*X'-1, 2*X'-1
	DB	34H;stk_data	;Y1, X'-1, X'-1, 2.5
				;Y2, 2*X'-1, 2*X'-1, 2.5
	DB	32H;exponent+82
	DB	20H;00,00,00
	DB	04H;multiply	;Y1, X'-1, 2.5*X'-2.5
				;Y2, 2*X'-1, 5*X'-2.5
	DB	0A2H;stk_half	;Y1, X'-1, 2.5*X'-2.5, .5
				;Y2, 2*X'-1, 5*X'-2.5
	DB	03H;subtract	;Y1, X'-1, 2.5*X'-3=Z
				;Y2, 2*X'-1, 5*X'-2=Z

;Perform step vi, passing to the SERIES GENERATOR the parameter '12' dec,
;and the twelwe constant required.
	DB	8CH;series_0C	;Y1, X'-1, Z or Y2, 2*X'-1, Z
	DB	11H;exponent+61
	DB	0ACH;00,00,00
	DB	14H;exponent+64
	DB	09H;00,00,00
	DB	56H;exponent+66
	DB	0DAH,0A5H;00,00
	DB	59H;exponent+69
	DB	30H,0C5H;00,00
	DB	5CH;exponent+6C
	DB	90H,0AAH;00,00
	DB	9EH;exponent+6E
	DB	70H,6FH,61H;00
	DB	0A1H;exponent+71
	DB	0CBH,0DAH,96H;00
	DB	0A4H;exponent+74
	DB	31H,9FH,0B4H;00
	DB	0E7H;exponent+77
	DB	0A0H,0FEH,5CH,0FCH
	DB	0EAH;exponent+7A
	DB	1BH,43H,0CAH,36H
	DB	0EDH;exponent+7D
	DB	0A7H,9CH,7EH,5EH
	DB	0F0H;exponent+80
	DB	6EH,23H,80H,93H

;At the end of the last loop the 'last value' is:
;	either	LD X'/(X'-1) for the larger values of X'
;	or	LD (2*X')/(2*X'-1) for the smaller values of X'.
;Perform step vii.
	DB	04H;multiply	;Y1=LN (2**e'), LN X'
				;Y2=LN (2**(e'-1)), LN (2*X')
	DB	0FH;addition	;LN (2**e'*X')	= LN X
				;LN (2**(e'-1)*2*X') = LN X
	DB	38H;end_calc	;LN X
	RET			;Finished: 'last value' is LN X.

;	THE 'REDUCE ARGUMENT' SUBROUTINE
;(Offset 39: 'get_argt')
;This subroutine transforms the argument X of SIN X or COS X into a
;value V.
;  The subroutine 1st finds a value Y such that:
;  Y=X/(2*PI)-INT(X/(2*PI)+.5), where Y is greater than, or equal to,
;-.5 but less than +.5.
;  The subroutine returns with:
;	V=4*Y		if -1<=4*Y<=1	- case i.
; or,	V=2-4*Y,	if 1<4*Y<2	- case ii.
; or,	V=-4*Y-2,	if -2<=4*Y<-1	- case iii.
;In each case, -1<=V<=1 and SIN(PI*V/2)=SIN X.
L3783:;get_argt
	RST	28H;FP_CALC	;X
	DB	3DH;re_stack	;X (in full flpnt form)
	DB	34H;stk_data	;X, 1/(2*PI)
	DB	0EEH
	DB	22H,0F9H,83H,6EH
	DB	04H;multiply	;X/(2*PI)
	DB	31H;duplicate	;X/(2*PI), X/(2*PI)
	DB	0A2H;stk_half	;X/(2*PI), X/(2*PI), .5
	DB	0FH;addition	;X/(2*PI), X/(2*PI)+.5
	DB	27H;int,1C46	;X/(2*PI), INT(X/(2*PI)+.5)
	DB	03H;subtract,174C ;X/(2*PI)-INT(X/(2*PI)+.5)=Y
;Note: Adding .5 and taking INT rounds the result to the nearest integer.
	DB	31H;duplicate	;Y, Y
	DB	0FH;addition	;2*Y
	DB	31H;duplicate	;2*Y, 2*Y
	DB	0FH;addition	;4*Y
	DB	31H;duplicate	;4*Y, 4*Y
	DB	2AH;abs		;4*Y, ABS(4*Y)
	DB	0A1H;stk_one	;4*Y, ABS(4*Y), 1
	DB	03H;subtract	;4*Y, ABS(4*Y)-1=Z
	DB	31H;duplicate	;4*Y, Z, Z
	DB	37H;greater_0	;4*Y, Z, (1/0)
	DB	0C0H;st_mem_0	;Mem_0 holds the result of the test.
	DB	00H;jump_true	;4*Y, Z
	DB	04H;to 37A1,ZPLUS ;4*Y, Z
	DB	02H;delete	;4*Y
	DB	38H;end_calc	;4*Y=V - case i.
	RET			;Finished.

;If the jump was made then continue.
L37A1:;ZPLUS
	DB	0A1H;stk_one	;4*Y, Z, 1
	DB	03H;subtract	;4*Y, Z-1
	DB	01H;exchange	;Z-1, 4*Y
	DB	36H;less_0	;Z-1, 4*Y
	DB	00H;jump_true	;Z-1
	DB	02H;to 37A8,YNEG
	DB	1BH;negate	;1-Z
L37A8:;YNEG
	DB	38H;end_calc	;1-Z=V - case ii.
				;Z-1=V - case iii.
	RET			;Finished.

;	THE 'COSINE' FUNCTION
;(Offset 20: 'cos')
;This subroutine handles the function COS X and returns a 'last value'
;that is an approximation to COS X.
;  The subroutine uses the expression:
;	COS X=SIN(PI*W/2), where -1<=W<=1.
;  In deriving W for X the subroutine uses the test result obtained
;in the previous subroutine and stored for this purpose in mem_0.
;It then jumps to the SINE subroutine, enetering at C_ENT, to produce
;a 'last value' of COS X.
L37AA:;sin
	RST	28H;FP_CALC	;X
	DB	39H;get_argt	;V
	DB	2AH;abs		;ABS V
	DB	0A1H;stk_one	;ABS V, 1
	DB	03H;subtract	;ABS V-1
	DB	0E0H;get_mem_0	;ABS V-1, (1/0)
	DB	00H;jump_true	;ABS V-1
	DB	06H;to 37B7,C_ENT
;If the jump was not made then continue.
	DB	1BH;negate	;1-ABS V
	DB	33H;jump	;1-ABS V=W
	DB	03H;to 37B7,C_ENT

;	THE 'SINE' FUNCTION
;(Offset 1F: 'sin')
;This subroutine handles the function SIN X and is the 3rd of the four
;routines that use SERIES GENERATOR to produce Chebyshev polynomials.
;  The approximation to SIN X is found as follows:
;   i. The argument X is reduced and in this case W=V directly.
;      Note that -1<=W<=1, as required for the series to converge.
;  ii. The argument Z is formed, such that Z=2*W*W-1.
; iii. The SERIES GENERATOR is used to return (SIN (PI*W/2))/W.
;  iv. Finally a simple multiplication gives SIN X.
L37B5:;sin
	RST	28H;FP_CALC	;X
;Perform step i.
	DB	39H;get_argt	;W
;Perform step ii. The subroutine from now on is common to both the SINE
;and COSINE functions.
L37B7:;C_ENT
	DB	31H;duplicate	;W, W
	DB	31H;duplicate	;W, W, W
	DB	04H;multiply	;W, W*W
	DB	31H;duplicate	;W, W*W, W*W
	DB	0FH;addition	;W, 2*W*W
	DB	0A1H;stk_one	;W, 2*W*W, 1
	DB	03H;subtract	;W, 2*W*W-1=Z
;Perform step iii, passing to the SERIES GENERATOR the parameter '6' and
;the six constants required.
	DB	86H;series_06	;W, Z
	DB	14H;exponent+64
	DB	0E6H;00,00,00
	DB	5CH;exponent+6C
	DB	1FH,0BH;00,00
	DB	0A3H;exponent+73
	DB	8FH,38H,0EEH;00
	DB	0E9H;exponent+79
	DB	15H,63H,0BBH,23H
	DB	0EEH;exponent+7E
	DB	92H,0DH,0CDH,0EDH
	DB	0F1H;exponent+81
	DB	23H,5DH,1BH,0EAH
;At the end of the last loop the 'last value' is (SIN(PI*W/2))/W.
;Perform step v.
	DB	04H;multiply	;SIN(PI*W/2)=SIN X (or COS X)
	DB	38H;end_calc
	RET			;Finished: 'last value'=SIN/COS X.

;	THE 'TAN' FUNCTION
;(Offset 21: 'tan')
;This subroutine handles the function TAN X. The subroutine simply returns
;SIN X/COS X, with arithmetic overflow if COS X=0.
L37DA:;tan
	RST	28H;FP_CALC	;X
	DB	31H;duplicate	;X, X
	DB	1FH;sin		;X, SIN X
	DB	01H;exchange	;SIN X, X
	DB	20H;cos		;SIN X, COS X
	DB	05H;division	;TAN X (Report arith ovfl if needed)
	DB	38H;end_calc
	RET			;Finished: 'last value' = TAN X.

;	THE 'ARCTAN' FUNCTION
;(Offset 24: 'atn')
;This subroutine handles the function ATN X and is the last of the four
;routines that use SERIES GENERATOR to produce Chebyshev polynomials.
;It returns a real number between -PI/2 and PI/2, which is equal to the
;value in radians of the angle whose tan is X.
;  The approximation to ATN X is found as follows:
;   i. The values W and Y are found for three cases of X, such that:
;      if    -1<X<1	then W=0	& Y=X	 - case i.
;      if         1<=X	then W=PI/2	& Y=-1/X - case ii.
;      if X<=-1		then W=-PI/2	& Y=-1/X - case iii.
;
;      In each case, -1<=Y<=1, as required for the series to converge.
;  ii. The argument Z is formed, such that:
;      if    -1<X<1	then Z=2*Y*Y-1=2*X*X-1	 - case i.
;      if         1<=X	then Z=2*Y*Y-1=2/(X*X)-1 - case ii.
;      if X<=-1		then Z=2*Y*Y-1=2/(X*X)-1 - case iii.
; iii. The SERIES GENERATOR is used to produce the required function.
;  iv. Finally a simple multiplication and addition give ATN X.
L37E2:;atn
;Perform stage i.
	CALL	L3297;RE_STACK	;Use the full flpnt form of X.
	LD	A,(HL)		;Fetch the exponent of X.
	CP	81H
	JR	C,L37F8;SMALL	;Jump forward for case i: Y=X.
	RST	28H;FP_CALC	;X
	DB	0A1H;stk_one	;X, 1
	DB	1BH;negate	;X, -1
	DB	01H;exchange	;-1, X
	DB	05H;division	;-1/X
	DB	31H;duplicate	;-1/X, -1/X
	DB	36H;less_0	;-1/X, (1/0)
	DB	0A3H;stk_pi/2	;-1/X, (1/0), PI/2
	DB	01H;exchange	;-1/X, PI/2, (1/0)
	DB	00H;jump_true	;-1/X, PI/2
	DB	06H;to 37FA,CASES ;Jump forward for case ii: Y=-1/X W=PI/2
	DB	1BH;negate	;-1/X, -PI/2
	DB	33H;jump	;-1/X, -PI/2
	DB	03H;to 37FA,CASES ;Jump forward for case iii: Y=-1/X W=-PI/2
L37F8:;SMALL
	RST	28H;FP_CALC	;Y
	DB	0A0H;stk_zero	;Y, 0
;Continue for case i: W=0
L37FA:;CASES
	DB	01H;exchange	;W, Y
	DB	31H;duplicate	;W, Y, Y
	DB	31H;duplicate	;W, Y, Y, Y
	DB	04H;multiply	;W, Y, Y*Y
	DB	31H;duplicate	;W, Y, Y*Y, Y*Y
	DB	0FH;addition	;W, Y, 2*Y*Y
	DB	0A1H;stk_one	;W, Y, 2*Y*Y, 1
	DB	03H;subtract	;W, Y, 2*Y*Y-1=Z
;Perform step iii, passing to the SERIES GENERATOR the parameter '12' dec,
;and the twelwe constants required.
	DB	8CH;series_0C	;W, Y, Z
	DB	10H;exponent+60
	DB	0B2H;00,00,00
	DB	13H;exponent+63
	DB	0EH;00,00,00
	DB	55H;exponent+65
	DB	0E4H,8DH;00,00
	DB	58H;exponent+68
	DB	39H,0BCH;00,00
	DB	5BH;exponent+6B
	DB	98H,0FDH;00,00
	DB	9EH;exponent+6E
	DB	00H,36H,75H;00
	DB	0A0H;exponent+70
	DB	0DBH,0E8H,0B4H;00
	DB	63H;exponent+73
	DB	42H,0C4H;00,00
	DB	0E6H;exponent+76
	DB	0B5H,09H,36H,0BEH
	DB	0E9H;exponent+79
	DB	36H,73H,1BH,5DH
	DB	0ECH;exponent+7C
	DB	0D8H,0DEH,63H,0BEH
	DB	0F0H;exponent+80
	DB	61H,0A1H,0B3H,0CH
;At the end of the last loop the 'last value' is:
;	ATN X/X			- case i.
;	ATN (-1/X)/(-1/X)	- case ii.
;	ATN (-1/X)/(-1/X)	- case iii.
;Perform step iv.
	DB	04H;multiply	;W, ATN X	- case i.
				;W, ATN (-1/X)	- cases ii,iii.
	DB	0FH;addition	;ATN X - all cases now.
	DB	38H;end_calc
	RET			;Finished: 'last value'=ATN X.

;	THE 'ARCSIN' FUNCTION
;(Offset 22: 'asn')
;This subroutine handles the function ASN X and return a real number from
;-PI/2 to PI/2 inclusive which is equal to the value in radians of the
;angle whose sine is X. Thereby if Y=ASN X then X=SIN Y.
;  This subroutine uses the trigonometric identity:
;    TAN(Y/2)=SIN Y/(1+COS Y)
;to obtain TAN(Y/2) and hence (using ATN) Y/2 and finally Y.
L3833:;asn
	RST	28H;FP_CALC	;X
	DB	31H;duplicate	;X, X
	DB	31H;duplicate	;X, X, X
	DB	04H;multiply	;X, X*X
	DB	0A1H;stk_one	;X, X*X, 1
	DB	03H;subtract	;X, X*X-1
	DB	1BH;negate	;X, 1-X*X
	DB	28H;sqr		;X, SQR(1-X*X)
	DB	0A1H;stk_one	;X, SQR(1-X*X), 1
	DB	0FH;addition	;X, 1+SQR(1-X*X)
	DB	05H;division	;X/(1+SQR(1-X*X))=TAN (Y/2)
	DB	24H;atn		;Y/2
	DB	31H;duplicate	;Y/2, Y/2
	DB	0FH;addition	;Y=ASN X
	DB	38H;end_calc
	RET			;Finished: 'last value'=ASN X.

;	THE 'ARCCOS' FUNCTION
;(Offset 23: 'acs')
;This subroutine handles the function ACS X and return a real number from
;zero to PI inclusive which is equal to the value in radians of the angle
;whose cosine is X.
;  This subroutine uses the relation:
;    ACS X=PI/2-ASN X
L3843:;acs
	RST	28H;FP_CALC	;X
	DB	22H;asn		;ASN X
	DB	0A3H;stk_pi/2	;ASN X, PI/2
	DB	03H;subtract	;ASN X-PI/2
	DB	1BH;negate	;PI/2-ASN X=ACS X
	DB	38H
	RET			;Finished: 'last value'=ACS X.

;	THE 'SQUARE ROOT' FUNCTION
;(Offset 28: 'sqr')
;This subroutine handles the function SQR X and returns the positive square
;root of the real number X if X is positive, and zero if X is zero.
;A negative value of X gives rise to report A - invalid argument (via ln
;in the EXPONENTATION subroutine).
;  This subroutine treats the square root operation as being X**.5 and
;therefore stacks the value .5 and proceeds directlyinto the EXPONENTATION
;subroutine.
L384A:;sqr
	RST	28H;FP_CALC	;X
	DB	31H;duplicate	;X, X
	DB	30H;not		;X, (1/0)
	DB	00H;jump_true	;X
	DB	1EH;to 386C,LAST
;The jump is made if X=0, otherwise continue with:
	DB	0A2H;stk_half	;X, .5
	DB	38H;end_calc
;and then find the result of X**.5.

;	THE 'EXPONENTATION' OPERATION
;(Offset 06: 'to_power')
;(This subroutine performs the binary operation of raising the 1st
;number, X, to the power of the 2nd number, Y.
;  The subroutine treats the result X**Y as being equivalent to
;EXP(Y*LN X). It returns this value unless X is zero, in which case
;it returns 1 if Y is also zero (0**0=1), returns zero if Y is positive
;and reports arith ovfl if Y is negative.
L3851:;to_power
	RST	28H;FP_CALC	;X, Y
	DB	01H;exchange	;Y, X
	DB	31H;duplicate	;Y, X, X
	DB	30H;not		;Y, X, (1/0)
	DB	00H;jump_true	;Y, X
	DB	07H;to 385D,XISO
;The jump is made if X=0, otherwise EXP(Y*LN X) is formed.
	DB	25H;ln		;Y, LN X (Report A if X is negative)
	DB	04H;multiply	;Y*LN X
	DB	38H;end_calc
	JP	36C4H;EXP	;Exit via EXP to form EXP (Y*LN X).
;The value of X is zero so consider the three possible cases involved.
L385D:;XISO
	DB	02H;delete	;Y
	DB	31H;duplicate	;Y, Y
	DB	30H;not		;Y, (1/0)
	DB	00H;jump_true	;Y
	DB	09H;to 386A,ONE
;The jump is made if X=0 and Y=0, otherwise proceed.
	DB	0A0H;stk_zero	;Y, 0
	DB	01H;exchange	;0, Y
	DB	37H;greater_0	;0, (1/0)
	DB	00H;jump_true	;0
	DB	06H;to 386C,LAST
;The jump is made if X=0 and Y is positive, otherwise proceed.
	DB	0A1H;stk_one	;0, 1
	DB	01H;exchange	;1, 0
	DB	05H;division	;Exit via 'division' as dividing by zero
				;gives 'arithmetic overflow'.
;The result is to be 1 for the operation.
L386A:;ONE
	DB	02H;delete
	DB	0A1H;stk_one	;1
;Now return with the 'last value' on the stack being 0**Y.
L386C:;LAST
	DB	38H;end_calc	;(1/0)
	RET			;Finished: 'last value' is 0 or 1.

;386E-3CFF	These locations are 'spare'. They all hold 0FFh.

;3D00-3FFF	These locations hold the 'char set'. There are 8 byte
;		representation for all the chars with codes 20h-7Fh.

;e.g. the letter 'A' has the representation 00 3C 42 42 7E 42 42 00
;and thereby the form:
;		00000000
;		00111100
;		01000010
;		01000010
;		01111110
;		01000010
;		01000010
;		00000000


