
;	THE COMMAND ROUTINES
;The section of the 16K monitor program from 1CEE to 23FA contains most of
;the command routines of the BASIC interpreter.

;	THE 'STOP' COMMAND ROUTINE
;The command routine for STOP contains only a call to the error handling
;routine.
L1CEE:;STOP (REPORT_9)
	RST	8;ERROR_1	;Call the error handling
	DB	08H		;routine.

;	THE 'IF' COMMAND ROUTINE
;On entry the value of the expression between the IF and the THEN is the
;'last value' on the calc stack. If this is logically true then the next
;stmt is considered; otherwise the line is considered to have been finished.
L1CF0:;IF
	POP	BC		;Drop the return addr - STMT_RET.
	CALL	L2530;SYNTAX_Z	;Jump forward if checking
	JR	Z,L1D00;IF_1	;syntax.

;Now use the calc to 'delete' the last value on the calc stack but leave
;the DE reg pair addressing the 1st byte of the value.
	RST	28H;FP_CALC	;Use the calc.
	DB	02H;delete	;The present 'last value' is
	DB	38H;end_calc	;deleted.
	EX	DE,HL		;Make HL point to the 1st
	CALL	L34E9;TEST_ZERO	;byte and call TEST_ZERO. If the value was
	JP	C,L1BB3;LINE_END ;'FALSE' jump to the next line.
L1D00:;IF_1
	JP	L1B29;STMT_1	;But if 'TRUE' jump to the next stmt
				;(after the THEN).

;	THE 'FOR' COMMAND ROUTINE
;This command routine is entered with the VALUE and the LIMIT of the FOR
;stmt already on the top of the calc stack.
L1D03:;FOR
	CP	0CDH		;Jump forward unless a 'STEP'
	JR	NZ,L1D10;F_USE_1 ;is given.
	RST	20H;NEXT_CHAR	;Advance CH_ADD and fetch the
	CALL	L1C82;EXPT_1NUM	;value of the STEP.
	CALL	L1BEE;CHECK_END	;Move on to the next stmt if checking
	JR	L1D16;F_REORDER	;syntax; otherwise jump forward.

;There has not been a STEP supplied so the value '1' is to be used.
L1D10:;F_USE_1
	CALL	L1BEE;CHECK_END	;Move on to the next stmt if checking
	RST	28H;FP_CALC	;syntax; otherwise use the calc to place
	DB	0A1H;stk_one	;a '1' on the calc stack.
	DB	38H;end_calc

;The three values on the calc stack are the VALUE (v), the LIMIT (l) and
;the STEP (s). These values now have to be manipulated.
L1D16:;F_REORDER
	RST	28H;FP_CALC	;v, i, s
	DB	0C0H;st_mem_0	;v, i, s	mem_0 = s
	DB	02H;delete	;v, i
	DB	01H;exchange	;i, v
	DB	0E0H;get_mem_0	;i, v, s
	DB	01H;exchange	;i, s, v
	DB	38H;end_calc

;A FOR control var is now established and treated as a temporary calculator
;memory area.
	CALL	L2AFF;LET	;The var is found, or created if needed
				;(v is used).
	LD	(L5C68),HL;MEM	;Make it a 'memory area'.

;The var that has been found may be a simple numeric var using anly six
;locs in which case it will need extending.
	DEC	HL		;Fetch the var's single
	LD	A,(HL)		;char name.
	SET	7,(HL)		;Ensure bit 7 of the name is set.
	LD	BC,6		;It will have six locs at least.
	ADD	HL,BC		;Make HL point after them.
	RLCA			;Rotate the name and jump if
	JR	C,L1D34;F_L&S	;it was already a FOR var.
	LD	C,0DH		;Otherwise create thirteen
	CALL	L1655;MAKE_ROOM	;more locs.
	INC	HL		;Again make HL point to the LIMIT position.

;The initial values for the LIMIT and the STEP are now added.
L1D34:;F_L&S
	PUSH	HL		;The pntr is saved.
	RST	28H;FP_CALC	;i, s
	DB	02H;delete	;i
	DB	02H;delete	;-
	DB	38H;end_calc	;DE still points to 'i'.
	POP	HL		;The pntr is restored and
	EX	DE,HL		;both pntrs exchanged.
	LD	C,0AH		;The ten bytes of the LIMIT
	LDIR			;and the STEP are moved.

;The looping line number and stmt number are now entered.
	LD	HL,(L5C45);PPC	;The current line number.
	EX	DE,HL		;Exchange the regs before
	LD	(HL),E		;adding the line number to the
	INC	HL		;FOR control var.
	LD	(HL),D
	LD	D,(IY+0DH);SUBPPC ;The looping stmt is
	INC	D		;always the next stmt -
	INC	HL		;whether it exists or not.
	LD	(HL),D

;The NEXT_LOOP subroutine is called to test the possibility of a 'pass'
;and a return is made if one is possible; otherwise the stmt after for
;FOR-NEXT loop has to be identified.
	CALL	L1DDA;NEXT_LOOP	;Is a 'pass' possible?
	RET	NC		;Return now if it is.
	LD	B,(IY+38H);STRLEN_lo ;Fetch the var's name.
	LD	HL,(L5C45);PPC	;Copy the present line number
	LD	(L5C42),HL;NEWPPC ;to NEWPPC.
	LD	A,(L5C47);SUBPPC ;Fetch the current stmt number
	NEG			;and 2's complement it.
	LD	D,A		;Transfer the result to the D reg.
	LD	HL,(L5C5D);CH_ADD ;Fetch the current value of CH_ADD.
	LD	E,0F3H		;The search will be for 'NEXT'.

;Now a search is made in the program area, from the present point onwards,
;for the 1st occurence of NEXT followed by the correct var.
L1D64:;F_LOOP
	PUSH	BC		;Save the var's name.
	LD	BC,(L5C55);NXTLIN ;Fetch the current value of NXTLIN.
	CALL	L1D86;LOOK_PROG	;The program area is now searched and BC
				;will change with each new line examined.
	LD	(L5C55),BC;NXTLIN ;Upon return save the pntr.
	POP	BC		;Restore the var's name.
	JR	C,L1D84;REPORT_I ;If there are no further NEXTs
				;then give an error.
	RST	20H;NEXT_CHAR	;Advance past the NEXT that was found.
	OR	' '		;Allow for upper and lower case letters
	CP	B		;before the new var name is tested.
	JR	Z,L1D7C;F_FOUND	;Jump forward if it matches.
	RST	20H;NEXT_CHAR	;Advance CH_ADD again and jump back
	JR	L1D64;F_LOOP	;if not the correct var.

;NEWPPC holds the line number of the line in which the correct NEXT was
;found. Now the stmt number has to be found and stored in NSPPC.
L1D7C:;F_FOUND
	RST	20H;NEXT_CHAR	;Advance CH_ADD.
	LD	A,1		;The stmt counter in the D reg counted
	SUB	D		;stmts back from zero so it has to be
	LD	(L5C44),A;NSPPC	;subtracted from '1'. The result is stored.
	RET			;Now return - to STMT_RET.

;Report I - FOR without NEXT
L1D84:;REPORT_I
	RST	8;ERROR_1	;Call the error handling
	DB	11H		;routine.

;	THE 'LOOK_PROG' SUBROUTINE
;This subroutine is used to find occurences of either DATA, DEF FN or NEXT.
;On entry the appropriate token code is in the E reg and the HL reg pair
;points to the starts of the search area.
L1D86:;LOOK_PROG
	LD	A,(HL)		;Fetch the present char.
	CP	':'		;Jump forward if it is a ':' which will
	JR	Z,1DA3H;LOOK_P_2 ;indicate there are more stmts in the
				;present line.

;Now a loop is entered to examine each further line in the program.
L1D8B:;LOOK_P_1
	INC	HL		;Fetch the high byte of the
	LD	A,(HL)		;line number and return with
	AND	0C0H		;carry set if there are no
	SCF			;further lines in the program.
	RET	NZ
	LD	B,(HL)		;The line number is fetched
	INC	HL		;and passed to NEWPPC.
	LD	C,(HL)
	LD	(L5C42),BC;NEWPPC
	INC	HL		;Then the length is collected.
	LD	C,(HL)
	INC	HL
	LD	B,(HL)
	PUSH	HL		;The pntr is saved whilst
	ADD	HL,BC		;the addr of the end of the
	LD	B,H		;line is formed in the BC
	LD	C,L		;reg pair.
	POP	HL		;The pntr is restored.
	LD	D,0		;Set the stmt counter to zero.
L1DA3:;LOOK_P_2
	PUSH	BC		;The end-of-line pntr is
	CALL	L198B;EACH_STMT	;saved whilst the stmts
	POP	BC		;of the line are examined.
	RET	NC		;Make a return if there was an 'occurence';
	JR	L1D8B;LOOK_P_1	;otherwise consider the next line.

;	THE 'NEXT' COMMAND ROUTINE
;The 'var in assignment' has already been determined (see CLASS_04, 1C6C);
;and it remains to change the VALUE as required.
L1DAB:;NEXT
	BIT	1,(IY+37H);FLAGX ;Jump to give the error report
	JP	NZ,L1C2E;REPORT_2 ;if the var was not found.
	LD	HL,(L5C4D);DEST	;The addr of the var
	BIT	7,(HL)		;is fetched and the name
	JR	Z,L1DD8;REPORT_1 ;tested further.

;Next the var's VALUE and STEP are manipulated by the calc.
	INC	HL		;Step past the name.
	LD	(L5C68),HL;MEM	;Make the var a temporary 'memory area'.
	RST	28H;FP_CALC	;-
	DB	0E0H;get_mem_0	;v
	DB	0E2H;get_mem_2	;v, s
	DB	0FH;addition	;v+s
	DB	0C0H;st_mem_0	;v+s
	DB	02H;delete	;-
	DB	38H;end_calc

;The result of adding the VALUE and the STEP is now tested against the
;LIMIT by calling NEXT_LOOP.
	CALL	L1DDA;NEXT_LOOP	;Test the new VALUE against the LIMIT.
	RET	C		;Return now if the FOR-NEXT loop
				;has been completed.

;Otherwise collect the 'looping' line number and stmt.
	LD	HL,(L5C68);MEM	;Find the addr of the
	LD	DE,0Fh		;low byte of the looping
	ADD	HL,DE		;line number.
	LD	E,(HL)		;Now fetch this line number.
	INC	HL
	LD	D,(HL)
	INC	HL
	LD	H,(HL)		;Followed by the stmt number.
	EX	DE,HL		;Exchange the numbers before jumping forward
	JP	L1E73;GO_TO_2	;to treat them as the destination line of
				;a GO TO command.

;Report 1 - NEXT without FOR
L1DD8:;REPORT_1
	RST	8;ERROR_1	;Call the error handling
	DB	00H		;routine.

;	THE 'NEXT_LOOP' SUBROUTINE
;This subroutine is used to determine whether the LIMIT has been exceeded
;by the present VALUE. Note has to be taken of the sign of the STEP.
;  The subroutine returns the carry flag set if the LIMIT is exceeded.
L1DDA:;NEXT_LOOP
	RST	28H;FP_CALC	;-
	DB	0E1H;get_mem_1	;l
	DB	0E0H;gem_mem_0	;l, v
	DB	0E2H;get_mem_2	;l, v, s
	DB	36H;less_0	;l, v, (1/0)
	DB	00H;jump_true	;l, v
	DB	02H;to NEXT_1
	DB	01H;exchange	;v, l
L1DE2:;NEXT_1
	DB	03H;subtract	;v-l or l-v
	DB	37H;greater_0	;(1/0)
	DB	00H;jump_true	;-
	DB	04H;to NEXT_2
	DB	38H;end_calc
	AND	A		;Clear the carry flag and
	RET			;return.

;However if the loop is impossible the carry flag has to be set.
L1DE9:;NEXT_2
	DB	38H;end_calc	;-
	SCF			;Set the carry flag and
	RET			;return.

;	THE 'READ' COMMAND ROUTINE
;The READ command allows for the reading of a DATA list and has an effect
;similar to a series of LET stmts.
;  Each assignment within a single READ stmt is dealt with in turn. The
;system var X_PTR is used as a storage 
L1DEC:;READ_3
	RST	20H;NEXT_CHAR	;Come here on each pass, after the 1st,
				;to move along the READ stmt.
L1DED:;READ
	CALL	L1C1F;CLASS_01	;Consider whether the var has been used
				;before; find the existing entry if it has.
	CALL	L2530;SYNTAX_Z	;Jump forward if checking
	JR	Z,L1E1E;READ_2	;syntax.
	RST	18H;GET_CHAR	;Syve the current pntr
	LD	(L5C5F),HL;X_PTR ;CH_ADD in X_PTR.
	LD	HL,(L5C57);DATADD ;Fetch the current DATA list
	LD	A,(HL)		;pntr and jump forward
	CP	','		;unless a new DATA stmt
	JR	Z,L1E0A;READ_1	;has to be found.
	LD	E,0E4H		;The search is for 'DATA'.
	CALL	L1D86;LOOK_PROG	;Jump forward if the search is
	JR	NC,L1E0A;READ_1	;successful.

;Report E - Out of DATA
L1E08:;REPORT_E
	RST	8;ERROR_1	;Call the error handling
	DB	0DH		;routine.

;Continue - picking up a value from the DATA list.
L1E0A:;READ_1
	CALL	L0077;TEMP_PTR1	;Advance the pntr along the DATA list
				;and set CH_ADD.
	CALL	L1C56;VAL_FET_1	;Fetch the value and assign it to the var.
	RST	18H;GET_CHAR	;Fetch the current value of CH_ADD and
	LD	(L5C57),HL;DATADD ;store it in DATADD.
	LD	HL,(L5C5F);X_PTR ;Fetch the pntr to the READ stmt
	LD	(IY+26H),0;X_PTR_hi ;and clear X_PTR.
	CALL	L0078;TEMP_PTR2	;Make CH_ADD once again point to the
				;READ stmt.
L1E1E:;READ_2
	RST	18H;GET_CHAR	;GET the present char and
	CP	','		;see if it is a ','.
	JR	Z,L1DEC;READ_3	;If it is then jump back as there are
				;further items;
	CALL	L1BEE;CHECK_END	;otherwise return either via CHECK_END
	RET			;(if checking syntax) or the RET instruction
				;(to STMT_RET).

;	THE 'DATA' COMMAND ROUTINE
;During syntax checking a DATA stmt is checked to ensure that it contains a
;series of valid expressions, separated by commas. But in 'run-time' the
;stmt is passed by.
L1E27:;DATA
	CALL	L2530;SYNTAX_Z	;Jump forward unless checking
	JR	NZ,L1E37;DATA_2	;syntax.

;A loop is now entered to deal with each expression in the DATA stmt.
L1E2C:;DATA_1
	CALL	L24FB;SCANNING	;Scan the next expression.
	CP	','		;Check for the correct separator - a ',';
	CALL	NZ,L1BEE;CHECK_END ;but move on to the next stmt if
				;not matched.
	RST	20H;NEXT_CHAR	;Whilst there are still expressions
	JR	L1E2C;DATA_1	;to be checked go around the loop.

;The DATA stmt has to be passed-by in 'run-time'.
L1E37:;DATA_2
	LD	A,0E4H		;It is a 'DATA' stmt that is to be passed-by.

;	THE 'PASS_BY' SUBROUTINE
;On entry the A reg will hold either the token 'DATA' or the token 'DEF FN'
;depending on the type of stmt that is being 'passed-by'.
L1E39:;PASS_BY
	LD	B,A		;Make the BC reg pair hold a very
				;high number.
	CPDR			;Look back along the stmt for the token.
	LD	DE,200h		;Now look along the line for the stmt
	JP	L198B;EACH_STMT	;after. (The 'D-1'th stmt from the
				;current position.)

;	THE 'RESTORE' COMMAND ROUTINE
;The operand for a RESTORE command is taken as a line number, zero being
;used if no operand is given.
;  The REST_RUN entry point is used by the RUN command routine.
L1E42:;RESTORE
	CALL	L1E99;FIND_INT2	;Compress the operand into the BC reg pair.
L1E45:;REST_RUN
	LD	H,B		;Transfer the result to the
	LD	L,C		;HL reg pair.
	CALL	L196E;LINE_ADDR	;Now find the addr of that line or the
	DEC	HL		;'1st line after'. Make DATADD point to
	LD	(L5C57),HL;DATADD ;the loc before.
	RET			;Return once it is done.

;	THE 'RANDOMIZE' COMMAND ROUTINE
;Once again the operand is compressed into the BC reg pair and transferred
;to the required system var. However if the operand is zero the value in
;FRAMES1 and FRAMES2 is used instead.
L1E4F:;RANDOMIZE
	CALL	L1E99;FIND_INT2	;Fetch the operand.
	LD	A,B		;Jump forward unless the
	OR	C		;value of the operand is
	JR	NZ,L1E5A;RAND_1	;zero.
	LD	BC,(L5C78);FRAMES1 ;Fetch the two low order bytes of
				;FRAMES instead.
L1E5A:;RAND_1
	LD	(L5C76),BC;SEED	;Now enter the result into the system var
	RET			;SEED before returning.

;	THE 'CONTINUE' COMMAND ROUTINE
;The required line number and stmt number within that line are made the
;object of a jump.
L1E5F:;CONTINUE
	LD	HL,(L5C6E);OLDPPC ;The line number.
	LD	D,(IY+36H);OSPCC  ;The stmt number.
	JR	L1E73;GO_TO_2	;Jump forward.

;	THE 'GO TO' COMMAND ROUTINE
;The operand of a GO TO ought to be a line number in the range '1' to '9999'
;but the actual test is against an upper value of '61439'.
L1E67:;GO_TO
	CALL	L1E99;FIND_INT2	;Fetch the operand and transfer
	LD	H,B		;it to the HL reg pair.
	LD	L,C
	LD	D,0		;Set the stmt number to zero.
	LD	A,H		;Give the error message
	CP	0F0H		;- Integer out of range -
	JR	NC,L1E9F;REPORT_B ;with lines over '61439'.

;The entry point GO_TO_2 is used to determine the line number of the next
;line to be handled in several instances.
L1E73:;GO_TO_2
	LD	(L5C42),HL;NEWPPC ;Enter the line number and
	LD	(IY+0AH),D;NSPPC  ;then the stmt number.
	RET			;Return; - to STMT_RET.

;	THE 'OUT' COMMAND ROUTINE
;The two parameters for the OUT instruction are fetched from the calc stack
;and used as directed.
L1E7A:;OUT
	CALL	L1E85;TWO_PARAM	;The operands are fetched.
	OUT	(C),A		;The actual OUT instruction.
	RET			;Return; - to STMT_RET.

;	THE 'POKE' COMMAND ROUTINE
;In a similar manner the POKE operation is performed.
L1E80:;POKE
	CALL	L1E85;TWO_PARAM	;The operands are fetched.
	LD	(BC),A		;The actual POKE operation.
	RET			;Return; - to STMT_RET.

;	THE 'TWO_PARAM' SUBROUTINE
;The topmost parameter on the calc stack must be compressible into a single
;reg. It is 2's complemented if it is negative. The 2nd parameter must be
;compressible into a reg pair.
L1E85:;TWO_PARAM
	CALL	L2DD5;FP_TO_A	;The parameter is fetched.
	JR	C,L1E9F;REPORT_B ;Give an error if it is too high a number.
	JR	Z,L1E8E;TWO_P_1	;Jump forward with positive numbers but 2's
	NEG			;complement negative numbers.
L1E8E:;TWO_P_1
	PUSH	AF		;Save the 1st parameter
	CALL	L1E99;FIND_INT2	;whilst the 2nd is fetched.
	POP	AF		;The 1st parameter is
	RET			;restored before returning.

;	THE 'FIND INTEGERS' SUBROUTINES
;The 'last value' on the calc stack is fetched and compressed into a single
;reg or a reg pair by entering at FIND_INT1 and FIND_INT2 respectively.
L1E94:;FIND_INT1
	CALL	L2DD5;FP_TO_A	;Fetch the 'last value'.
	JR	L1E9C;FIND_I_1	;Jump forward.
L1E99:;FIND_INT2
	CALL	L2DA2;FP_TO_BC	;Fetch the 'last value'.
L1E9C:;FIND_I_1
	JR	C,L1E9F;REPORT_B ;In both cases overflow is
				;indicated by a set carry flag.
	RET	Z		;Return with all positive numbers
				;that are in range.

;Report B - Integer out of range
L1E9F:;REPORT_B
	RST	8;ERROR_1	;Call the error handling
	DB	0AH		;routine.

;	THE 'RUN' COMMAND ROUTINE
;The parameter of the RUN command is passed to NEWPPC by calling the GO TO
;command routine. The operations of 'RESTORE 0' and 'CLEAR 0' are then
;performed before a return is made.
L1EA1:;RUN
	CALL	L1E67;GO_TO	;Set NEWPPC as required.
	LD	BC,0		;Now perform a 'RESTORE 0'.
	CALL	L1E45;REST_RUN
	JR	L1EAF;CLEAR_1	;Exit via the CLEAR command routine.

;	THE 'CLEAR' COMMAND ROUTINE
;This routine allows for the vars area to be cleared, the display area cleared
;and RAMTOP moved. In consequence of the last operation the machine stack is
;rebuilt thereby having the effect of also clearing the GO SUB stack.
L1EAC:;CLEAR
	CALL	L1E99;FIND_INT2	;Fetch the operand - using zero by default.
L1EAF:;CLEAR_RUN
	LD	A,B		;Jump forward if the operand is
	OR	C		;other than zero. When called
	JR	NZ,L1EB7;CLEAR_1 ;from RUN there is no jump.
	LD	BC,(L5CB2);RAMTOP ;If zero use the existing value in RAMTOP.
L1EB7:;CLEAR_1
	PUSH	BC		;Save the value.
	LD	DE,(L5C4B);VARS	;Next reclaim all the bytes
	LD	HL,(L5C59);E_LINE ;of the present vars
	DEC	HL		;area.
	CALL	L19E5;RECLAIM_1
	CALL	L0D6B;CLS	;Clear the display area.

;The value in the BC reg pair which will be used as RAMTOP is tested to
;ensure it is neither too low nor too high.
	LD	HL,(L5C65);STKEND ;The current value of STKEND
	LD	DE,32h		;is increased by '50' before
	ADD	HL,DE		;being tested. This forms the
	POP	DE		;lower limit.
	SBC	HL,DE
	JR	NC,L1EDA;REPORT_M ;RAMTOP will be too low.
	LD	HL,(L5CB4);P_RAMT ;For the upper test the value
	AND	A		;for RAMTOP is tested against
	SBC	HL,DE		;P_RAMT.
	JR	NC,L1EDC;CLEAR_2 ;Jump forward if acceptable.

;Report M - RAMTOP no good
L1EDA:;REPORT_M
	RST	8;ERROR_1	;Call the error handling
	DB	15H		;routine.

;Continue with the CLEAR operation.
L1EDC:;CLEAR_3
	EX	DE,HL		;Now the value can actually be
	LD	(L5CB2),HL;RAMTOP ;passed to RAMTOP.
	POP	DE		;Fetch the addr - STMT_RET.
	POP	BC		;Fetch the 'error addr'.
	LD	(HL),3Eh	;Enter a GO SUB stack end marker.
	DEC	HL		;Leave one loc.
	LD	SP,HL		;Make the SP point to an empty GO SUB stack.
	PUSH	BC		;Next pass the 'error addr' to the stack
	LD	(L5C3D),SP;ERR_SP ;and save its addr in ERR_SP.
	EX	DE,HL		;An indirect return is now
	JP	(HL)		;made to STMT_RET.
;Note: When the routine is called from RUN the values of NEWPPC & NSPPC will
;have been affected and no stmts coming after RUN can ever be found before
;the jump is taken.

;	THE 'GO SUB' COMMAND ROUTINE
;The present value of PPC and the incremented value of SUBPPC are stored
;on the GO SUB stack.
L1EED:;GO_SUB
	POP	DE		;Save the addr - STMT_RET.
	LD	H,(IY+0DH);SUBPPC ;Fetch the stmt number
	INC	H		;and increment it. Exchange the 'error
	EX	(SP),HL		;addr' with the stmt number.
	INC	SP		;Reclaim the use of a loc.
	LD	BC,(L5C45);PPC	;Next save the present line number.
	PUSH	BC
	PUSH	HL		;Return the 'error addr' to the machine
	LD	(L5C3D),SP;ERR_SP ;stack and reset ERR_SP to point to it.
	PUSH	DE		;Return the addr - STMT_RET.
	CALL	L1E67;GO_TO_1	;Now set NEWPPC & NSPPC to the required
				;values.
	LD	BC,14H		;But before making the jump make
				;a test for room.

;	THE 'TEST_ROOM' SUBROUTINE
;A series of tests is performed to ensure that there is sufficient free
;memory available for the task being undertaken.
L1F05:;TEST_ROOM
	LD	HL,(L5C65);STKEND ;Increase the value taken from STKEND by
	ADD	HL,BC		;the value carried into the routine by BC.
	JR	C,L1F15;REPORT_4 ;Jump forward if the result is over 0FFFFh.
	EX	DE,HL		;Try it again allowing for a
	LD	HL,50H		;further eighty bytes.
	ADD	HL,DE
	JR	C,L1F15;REPORT_4
	SBC	HL,SP		;Finally test the value against the addr of
	RET	C		;the machine stack. Return if satisfactory.

;Report 4 - Out of memory
L1F15:;REPORT_4
	LD	L,3		;This is a 'run-time' error and the
	JP	L0055;ERROR_3	;error marker is not to be used.

;	THE 'FREE MEMORY' SUBROUTINE
;There is no BASIC command 'FRE' in the SPECTRUM but there is a subroutine
;for performing such a task.
;  An estimate of the amount of free space can be found at any time by using:
;	PRINT 65536-USR 7962
L1F1A:;FREE_MEM
	LD	BC,0		;Do not allow any overhead.
	CALL	L1F05;TEST_ROOM	;Make the test and pass the
	LD	B,H		;result to the BC reg
	LD	C,L		;before returning.
	RET

;	THE 'RETURN' COMMAND ROUTINE
;The line number and the stmt number that are to be made the object of a
;'return' are fetched from the GO SUB stack.
L1F23:;RETURN
	POP	BC		;Fetch the addr - STMT_RET.
	POP	HL		;Fetch the 'error addr'.
	POP	DE		;Fetch the last entry on the GO SUB stack.
	LD	A,D		;The entry is tested to see if
	CP	3Eh		;it is the GOSUB stack end
	JR	Z,L1F36;REPORT_7 ;marker; jump it it is.
	DEC	SP		;The full entry uses three locs only.
	EX	(SP),HL		;Exchange the stmt number with the
				;'error addr'.
	EX	DE,HL		;Move the stmt number.
	LD	(L5C3D),SP;ERR_SP ;Reset the error pntr.
	PUSH	BC		;Replace the addr - STMT_RET.
	JP	L1E73;GO_TO_2	;Jump back to change NEWPPC & NSPPC.

;Report 7 - RETURN without GOSUB
L1F36:;REPORT_7
	PUSH	DE		;Replace the end marker and
	PUSH	HL		;the 'error addr'.
	RST	8;ERROR_1	;Call the error handling
	DB	06H		;routine.

;	THE 'PAUSE' COMMAND ROUTINE
;The period of the PAUSE is determined by counting the number of maskable
;interrupts as the occur every 1/50 th. of a second.
;  A PAUSE is finished either after the appropriate number of interrupts
;or by the system var FLAGS indicating that a key has been pressed.
L1F3A:;PAUSE
	CALL	L1E99;FIND_INT2	;Fetch the operand.
L1F3D:;PAUSE_1
	HALT			;Wait for a maskable interrupt.
	DEC	BC		;Decrease the counter.
	LD	A,B		;If the counter is thereby
	OR	C		;reduced to zero the PAUSE
	JR	Z,L1F4F;PAUSE_END ;has come to an end.
	LD	A,B		;If the operand was zero BC
	AND	C		;will now hold 0FFFFh and this
	INC	A		;value will be returned to
	JR	NZ,L1F49;PAUSE_2 ;zero. Jump will all other
	INC	BC		;operand values.
L1F49:;PAUSE_2
	BIT	5,(IY+1);FLAGS	;Jump back unless a key has
	JR	Z,L1F3D;PAUSE_1	;been pressed.

;The period of the PAUSE has now finished.
L1F4F:;PAUSE_END
	RES	5,(IY+1);FLAGS	;Signal 'no key pressed'.
	RET			;Now return; - to STMT_RET.

;	THE 'BREAK_KEY' SUBROUTINE
;This subroutine is called in several instances to read the BREAK key.
;The carry flag is returned reset only if the SHIFT and the BREAK keys
;are both being pressed.
L1F54:;BREAK_KEY
	LD	A,7FH		;Form the port addr
	IN	A,(0FEH)	;07FFEh and read in a byte.
	RRA			;Examine only bit 0 by shifting
				;it into the carry position.
	RET	C		;Return if the BREAK key is not
				;being pressed.
	LD	A,0FEH		;Form the port addr
	IN	A,(0FEH)	;0FEFEh and read in a byte.
	RRA			;Again examine bit 0.
	RET			;Return with carry reset if
				;both keys are being pressed.

;	THE 'DEF FN' COMMAND ROUTINE
;During syntax checking a DEF FN stmt is checked to ensure that it has
;the correct form. Space is also made available for the result of evaluating
;the function.
;  But in 'run-time' a DEF FN stmt is passed-by.
L1F60:;DEF_FN
	CALL	L2530;SYNTAX_Z	;Jump forward if checking
	JR	Z,L1F6A;DEF_FN_1 ;syntax.
	LD	A,0CEH		;Otherwise pass by the
	JP	L1E39;PASS_BY	;'DEF FN' stmt.

;1st consider the var of the function.
L1F6A:;DEF_FN_1
	SET	6,(IY+1);FLAGS	;Signal 'a numeric var'.
	CALL	L2C8D;ALPHA	;Check that the present code is a letter.
	JR	NC,L1F89;DEF_FN_4 ;Jump forward if not.
	RST	20H;NEXT_CHAR	;Fetch the next char.
	CP	'$'		;Jump forward unless it is
	JR	NZ,L1F7D;DEF_FN_2 ;a '$'.
	RES	6,(IY+1);FLAGS	;Change bit 6 as it is a string var.
	RST	20H;NEXT_CHAR	;Fetch the next char.
L1F7D:;DEF_FN_2
	CP	'('		;A '(' must follow the
	JR	NZ,L1FBD;DEF_FN_7 ;var's name.
	RST	20H;NEXT_CHAR	;Fetch the next char.
	CP	')'		;Jump forward if it is a ')' as there are
	JR	Z,L1FA6;DEF_FN_6 ;no parameters of the function.

;A loop is now entered to deal with each parameter in turn.
L1F86:;DEF_FN_3
	CALL	L2C8D;ALPHA	;The present code must be
L1F89:;DEF_FN_4
	JP	NC,L1C8A;REPORT_C ;a letter.
	EX	DE,HL		;Save the pntr in DE.
	RST	20H;NEXT_CHAR	;Fetch the next char.
	CP	'$'		;Jump forward unless it is a
	JR	NZ,L1F94;DEF_FN_5 ;'$'.
	EX	DE,HL		;Otherwise save the new pntr in DE instead.
	RST	20H;NEXT_CHAR	;Fetch the next char.
L1F94:;DEF_FN_5
	EX	DE,HL		;Move the pntr to the last char of the
				;name to the HL reg pair.
	LD	BC,6		;Now make six locs after
	CALL	L1655;MAKE_ROOM	;that last char and
	INC	HL		;enter a 'number marker' into
	INC	HL		;the 1st of the new
	LD	(HL),0EH	;locs.
	CP	','		;If the present char is
	JR	NZ,L1FA6;DEF_FN_6 ;a ',' then jump back as
	RST	20H;NEXT_CHAR	;there should be a further parameter;
	JR	L1F86;DEF_FN_3	;otherwise jump out of the loop.

;Next the definition of the function is considered.
L1FA6:;DEF_FN_6
	CP	')'		;Check that the ')' does
	JR	NZ,L1FBD;DEF_FN_7 ;exist.
	RST	20H;NEXT_CHAR	;The next char is fetched.
	CP	'='		;It must be an '='.
	JR	NZ,L1FBD;DEF_FN_7
	RST	20H;NEXT_CHAR	;Fetch the next char.
	LD	A,(L5C3B);FLAGS	;Save the nature - numeric or
	PUSH	AF		;string - of the var.
	CALL	L24FB;SCANNING	;Now consider the definition
				;as an expression.
	POP	AF		;Fetch the nature of the
	XOR	(IY+1);FLAGS	;var and check that it is of the same type
	AND	40h		;as found for the definition.
L1FBD:;DEF_FN_7
	JP	NZ,L1C8A;REPORT_C ;Give an error report if it is required.
	CALL	L1BEE;CHECK_END	;Exit via the CHECK_END subroutine.
				;(Thereby moving on to consider the next
				;stmt in the line.)

;	THE 'UNSTACK_Z' SUBROUTINE
;This subroutine is called in several instances in order to 'return early'
;from a subroutine when checking syntax. The reason for this is to avoid
;actually printing chars or passing values to/from the calc stack.
L1FC3:;UNSTACK_Z
	CALL	L2530;SYNTAX_Z	;Is syntax being checked?
	POP	HL		;Fetch the return addr but ignore it in
	RET	Z		;'syntax-time'. In 'run-time' make a
	JP	(HL)		;simple return to the calling routine.

