
;	THE 'PIXEL ADDRESS' SUBROUTINE
;This subroutine is called by the POINT subroutine and by the PLOT command
;routine. It is entered with the co-ordinates of a pixel in the BC reg pair
;and returns with HL holding the addr of the display file byte which
;contains that pixel and A pointing to the position of the pixel within
;the byte.
L22AA:;PIXEL_ADD
	LD	A,0AFH		;Test that the y co-ordinate (in
	SUB	B		;B) is not greater than 175.
	JP	C,L24F9;REPORT_B
	LD	B,A		;B now contains 175 minus y.
	AND	A		;A holds b7b6b5b4b3b2b1b0, the byte of B,
	RRA			;and now 0b7b6b5b4b3b2b1.
	SCF
	RRA			;Now 10b7b6b5b4b3b2.
	AND	A
	RRA			;Now 010b7b6b5b4b3.
	XOR	B
	AND	0F8H		;Finally 010b7b6b2b1b0, so that
	XOR	B		;H becomes 64+8*INT(B/64)+
	LD	H,A		;B(mod 8), the high byte of the
	LD	A,C		;pixel addr. C contains X.
	RLCA			;A starts as c7c6c5c4c3c2c1c0.
	RLCA
	RLCA
	XOR	B
	AND	0C7H
	XOR	B		;Now c2c1b5b4b3c5c4c3.
	RLCA
	RLCA			;Finally b5b4b3c7c6c5c4c3, so
	LD	L,A		;that L becomes 32*INT(B(mod 64)/8)
	LD	A,C		;+INT(X/8), the low byte.
	AND	7		;A holds X(mod 8): so the pixel
	RET			;is bit (A-7) within the byte.

;	THE 'POINT' SUBROUTINE
;This subroutine is called by the POINT function in SCANNING. It is entered
;with the co-ordinates of a pixel on the calc stack, and returns a last
;value of 1 if that pixel is ink colour, and 0 if it is paper colour.
L22CB:;POINT_SUB
	CALL	L2307;STK_TO_BC	;Y co-ordinate to B, X to C.
	CALL	L22AA;PIXEL_ADD	;Pixel addr to HL.
	LD	B,A		;B will count A+1 loops to get
	INC	B		;the wanted bit of (HL) to
	LD	A,(HL)		;loc 0.
L22D4:;POINT_LP
	RLCA			;The shifts.
	DJNZ	L22D4;POINT_LP
	AND	1		;The bit is 1 for ink, 0 for paper.
	JP	L2D28;STACK_A	;It is put on the calc stack.

;	THE 'PLOT' COMMAND ROUTINE
;This routine consists of a main subroutine plus one line to call it and one
;line to exit from it. The main routine is used twice by CIRCLE and the
;subroutine is called by DRAW. The routine is entered with the co-ordinates
;of a pixel on the calc stack. It finds the addr of that pixel and plots
;it, taking account of the status of INVERSE and OVER held in the P_FLAG.
L22DC:;PLOT
	CALL	L2307;STK_TO_BC	;Y co-ordinate to B, X to C.
	CALL	L22E5;PLOT_SUB	;The subroutine is called.
	JP	L0D4D;TEMPS	;Exit, setting temporary colours.
L22E5:;PLOT_SUB
	LD	(L5C7D),BC;COORDS ;The system var is set.
	CALL	L22AA;PIXEL_ADD	;Pixel addr in HL.
	LD	B,A		;B will count A+1 loops to get a
	INC	B		;zero to the correct place in A.
	LD	A,0FEH		;The zero is entered.
L22F0:;PLOT_LOOP
	RRCA			;Then lined up with the pixel
	DJNZ	L22F0;PLOT_LOOP	;bit position in the byte.
	LD	B,A		;Then copied to B.
	LD	A,(HL)		;The pixel-byte is obtained in A.
	LD	C,(IY+57H);P_FLAG ;P_FLAG is obtained and 1st
	BIT	0,C		;tested for OVER.
	JR	NZ,L22FD;PL_TST_IN ;Jump if OVER 1.
	AND	B		;OVER 0 1st makes the pixel zero.
L22FD:;PL_TST_IN
	BIT	2,C		;Test for INVERSE.
	JR	NZ,L2303;PLOT_END ;INVERSE 1 just leaves the pixel as
				;it was (OVER 1) or zero (OVER 0).
	XOR	B		;INVERSE 0 leaves the pixel complemented
	CPL			;(OVER 1) or 1 (OVER 0).
L2303:;PLOT_END
	LD	(HL),A		;The byte is entered. Its other bits are
	JP	L0BDB;PO_ATTR	;unchanged in every case. Exit, setting
				;attribute byte.

;	THE 'STK_TO_BC' SUBROUTINE
;This subroutine loads two flpnt numbers into the BC reg pair. It is thus
;used to pick up params in the range 0-0FFh. It also obtains in DE the
;'diagonal move' values (+-1,+-1) which are used in the line drawing
;subroutine of DRAW.
L2307:;STK_TO_BC
	CALL	L2314;STK_TO_A	;1st number to A.
	LD	B,A		;Hence to B.
	PUSH	BC		;Save it briefly.
	CALL	L2314;STK_TO_A	;2nd number to A.
	LD	E,C		;Its sign indicator to E.
	POP	BC		;Restore 1st number.
	LD	D,C		;Its sign indicator to D.
	LD	C,A		;2nd number to C.
	RET			;BC, DE are now as required.

;	THE 'STK_TO_A' SUBROUTINE
;This subroutine loads the A reg with the flpnt number held at the top of
;the calc stack. The number must be in the range 0-0FFh.
L2314:;STK_TO_A
	CALL	L2DD5;FP_TO_A	;Modulus of rounded last value to
	JP	C,L24F9;REPORT_B ;A if possible; else, error report.
	LD	C,1		;One to C for positive last value.
	RET	Z		;Return if value was positive.
	LD	C,0FFH		;Else change C to 0FFh (i.e. minus
	RET			;one). Finished.

;	THE 'CIRCLE' COMMAND ROUTINE
;This routine draws an approximation to the circle with centre co-ordinates
;X and Y and radius Z. These numbers are rounded to the nearest integer
;before use. Thus Z must be less than 87.5, even when (X,Y) is in the centre
;of the screen. The method used is to draw a series of arcs approximated
;by straight lines. It is illustarted in a BASIC program in the appendix.
;The notation of that program is followed here.
;  CIRCLE has four parts:
;   i. Test the radius. If its modulus is less than 1, just plots X,Y;
;  ii. Cll CD_PRMS1 at 2470-24B6, which is used to set the initial params
;      for both CIRLE and DRAW;
; iii. Sets up the ramaining params for CIRCLE, including the initial
;      displacement for the 1st 'arc' (a straight line, in fact);
;  iv. Jumps into DRAW to use the arc-drawing loop at 2420-24FA.
;Parts i. to iii. will now be explained in turn.

;   i. 2320-23AA. The radius, say Z', is obtained from the calc stack. Its
;modulus Z is formed and used from now on. If Z is less than 1, it is deleted
;from the stack and the point X,Y is plotted by a jump to PLOT.
L2320:;CIRCLE
	RST	18H;GET_CHAR	;Get the present char.
	CP	','		;Test for comma.
	JP	NZ,L1C8A;REPORT_C ;If not so, report the error.
	RST	20H;NEXT_CHAR	;Get next char (the radius).
	CALL	L1C82;EXPT_1NUM	;Radius to calc stack.
	CALL	L1BEE;CHECK_END	;Move to consider next stmt
				;if checking syntax.
	RST	28H;FP_CALC	;Use calc: the stack holds:
	DB	2AH;abs		;X, Y, Z
	DB	3DH;re_stack	;Z is re-stacked; its exponent
	DB	38H		;is therefore available.
	LD	A,(HL)		;Get exponent of radius.
	CP	81H		;Test whether radius less than 1.
	JR	NC,L233B;C_R_GRE_1 ;If not, jump.
	RST	28H;FP_CALC	;If less, delete it from stack.
	DB	02H;delete	;The stack holds X, Y.
	DB	38H;end_calc
	JR	L22DC;PLOT	;Just plot the point X, Y.

;  ii. 233B-2346 and the call to CD_PRMS1. 2*PI is stored in mem_5 and
;CD_PRMS1 is called. This subroutine stores in the B reg the number of
;arcs required for the circle, viz. A=4*INT(PI*SQR Z/4)+4, hence 4, 8,
;12..., up to a maximum of 32. It also stores in mem_0 to mem_4 the
;quantities 2*PI/A, SIN(PI/A), 0, COS(2*PI/A) and SIN(2*PI/A).
L233B:;C_R_GRE_1
	RST	28H;FP_CALC
	DB	0A3H;stk_pi/2	;X, Y, Z, PI/2
	DB	38H;end_calc	;Now increase exponent to 83h,
	LD	(HL),83H	;changing PI/2 int 2*PI.
	RST	28H;FP_CALC	;X, Y, Z, 2*PI
	DB	0C5H;st_mem_5	;(2*PI is copied to mem_5).
	DB	02H;delete	;X, Y, Z
	DB	38H;end_calc
	CALL	L247D;CD_PRMS1	;Set the initial params.

; iii. 2347-2381: the remaining params and the jump to DRAW. A test is made
;to see whether the initial 'arc' length is less than 1. If it is, a jump is
;made simply to plot X, Y. Otherwise, the params are set: X+Z and
;Y-Z*SIN(PI/A) are stacked twice as start and end point, and copied to
;COORDS as well; zero and 2*Z*SIN(PI/A) are stored in mem_1 and mem_2 as
;initial increments, giving as 1st 'arc' the vertical straight line joining
;X+Z, Y-Z*SIN(PI/A) and X+Z, Y+Z*SIN(PI/A). The arc-drawing loop of DRAW
;will ensure that all subsequent points remain on the same circle as these
;2 points, with incremental angle 2*PI/A. But it is clear that these 2 points
;in fact subtend this angle at the point X+Z*(1-COS(PI/A)),Y not at X, Y.
;Hence the end points of each arc of the circle are displaced right by an
;amount 2*(1-COS(PI/A)), which is less than half a pixel, and rounds to one
;pixel at most.
	PUSH	BC		;Save the arc-count in B.
	RST	28H;FP_CALC	;X, Y, Z
	DB	31H;duplicate	;X, Y, Z, Z
	DB	0E1H;get_mem_1	;X, Y, Z, Z, SIN(PI/A)
	DB	04H;multiply	;X, Y, Z, Z*SIN(PI/A)
	DB	38H;end_calc	;SIN(PI/A) is half the initial
	LD	A,(HL)		;'arc' length; it is tested to see
	CP	80H		;whether it is less than 0.5.
	JR	NC,L235A;C_ARC_GE1 ;If not, the jump is made.
	RST	28H;FP_CALC	;Otherwise, Z is deleted from the
	DB	02H;delete	;stack, with the half-arc too; the
	DB	02H;delete	;machine stack is cleared; and a
	DB	38H;end_calc	;jump is made to plot X, Y.
	POP	BC
	JP	L22DC;PLOT
L235A:;C_ARC_GE1
	RST	28H;FP_CALC	;X, Y, Z, Z*SIN(PI/A)
	DB	0C2H;st_mem_2	;(Z*SIN(PI/A) to mem_2 for now).
	DB	01H;exchange	;X, Y, Z*SIN(PI/A), Z
	DB	0C0H;st_mem_0	;X, Y, Z*SIN(PI/A), Z
	DB	02H;delete	;X, Y, Z*SIN(PI/A)
	DB	03H;subtract	;X, Y-Z*SIN(PI/A)
	DB	01H;exchange	;Y-Z*SIN(PI/A), X
	DB	0E0H;get_mem_0	;Y-Z*SIN(PI/A), X, Z
	DB	0FH;addition	;Y-Z*SIN(PI/A), X+Z
	DB	0C0H;st_mem_0	;Y-Z*SIN(PI/A), X+Z
	DB	01H;exchange	;X+Z, Y-Z*SIN(PI/A)
	DB	31H;duplicate	;X+Z, Y-Z*SIN(PI/A), Y-Z*SIN(PI/A)
	DB	0E0H;get_mem_0	;sa, sb, sb, sa
	DB	01H;exchange	;sa, sb, sa, sb                   
	DB	31H;duplicate	;sa, sb, sa, sb, sb
	DB	0E0H;get_mem_0	;sa, sb, sa, sb, sb, sa
	DB	0A0H;stk_zero	;sa, sb, sa, sb, sb, sa, 0
	DB	0C1H;st_mem_1	;(mem_1 is set to zero)
	DB	02H;delete	;sa, sb, sa, sb, sb, sa
	DB	38H;end_calc

;(Here sa denotes X+Z and sb denotes Y-Z*SIN(PI/A)).
	INC	(IY+62H);mem_2_1st ;Incrementing the exponent byte of mem_2
				;sets mem_2 to 2*Z*SIN(PI/A).
	CALL	L1E94;FIND_INT1	;The last value X+Z is moved from the
	LD	L,A		;stack to A and copied to L.
	PUSH	HL		;It is saved in HL.
	CALL	L1E94;FIND_INT1	;Y-Z*SIN(PI/A) goes from the
	POP	HL		;stack to A and is copied to H.
	LD	H,A		;HL now holds the initial point.
	LD	(L5C7D),HL;COORDS ;It is copied to COORDS.
	POP	BC		;The arc-count is restored.
	JP	L2420;DRW_STEPS	;The jump is made to DRAW.

;	THE 'DRAW' COMMAND ROUTINE
;This routine is entered with the co-ordinates of a point X0, Y0, say, in
;COORDS. If only two params X, Y are given with the DRAW command, it draws
;an approximation to a straight line from the point X0, Y0 to X0+X, Y0+Y.
;If a 3rd param G is given, it draws an approximation to a circular arc
;from X0, Y0 to X0+X, Y0+Y turning anticlockwise through an angle G radians.
;  The routine has four parts:
;    i. Just draws a line if only 2 params are given or if the diameter of
;	the implied circle is less than 1;
;   ii. Call CD_PRMS1 at 247D-24B6 to set the 1st params;
;  iii. Sets up the remaining params, including the initial displacements
;	for the 1st arc;
;   iv. Enters the arc-drawing loop and draws the arc as a series of smaller
;	arcs approximated by straight lines, calling the line-drawing
;	subroutine at 24B7-24FA as necessary.
;Two subroutine, CD_PRMS1 and DRAW_LINE, follow the main routine. The above
;4 parts of the main routine will now be treated in turn.

;    i. If there are only 2 params, a jump is made to LINE_DRAW at 2477.
;A line is also drawn if the quantity Z=(ABS X+ABS Y)/ABS SIN(G/2) is less
;than 1. Z lies between 1 and 1.5 times the diameter of the implied circle.
;In this section mem_0 is set to SIN(G/2), mem_1 to Y, and mem_5 to G.
L2382:;DRAW
	RST	18H;GET_CHAR	;Get the current char.
	CP	','		;If it is a comma,
	JR	Z,L238D;DR_3_PRMS ;then jump.
	CALL	L1BEE;CHECK_END	;Move on to the next stmt if checking syntax.
	JP	L2477;LINE_DRAW	;Jump to just draw the line.
L238D:;DR_3_PRMS
	RST	20H;NEXT_CHAR	;Get next char (the angle).
	CALL	L1C82;EXPT_1NUM	;Angle the calc stack.
	CALL	L1BEE;CHECK_END	;Move on to the next stmt if checking syntax.
	RST	28H;FP_CALC	;X, Y, G are on the stack.
	DB	0C5H;st_mem_5	;(G is copied to mem_5).
	DB	0A2H;stk_half	;X, Y, G, .5
	DB	04H;multiply	;X, Y, G/2
	DB	1FH;sin		;X, Y, SIN(G/2)
	DB	31H;duplicate	;X, Y, SIN(G/2), SIN(G/2)
	DB	30H;not		;X, Y, SIN(G/2), (0/1)
	DB	30H;not		;X, Y, SIN(G/2), (1/0)
	DB	00H;jump_true	;X, Y, SIN(G/2)
	DB	06H;to DR_SIN_NZ ;(if SIN(G/2)=0 i.e. G=2*N*PI
	DB	02H;delete	;just draw a straight line).
	DB	38H;end_calc	;X, Y
	JP	L2477;LINE_DRAW	;Line X0, Y0 to X0+X, Y0+Y.
L23A3:;DR_SIN_NZ
	DB	0C0H;st_mem_0	;(SIN G/2 is copied to mem_0).
	DB	02H;delete	;X, Y
	DB	0C1H;st_mem_1	;(Y is copied to mem_1).
	DB	02H;delete	;X
	DB	31H;duplicate	;X, X
	DB	2AH;abs		;X, X' (X'=ABS X)
	DB	0E1H;get_mem_1	;X, X', Y
	DB	01H;exchange	;X, Y, X'
	DB	0E1H;get_mem_1	;X, Y, X', Y
	DB	2AH;abs		;X, Y, X', Y' (Y'=ABS Y)
	DB	0FH;addition	;X, Y, X'+Y'
	DB	0E0H;get_mem_0	;X, Y, X'+Y', SIN(G/2)
	DB	05H;division	;X, Y, (X'+Y')/SIN(G/2)=Z', say
	DB	2AH;abs		;X, Y, Z (Z=ABS Z')
	DB	0E0H;get_mem_0	;X, Y, Z, SIN(G/2)
	DB	01H;exchange	;X, Y, SIN(G/2), Z
	DB	3DH;re_stack	;(Z is re-stacked to make sure
	DB	38H;end_calc	;that its exponent is available).
	LD	A,(HL)		;Get exponent of Z.
	CP	81H		;If Z is greater than or equal
	JR	NC,L23C1;DR_PRMS ;to 1, jump.
	RST	28H;FP_CALC	;X, Y, SIN(G/2), Z
	DB	02H;delete	;X, Y, SIN(G/2)
	DB	02H;delete	;X, Y
	DB	38H;end_calc	;Just draw the line from X0, Y0
	JP	L2477;LINE_DRAW	;to X0+X, Y0+Y.

;   ii. Just calls CD_PRMS1. This subroutine saves in the B reg the number
;of shorter arcs required for the complete arc, viz. A=4*INT(G'*SQR Z/8)+4,
;where G'=mod G, or 252 if this expression exceeds 252 (as can happen with
;a large chord and a small angle). So A is 4, 8, 12, ..., up to 252. The
;subroutine also stores in mem_0 to mem_4 the quantities G/A, SIN(G/2*A),
;0, COS(G/A), SIN(G/A).
L23C1:;DR_PRMS
	CALL	L247D;CD_PRMS1	;The subroutine is called.

;  iii. Sets up the rest of the params as follow. The stack will hold these
;4 items, reading up to the top: X0+X and Y0+Y as end of last arc;
	PUSH	BC		;Save the arc-counter in B.
	RST	28H;FP_CALC	;X, Y, SIN(G/2), Z
	DB	02H;delete	;X, Y, SIN(G/2)
	DB	0E1H;get_mem_1	;X, Y, SIN(G/2), SIN(G/2*A)
	DB	01H;exchange	;X, Y, SIN(G/2*A), SIN(G/2)
	DB	05H;division	;X, Y, SIN(G/2*A)/SIN(G/2)=W
	DB	0C1H;st_mem_1	;(W is copied to mem_1).
	DB	02H;delete	;X, Y
	DB	01H;exchange	;Y, X
	DB	31H;duplicate	;Y, X, X
	DB	0E1H;get_mem_1	;Y, X, X, W
	DB	04H;multiply	;Y, X, X*W
	DB	0C2H;st_mem_2	;(X*W is copied to mem_2).
	DB	02H;delete	;Y, X
	DB	01H;exchange	;X, Y
	DB	31H;duplicate	;X, Y, Y
	DB	0E1H;get_mem_1	;X, Y, Y, W
	DB	04H;multiply	;X, Y, Y*W
	DB	0E2H;get_mem_2	;X, Y, Y*W, X*W
	DB	0E5H;get_mem_5	;X, Y, Y*W, X*W, G
	DB	0E0H;get_mem_0	;X, Y, Y*W, X*W, G, G/A
	DB	03H;subtract	;X, Y, Y*W, X*W, G-G/A
	DB	0A2H;stk_half	;X, Y, Y*W, X*W, G-G/A, .5
	DB	04H;multiply	;X, Y, Y*W, X*W, G/2-G/2*A=F
	DB	31H;duplicate	;X, Y, Y*W, X*W, F, F
	DB	1FH;sin		;X, Y, Y*W, X*W, F, SIN F
	DB	0C5H;st_mem_5	;(SIN F is copied to mem_5).
	DB	02H;delete	;X, Y, Y*W, X*W, F
	DB	20H;cos		;X, Y, Y*W, X*W, COS F
	DB	0C0H;st_mem_0	;(COS F is copied to mem_0).
	DB	02H;delete	;X, Y, Y*W, X*W
	DB	0C2H;st_mem_2	;(X*W is copied to mem_2).
	DB	02H;delete	;X, Y, Y*W
	DB	0C1H;st_mem_1	;(Y*W is copied to mem_1).
	DB	0E5H;get_mem_5	;X, Y, Y*W, SIN F
	DB	04H;multiply	;X, Y, Y*W*SIN F
	DB	0E0H;get_mem_0	;X, Y, Y*W*SIN F, COS F
	DB	0E2H;get_mem_2	;X, Y, Y*W*SIN F, COS F, X*W
	DB	04H;multiply	;X, Y, Y*W*SIN F, X*W*COS F
	DB	0FH;addition	;X, Y, Y*W*SIN F+X*W*COS F=U
	DB	0E1H;get_mem_1	;X, Y, U, Y*W
	DB	01H;exchange	;X, Y, Y*W, U
	DB	0C1H;st_mem_1	;(U is copied to mem_1).
	DB	02H;delete	;X, Y, Y*W
	DB	0E0H;get_mem_0	;X, Y, Y*W, COS F
	DB	04H;multiply	;X, Y, Y*W*COS F
	DB	0E2H;get_mem_2	;X, Y, Y*W*COS F, X*W
	DB	0E5H;get_mem_5	;X, Y, Y*W*COS F, X*W, SIN F
	DB	04H;multiply	;X, Y, Y*W*COS F, X*W*SIN F
	DB	03H;subtract	;X, Y, Y*W*COS F-X*W*SIN F=V
	DB	0C2H;st_mem_2	;(V is copied to mem_2).
	DB	2AH;abs		;X, Y, V' (V'=ABS V)
	DB	0E1H;get_mem_1	;X, Y, V', U
	DB	2AH;abs		;X, Y, V', U' (U'=ABS U)
	DB	0FH;addition	;X, Y, U'+V'
	DB	02H;delete	;X, Y
	DB	38H;end_calc	;(DE now points to U'+V').
	LD	A,(DE)		;Get exponent of U'+V'.
	CP	81H		;If U'+V' is less than 1, just
	POP	BC		;tidy the stack and draw the line
	JP	C,L2477;LINE_DRAW ;from X0, Y0 to X0+X, Y0+Y.
	PUSH	BC		;Otherwise, continue with the
	RST	28H;FP_CALC	;params: X, Y, on the stack.
	DB	01H;exchange	;Y, X
	DB	38h;end_calc
	LD	A,(L5C7D);COORDS_lo ;Get X0 into A and so
	CALL	L2D28;STACK_A	;on to the stack.
	RST	28H;FP_CALC	;Y, X, X0
	DB	0C0H;st_mem_0	;(X0 is copied to mem_0).
	DB	0FH;addition	;Y, X0+X
	DB	01H;exchange	;X0+X, Y
	DB	38H;end_calc
	LD	A,(L5C7E);COORDS_hi ;Get Y0 into A and so
	CALL	L2D28;STACK_A	;on to the stack.
	RST	28H;FP_CALC	;X0+X, Y, Y0
	DB	0C5H;st_mem_5	;(Y0 is copied to mem_5).
	DB	0FH;addition	;X0+X, Y+Y0
	DB	0E0H;get_mem_0	;X0+X, Y0+Y, X0
	DB	0E5H;get_mem_5	;X0+X, Y0+Y, X0, Y0
	DB	38H;end_calc
	POP	BC		;Restore the arc-counter in B.

;   iv. The arc-drawing loop. This is entered at 2439 with the co-ordinates
;of the starting point on top of the stack, and the initial displacements for
;the 1st arc in mem_1 and mem_2. It uses simple trigonometry to ensure that
;all subsequent arcs will be drawn to points that lie on the same circle as
;the 1st two, subtending the same angle at the centre. It can be shown that
;if 2 points X1, Y1 and X2, Y2 lie on a circle and subtend an angle N at
;the centre, which is also the origin of co-ordinates, then X2=X1*COS N-
;Y1*SIN N, and Y2=X1*SIN N+Y1*COS N. But because the origin is here at the
;bottom left-hand corner of the screen, the arc-drawing loop applies these
;relations to increments, say Un=Xn+1-Xn and Vn=Yn+1-Yn, thus achieving the
;desired result. The stack is shown below on the (n+1)th pass through the
;loop, as Xn and Yn are incremented by Un and Vn, after there are obtained
;from Un-1 and Vn-1. The 4 values on the top of the stack at 2425 are, in
;DRAW, reading upwards, X0+X, Y0+Y, Xn and Yn but to save space these are
;not shown until 2439. For the initial values in CIRCLE, see the end of
;CIRCLE, above. In CIRCLE too, the angle G must be taken to be 2*PI.
L2420:;DRW_STEPS
	DEC	B		;B counts the passes through the loop.
	JR	Z,L245F;ARC_END	;Jump when B has reached zero.
	JR	L2439;ARC_START	;Jump into the loop to start.
L2425:;ARC_LOOP
	RST	28H;FP_CALC	;(See text above for the stack).
	DB	0E1H;get_mem_1	;Un-1
	DB	31H;duplicate	;Un-1, Un-1
	DB	0E3H;get_mem_3	;Un-1, Un-1, COS(G/A)
	DB	04H;multiply	;Un-1, Un-1*COS(G/A)
	DB	0E2H;get_mem_2	;Un-1, Un-1*COS(G/A), Vn-1
	DB	0E4H;get_mem_4	;Un-1, Un-1*COS(G/A), Vn-1, SIN(G/A)
	DB	04H;multiply	;Un-1, Un-1*COS(G/A), Vn-1*SIN(G/A)
	DB	03H;subtract	;Un-1, Un-1*COS(G/A)-Vn-1*SIN(G/A)=Un
	DB	0C1H;st_mem_1	;(Un is copied to mem_1)
	DB	02H;delete	;Un-1
	DB	0E4H;get_mem_4	;Un-1, SIN(G/A)
	DB	04H;multiply	;Un-1*SIN(G/A)
	DB	0E2H;get_mem_2	;Un-1*SIN(G/A), Vn-1
	DB	0E3H;get_mem_3	;Un-1*SIN(G/A), Vn-1, COS(G/A)
	DB	04H;multiply	;Un-1*SIN(G/A), Vn-1*COS(G/A)
	DB	0FH;addition	;Un-1*SIN(G/A)+Vn-1*COS(G/A)=Vn
	DB	0C2H;st_mem_2	;(Vn is copied to mem_2)
	DB	02H;delete	;(As noted in the text, the stack
	DB	38H;end_calc	;in fact holds X0+X, Y0+Y, Xn and Yn).
L2439:;ARC_START
	PUSH	BC		;Save the arc-counter.
	RST	28H;FP_CALC	;X0+X, Y0+Y, Xn, Yn
	DB	0C0H;st_mem_0	;(Yn is copied to mem_0)
	DB	02H;delete	;X0+X, Y0+Y, Xn
	DB	0E1H;get_mem_1	;X0+X, Y0+Y, Xn, Un
	DB	0FH;addition	;X0+X, Y0+Y, Xn+Un=Xn+1
	DB	31H;duplicate	;X0+X, Y0+Y, Xn+1, Xn+1
	DB	38H;end_calc	;Next Xn', the approximate value of Xn
				;reached by the line-drawing subroutine
	LD	A,(L5C7D);COORDS_lo ;is copied to A
	CALL	L2D28;STACK_A	;and hence to the stack.
	RST	28H;FP_CALC	;X0+X, Y0+Y, Xn+1, Xn+1, Xn'
	DB	03H;subtract	;X0+X, Y0+Y, Xn+1, Xn+1-Xn'=Un'
	DB	0E0H;get_mem_0	;X0+X, Y0+Y, Xn+1, Un', Yn 
	DB	0E2H;get_mem_2	;X0+X, Y0+Y, Xn+1, Un', Yn, Vn
	DB	0FH;addition	;X0+X, Y0+Y, Xn+1, Un', Yn+Vn=Yn+1
	DB	0C0H;st_mem_0	;(Yn+1 is copied to mem_0)
	DB	01H;exchange	;X0+X, Y0+Y, Xn+1, Yn+1, Un'
	DB	0E0H;get_mem_0	;X0+X, Y0+Y, Xn+1, Yn+1, Un', Yn+1
	DB	38H;end_calc
	LD	A,(L5C7E);COORDS_hi ;Yn', approximate like Xn', is copied
	CALL	L2D28;STACK_A	;to A and hence to the stack.
	RST	28H;FP_CALC	;X0+X, Y0+Y, Xn+1, Yn+1, Un', Yn+1, Yn'
	DB	03H;subtract	;X0+X, Y0+Y, Xn+1, Yn+1, Un', Vn'
	DB	38H;end_calc
	CALL	L24B7;DRAW_LINE	;The next 'arc' is drawn.
	POP	BC		;The arc-counter is restored.
	DJNZ	L2425;ARC_LOOP	;Jump if more arcs to draw.
L245F:;ARC_END
	RST	28H;FP_CALC	;The co-ordinates of the end
	DB	02H;delete	;of the last arc that was drawn
	DB	02H;delete	;are now deleted from the stack.
	DB	01H;exchange	;Y0+Y, X0+X
	DB	38H;end_calc
	LD	A,(L5C7D);COORDS_lo ;The X-co-ordinate of the end of
	CALL	L2D28;STACK_A	;the last arc that was drawn, say
	RST	28H;FP_CALC	;Xz', is copied to the stack.
	DB	03H;subtract	;Y0+Y, X0+X-Xz'
	DB	01H;exchange	;X0+X-Xz', Y0+Y
	DB	38H;end_calc
	LD	A,(L5C7E);COORDS_hi ;The Y-co-ordinate is obtained.
	CALL	L2D28;STACK_A
	RST	28H;FP_CALC	;X0+X-Xz', Y0+Y, Yz'
	DB	03H;subtract	;X0+X-Xz', Y0+Y-Yz'
	DB	38H;end_calc
L2477:;LINE_DRAW
	CALL	L24B7;DRAW_LINE	;The final arc is drawn to reach X0+X, Y0+Y
				;(or close the circle).
	JP	L0D4D;TEMPS	;Exit, setting temporary colours.

;	THE 'INITIAL PARAMETERS' SUBROUTINE
;This subroutine is called by both CIRCLE and DRAW to set their initial
;params. It is called by CIRCLE with X, Y and the radius Z on the top of the
;stack, reading upwards. It is called by DRAW with its own X, Y, SIN(G/2)
;and Z, as defined in DRAW i. above, on the top of the stack. In what 
;follows the stack is only shown from Z upwards.
;  The subroutine returns in B the arc-count A as explained in both CIRCLE
;and DRAW above, and in mem_0 to mem_5 the quantities G/A, SIN(G/2*A), 0,
;COS(G/A), SIN(G/A) and G. For a circle, G must be taken to be equal to 2*PI.
L247D:;CD_PRMS1
	RST	28H;FP_CALC	;Z
	DB	31H;duplicate	;Z, Z
	DB	28H;sqr		;Z, SQR Z
	DB	34H;stk_data	;Z, SQR Z, 2
	DB	32H;exponent+82 
	DB	00H;00,00,00
	DB	01H;exchange	;Z, 2, SQR Z
	DB	05H;division	;Z, 2/SQR Z
	DB	0E5H;get_mem_5	;Z, 2/SQR Z, G
	DB	01H;exchange	;Z, G, 2/SQR Z
	DB	05H;division	;Z, G*SQR Z/2
	DB	2AH;abs		;Z, G'*SQR Z/2 (G'=mod G)
	DB	38H;end_calc	;Z, G'*SQR Z/2=a1, say
	CALL	L2DD5;FP_TO_A	;A1 to A from the stack, if possible.
	JR	C,L2495;USE_252	;If A1 rounds to 256 or more, use 252.
	AND	0FCH		;4*INT(A1/4) to A.
	ADD	A,4		;Add 4, giving the arc-count A.
	JR	NC,L2497;DRAW_SAVE ;Jump if still under 256.
L2495:;USE_252
	LD	A,0FCH		;Here, just use 252 dec.
	PUSH	AF		;Now save the arc-count.
	CALL	L2D28;STACK_A	;Copy it to calc stack too.
	RST	28H;FP_CALC	;Z, A
	DB	0E5H;get_mem_5	;Z, A, G
	DB	01H;exchange	;Z, G, A
	DB	05H;division	;Z, G/A
	DB	31H;duplicate	;Z, G/A, G/A
	DB	1FH;sin		;Z, G/A, SIN(G/A)
	DB	0C4H;st_mem_4	;(SIN(G/A) is copied to mem_4)
	DB	02H;delete	;Z, G/A
	DB	31H;duplicate	;Z, G/A, G/A
	DB	0A2H;stk_half	;Z, G/A, G/A, .5
	DB	04H;multiply	;Z, G/A, G/2*A
	DB	1FH;sin		;Z, G/A, SIN(G/2*A)
	DB	0C1H;st_mem_1	;(SIN(G/2*A) is copied to mem_1)
	DB	01H;exchange	;Z, SIN(G/2*A), G/A
	DB	0C0H;st_mem_0	;(G/A is copied to mem_0)
	DB	02H;delete	;Z, SIN(G/2*A)=S
	DB	31H;duplicate	;Z, S, S
	DB	04H;multiply	;Z, S*S
	DB	31H;duplicate	;Z, S*S, S*S
	DB	0FH;addition	;Z, 2*S*S
	DB	0A1H;stk_one	;Z, 2*S*S, 1
	DB	03H;subtract	;Z, 2*S*S-1
	DB	1BH;negate	;Z, 1-2*S*S=COS(G/A)
	DB	0C3H;st_mem_3	;(COS(G/A) is copied to mem_3)
	DB	02H;delete      ;Z
	DB	38H;end_calc
	POP	BC          	;Restore the arc-count to B.
	RET			;Finished.

;	THE LINE-DRAWING SUBROUTINE
;This subroutine is called by DRAW to draw an approximation to a straight
;line from the point X0, Y0 held in COORDS to the point X0+X, Y0+Y, where
;the increments X and Y are on the top of the calc stack. The subroutine
;was originally intended for the ZX80 and ZX81 8K ROM, and it is described
;in a BASIC program on page 121 of the ZX81 manual. It is also illustrated
;here in the Circle program in the appendix.
;  The method is to intersperse as many horizontal or vertical steps as are
;needed among a basic set of diagonal steps, using an algorithm that spaces
;the horizontal or vertical steps as evenly as possible.
L24B7:;DRAW_LINE
	CALL	L2307;STK_TO_BC	;ABS Y to B; ABS X to C;
				;SGN Y to D; SGN X to E.
	LD	A,C		;Jump if ABS X is greater than
	CP	B		;or equal to ABS Y, so that the
	JR	NC,L24C4;DL_X_GE_Y ;smaller goes to L, and the
	LD	L,C		;larger (later) goes to H.
	PUSH	DE		;Save diag. step (+-1,+-1) in DE.
	XOR	A		;Insert a vertical step (+-1,0)
	LD	E,A		;into DE (D holds SGN Y).
	JR	L24CB;DL_LARGER	;Now jump to set H.
L24C4:;DL_X_GE_Y
	OR	C		;Return if ABS X and ABS Y
	RET	Z		;are both zero.
	LD	L,B		;The smaller (ABS Y here) goes to L.
	LD	B,C		;ABS X to B here, for H.
	PUSH	DE		;Save the diag. step here too.
	LD	D,0		;Hor. step (0,+-1) to DE here.
L24CB:;DL_LARGER
	LD	H,B		;Larger of ABS X, ABS Y to H now.

;The algorithm starts here. The larger of ABS X and ABS Y, say H, is put
;into A and reduced to INT(H/2). The H-L horizontal or vertical steps and
;L diagonal steps are taken (where L is the smaller of ABS X and ABS Y) in
;this way: L is added to A; if A now equals or exceeds H, it is reduced by
;H and a diagonal step is taken; otherwise a horizontal or vertical step is
;taken. This is repeated H times (B also holds H). Note that meanwhile the
;exchange regs H' and L' are used to hold COORDS.
	LD	A,B		;B to A as well as to H.
	RRA			;A starts at INT(H/2).
L24CE:;D_L_LOOP
	ADD	A,L		;L is added to A.
	JR	C,L24D4;D_L_DIAG ;If 256 or more, jump - diag. step.
	CP	H		;If A is less than H, jump for
	JR	C,L24DB;D_L_HR_VT ;horizontal or vertical step.
L24D4:;D_L_DIAG
	SUB	H		;Reduce A by H.
	LD	C,A		;Restore it to C.
	EXX			;Now use the exchange regs.
	POP	BC		;Diag. step to B'C'.
	PUSH	BC		;Save it too.
	JR	L24DF;D_L_STEP	;Jump to take the step.
L24DB:;D_L_HR_VT
	LD	C,A		;Save A (unreduced) in C.
	PUSH	DE		;Step to stack briefly.
	EXX			;Get exchange regs.
	POP	BC		;Step to B'C' now.
L24DF:;D_L_STEP
	LD	HL,(L5C7D);COORDS ;Now take the step: 1st, COORDS
				;to H'L' as the start point.
	LD	A,B		;Y-step from B' to A.
	ADD	A,H		;Add in H'.
	LD	B,A		;Result to B'.
	LD	A,C		;Now the X-step; it will be tested for
	INC	A		;range (Y will be tested in PLOT).
	ADD	A,L		;Add L' to C' in A, jump on
	JR	C,L24F7;D_L_RANGE ;carry for further test.
	JR	Z,L24F9;REPORT_B ;Zero after no carry denotes
				;X-position -1, out of range.
L24EC:;D_L_PLOT
	DEC	A		;Restore true value to A.
	LD	C,A		;Value to C' for plotting.
	CALL	L22E5;PLOT_SUB	;Plot the step.
	EXX			;Restore main regs.
	LD	A,C		;C back to A to continue alogrithm.
	DJNZ	L24CE;D_L_LOOP	;Loop back for B steps (i.e. H steps).
	POP	DE		;Clear machine stack.
	RET			;Finished.
L24F7:;D_L_RANGE
	JR	Z,L24EC;D_L_PLOT ;Zero after carry denotes X-position
				;255, in range.

;Report B - Integer out of range
L24F9:;REPORT_B
	RST	8;ERROR_1	;Call the error handling
	DB	0AH		;routine.

