

;	THE KEYBOARD ROUTINES

;	THE 'KEYBOARD SCANNING' SUBROUTINE
;This very important subroutine is called by both the main keyboard subr
;and the INKEY$ routine (in SCANNING).
;  In all instances the E reg is returned with a value in the range 00-27,
;the value being different for each of the 40 keys of the keyboard, or the
;value 0FFh for no key.
;  The D reg is returned with a value that indicates which single shift
;key is being pressed. If both shift keys are being pressed then the D and
;E regs are returned with the values for the CAPS SHIFT and SYMBOL SHIFT
;respectively.
;  If no key is being pressed then the DE reg pair is returned holding FFFF.
;  The zero flag is returned reset if more than two keys are being pressed,
;or neither key of a pair of keys is a shift key.
L028E:;KEY_SCAN
	LD	L,2FH		;The initial value for each line will be
				;2F,2E,...,28. (8 lines)
	LD	DE,0FFFFH	;Init DE to 'no-key'.
	LD	BC,0FEFEH	;C=port address, B=counter.
L0296:;KEY_LINE
	IN	A,(C)
	CPL			;A pressed key in the line will set its
	AND	1FH		;respective bit(from b0-outer key to b4-inner).
	JR	Z,L02AB;KEY_DONE;Jump forward if none of the 5 keys in the line
				;are being pressed.
	LD	H,A		;The key-bits go to the H reg
	LD	A,L		;whilst the initial key value is fetched.
L029F:;KEY_3KEYS
	INC	D		;If 3 keys are being pressed on the keyboard
	RET	NZ		;then the D reg will no longer hold FF,
				;so return.
L02A1:;KEY_BITS
	SUB	8		;Repeatedly subtract '8' from the present key
	SRL	H		;value until a key-bit is found.
	JR	NC,L02A1;KEY_BITS
	LD	D,E		;Copy any earlier key value to the D reg.
	LD	E,A		;Pass the new key value to the E reg.
	JR	NZ,L029F;KEY_3KEYS  ;If there is a second, or possibly a third,
				;pressed key in this line then jump back.
L02AB:;KEY_DONE
	DEC	L		;The line has been scanned so the initial key
				;value is reduced for the next pass.
	RLC	B		;The counter is shifted and the
	JR	C,L0296;KEY_LINE;jump taken if there are still lines to be
				;scanned.
;Four tests are now made.
	LD	A,D		;Accept any key value which still has the D reg
	INC	A		;holding FF. i.e. A single key pressed
	RET	Z		;or 'no-key'.
	CP	28H		;Accept the key value for a pair of keys if the
	RET	Z		;'D' key is CAPS SHIFT.
	CP	19H		;Accept the key value for a pair of keys if the
	RET	Z		;'D' key is SYMBOL SHIFT.
	LD	A,E		;It is however possible for the 'E' key of a
	LD	E,D		;pair to be SYMBOL SHIFT - so this has to be
	LD	D,A		;considered.
	CP	18H		;Return with the zero flag set if it was SYMBOL
	RET			;SHIFT and 'another key', otherwise reset.

;	THE 'KEYBOARD' SUBROUTINE
;This subroutine is called on every occasion that a maskable interrupt occurs.
;In normal operation this will happen once every 20ms. The purpose of this subr
;is to scan the keyboard and decode the key value. The code produced will, if
;the 'repeat' status allows it, be passed to the system var LAST_K. When a
;code is put into this system var bit 5 of FLAGS is set to show that a
;'new' key has been pressed.
L02BF:;KEYBOARD
	CALL	L028E;KEY_SCAN	;Fetch a key value in the DE reg pair but
	RET	NZ		;return immediately if the zero flag is reset.

;A double system of 'KSTATE system variables' (KSTATE0-3 and KSTATE4-7) is used
;from now on.
;  The two sets allow for the detection of a new key being pressed (using one
;set) whilst still within the 'repeat period' of the previous key to have been
;pressed (details in the other set).
;  A set will only become free to handle a new key if the key is held down for
;about 1/10 th. of a second. i.e. Five calls to KEYBOARD.

	LD	HL,L5C00	;Start with KSTATE0.
L02C6:;K_ST_LOOP
	BIT	7,(HL)		;Jump forward if a 'set if free';
	JR	NZ,L02D1	;i.e. KSTATE0/4 holds FF.
	INC	HL		;However if the set is not free
	DEC	(HL)		;decrease its '5 call counter'
	DEC	HL		;and when it reaches zero signal
	JR	NZ,L02D1;K_CH_SET ;the set is free.
	LD	(HL),0FFH

;After considering the 1st set change the pointer and consider the 2nd set.
L02D1:;K_CH_SET
	LD	A,L		;Fetch the low byte of the address
	LD	HL,L5C04;KSTATE4 ;and jump back if the second set has still
	CP	L		;to be considered.
	JR	NZ,L02C6;K_ST_LOOP

;Return now if the key value indicates 'no-key' or a shift key only.
	CALL	L031E;K_TEST	;Make the necessary tests and return if needed.
	RET	NC		;Also change the key value to a 'main code'.

;A key stroke that is being repeated (held down) is now separated from a new
;key stroke.
	LD	HL,L5C00	;Look first at KSTATE0.
	CP	(HL)		;Jump forward if the codes
	JR	Z,L0310;K_REPEAT;match - indicating a repeat.
	EX	DE,HL		;Save the addr of KSTATE0.
	LD	HL,L5C04	;Now look at KSTATE4.
	CP	(HL)		;Jump forward if the codes
	JR	Z,L0310;K_REPEAT;match - indicating a repeat.

;But a new key will not be accepted unless one of the sets of KSTATE system
;variables is 'free'.
	BIT	7,(HL)		;Consider the second set.
	JR	NZ,L02F1;K_NEW	;Jump forward if 'free'.
	EX	DE,HL		;Now consider the first set.
	BIT	7,(HL)		;Continue if the set if 'free' but
	RET	Z		;exit from the KEYBOARD subroutine if not.

;The new key is to be accepted. But before the system variable LAST_K can be
;filled, the KSTATE system variables, of the set being used, have to be
;initialised to handle any repeats and the key's code has to be decoded.
L02F1:;K_NEW
	LD	E,A		;The code is passed to the E reg
	LD	(HL),A		;and to KSTATE0/4.
	INC	HL		;The '5 call counter' for this set
	LD	(HL),5		;is reset to '5'.
	INC	HL		;The third system variable of the set holds the
	LD	A,(L5C09)	;REPDEL value
	LD	(HL),A		;(normally 0.7 secs).
	INC	HL		;Point to KSTATE3/7.

;The decoding of a 'main code' depends upon the present state of MODE,
;bit 3 of FLAGS and the 'shift byte'.
	LD	C,(IY+07H)	;Fetch MODE.
	LD	D,(IY+01H)	;Fetch FLAGS.
	PUSH	HL		;Save the pointer whilst the
	CALL	L0333;K_DECODE	;'main code' is decoded.
	POP	HL
	LD	(HL),A		;The final code value is saved in KSTATE3/7;
			       ;from where it is collected in case of a repeat.
L0308:;K_END
	LD	(L5C08),A	;Enter the final code value into LAST_K
	SET	5,(IY+01H);FLAGS ;and signal a 'new key'.
	RET			;Finally return.

;	THE 'REPEATING KEY' SUBROUTINE
;A key will 'repeat' on the first occasion after the delay period - REPDEL
;(normally 0.7 secs) and on subsequent occasions after the delay period -
;REPPER (normally 0.1 secs).
L0310:;K_REPEAT
	INC	HL		;Point to the '5 call counter'
	LD	(HL),5		;of the set being used and reset it to '5'.
	INC	HL		;Point to the third system variable -
	DEC	(HL)		;REPDEL/REPPER, and decrement it.
	RET	NZ		;Exit from the KEYBOARD subroutine if the
				;delay period has not passed.
	LD	A,(L5C0A);REPPER ;However once it has passed the delay period
	LD	(HL),A		;for the next repeat is to be REPPER.
	INC	HL		;The repeat has been accepted so the final
	LD	A,(HL)		;code value is fetched from KSTATE3/7 and
	JR	L0308;K_END	;passed to K_END.

;	THE 'K_TEST' SUBROUTINE
;The key value is tested and a return made if 'no-key' or 'shift only';
;otherwise the 'main code' for that key is found.
L031E:;K_TEST
	LD	B,D		;Copy the shift byte.
	LD	D,0		;Clear the D reg for later.
	LD	A,E		;Move the key number.
	CP	27H		;Return now if the key was
	RET	NC		;'CAPS SHIFT' only or 'no-key'.
	CP	18H		;Jump forward unless the 'E' key
	JR	NZ,L032C;K_MAIN	;was SYMBOL SHIFT.
	BIT	7,B		;However accept SYMBOL SHIFT and another key;
	RET	NZ		;return with SYMBOL SHIFT only.

;The 'main code' is found by indexing into the main key table.
L032C:;K_MAIN
	LD	HL,L0205	;The base address of the table.
	ADD	HL,DE		;Index into the table
	LD	A,(HL)		;and fetch the 'main code'.
	SCF			;Signal 'valid keystroke'
	RET			;before returning.

;	THE 'KEYBOARD DECODING' SUBROUTINE
;This subroutine is entered with the 'main code' in the E reg, the value of
;FLAGS in the D reg, the value of MODE in the C reg and the 'shift byte' in
;the B reg.
;  By considering these four values and referring, as necessary, to the six
;key tables a 'final code' is produced. This is returned in the A reg.
L0333:;K_DECODE
	LD	A,E		;Copy the 'main code'.
	CP	'9'+1		;Jump forward if a digit key is being
	JR	C,L0367;K_DIGIT	;considered; also SPACE, ENTER & both shifts.
	DEC	C		;Decrement the MODE value.
	JP	M,L034F;K_KLC_LET ;Jump forward, as needed, for modes 'K',
	JR	Z,L0341;K_E_LET	;'L', 'C' & 'E'.

;Only 'graphics' mode remains and the 'final code' for letter keys in graphics
;mode is computed from the 'main code'.
	ADD	A,4FH		;Add the offset.
	RET			;Return with the 'final code'.

;Letter keys in extended mode are considered next.
L0341:;K_E_LET
	LD	HL,L01EB;TABB-'A' ;The base addr for table 'b'.
	INC	B		;Jump forward to use this table if neither
	JR	Z,L034A;K_LOOK_UP ;shift key is being pressed.
	LD	HL,L0205;TABC-'A' ;Otherwise use the base addr for table 'c'.

;Key tables 'b-f' are all served by the following look-up routine. In all
;cases a 'final code' is found and returned.
L034A:;K_LOOK_UP
	LD	D,0
	ADD	HL,DE		;Index into required table
	LD	A,(HL)		;and fetch the 'final code'.
	RET			;Then return.

;Letter keys in 'K', 'L' or 'C' modes are now considered. But first the
;special SYMBOL SHIFT codes have to be dealt with.
L034F:;K_KLC_LET
	LD	HL,L0229;TABE-'A' ;The base addr for table 'e'.
	BIT	0,B		;Jump back if using the SYMBOL SHIFT key
	JR	Z,L034A;K_LOOK_UP ;and a letter key.
	BIT	3,D		;Jump forward if currently in 'K' mode.
	JR	Z,L0364;K_TOKENS
	BIT	3,(IY+30H);FLAGS2 ;If CAPS LOCK is set then
	RET	NZ		;return with the 'main code'.
	INC	B		;Also return in the same manner
	RET	NZ		;if CAPS SHIFT is being pressed.
	ADD	A,20H		;However if lower case codes are required
	RET			;then 20 has to be added to the 'main code'
				;to give the correct 'final code'.

;The 'final code' values for tokens are found by adding A5 to the 'main code'.
L0364:;K_TOKENS
	ADD	A,0A5H
	RET

;Next the digit keys; and SPACE, ENTER & both shifts are considered.
L0367:;K_DIGIT
	CP	'0'		;Proceed only with the digit keys.
	RET	C		;i.e. Return with SPACE (20), ENTER (0D),
				;both shifts (0E).
	DEC	C		;Now separate the digit keys into three
				;groups - according to mode.
	JP	M,L039D;K_KLC_DGT ;Jump with 'K', 'L' & 'C' modes;
	JR	NZ,L0389;K_GRA_DGT ;and also with 'G' mode.
				;Continue with 'E' mode.
	LD	HL,L0254;TABF-'0' ;The base addr of table 'f'.
	BIT	5,B		;Use this table for SYMBOL SHIFT & a digit
	JR	Z,L034A;K_LOOK_UP ;key in extended mode.
	CP	'8'		;Jump forward with digit keys
	JR	NC,L0382;K_8_&_9 ;'8' and '9'.

;The digit keys '0' to '7' in extended mode are to give either a 'paper
;colour code' or an 'ink colour code' depending on the use of CAPS SHIFT.
	SUB	20H		;Reduce the range 30-37 giving 10-17.
	INC	B		;Return with this 'paper colour code' if the
	RET	Z		;CAPS SHIFT is not being used.
	ADD	A,8		;But if it is then the range is to be 18 to 1F
	RET			;instead - indicating an 'ink colour code'.

;The digit keys '8' and '9' are to give 'BRIGHT' & 'FLASH' codes.
L0382:;K_8_&_9
	SUB	36H		;38-39 go to 02-03.
	INC	B		;Return with these codes if CAPS SHIFT is not
	RET	Z		;being used. (These are 'BRIGHT' codes.)
	ADD	A,0FEH		;Subtract '2' if CAPS SHIFT is being used;
	RET			;giving 00&01 (as 'FLASH' codes).

;The digit keys in graphics mode are to give the block graphic chars (80-8F),
;the GRAPHICS code (0F) and the DELETE code (0C).
L0389:;K_GRA_DGT
	LD	HL,L0230;TABD-'0' ;The base addr of table 'd'.
	CP	'9'		;Use this table directly for both digit key
	JR	Z,L034A;K_LOOK_UP ;'9' that is to give GRAPHICS,
	CP	'0'		;and digit key '0'
	JR	Z,L034A;K_LOOK_UP ;that is to give DELETE.
	AND	7		;For keys '1' to '8' make the range 80-87.
	ADD	A,80H
	INC	B		;Return with a value from this range if
	RET	Z		;neither shift key is being pressed.
	XOR	0FH		;But if 'shifted' make the range 88-8F.
	RET

;Finally consider the digit keys in 'K', 'L' & 'C' modes.
L039D:;K_KLC_DGT
	INC	B		;Return directly if neither shift key is
	RET	Z		;being used. (Final codes 30-39.)
	BIT	5,B		;Use table 'd' if the CAPS SHIFT key
	LD	HL,L0230;TABD-'0' ;is also being pressed.
	JR	NZ,L034A;K_LOOK_UP

;The codes for the various digit keys and SYMBOL SHIFT can now be found.
	SUB	10H		;Reduce the range to give 20-29.
	CP	22H		;Separate the '@' char from the others.
	JR	Z,L03B2;K_@_CHAR
	CP	' '		;The '_' char has also to be separated.
	RET	NZ		;Return now with the 'final codes' 21,23-29.
	LD	A,5FH		;Give the '_' char a code of 5F.
	RET
L03B2:;K_@_CHAR
	LD	A,40H		;Give the '@' char a code of 40.
	RET

