

;	THE SCREEN & PRINTER HANDLING ROUTINES

;	THE 'PRINT-OUT' ROUTINES
;All of the printing to the main part of the screen, the lower part of the
;screen and the printer is handled by this set of routines.
;  The PRINT_OUT routine is entered with A reg holding the code for a control
;char, a printable char or a token.
L09F4:;PRINT_OUT
	CALL	L0B03;PO_FETCH	;The current print position.
	CP	' '		;If the code represents a printable char
	JP	NC,L0AD9;PO_ABLE;then jump.
	CP	6		;Print a question mark for codes
	JR	C,L0A69;PO_QUEST;in range 00-05.
	CP	18H		;And also for codes 18-1F.
	JR	NC,L0A69;PO_QUEST
	LD	HL,L0A0B;CTRLTAB-6 ;Base of 'control' table.
	LD	E,A
	LD	D,0
	ADD	HL,DE		;Index into the table
	LD	E,(HL)		;and fetch the offset.
	ADD	HL,DE		;Add the offset and make
	PUSH	HL		;an indirect jump to the
	JP	L0B03;PO_FETCH	;appropriate subroutine.
;OFFSETS TO ROUTINES
L0A11:;CTRLTAB
	DB	4EH,57H		;PRINT comma		EDIT
	DB	10H,29H		;cursor left		cursor right
	DB	54H,53H		;cursor down		cursor up
	DB	52H,37H		;DELETE			ENTER
	DB	50H,4FH		;unused			unused
	DB	5FH,5EH		;INK control		PAPER control
	DB	5DH,5CH		;FLASH control		BRIGHT control
	DB	5BH,5AH		;INVERSE control	OVER control
	DB	54H,53H		;AT control		TAB control

;	THE 'CURSOR LEFT' SUBROUTINE
;The subroutine is entered with the B reg holding the current line number
;and the C reg with the current column number.
L0A23:;PO_BACK_1
	INC	C		;Move leftwards by one column.
	LD	A,22H		;Accept the change unless
	CP	C		;up against the lefthand side.
	JR	NZ,L0A3A;PO_BACK_3
	BIT	1,(IY+01H);FLAGS ;If dealing with the printer
	JR	NZ,L0A38;PO_BACK_2 ;jump forward.
	INC	B		;Go up one line.
	LD	C,2		;Set column value.
	LD	A,18H		;Test against top line.
	CP	B		;Note: This ought to be 19H.
	JR	NZ,L0A3A;PO_BACK_3 ;Accept the change unless at the top
				;of the screen.
	DEC	B		;Unacceptable so down a line.
L0A38:;PO_BACK_2
	LD	C,21H		;Set to lefthand column.
L0A3A:;PO_BACK_3
	JP	L0DD9;CL_SET	;Make an indirect return via CL_SET&PO_STORE.

;	THE 'CURSOR RIGHT' SUBROUTINE
;This subroutine performs an operation identical to the BASIC statement:
;PRINT OVER 1;CHR$ 32;
L0A3D:;PO_RIGHT
	LD	A,(L5C91)	;Fetch P_FLAG and save it
	PUSH	AF		;on the machine stack.
	LD	(IY+57H),1	;Set P_FLAG to OVER 1.
	LD	A,' '		;A 'space'.
	CALL	L0B65;PO_CHAR	;Print the char.
	POP	AF		;Fetch the old value 
	LD	(L5C91),A	;of P_FLAG.
	RET			;Finished. Note: The programmer has forgotten
				;to exit via PO_STORE. 

;	THE 'CARRIAGE RETURN' SUBROUTINE
;If the printing being handled is going to the printer then a carriage return
;char leads to the printer buffer being emptied. If the printing is to the
;screen then a test for 'scroll?' is made before decreasing the line number.
L0A4F:;PO_ENTER
	BIT	1,(IY+01H);FLAGS ;Jump forward if handling
	JP	NZ,L0ECD;COPY_BUFF ;the printer.
	LD	C,21H		;Set to lefthand column.
	CALL	L0C55;PO_SCR	;Scroll if necessary.
	DEC	B		;Now down a line.
	JP	L0DD9;CL_SET	;Make an indirect return via CL_SET&PO_STORE.

;	THE 'PRINT COMMA' SUBROUTINE
;The current column value is manipulated and the A reg set to hold 00 (for
;TAB 0) or 10 (for TAB 16).
L0A5F:;PO_COMMA
	CALL	L0B03;PO_FETCH	;Why again?
	LD	A,C		;Current column number.
	DEC	A		;Move rightwards by two columns
	DEC	A		;and then test.
	AND	10H		;The A reg will be 00 or 10.
	JR	L0AC3;PO_FILL	;Exit via PO_FILL.

;	THE 'PRINT A QUESTION MARK' SUBROUTINE
;A question mark is printed whenever an attempt is made to print an
;unprintable code.
L0A69:;PO_QUEST	
	LD	A,'?'
	JR	L0AD9;PO_ABLE	;Now print this char instead.

;	THE 'CONTROL CHAR WITH OPERANDS' ROUTINE
;The control chars from INK to OVER require a single operand whereas the
;control chars AT & TAB are required to be followed by two operands.
;  The present routine leads to the ctrl char code being saved in TVDATA-lo,
;the 1st operand in TVDATA-hi or the A reg if there is only a single operand
;required, and the 2nd operand in the A reg.
L0A6D:;PO_TV_2
	LD	DE,L0A87;PO_CONT;Save the 1st operand in TVDATA-hi and
	LD	(L5C0F),A	;change the addr of the 'output' routine
	JR	L0A80;PO_CHANGE	;to PO_CONT.

;Enter here when handling the chars AT & TAB.
L0A75:;PO_2_OPER
	LD	DE,L0A6D;PO_TV_2;The char code will be saved in TVDATA-lo
	JR	L0A7D;PO_TV_1	;and the addr of the 'output' routine
				;changed to PO_TV_2.

;Enter here when handling the colour items - INK to OVER.
L0A7A:;PO_1_OPER
	LD	DE,L0A87	;The 'output' routine is to be changed
				;to PO_CONT.
L0A7D:;PO_TV_1
	LD	(L5C0E),A	;Save the control char code in TVDATA-lo.

;The current 'output' routine addr is changed temporarily.
L0A80:;PO_CHANGE
	LD	HL,(L5C51);CURCHL ;HL will point to the 'output' routine addr.
	LD	(HL),E		;Enter the new 'output' routine addr
	INC	HL		;and thereby force the next char code
	LD	(HL),D		;to be considered as an operand.
	RET

;Once the operands have been collected the routine continues.
L0A87:;PO_CONT
	LD	DE,L09F4	;Restore the original addr for PRINT_OUT.
	CALL	L0A80;PO_CHANGE
	LD	HL,(L5C0E);TVDATA ;Fetch the control code and the 1st
				;operand if there are indeed two operands.
	LD	D,A		;The 'last' operand and the
	LD	A,L		;control code are moved.
	CP	16H		;Jump forward if handling
	JP	C,L2211;CO_TEMPS;INK to OVER.
	JR	NZ,L0AC2;PO_TAB	;Jump forward if handling TAB.

;Now deal with the AT control char.
	LD	B,H		;The line number.
	LD	C,D		;The column number.
	LD	A,1FH		;Reverse the column number;
	SUB	C		;i.e. 00-1F becomes 1F-00.
	JR	C,L0AAC;PO_AT_ERR ;Must be in range.
	ADD	A,2		;Add in the offset to give
	LD	C,A		;C holding 21..02.
	BIT	1,(IY+01H);FLAGS ;Jump forward if handling
	JR	NZ,L0ABF;PO_AT_SET ;the printer.
	LD	A,16H		;Reverse the line number:
	SUB	B		;i.e. 00-15 becomes 16-01.
L0AAC:;PO_AT_ERR	
	JP	C,L1E9F;REPORT_B;If appropriate jump forward.
	INC	A		;The range 16-01 becomes 17-02.
	LD	B,A
	INC	B		;And now 18-03.
	BIT	0,(IY+02H);TV_FLAG; If printing in the lower part of
	JP	NZ,L0C55;PO_SCR	;the screen the consider whether scrolling
				;is needed.
	CP	(IY+31H);DF_SZ	;Give report 5 - Out of screen,
	JP	C,L0C86;REPORT_5;if required.
L0ABF:;PO_AT_SET
	JP	L0DD9;CL_SET	;Return via CL_SET & PO_STORE.

;And the TAB control char.
L0AC2:;PO_TAB
	LD	A,H		;Fetch the 1st operand.
L0AC3:;PO_FILL
	CALL	L0B03;PO_FETCH	;The current print position.
	ADD	A,C		;Add the current column value.
	DEC	A		;Find how many 'spaces', modulo 32,
	AND	1FH		;are required and return
	RET	Z		;if result is zero.
	LD	D,A		;Use D as the counter.
	SET	0,(IY+01H);FLAGS ;Suppress 'leading space'.
L0AD0:;PO_SPACE
	LD	A,' '		;Print 'D number' of
	CALL	L0C3B;PO_SAVE	;spaces.
	DEC	D
	JR	NZ,L0AD0;PO_SPACE
	RET			;Now finished.

;	PRINTABLE CHAR CODES
;The required char (or chars) is printed by calling PO_ANY followed
;by PO_STORE.
L0AD9:;PO_ABLE
	CALL	L0B24;PO_ANY	;Print the char(s) and continue
				;into PO_STORE.

;	THE 'POSITION STORE' SUBROUTINE
;The new position's 'line & column' values and the 'pixel' addr are stored
;in the appropriate system variables.
L0ADC:;PO_STORE
	BIT	1,(IY+01H);FLAGS ;Jump forward if handling
	JR	NZ,L0AFC;PO_ST_PR ;the printer.
	BIT	0,(IY+02H);TV_FLAG ;Jump forward if handling
	JR	NZ,L0AF0;PO_ST_E;the lower part of the screen.
	LD	(L5C88),BC;S_POSN ;Save the values that relate to the
	LD	(L5C84),HL;DF_CC ;main part of the screen.
	RET
L0AF0:;PO_ST_E
	LD	(L5C8A),BC;S_POSNL ;Save the values that relate to the
	LD	(L5C82),BC;ECHO_E ;lower part of the screen.
	LD	(L5C86),HL;DF_CCL
	RET
L0AFC:;PO_ST_PR
	LD	(IY+45H),C;P_POSN ;Save the values that relate
	LD	(L5C80),HL;PR_CC ;to the printer buffer.
	RET

;	THE 'POSITION FETCH' SUBROUTINE
;The current position's parameters are fetched from the appropriate system
;variables.
L0B03:;PO_FETCH
	BIT	1,(IY+01H)	;FLAGS: Jump forward if handling
	JR	NZ,L0B1D;PO_F_PR;the printer.
	LD	BC,(L5C88);S_POSN ;Fetch the values relating to the
	LD	HL,(L5C84);DF_CC ;main part of the screen
	BIT	0,(IY+02H);TV_FLAG ;and return if this was
	RET	Z		;the intention.
	LD	BC,(L5C8A);S_POSNL ;Otherwise fetch the values relating
	LD	HL,(L5C86);DF_CCL ;to the lower part of the screen.
	RET
L0B1D:;PO_F_PR
	LD	C,(IY+45H);P_POSN ;Fetch the values relating to the
	LD	HL,(L5C80);PR_CC ;printer buffer.
	RET

;	THE 'PRINT ANY CHAR(S)' SUBROUTINE
;Ordinary char codes, token codes and user-defined graphic codes, and
;graphic codes are dealt with separately.
L0B24:;PO_ANY
	CP	80H		;Jump forward with ordinary
	JR	C,L0B65;PO_CHAR	;char codes.
	CP	90H		;Jump forward with token codes
	JR	NC,L0B52;PO_T&UDG ;and UDG codes.
	LD	B,A		;Move the graphic code.
	CALL	L0B38;PO_GR_1	;Construct the graphic form.
	CALL	L0B03;PO_FETCH	;HL has been disturbed so 'fetch' again.
	LD	DE,L5C92;MEMBOT	;Make DE point to the start of the graphic
				;form; i.e. MEMBOT.
	JR	L0B7F;PO_ALL	;Jump forward to print the graphic char.

;Graphic chars are constructed in an Ad Hoc manner in the calculator's memory
;area; i.e. MEM_0 & MEM_1.
L0B38:;PO_GR_1
	LD	HL,L5C92	;This is MEMBOT.
	CALL	L0B3E;PO_GR_2	;In effect call the following subr twice.
L0B3E:;PO_GR_2
	RR	B		;Determine bit 0 (and later bit 2)
	SBC	A,A		;of the graphic code. The A reg will hold
	AND	0FH		;00 or 0F depending on the value of bit in
	LD	C,A		;the code. Save the result in C.
	RR	B		;Determine bit 1 (and later bit 3) of the
	SBC	A,A		;graphic code.
	AND	0F0H		;The A reg will hold 00 or F0.
	OR	C		;The two results are combined.
	LD	C,4		;The A reg holds half the
L0B4C:;PO_GR_3
	LD	(HL),A		;char form and has to be
	INC	HL		;used four times.
	DEC	C		;This is done for the upper
	JR	NZ,L0B4C;PO_GR_3;half of the char form
	RET			;and then the lower.

;Token codes and user-defined graphic codes are now separated.
L0B52:;PO_T&UDG
	SUB	0A5H		;Jump forward with token codes.
	JR	NC,L0B5F;PO_T
	ADD	A,15H		;UDG codes are now 00-0F.
	PUSH	BC		;Save the current position
				;values on the machine stack.
	LD	BC,(L5C7B);UDG	;Fetch the base addr of the UDG
	JR	L0B6A;PO_CHAR_2	;area and jump forward.
L0B5F:;PO_T
	CALL	L0C10;PO_TOKENS	;Now print the token and return
	JP	L0B03;PO_FETCH	;via PO_FETCH.

;The required char form is identified.
L0B65:;PO_CHAR
	PUSH	BC		;The current position is saved.
	LD	BC,(L5C36);CHARS;The base addr of the char area is fetched.
L0B6A:;PO_CHAR_2
	EX	DE,HL		;The print addr is saved.
	LD	HL,L5C3B	;This is FLAGS.
	RES	0,(HL)		;Allow for a leading space.
	CP	' '		;Jump forward if the char
	JR	NZ,L0B76;PO_CHAR_3 ;is not a 'space'.
	SET	0,(HL)		;But 'suppress' if it is.
L0B76:;PO_CHAR_3
	LD	H,0		;Now pass the char code
	LD	L,A		;to the HL reg pair.
	ADD	HL,HL		;The char code is in effect
	ADD	HL,HL		;multiplied by 8.
	ADD	HL,HL
	ADD	HL,BC		;The base addr of the char form is found.
	POP	BC		;The current position is fetched
	EX	DE,HL		;and the base addr passed to the DE reg pair.

;	THE 'PRINT ALL CHARS' SUBROUTINE
;This subroutine is used to print all '8*8' bit chars. On entry the DE reg
;pair holds the base addr of the char form, the HL reg the destination addr
;and the BC reg pair the current 'line & column' values.
L0B7F:;PR_ALL
	LD	A,C		;Fetch the column number.
	DEC	A		;Move one column rightwards.
	LD	A,21H		;Jump forward unless a new
	JR	NZ,L0B93;PR_ALL_1 ;line is indicated.
	DEC	B		;Move down one line.
	LD	C,A		;Column number is 21H.
	BIT	1,(IY+01H);FLAGS;Jump forward if handling
	JR	Z,L0B93;PR_ALL_1;the screen.
	PUSH	DE		;Save the base addr whilst
	CALL	L0ECD;COPY_BUFF	;the printer buffer is
	POP	DE		;emptied.
	LD	A,C		;Copy the new column number.
L0B93:;PR_ALL_1
	CP	C		;Test whether a new line is
	PUSH	DE		;being used. If it is
	CALL	Z,L0C55;PO_SCR	;see if the display requires
	POP	DE		;to be scrolled.

;Now consider the present state of INVERSE & OVER.
	PUSH	BC		;Save the position values and the
	PUSH	HL		;destination addr on the machine stack.
	LD	A,(L5C91)	;Fetch P_FLAG and read bit 0.
	LD	B,0FFH		;Prepare the 'OVER mask' in
	RRA			;the B reg; i.e. OVER 0
	JR	C,L0BA4;PR_ALL_2; = 00 & OVER 1 = FF.
	INC	B
L0BA4:;PR_ALL_2
	RRA			;Read bit 2 of P_FLAG and
	RRA			;prepare the 'INVERSE mask'
	SBC	A,A		;in the C reg; i.e. INVERSE 0
	LD	C,A		; = 00 & INVERSE 1 = FF.
	LD	A,8		;Set the A reg to hold the 'pixel-line'
	AND	A		;counter and clear the carry flag.
	BIT	1,(IY+01H);FLAGS;Jump forward if handling the screen.
	JR	Z,L0BB6;PR_ALL_3
	SET	1,(IY+30H);FLAGS2 ;Signal 'printer buffer no longer empty'.
	SCF			;Set the carry flag to show that
				;the printer is being used.
L0BB6:;PR_ALL_3
	EX	DE,HL		;Exchange the destination addr with the base
				;addr before entering the loop.

;The char can now be printed. Eight passes of the loop are made - one for
;each 'pixel-line'.
L0BB7:;PR_ALL_4
	EX	AF,AF'		;The carry flag is set when using the printer.
				;Save this flag in F'.
	LD	A,(DE)		;Fetch the existing 'pixel-line'.
	AND	B		;Use the 'OVER mask' and then
	XOR	(HL)		;XOR the result with the 'pixel-line'
				;of the char form.
	XOR	C		;Finally consider the 'INVERSE mask'.
	LD	(DE),A		;Enter the result.
	EX	AF,AF'		;Fetch the printer flag and
	JR	C,L0BD3;PR_ALL_6;jump forward if required.
	INC	D		;Update the destination addr.
L0BC1:;PR_ALL_6
	INC	HL		;Update the 'pixel-line' of the char form.
	DEC	A		;Decrease the counter and loop
	JR	NZ,L0BB7;PR_ALL_4 ;back unless it is zero.

;Once the char has been printed the attribute byte is to set as required.
	EX	DE,HL		;Make the H reg hold a correct high addr
	DEC	H		;for the char area.
	BIT	1,(IY+01H);FLAGS;Set the attribute byte only if handling
	CALL	Z,L0BDB;PO_ATTR	;the screen.
	POP	HL		;Restore the original destination addr
	POP	BC		;and the position values.
	DEC	C		;Decrease the column number
	INC	HL		;and increase the destination addr
	RET			;before returning.

;When the printer is being used the destination addr has to be updated in
;increments of 20H.
L0BD3:;PR_ALL_6
	EX	AF,AF'		;Save the 'printer flag' in F'.
	LD	A,20H		;The required increment value.
	ADD	A,E		;Add the value and pass the
	LD	E,A		;result back to the E reg.
	EX	AF,AF'		;Fetch the flag.
	JR	L0BC1;PR_ALL_5	;Jump back into the loop.

;	THE 'SET ATTRIBUTE BYTE' SUBROUTINE
;The appropriate attribute byte is identified and fetched. The new value is
;formed by manipulating the old value, ATTR_T, MASK_T and P_FLAG. Finally
;this new value is copied to the attribute area.
L0BDB:;PO_ATTR
	LD	A,H		;The high byte of the
	RRCA			;destination addr is divided
	RRCA			;by 8 and ANDed with 3 to determine which
	RRCA			;third of the screen is being addressed;
	AND	3               ;i.e. 00, 01 or 02.
	OR	58H		;The high byte for the
	LD	H,A		;attribute area is then formed.
	LD	DE,(L5C8F)	;D holds ATTR_T & E holds MASK_T.
	LD	A,(HL)		;The old attribute value.
	XOR	E		;The values of MASK_T and
	AND	D		;ATTR_T are taken into
	XOR	E		;account.
	BIT	6,(IY+57H);P_FLAG ;Jump forward unless dealing
	JR	Z,L0BFA;PO_ATTR_1 ;with PAPER 9.
	AND	0C7H		;The old paper colour is ignored and depending
	BIT	2,A		;on whether the ink colour is light or dark
	JR	NZ,L0BFA;PO_ATTR_1 ;the new paper colour will be
	XOR	38H		;black (000) or white (111).
L0BFA:;PO_ATTR_1
	BIT	4,(IY+57H);P_FLAG ;Jump forward unless dealing
	JR	Z,L0C08;PO_ATTR_2 ;with INK 9.
	AND	0F8H		;The old ink colour is ignored and depending
	BIT	5,A		;on whether the paper colour is light or dark
	JR	NZ,L0C08;PO_ATTR_2 ;the new ink colour will be black (000)
	XOR	7		;or white (111).
L0C08:;PO_ATTR_3
	LD	(HL),A		;Enter the new attribute value
	RET			;and return.

;	THE 'MESSAGE PRINTING' SUBROUTINE
;This subroutine is used to print messages and tokens. The A reg holds the
;'entry number' of the message or token in a table. The DE reg pair holds
;the base addr of the table.
L0C0A:;PO_MSG
	PUSH	HL		;The high byte of the last entry
	LD	H,0		;on the machine stack is made zero
	EX	(SP),HL		;so as to suppress trailing spaces (see
	JR	L0C14;PO_TABLE	;below). Jump forward.

;Enter here when expanding token codes.
L0C10:;PO_TOKENS
	LD	DE,L0095	;The base addr of the token table.
	PUSH	AF		;Save the code on the stack.
				;Range 00-5A : RND-COPY
;The table is searched and the correct entry printed.
L0C14:;PO_TABLE
	CALL	L0C41;PO_SEARCH	;Locate the required entry.
	JR	C,L0C22;PO_EACH	;Print the message/token.
	LD	A,' '		;A 'space' will be printed
	BIT	0,(IY+01H);FLAGS;before the message/token
	CALL	Z,L0C3B;PO_SAVE	;if required.

;The chars of the message/token are printed in turn.
L0C22:;PO_EACH
	LD	A,(DE)		;Collect a code.
	AND	7FH		;Cancel any 'inverted bit'.
	CALL	L0C3B;PO_SAVE	;Print the char.
	LD	A,(DE)		;Collect the code again.
	INC	DE		;Advance the pointer.
	ADD	A,A		;The 'inverted bit' goes to the carry flag
	JR	NC,L0C22;PO_EACH;and signals the end of the message/token;
				;otherwise jump back.

;Now consider whether a 'trailing space' is required.
	POP	DE		;For messages - D hold 00; for tokens - 00-5A.
	CP	'$'+'$'		;Jump forward if the last
	JR	Z,L0C35;PO_TRSP	;char was a '$'.
	CP	'A'+'A'		;Return if the last char
	RET	C		;was any other before 'A'.
L0C35:;PO_TRSP
	LD	A,D		;Examine the value in D and
	CP	3		;return if it indicates a
	RET	C		;message, RND, INKEY$ or PI. All other
	LD	A,' '		;cases will require 'trailing space'.

;	THE 'PO_SAVE' SUBROUTINE
;The subroutine is for chars to be printed 'recursively'. The appropriate
;pointers are saved whilst 'PRINT_OUT' is called.
L0C3B:;PO_SAVE
	PUSH	DE		;Save the DE reg pair.
	EXX			;Save HL & BC.
	RST	10H;PRINT_A_1	;Print the single char.
	EXX			;Restore HL & BC.
	POP	DE		;Restore DE.
	RET			;Finished.

;	THE 'TABLE SEARCH' SUBROUTINE
;The subroutine return with the DE reg pair pointing to the initial char of
;the required entry and the carry flag reset if a 'leading space' is to be
;considered.
L0C41:;PO_SEARCH
	PUSH	AF		;Save the 'entry number'.
	EX	DE,HL		;HL now holds the base addr.
	INC	A		;Make the range 01-XX.
L0C44:PO_STEP
	BIT	7,(HL)		;Wait for an 'inverted
	INC	HL		;char'.
	JR	Z,L0C44;PO_STEP
	DEC	A		;Count through the entries
	JR	NZ,L0C44;PO_STEP;until the correct one is found.
	EX	DE,HL		;DE points to the initial char.
	POP	AF		;Fetch the 'entry number' and
	CP	' '		;return with carry set for the
	RET	C		;1st 32 entries.
	LD	A,(DE)		;However if the initial
	SUB	'A'		;char is a letter then a
	RET			;leading space may be needed.

