
;	THE 'TEST FOR SCROLL' SUBROUTINE
;This subr is used whenever there might be the need to scroll the display.
;This occurs on three occasions: i. when handling a 'carriage return' char;
;ii. when using ... INPUT line; iii. when the current line is full and the
;next line has to be used.
;  On entry the B reg holds the line number under test.
L0C55:;PO_SCR
	BIT	1,(IY+01H);FLAGS;Return immediately if the
	RET	NZ		;printer is being used.
	LD	DE,L0DD9	;Pre-load the machine stack
	PUSH	DE		;with the addr of 'CL_SET'.
	LD	A,B		;Transfer the line number.
	BIT	0,(IY+02H);TV_FLAG ;Jump forward if considering
	JP	NZ,L0D02;PO_SCR_4  ;'INPUT ... AT ...'.
	CP	(IY+31H);DF_SZ	;Return, via CL_SET, if the line number is
	JR	C,L0C86;REPORT_5;greater than the value of DF_SZ; give
	RET	NZ		;report 5 if it is less; otherwise continue.
	BIT	4,(IY+02H);TV_FLAG ;Jump forward unless dealing with
	JR	Z,L0C88;PO_SCR_2   ;an 'automatic listing'.
	LD	E,(IY+2DH);BREG	;Fetch the line counter.	
	DEC	E		;Decrease this counter.
	JR	Z,L0CD2;PO_SCR_3;Jump forward if the listing is to be
	LD	A,0		;scrolled. Otherwise open channel 'K',
	CALL	L1601;CHAN_OPEN	;restore the stack pointer,
	LD	SP,(L5C3F);LIST_SP ;flag that the automatic
	RES	4,(IY+02H);TV_FLAG ;listing has finished and
	RET			;return via CL_SET.

;Report 5 - Out of screen
L0C86:;REPORT_5
	RST	8;ERROR_1	;Call the error handling routine.
	DB	4

;Now consider if the prompt 'scroll?' is required.
L0C88:;PO_SCR_2
	DEC	(IY+52H);SCR_CT	;Decrease the scroll counter and proceed to
	JR	NZ,L0CD2;PO_SCR_3 ;give the prompt only if is becomes zero.

;Proceed to give the prompt message.
	LD	A,18H		;The counter is reset.
	SUB	B
	LD	(L5C8C),A;SCR_CT
	LD	HL,(L5C8F)	;The current values of ATTR_T
	PUSH	HL		;and MASK_T are saved.
	LD	A,(L5C91)	;The current value of P_FLAG
	PUSH	AF		;is saved.
	LD	A,0FDH		;Channel 'K' is opened.
	CALL	L1601;CHAN_OPEN
	XOR	A		;The message 'scroll?' is
	LD	DE,L0CF8	;message '0'. This message is
	CALL	L0C0A;PO_MSG	;now printed.
	SET	5,(IY+02H);TV_FLAG ;Signal 'clear the lower screen
				;after a keystroke'.
	LD	HL,L5C3B	;This is FLAGS.
	SET	3,(HL)		;Signal 'L mode'.
	RES	5,(HL)		;Signal 'no key yet'.
	EXX			;NOTE: DE should be pushed also.
	CALL	L15D4;WAIT_KEY	;Fetch a single key code.
	EXX			;Restore the regs.
	CP	' '		;There is a jump forward to
	JR	Z,L0D00;REPORT_D;REPORT_D - 'BREAK - CONT repeats' - if
	CP	0E2H		;the keystroke
	JR	Z,L0D00;REPORT_D;was 'BREAK', 'STOP', 'N' or
	OR	' '		;'n'; otherwise accept the
	CP	'n'		;keystroke as indicating the need
	JR	Z,L0D00		;to scroll the display.
	LD	A,0FEH		;Open channel 'S'.
	CALL	L1601;CHAN_OPEN
	POP	AF		;Restore the value of
	LD	(L5C91),A	;P_FLAG.
	POP	HL		;Restore the values of ATTR_T
	LD	(L5C8F),HL	;and MASK_T.

;The display is now scrolled.
L0CD2:;PO_SCR_3
	CALL	L0DFE;CL_SC_ALL	;The whole display is scrolled.
	LD	B,(IY+31H);DF_SZ;The line and column numbers
	INC	B		;for the start of the line
	LD	C,21H		;above the lower part of the
	PUSH	BC		;display are found and saved.
	CALL	L0E9B;CL_ADDR	;The corresponding attribute
	LD	A,H		;byte for this char area is
	RRCA			;then found. The HL reg pair
	RRCA			;holds the addr of the byte.
	RRCA
	AND	3
	OR	58H
	LD	H,A

;The line in question will have 'lower part' attribute values and the
;new line at the bottom of the display may have 'ATTR_P' values so the
;attribute values are exchanged.
	LD	DE,L5AE0	;DE points to the 1st attribute byte
				;of the bottom line.
	LD	A,(DE)		;The value is fetched.
	LD	C,(HL)		;The 'lower part' value.
	LD	B,20H		;There are 32 bytes.
	EX	DE,HL		;Exchange the pointers.
L0CF0:;PO_SCR_3A
	LD	(DE),A		;Make the 1st exchange
	LD	(HL),C		;and then proceed to use the
	INC	DE		;same values for the 32
	INC	HL		;attribute bytes of the 2
	DJNZ	L0CF0;PO_SCR_3A	;lines being handled.
	POP	BC		;The line and column numbers of the bottom
	RET			;line of the 'upper part' are fetched
				;before returning.

;The 'scroll?' message.
L0CF8:	
	DB	80H
	DB	'scroll','?'+80H

;Report D - BREAK-CONT repeats
L0D00:;REPORT_D
	RST	8;ERROR_1	;Call the error handling routine.
	DB	0CH

;The lower part of the display is handled as follows:
L0D02:;PO_SCR_4
	CP	2		;The 'out of screen' error is
	JR	C,L0C86;REPORT_5;given if the lower part is
	ADD	A,(IY+31H);DF_SZ;going to be 'too large' and a
	SUB	19H		;return made if scrolling is
	RET	NC		;unnecessary.
	NEG			;The A reg will now hold 'the number of
				;scrolls to be made'.
	PUSH	BC		;The line and column numbers are now saved.
	LD	B,A		;The 'scroll number',
	LD	HL,(L5C8F)	;ATTR_T, MASK_T
	PUSH	HL
	LD	HL,(L5C91)	;& P_FLAG are all saved.
	PUSH	HL
	CALL	L0D4D;TEMPS	;The 'permanent' colour items aro to be used.
	LD	A,B		;The 'scroll number' is fetched.

;The lower part of the screen is now scrolled 'A' number of times.
L0D1C:;PO_SCR_4A
	PUSH	AF		;Save the 'number'.
	LD	HL,L5C6B	;This is DF_SZ.
	LD	B,(HL)		;The value in DF_SZ is
	LD	A,B		;incremented; the B reg
	INC	A		;set to hold the former value and
	LD	(HL),A		;the A reg the new value.
	LD	HL,L5C89	;This is S_POSN_hi.
	CP	(HL)		;The jump is taken if only the lower parts of
	JR	C,L0D2D;PO_SCR_4B ;display is to be scrolled. (B = old DF_SZ)
	INC	(HL)		;Otherwise S_POSN_hi is incremented and
	LD	B,18H		;the whole display scrolled. (B = 18)
L0D2D:;PO_SCR_4B
	CALL	L0E00;CL_SCROLL	;Scroll 'B' lines.
	POP	AF		;Fetch and decrement the
	DEC	A		;'scroll number'.
	JR	NZ,L0D1C;PO_SCR_4A ;Jump back until finished.
	POP	HL		;Restore the value of
	LD	(IY+57H),L	;P_FLAG.
	POP	HL		;Restore the values of MASK_T
	LD	(L5C8F),HL	;& ATTR_T.
	LD	BC,(L5C88)	;In case S_POSN has been
	RES	0,(IY+02H);TV_FLAG ;changed CL_SET is called to
	CALL	L0DD9;CL_SET	;give a matching value to DF_CC.
	SET	0,(IY+02H);TV_FLAG ;Reset the flag to indicate that the
	POP	BC		;lower screen is being handled, fetch the
	RET			;line and column numbers, and then return.

;	THE 'TEMPORARY COLOUR ITEMS' SUBROUTINE
;This is a most important subroutine. It is used whenever the 'permanent'
;details are required to be copied to the 'temporary' system variables. 1st
;ATTR_T & MASK_T are considered.
L0D4D:;TEMPS
	XOR	A		;A is set to hold 00.
	LD	HL,(L5C8D)	;The current values of ATTR_P&MASK_P are
	BIT	0,(IY+02H);TV_FLAG ;fetched. Jump forward if handling
	JR	Z,L0D5B;TEMPS_1	;the main part of the screen.
	LD	H,A		;Otherwise use 00 and the
	LD	L,(IY+0EH)	;value in BORDCR instead.	
L0D5B:;TEMPS_1
	LD	(L5C8F),HL	;Now set ATTR_T & MASK_T.

;Next P_FLAG is considered.
	LD	HL,L5C91	;This is P_FLAG.
	JR	NZ,L0D65;TEMPS_2;Jump forward if dealing with
				;the lower part of the screen (A = 00).
	LD	A,(HL)		;Otherwise fetch the value of P_FLAG and
	RRCA			;move the odd bits to the even bits.
L0D65:;TEMPS_2
	XOR	(HL)		;Proceed to copy the even bits
	AND	01010101B	;of A to P_FLAG.
	XOR	(HL)
	LD	(HL),A
	RET

;	THE 'CLS COMMAND' ROUTINE
;In the 1st instance the whole of the display is 'cleared' - the data bytes
;are all reset and the attribute bytes are set to equal the value in ATTR_P -
;then the lower part of the display is reformed.
L0D6B:;CLS
	CALL	L0DAF;CL_ALL	;The whole of the display is 'cleared'.
L0D6E:;CLS_LOWER
	LD	HL,L5C3C	;This is TV_FLAG.
	RES	5,(HL)		;Signal 'do not clear the lower screen
	SET	0,(HL)		;after keystroke'. Signal 'lower part'.
	CALL	L0D4D;TEMPS	;Use the permanent values, i.e. ATTR_T
				;is copied from BORDCR.
	LD	B,(IY+31H);DF_SZ;The lower part of the screen is now
	CALL	L0E44;CL_LINE	;'cleared' with these values.

;With the exception of the attribute bytes for lines '22' & '23' the
;attribute bytes for the lines in the lower part of the display will
;need to be made equal to ATTR_P.
	LD	HL,L5AC0	;Attribute byte at start of line '22'.
	LD	A,(L5C8D)	;Fetch ATTR_P.
	DEC	B		;The line counter.
	JR	L0D8E;CLS_3
L0D87:;CLS_1
	LD	C,20H		;20H chars per line.
L0D89:;CLS_2
	DEC	HL		;Go back along the line setting
	LD	(HL),A		;the attribute bytes.
	DEC	C
	JR	NZ,L0D89;CLS_2
L0D8E:;CLS_3
	DJNZ	L0D87;CLS_1	;Loop back until finished.

;The size of the lower part of the display can now be fixed.
	LD	(IY+31H),2;DF_SZ;It will be 2 lines in size.

;It now remains for the following 'house keeping' tasks to be performed.
L0D94:;CL_CHAN
	LD	A,0FDH		;Open channel 'K'.
	CALL	L1601;CHAN_OPEN
	LD	HL,(L5C51);CURCHL  ;Fetch the addr of the current
	LD	DE,L09F4;PRINT_OUT ;channel and make the output addr
	AND	A		;PRINT_OUT
L0DA0:;CL_CHAN_A
	LD	(HL),E		;and the input address KEY_INPUT.
	INC	HL
	LD	(HL),D
	INC	HL
	LD	DE,L10A8;KEY_INPUT
	CCF			;1st the output addr
	JR	C,L0DA0;CL_CHAN_A ;then the input addr.
	LD	BC,1721H	;As the lower part of the display is being
				;handled the 'lower print line' will be
	JR	L0DD9;CL_SET	;line 23. Return via CL_SET.

;	THE 'CLEARING THE WHOLE DISPLAY AREA' SUBROUTINE
;This subroutine is called from: i. the CLS command routine, ii. the main
;execution routine, and iii. the automatic listing routine.
L0DAF:;CL_ALL
	LD	HL,0		;The system variable
	LD	(L5C7D),HL	;COORDS is reset to zero.
	RES	0,(IY+30H);FLAGS2 ;Signal 'the screen is clear'.
	CALL	L0D94;CL_CHAN	;Perform the 'house keeping' tasks.
	LD	A,0FEH		;Open channel 'S'.
	CALL	L1601;CHAN_OPEN
	CALL	L0D4D;TEMPS	;Use the 'permanent' values.
	LD	B,18H		;Now 'clear' the 24. lines
	CALL	L0E44;CL_LINE	;of the display.
	LD	HL,(L5C51);CURCHL ;Ensure that the current output addr
	LD	DE,L09F4	;is PRINT_OUT.
	LD	(HL),E
	INC	HL
	LD	(HL),D
	LD	(IY+52H),1;SCR_CT ;Reset the scroll counter.
	LD	BC,1821H	;As the upper part of the display is being
				;handled the 'upper print line' will be
				;line '0'. Continue into CL_SET.

;	THE 'CL_SET' SUBROUTINE
;This subr is entered with the BC reg pair holding the line and column
;numbers of a char areas, or the C reg holding the column number within the
;printer buffer. The appropriate address of the 1st char bit is then found.
;The subroutine returns via PO_STORE so as to store all the values in the
;required system variables.
L0DD9:;CL_SET
	LD	HL,L5B00	;The start of the printer buffer.
	BIT	1,(IY+01H);FLAGS;Jump forward if handling the
	JR	NZ,L0DF4;CL_SET_2 ;printer buffer.
	LD	A,B		;Transfer the line number.
	BIT	0,(IY+02H);TV_FLAG ;Jump forward if handling the
	JR	Z,L0DEE;CL_SET_1   ;main part of the display.
	ADD	A,(IY+31H);DF_SZ   ;The top line of the lower part part
	SUB	18H		;of the display is called 'line 18' and
				;this has to be converted.
L0DEE:;CL_SET_1
	PUSH	BC		;The line & column numbers are saved.
	LD	B,A		;The line number is moved.
	CALL	L0E9B;CL_ADDR	;The addr for the start of the
				;line is formed in HL.
	POP	BC		;The line & column numbers are fetched back.
L0DF4:;CL_SET_2
	LD	A,21H		;The column number is now
	SUB	C		;reversed and transferred to
	LD	E,A		;the DE reg pair.
	LD	D,0
	ADD	HL,DE		;The required addr is now formed; and the
	JP	L0ADC;PO_STORE	;addr and the line and column numbers are
				;stored by jumping to PO_STORE.

;	THE 'SCROLLING' SUBROUTINE
;The number of lines of the display that are to be scrolled has to be held,
;on entry to the main subroutine, in the B reg.
L0DFE:;CL_SC_ALL
	LD	B,17H		;The entry point after 'scroll?'

;The main entry point - from above and when scrolling for INPUT_AT.
L0E00:;CL_SCROLL
	CALL	L0E9B;CL_ADDR	;Find the starting addr of the line.
	LD	C,8		;There are 8 pixel lines to a complete line.

;Now enter the main scrolling loop. The B reg holds the number of the
;top line to be scrolled, the HL reg pair the starting addr in the display
;area of this line and the C reg the pixel line counter.
L0E05:;CL_SCR_1
	PUSH	BC		;Save both counters.
	PUSH	HL		;Save the starting addr.
	LD	A,B		;Jump forward unless
	AND	7		;dealing at the present
	LD	A,B		;moment with a '3rd' of
	JR	NZ,L0E19;CL_SCR_3 ;the display.

;The pixel lines of the top lines of the '3rds' of the display have to be
;moved across the 2K boundaries. (Each '3rd' = 2K.)
L0E0D:;CL_SCR_2
	EX	DE,HL		;The result of this
	LD	HL,LF8E0	;manipulation is to leave HL
	ADD	HL,DE		;unchanged and DE pointing to
	EX	DE,HL		;the required destination.
	LD	BC,20H		;The are 20H chars.
	DEC	A		;Decrease the counter as one line is
				;being dealt with.
	LDIR			;Now move the thirty two bytes.

;The pixel lines within the '3rds' can now be scrolled. The A reg holds,
;on the 1st pass, 01-07, 09-0F or 11-17.
L0E19:;CL_SCR_3
	EX	DE,HL		;Again DE is made to point
	LD	HL,0FFE0H	;to the required destination.
	ADD	HL,DE		;This time only 32
	EX	DE,HL		;locs away.
	LD	B,A		;Save the line number in B.
	AND	7		;Now find how many chars
	RRCA			;there are remaining in the '3rd'.
	RRCA
	RRCA
	LD	C,A		;Pass the 'char total' to the C reg.
	LD	A,B		;Fetch the line number.
	LD	B,0		;BC holds the 'char total' and a pixel line
	LDIR			;from each of the chars is 'scrolled'.
	LD	B,7		;Now prepare to increment the addr to jump
				;across a '3rd' boundary.
	ADD	HL,BC		;Increase HL by 0700.
	AND	0F8H		;Jump back if there are any
	JR	NZ,L0E0D;CL_SCR_2 ;'3rds' left to consider.

;Now find if the loop has been used 8 times - once for each pixel line.
	POP	HL		;Fetch the original addr.
	INC	H		;Address the next pixel line.
	POP	BC		;Fetch the counters.
	DEC	C		;Decrease the pixel line counter
	JR	NZ,L0E05;CL_SCR_1 ;and jump back unless 8 lines
				;have been moved.

;Next the attribute bytes are scrolled. Note that the B reg still holds the
;number of lines to be scrolled and C reg holds zero.
	CALL	L0E88;CL_ATTR	;The required addr in the attribute area
				;and the number of chars in 'B' lines
				;are found.
	LD	HL,0FFE0H	;The displacement for all
	ADD	HL,DE		;the attribute bytes is
	EX	DE,HL		;32 locs away.
	LDIR			;The attribute bytes are 'scrolled'.

;It remains now to clear the bottom line of the display.
	LD	B,1		;The B reg is loaded with 01 and
				;CL_LINE is entered.

;	THE 'CLEAR LINES' SUBROUTINE
;This subroutine will clear the bottom 'B' lines of the display.
L0E44:;CL_LINE
	PUSH	BC		;The line number is saved for the
				;duration of the subroutine.
	CALL	L0E9B;CL_ADDR	;The starting addr for the line
				;is formed in HL.
	LD	C,8		;Again there are eight pixel
				;lines to be considered.
L0E4A:;CL_LINE_1
	PUSH	BC		;Save the line number and the
				;pixel line counter.
	PUSH	HL		;Save the addr.
	LD	A,B		;Save the line number in A.
L0E4D:;CL_LINE_2
	AND	7		;Find how many chars are
	RRCA			;involved in 'B mod 8' lines.
	RRCA			;Pass the result to the
	RRCA			;C reg. (C will hold 00
	LD	C,A		;i.e. 256 dec. for a '3rd'.)
	LD	A,B		;Fetch the line number.
	LD	B,0		;Make the BC reg pair hold 'one less'
	DEC	C		;than the number of chars.
	LD	D,H		;Make DE point to the 1st char.
	LD	E,L
	LD	(HL),0		;Clear the pixel-byte of the 1st char.
	INC	DE		;Make DE point to the 2nd char and then
	LDIR			;clear the pixel-bytes of all the other
	LD	DE,0701H	;chars. For each '3rd' of the display
	ADD	HL,DE		;HL has to be increased by 0701.
	DEC	A		;Now decrease the line number.
	AND	0F8H		;Discard any extra lines and pass
	LD	B,A		;the '3rd' count to B.
	JR	NZ,L0E4D;CL_LINE_2 ;Jump back if there are still
				;'3rds' to be dealt with.

;Now find if the loop has been used 8 times.
	POP	HL		;Update the addr for each
	INC	H		;pixel line.
	POP	BC		;Fetch the counters.
	DEC	C		;Decrease the pixel line counter and
	JR	NZ,L0E4A;CL_LINE_1 ;jump back unless finished.

;Next the attribute bytes are set as required. The value in ATTR_P will
;be used when handling the main part of the display and the value
;in BORDCR when handling the lower part.
	CALL	L0E88;CL_ATTR	;The addr of the 1st attribute byte and
				;the number of bytes are found.
	LD	H,D		;HL will point to the 1st attribute byte
	LD	L,E		;and DE the 2nd.
	INC	DE
	LD	A,(L5C8D)	;Fetch the value in ATTR_P.
	BIT	0,(IY+02H);TV_FLAG ;Jump forward if handling the
	JR	Z,L0E80;CL_LINE_3  ;main part of the screen.
	LD	A,(L5C48)	;Otherwise use BORDCR instead.
L0E80:;CL_LINE_3
	LD	(HL),A		;Set the attribute byte.
	DEC	BC		;One byte has been done. Now copy the
	LDIR			;value to all the attribute bytes.
	POP	BC		;Restore the line number.
	LD	C,21H		;Set the column number to the lefthand
	RET			;column and return.

;	THE 'CL_ATTR' SUBROUTINE
;This subroutine has two separate functions:
;
;i.   For a given display area address the appropriate attribute addr is
;     returned in the DE reg pair. Note that the value on entry points to
;     the '9th' line of a char.
;ii.  For a given number, in the B reg, the number of char areas in the
;     display from the start of that line onwards is returned in the
;     BC reg pair.
L0E88:;CL_ATTR
	LD	A,H		;Fetch the high byte.
	RRCA			;Multiply this value by 32.
	RRCA
	RRCA
	DEC	A		;Go back to the 'eight' line.
	OR	50H		;Address the attribute area.
	LD	H,A		;Restore to the high byte and
	EX	DE,HL		;transfer the addr to DE.
	LD	H,C		;This is always zero.
	LD	L,B		;The line number.
	ADD	HL,HL		;Multiply by 32.
	ADD	HL,HL
	ADD	HL,HL
	ADD	HL,HL
	ADD	HL,HL
	LD	B,H		;Move the result to the
	LD	C,L		;BC reg pair before
	RET			;returning.

;	THE 'CL_ADDR' SUBROUTINE
;For a given line number, in the B reg, the appropriate display file
;addr is formed in the HL reg pair.
L0E9B:;CL_ADDR
	LD	A,18H		;The line number has to be
	SUB	B		;reversed.
	LD	D,A		;The result is saved in D.
	RRCA			;In effect '(A mod 8)*32'.
	RRCA			;In a '3rd' of the display
	RRCA			;the low byte for the:
	AND	0E0H		;1st. line = 00,
				;2nd. line = 20, etc.
	LD	L,A		;The low byte goes into L.
	LD	A,D		;The true line number is fetched.
	AND	18H		;In effect '64+8*int(A/8)'.
	OR	40H		;For the upper '3rd' of the display the
				;high byte=40, middle '3rd'=48, and the
				;lower '3rd'=50.
	LD	H,A		;The high byte goes to H.
	RET			;Finished.

;	THE 'COPY' COMMAND ROUTINE
;The one hundred and seventy six pixel lines of the display are dealt
;with one by one.
L0EAC:;COPY
	DI			;The maskable interrupt is disabled
				;during COPY.
	LD	B,0B0H		;The '176' lines.
	LD	HL,L4000	;The base addr of the display.

;The following loop is now entered.
L0EB2:;COPY_1
	PUSH	HL		;Save the base addr and
	PUSH	BC		;the number of the line.
	CALL	L0EF4;COPY_LINE	;It is called '176' times.
	POP	BC		;Fetch the line number and
	POP	HL		;the base addr.
	INC	H		;The base addr is updated by '256' locs
				;for each line of pixels.
	LD	A,H		;Jump forward and hence round
	AND	7		;the loop again directly for the
	JR	NZ,L0EC9;COPY_2	;8 pixel lines of a char line.

;For each new line of chars the base addr has to be updated.
	LD	A,L		;Fetch the low byte.
	ADD	A,20H		;Update it by 20H bytes.
	LD	L,A		;The carry flag will be reset when
				;'within 3rds' of the display.
	CCF			;Change the carry flag.
	SBC	A,A		;The A reg will hold F8 when within a '3rd'
	AND	0F8H		;but 00 when a new '3rd' is reached.
	ADD	A,H		;The high byte of the
	LD	H,A		;addr is now updated.
L0EC9:;COPY_2
	DJNZ	L0EB2		;Jump back until '176' lines have
				;been printed.
	JR	L0EDA;COPY_END	;Jump forward to the end routine.

;	THE 'COPY_BUFF' SUBROUTINE
L0ECD:;COPY_BUFF
	DI			;Disable the maskable interrupt.
	LD	HL,L5B00	;The base addr of the printer buffer.
	LD	B,8		;There are 8 pixel lines.
L0ED3:;COPY_3
	PUSH	BC		;Save the line number.
	CALL	L0EF4;COPY_LINE	;It is called '8' times.
	POP	BC		;Fetch the line number.
	DJNZ	L0ED3;COPY_3	;Jump back until '8' lines have been printed.

;Continue into the COPY_END routine.
L0EDA:;COPY_END
	LD	A,4		;Stop the printer motor.
	OUT	(0FBH),A
	EI			;Enable the maskable interrupt and
				;continue into CLEAR_PRB.

;	THE 'CLEAR PRINTER BUFFER' SUBROUTINE
;The printer buffer is cleared by calling this subroutine.
L0EDF:;CLEAR_PRB
	LD	HL,L5B00	;The base addr of the printer buffer.
	LD	(IY+46H),L;PR_CC;Reset the printer 'column'.
	XOR	A		;Clear the A reg.
	LD	B,A		;Alse clear the B reg (in effect B
				;holds dec. 256).
L0EE7:;PRB_BYTES
	LD	(HL),A		;The '256' bytes of the
	INC	HL		;printer buffer are still
	DJNZ	L0EE7;PRB_BYTES	;cleared in turn.
	RES	1,(IY+30H);FLAGS2 ;Signal 'the buffer is empty'.
	LD	C,21H		;Set the printer position and
	JP	L0DD9;CL_SET	;return via CL_SET & PO_STORE.

;	THE 'COPY_LINE' SUBROUTINE
;The subroutine is entered with the HL reg pair holding the base addr
;of the 32 bytes that form the pixel-line and the B reg holding the
;pixel-line number.
L0EF4:;COPY_LINE
	LD	A,B		;Copy the pixel-line number.
	CP	3		;The A reg will hold
	SBC	A,A		;00 until the last two lines
	AND	2		;are being handled.
	OUT	(0FBH),A	;Slow the motor for the last two
				;pixel lines only.
	LD	D,A		;The D reg will hold either 00 or 02.

;There are three tests to be made before doing any 'printing'.
L0EFD:;COPY_L_1
	CALL	L1F54;BREAK_KEY	;Jump forward unless the
	JR	C,L0F0C;COPY_L_2;BREAK key is being pressed.
	LD	A,4		;But if it is then:
	OUT	(0FBH),A	;stop the motor,
	EI			;enable the maskable interrupt,
	CALL	L0EDF;CLEAR_PRB	;clear the printer buffer and exit
	RST	8;ERROR_1	;via the error handling routine
	DB	0CH		;- 'BREAK-CONT repeats'.
L0F0C:;COPY_L_2
	IN	A,(0FBH)	;Fetch the status of the printer.
	ADD	A,A		;Make an immediate return if the
	RET	M		;printer is not present.
	JR	NC,L0EFD;COPY_L_1 ;Wait for the stylos.
	LD	C,20H		;There are 32 bytes.

;Now enter a loop to handle these bytes.
L0F14:;COPY_L_3
	LD	E,(HL)		;Fetch a byte.
	INC	HL		;Update the pointer.
	LD	B,8		;8 bits per byte.
L0F18:;COPY_L_4
	RL	D		;Move D left.
	RL	E		;Move each bit into the carry.
	RR	D		;Move D back again, picking up
				;the carry from E.
L0F1E:;COPY_L_5
	IN	A,(0FBH)	;Again fetch the status of the
	RRA			;printer and wait for the
	JR	NC,L0F1E;COPY_L_5 ;signal from the encoder.
	LD	A,D		;Now go ahead and pass the
	OUT	(0FBH),A	;'bit' to the printer:
			;Note: bit 2 - low starts the motor
			;      bit 1 - high slows the motor
			; and  bit 7 is high for the actual 'printing'.
	DJNZ	L0F18;COPY_L_4	;'Print' each bit.
	DEC	C		;Decrease the byte counter.
	JR	NZ,L0F14;COPY_L_3 ;Jump back whilst there are
	RET			;still bytes; otherwise return.

