

;	THE RESTART ROUTINES and THE TABLES

;	THE 'START'
;The maskable interrupt is disabled and the DE reg pair set to hold the
;'top of possible RAM'.
L0000:;START
	DI			;Disable the 'keyboard interrupt'.
	XOR	A		;00 for 'start' (0FFh for 'NEW')
	LD	DE,0FFFFH	;Top of possible RAM.
	JP	L11CB;START/NEW	;Jump forward.

;	THE 'ERROR' RESTART
;The error pntr is made to point to the position of the error.
L0008:;ERROR_1	
	LD	HL,(L5C5D);CH_ADD ;The addr reached by the interpreter
	LD	(L5C5F),HL;X_PTR ;is copied to the error pntr before
	JR	L0053;ERROR_2	;proceeding.

;	THE 'PRINT A CHARACTER' RESTART
;The A reg holds the code of the char that is to be printed.
L0010:;PRINT_A_1
	JP	L15F2;PRINT_A_2	;Jump forward immediately.
	DB	0FFH,0FFH,0FFH,0FFH,0FFH

;	THE 'COLLECT CHARACTER' RESTART
;The contents of the location currently addressed by CH_ADD are fetched.
;A return is made if the value represents a printable char, otherwise
;CH_ADD is incremented and the tests repeated.
L0018:;GET_CHAR
	LD	HL,(L5C5D);CH_ADD ;Fetch the value that is addressed
	LD	A,(HL)		;by CH_ADD.
L001C:;TEST_CHAR
	CALL	L007D;SKIP_OVER	;Find out if the char is printable.
	RET	NC		;Return if it is so.

;	THE 'COLLECT NEXT CHARACTER' RESTART
;As a BASIC line is interpreted, this routine is called repeatedly to step
;along the line.
L0020:;NEXT_CHAR
	CALL	L0074;CH_ADD+1	;CH_ADD needs to be incremented.
	JR	L001C;TEST_CHAR	;Jump back to test the new value.
	DB	0FFH,0FFH,0FFH

;	THE 'CALCULATOR' RESTART
;The flpnt calculator is entered at 335B.
L0028:;FP_CALC
	JP	L335B;CALCULATE	;Jump forward immediately.
	DB	0FFH,0FFH,0FFH,0FFH,0FFH

;	THE 'MAKE BC SPACES' RESTART
;This routine creates free locs in the work space. The number of
;locs is determined by the current contents of the BC reg pair.
L0030:;BC_SPACES
	PUSH	BC		;Save the 'number'.
	LD	HL,(L5C61);WORKSP ;Fetch the present addr of the start
	PUSH	HL		;of thework space and save that also
	JP	L169E;RESERVE	;before proceeding.

;	THE 'MASKABLE INTERRUPT' ROUTINE
;The real time clock is incremented and the keyboard scanned whenever
;a maskable interrupt occurs.
L0038:;MASK_INT
	PUSH	AF		;Save the current values held in
	PUSH	HL		;these regs.
	LD	HL,(L5C78);FRAMES ;The lower two bytes of the frame counter
	INC	HL		;are incremented every 20ms. (U.K.)
	LD	(L5C78),HL;FRAMES
	LD	A,H		;The highest byte of the frame counter
	OR	L		;is only incremented when the value
	JR	NZ,L0048;KEY_INT;of the lower two bytes is zero.
	INC	(IY+40H);FRAMES_3
L0048:;KEY_INT
	PUSH	BC
	PUSH	DE
	CALL	L02BF;KEYBOARD	;Scan the keyboard.
	POP	DE		;Restore the values.
	POP	BC
	POP	HL
	POP	AF
	EI			;The maskable interrupt is enabled
	RET			;before returning.

;	THE 'ERROR_2' ROUTINE
;The return addr to the interpreter points to the 'DB' that signifies
;which error has occured. This 'DB' is fetched and transferred to ERR_NR.
;The machine stack is cleared before jumping forward to clear the calc
;stack.
L0053:;ERROR_2
	POP	HL		;The address on the stack points to
	LD	L,(HL)		;the error code.
L0055:;ERROR_3
	LD	(IY+0),L;ERR_NR	;It is transferred to ERR_NR.
	LD	SP,(L5C3D);ERR_SP ;The machine is cleared before
	JP	L16C5;SET_STK	;exiting via SET_STK.
	DB	0FFH,0FFH,0FFH
	DB	0FFH,0FFH,0FFH,0FFH

;	THE 'NON-MASKABLE INTERRUPT' ROUTINE
;This routine isn't used in the standard Spectrum but the code allows for a
;system reset to occur following activation of the NMI line. The system var
;NMIADD has to have the value zero for the reset to occur.
L0066:;RESET
	PUSH	AF
	PUSH	HL
	LD	HL,(L5CB0);NMIADD ;The two bytes of NMIADD must
	LD	A,H		;both be zero for the reset
	OR	L		;to occur.
	JR	NZ,L0070;NO_RESET ;Note: This should have been 'JR Z'!
	JP	(HL)		;Jump to START.
L0070:;NO_RESET
	POP	HL
	POP	AF
	RETN

;	THE 'CH_ADD+1' SUBROUTINE
;The addr held in CH_ADD is fetched, incremented and restored. The contents
;of the loc now addressed by CH_ADD is fetched. The entry points of
;TEMP_PTR1 and TEMP_PTR2 are used to set CH_ADD for a temporary period.
L0074:;CH_ADD+1
	LD	HL,(L5C5D);CH_ADD ;Fetch the addr.
L0077:;TEMP_PTR1
	INC	HL		;Increment the pntr.
L0078:;TEMP_PTR2
	LD	(L5C5D),HL	;Set CH_ADD.
	LD	A,(HL)		;Fetch the addressed value
	RET			;and then return.

;	THE 'SKIP-OVER' SUBROUTINE
;The value brought to the subr in the A reg is tested to see if it is
;printable. Various special codes lead to HL being incremented once,
;or twice, and CH_ADD amended accordingly.
L007D:;SKIP_OVER
	CP	'!'		;Return with the carry flag reset if
	RET	NC		;ordinary char code.
	CP	0DH		;Return if the end of the line
	RET	Z		;has been reached.
	CP	10H		;Return with codes 00 to 0Fh
	RET	C		;but with carry set.
	CP	18H		;Return with codes 18h to 20h
	CCF			;again with carry set.
	RET	C
	INC	HL		;Skip-over once.
	CP	16H		;Jump forward with codes 10h to 15h
	JR	C,L0090;SKIPS	;(INK to OVER).
	INC	HL		;Skip-over once more (AT & TAB).
L0090:;SKIPS
	SCF			;Return with the carry flag set and
	LD	(L5C5D),HL;CH_ADD ;CH_ADD holding the appropriate addr.
	RET

;	THE TOKEN TABLE
;All the tokens used by the Spectrum are expanded by reference to this
;table. The last code of each token is 'inverted' by having its bit 7 set.
x	EQU	80h
L0095:	
	DB	'?'+x
	DB	'RN','D'+x	;A5
	DB	'INKEY','$'+x	;A6
	DB	'P','I'+x	;A7
	DB	'F','N'+x	;A8
	DB	'POIN','T'+x	;A9
	DB	'SCREEN','$'+x	;AA
	DB	'ATT','R'+x	;AB
	DB	'A','T'+x	;AC
	DB	'TA','B'+x	;AD
	DB	'VAL','$'+x	;AE
	DB	'COD','E'+x	;AF
	DB	'VA','L'+x	;B0
	DB	'LE','N'+x	;B1
	DB	'SI','N'+x	;B2
	DB	'CO','S'+x	;B3
	DB	'TA','N'+x	;B4
	DB	'AS','N'+x	;B5
	DB	'AC','S'+x	;B6
	DB	'AT','N'+x	;B7
	DB	'L','N'+x	;B8
	DB	'EX','P'+x	;B9
	DB	'IN','T'+x	;BA
	DB	'SQ','R'+x	;BB
	DB	'SG','N'+x	;BC
	DB	'AB','S'+x	;BD
	DB	'PEE','K'+x	;BE
	DB	'I','N'+x	;BF
	DB	'US','R'+x	;C0
	DB	'STR','$'+x	;C1
	DB	'CHR','$'+x	;C2
	DB	'NO','T'+x	;C3
	DB	'BI','N'+x	;C4
	DB	'O','R'+x	;C5
	DB	'AN','D'+x	;C6
	DB	'<','='+x	;C7
	DB	'>','='+x	;C8
	DB	'<','>'+x	;C9
	DB	'LIN','E'+x	;CA
	DB	'THE','N'+x	;CB
	DB	'T','O'+x	;CC
	DB	'STE','P'+x	;CD
	DB	'DEF F','N'+x	;CE
	DB	'CA','T'+x	;CF
	DB	'FORMA','T'+x	;D0
	DB	'MOV','E'+x	;D1
	DB	'ERAS','E'+x	;D2
	DB	'OPEN ','#'+x	;D3
	DB	'CLOSE ','#'+x	;D4
	DB	'MERG','E'+x	;D5
	DB	'VERIF','Y'+x	;D6
	DB	'BEE','P'+x	;D7
	DB	'CIRCL','E'+x	;D8
	DB	'IN','K'+x	;D9
	DB	'PAPE','R'+x	;DA
	DB	'FLAS','H'+x	;DB
	DB	'BRIGH','T'+x   ;DC
	DB	'INVERS','E'+x	;DD
	DB	'OVE','R'+x	;DE
	DB	'OU','T'+x	;DF
	DB	'LPRIN','T'+x	;E0
	DB	'LLIS','T'+x	;E1
	DB	'STO','P'+x	;E2
	DB	'REA','D'+x	;E3
	DB	'DAT','A'+x	;E4
	DB	'RESTOR','E'+x	;E5
	DB	'NE','W'+x	;E6	;A
	DB	'BORDE','R'+x	;E7	;B
	DB	'CONTINU','E'+x	;E8	;C
	DB	'DI','M'+x	;E9	;D
	DB	'RE','M'+x	;EA	;E
	DB	'FO','R'+x	;EB	;F
	DB	'GO T','O'+x	;EC	;G
	DB	'GO SU','B'+x	;ED	;H
	DB	'INPU','T'+x	;EE	;I
	DB	'LOA','D'+x	;EF	;J
	DB	'LIS','T'+x	;F0	;K
	DB	'LE','T'+x	;F1	;L
	DB	'PAUS','E'+x	;F2	;M
	DB	'NEX','T'+x	;F3	;N
	DB	'POK','E'+x	;F4	;O
	DB	'PRIN','T'+x	;F5	;P
	DB	'PLO','T'+x	;F6	;Q
	DB	'RU','N'+x	;F7	;R
	DB	'SAV','E'+x	;F8	;S
	DB	'RANDOMIZ','E'+x;F9	;T
	DB	'I','F'+x	;FA	;U
	DB	'CL','S'+x	;FB	;V
	DB	'DRA','W'+x	;FC	;W
	DB	'CLEA','R'+x	;FD	;X
	DB	'RETUR','N'+x	;FE	;Y
	DB	'COP','Y'+x	;FF	;Z

;	THE KEY TABLES
;There are six separate key tables. The final char code obtained depends of
;the particular key pressed and the 'mode' been used.

L0205:;TABA		;The main key table - L mode and CAPS SHIFT.
	DB	'BHY65TGV'
	DB	'NJU74RFC'
	DB	'MKI83EDX'
	DB	0EH,'LO92WSZ'		;0E - SYMBOL SHIFT
	DB	' ',0DH,'P01QA'		;0D - CR

L022C:;TABB		;Extended mode. Letter keys and unshifted
	DB	0E3H,0C4H,0E0H,0E4H	;READ	BIN	LPRINT	DATA
	DB	0B4H,0BCH,0BDH,0BBH	;TAN	SGN	ABS	SQR
	DB	0AFH,0B0H,0B1H,0C0H	;CODE	VAL	LEN	USR
	DB	0A7H,0A6H,0BEH,0ADH	;PI	INKEY$	PEEK	TAB
	DB	0B2H,0BAH,0E5H,0A5H	;SIN	INT	RESTORE	RND
	DB	0C2H,0E1H,0B3H,0B9H	;CHR$	LLIST	COS	EXP
	DB	0C1H,0B8H		;STR$	LN

L0246:;TABC		;Extended mode. Letter keys and either shift
	DB	'~',0DCH,0DAH,'\'	;~	BRIGHT	PAPER	\
	DB	0B7H,'{}',0D8H		;ATN	{	}	CIRCLE
	DB	0BFH,0AEH,0AAH,0ABH	;IN	VAL$	SCREEN$	ATTR
	DB	0DDH,0DEH,0DFH,7FH	;INVERSE OVER	OUT	copyright
	DB	0B5H,0D6H,'|',0D5H	;ASN	VERIFY	|	MERGE
	DB	']',0DBH,0B6H,0D9H	;]	FLASH	ACS	INK
	DB	'[',0D7H		;[	BEEP

L0260:;TABD		;Control codes. Digit keys and CAPS SHIFT
	DB	0CH,7,6,4		;DELETE	EDIT	CAPS_LOCK TRUE_VIDEO
	DB	5,8,0AH,0BH		;INV_VIDEO left down	up
	DB	9,0FH			;right	GRAPHICS

L026A:;TABE		;Symbol codes. Letter keys and symbol shift
	DB	0E2H,'*?',0CDH		;STOP	*	?	STEP
	DB	0C8H,0CCH,0CBH,'^'	;>=	TO	THEN	^
	DB	0ACH,'-+='		;AT	-	+	=
	DB	'.,;"'			;.	,	;	"
	DB	0C7H,'<',0C3H,'>'	;<=	<	NOT	>
	DB	0C5H,'/',0C9H,'`'	;OR	/	<>	`
	DB	0C6H,':'		;AND	:

L0284:;TABF		;Extended mode. Digit keys and symbol shift
	DB	0D0H,0CEH,0A8H,0CAH	;FORMAT	DEF FN	FN	LINE
	DB	0D3H,0D4H,0D1H,0D2H	;OPEN#	CLOSE#	MOVE	ERASE
	DB	0A9H,0CFH		;POINT	CAT

